<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: be576db4 (2024-10-06 12:37:19)
// 

use ArfaCMS\Core\BaseObject;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Util;
use ArfaCMS\DB\Database;

defined('_ARFACMS_VALID') or die("Access denied");

class Installer extends BaseObject
{

    //---------- Singleton implementation ------------
    private static $_instance = null;
    private $_delimiter = null;

    private static $_ver_revision = 0;
    private static $_ver_build = 0;
    private static $_ver_build_date = "";

    public static function createInstance()
    {
        if (self::$_instance == null) {
            self::$_instance = new self();
        }
    }

    public static function getInstance()
    {
        self::createInstance();
        return self::$_instance;
    }

    //------------------------------------------------

    private $_existedCfg = "";

    private function __construct()
    {
        $this->initObj();
        $this->initVersion();
        $this->initializeRegistry();
        $this->prepare();

    }

    private function initVersion()
    {
        // Шаблоны переменных заменяются при создании дистрибутива
        self::$_ver_revision = intval("3661");
        self::$_ver_build = intval("3661");
        self::$_ver_build_date = "2025-01-21 19:11:35";
    }

    private function initializeRegistry()
    {
        $returnUrl = Request::getSafe('return_url', '');
        if ($returnUrl != '') $returnUrl = base64_decode($returnUrl);
        else $returnUrl = 'index.php';
        $step = Request::getInt('step', 0);
        $this->set('returnUrl', $returnUrl, true);
        $this->set('step', $step, true);
    }

    private function prepare()
    {
        //
    }

    private function initDB()
    {   $dbHost= Request::getSafe('dbHost', '');
        \DatabaseConfig::$dbHost = $dbHost;
        $dbPort = Request::getSafe('dbPort', '');
        \DatabaseConfig::$dbPort = $dbPort;
        $dbUser = Request::getSafe('dbUser', '');
        \DatabaseConfig::$dbUser = $dbUser;
        \DatabaseConfig::$dbPassword = Request::getSafe('dbPassword', '');
        $dbName = Request::getSafe('dbName', '');
        \DatabaseConfig::$dbName = $dbName;
        $dbPrefix = Request::getSafe('dbPrefix', '');
        \DatabaseConfig::$dbPrefix = $dbPrefix;


        $db = Database::getInstance(false);

        if ($db->isConnected() == false) {
            Util::redirect("install.php?step=2&fail=1&dbHost=".$dbHost."&dbPort=".$dbPort."&dbUser=".$dbUser."&dbName=".$dbName."&dbPrefix=".$dbPrefix);
        }
        $this->_delimiter = $db->getDelimiter();
        return $db;
    }

    private function populateDatabase($db, $sqlfile, &$err_array)
    {
        if (!($buffer = file_get_contents($sqlfile))) return false;
//		$queries = preg_split ("/[".self::$query_delimiter."]+/", $buffer);
        $queries = preg_split("/" . $this->_delimiter . "/", $buffer);
        foreach ($queries as $query) {
            $query = trim($query);
            if ($query != '' && $query[0] != '#') {
                $db->setQuery($query);
                if (!$db->query()) {
                    $err_array[] = "<p class=\"sql-error\">" . "SQL Error :" . $db->getLastError() . "</p>" . $db->getQuery();
                    return false;
                }
            }
        }
        return true;
    }

    private function getLicense()
    {
        $licPath = PATH_MODULES . 'help' . DS . 'views' . DS . 'admin' . DS . 'layouts' . DS . 'default' . DS . 'license.php';
        require_once $licPath;
    }

    private function writeConfigToDatabase($db)
    {
        $sql = "INSERT INTO #__config VALUES ('site','siteDomain','" . Request::getSafe('siteDomain', '') . "')";
        $db->setQuery($sql);
        if (!$db->query()) return false;
        $sql = "INSERT INTO #__config VALUES ('site','sitePort','" . Request::getInt('sitePort', '') . "')";
        $db->setQuery($sql);
        if (!$db->query()) return false;
        $sql = "INSERT INTO #__config VALUES ('site','siteSSLPort','" . Request::getInt('siteSSLPort', '') . "')";
        $db->setQuery($sql);
        if (!$db->query()) return false;
        $sql = "INSERT INTO #__config VALUES ('site','metaTitle','" . stripcslashes(Request::getSafe('siteTitle', '')) . "')";
        $db->setQuery($sql);
        if (!$db->query()) return false;
        $sql = "INSERT INTO #__config VALUES ('site','siteTemplate','" . Request::getSafe('siteTemplate', 'html5') . "')";
        $db->setQuery($sql);
        if (!$db->query()) return false;
        return true;
    }

    private function writeConfig()
    {
        // database.php
        $siteCfgPath = PATH_CONFIG . 'site.php';
        $databaseCfgPath = PATH_CONFIG . 'database.php';
        $buffer = '<?php defined("_ARFACMS_VALID") or die("Access denied");
class DatabaseConfig {
	// Database configuration
	public static	' . '$' . 'dbHost		= "' . \DatabaseConfig::$dbHost . '";
	public static	' . '$' . 'dbPort		= "' . \DatabaseConfig::$dbPort . '";
	public static	' . '$' . 'dbName		= "' . \DatabaseConfig::$dbName . '";
	public static	' . '$' . 'dbUser		= "' . \DatabaseConfig::$dbUser . '";
	public static	' . '$' . 'dbPassword	= "' . \DatabaseConfig::$dbPassword . '";
	public static	' . '$' . 'dbPrefix		= "' . \DatabaseConfig::$dbPrefix . '";
	public static	' . '$' . 'dbSecret		= "' . \DatabaseConfig::$dbSecret . '";
	}
?>';
        $f = fopen($databaseCfgPath, "wt");
        if ($f) {
            fwrite($f, $buffer);
            fclose($f);
        } else return false;

        // site.php
        $buffer = file_get_contents($siteCfgPath);
        $buffer = str_replace('###SITE_DOMAIN###', Request::getSafe('siteDomain', ''), $buffer);
        $buffer = str_replace('###SITE_PORT###', Request::getInt('sitePort', ''), $buffer);
        $buffer = str_replace('###SITE_SSL_PORT###', Request::getInt('siteSSLPort', ''), $buffer);
        $buffer = str_replace('###SITE_TITLE###', stripcslashes(Request::getSafe('siteTitle', '')), $buffer);
        $buffer = str_replace('###SITE_TEMPLATE###', Request::getSafe('siteTemplate', 'html5'), $buffer);
        $f = fopen($siteCfgPath, "wt");
        if ($f) {
            fwrite($f, $buffer);
            fclose($f);
        } else return false;
        return true;
    }

    public function render()
    {
        $step = $this->get('step');
        ob_start();
        require_once PATH_INSTALL . 'wizard' . DS . 'step' . $step . '.php';
        $stepHTML = ob_get_contents();
        ob_end_clean();

        require_once PATH_INSTALL . 'template.php';
    }

    public static function getVersionBuild()
    {
        return self::$_ver_build;
    }

    public static function getVersion()
    {
        return Portal::getVersionMajor() . "." . Portal::getVersionMinor() . "." . intval(self::$_ver_revision);
    }

    public static function getVersionBuildDate()
    {
        return self::$_ver_build_date;
    }

    public static function getVersionRevision()
    {
        return self::$_ver_revision;
    }
}

?>