<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 680a4453 (2023-07-28 15:46:25)
// 

use ArfaCMS\Core\Request;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\User;
use ArfaCMS\Core\Util;
use ArfaCMS\MVC\Module;
use ArfaCMS\Sprav\SpravController;

defined('_ARFACMS_VALID') or die("Access denied");

class aclmgrControllerDefault extends SpravController {
	// работа с правами
	public function showRules() {
		$view = $this->getView();
		
		$roleId = Request::getInt('roleid',1);
		
		$message=array();
		if(Module::getHelper("acl", "aclmgr")->checkModulesAccess($message)){ // True if all rules exists and role has access for all modules.
			$message=array(); // reset messages if everything OK
			if(Module::getHelper("acl", "aclmgr")->checkModulesAcl($message)) $message=array();  // reset messages if everything OK
		} elseif(User::getInstance()->getRole() != $roleId) {
			 Util::redirect("index.php?module=aclmgr&view=roles", Text::_("Check your role module access rules first")); 
		}
		$view->assign('message',$message);
		
		$roleModel = $this->getModel('roles');
		$ruleModel = $this->getModel('rules');

		$tabId = Request::getSafe('tabid','frontend');

		$objectsFE = $ruleModel->getObjects(0);
		$objectsBE = $ruleModel->getObjects(1);
		$rulesFE = $ruleModel->getRules($roleId,$objectsFE);
		$rulesBE = $ruleModel->getRules($roleId,$objectsBE);

		foreach ($rulesFE as $rf) { $objectsFE[$rf->acl_object_id]->canAccess = $rf->acl_access; }
		foreach ($rulesBE as $rb) { $objectsBE[$rb->acl_object_id]->canAccess = $rb->acl_access; }

		$view->assign('roleName',$roleModel->getRoleName($roleId));
		$view->assign('roleId',$roleId);
		$view->assign('tabId',$tabId);
		$view->assign('objectsFE',$objectsFE);
		$view->assign('objectsBE',$objectsBE);
	}
	
	public function update() {
		$roleId = Request::getInt('roleId',0);
		$access = Request::getSafe('access',array());
		$oldAccess = Request::getSafe('oldAccess',array());
		$tabId = Request::getSafe('tabId','');

		$model = $this->getModel('rules');
		foreach ($access as $objectId=>$state) {
			if ($state=="on")
				$model->setRule($objectId,$roleId,1);
		}
		foreach ($oldAccess as $objectId=>$state) {
			if (!array_key_exists($objectId,$access) && $state=="1")
				$model->setRule($objectId,$roleId,0);
		}

		$this->setRedirect("index.php?module=aclmgr&view=rules&roleid=".$roleId."&tabid=".$tabId,Text::_('ACL updated'));
	}
	// работа с ролями (override справочника)
	public function showRoles() {
		$this->showData();
	}
	
	public function delete() {
		$model = $this->getModel();
		$sr = $model->getSystemRoles();
		$dr = User::getDefaultRole();
        $psid = $this->getPsid();
		if ($psid) {
			if($dr==$psid) $this->setRedirect("index.php?module=aclmgr",Text::_('It is default role'));
			elseif(array_key_exists($psid, $sr)) $this->setRedirect("index.php?module=aclmgr",Text::_('It is system role'));
			else parent::delete();
		} else {
			$this->setRedirect("index.php?module=aclmgr",Text::_('Absent role'));
		}
	}
}
?>