<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 637ec2ba (2023-09-02 13:38:06)
// 

use ArfaCMS\Core\Date;
use ArfaCMS\Core\Event;
use ArfaCMS\Core\Registry;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Translator;
use ArfaCMS\Core\User;
use ArfaCMS\Core\Util;
use ArfaCMS\MVC\Controller;
use ArfaCMS\MVC\Module;

defined('_ARFACMS_VALID') or die("Access denied");

class articleControllerDefault extends Controller {
	private  $arrTranslateData=array();
	public function showRead($childs_order='title') {
		$page404	= Request::getBool('page404', false);
		if ($page404) $this->setRedirect('index.php',Text::_('Article not found'),404);
		
		$psid=$this->getPsid();
		$layout = $this->get('layout','read');
		$notmpl = Request::getInt('notmpl',0);
		$noctrl = Request::getInt('noctrl',$notmpl);
		$addbreadcrumb = Request::getBool('bc',true);
		$view = $this->getView();
		$model = $this->getModel('article');
		$view->assign("use_rating",$this->getConfigVal("use_rating"));
        $canModify = $this->checkACL("articleEditing",false);
        $view->assign('canModify', $canModify);
		if (!$psid) {
			if ( \siteConfig::$defaultModule == 'article') {
				$view->assign('art',false);
			} else {
				$this->setRedirect('index.php',Text::_('Article not found'),404); 
			}
		} else {
			// Article
			if ($canModify) $art = $model->getArticle($psid, 0, 0);
			else $art = $model->getArticle($psid);
			if (!$art) {
				if ($noctrl) { Util::halt(Text::_('Article not found')); }
				else {
					if (\siteConfig::$defaultModule == 'article' && $psid == $this->getConfigVal("Default_item_ID")) {
						$view->assign('art',false);
					} else $this->setRedirect('index.php',Text::_('Article not found'),404);
				}
			} else {
				Module::getInstance()->setBreadCrumbVisibility($art->a_show_breadcrumb);
				if ($art->a_show_childs == '1') $arrChilds=$model->getChildsArray($art->a_id, true, $art->a_childs_order_by, $art->a_childs_order_dir);
				else $arrChilds=array();
				$user = User::getInstance();
				$artUrl = 'index.php?module=article&amp;view=read&amp;psid='.$art->a_id;
				if ($art->a_alias) $artUrl.="&alias=".$art->a_alias;
				if (!$noctrl){
					if ($art->a_id != $this->getConfigVal("Default_item_ID")) {
						if($addbreadcrumb) $this->createUpBreadcrumb($view,$art->a_id);
					}
				}
				if ($art->a_meta_keywords) $view->setMeta("keywords",$art->a_meta_keywords);
				if ($art->a_meta_title) $view->setMeta("title",$art->a_meta_title);
				else $view->setMeta("title",$art->a_title);
				if ($art->a_meta_description) $view->setMeta("description",$art->a_meta_description);
				$authorProfileUrl = Router::_("index.php?module=user&amp;view=info&amp;psid=".$art->a_author_id);
				$view->assign("comm",$this->initComments($art->a_id));
				$view->assign('articleAuthor',$art->author);
				$view->assign('articleRating',$art->a_rating);
				$view->assign('canVote',$this->checkACL("articleVoting",false));
				$view->assign('articleAuthorProfileUrl',$authorProfileUrl);
				$view->assign('articleDate',$art->a_date);
				$articleTitle = $art->a_title;
				$view->assign('articleTitle',$articleTitle);
				$view->assign('articleHTML',$art->a_text);
				$view->assign('art',$art);
				$view->assign("notmpl",$notmpl);
				$view->assign('articleShowInfo',$art->a_show_info);
				$view->assign('showTitle',$art->a_show_title);
				$view->assign('arrChilds',$arrChilds);
			}
		}
		$this->set("layout","default", true);
		if ($layout) {$view->setLayout($layout);}
	}
	//
    public function saveComment() {
        $layout	= Request::getSafe('layout');
        if ($layout=="alias") $this->set("layout","",true);
        parent::saveComment();
    }
	public function ajaxVote() {
		$psid = Request::getInt('psid',0);
		$element = Request::getSafe("element",false);
		switch ($element) {
			case "object":
				if ($this->checkACL("articleVoting",false) && $this->getConfigVal("use_rating")){
					$model = $this->getModel('article');
					$psid = Request::getInt('psid',0);
					$author= $model->getAuthor($psid);
					$direction = Request::getSafe('dir',"up");
					$params = array("module"=>"article","element"=>"object","psid"=>$psid,"direction"=>$direction, "author"=>$author);
					$data_voted=Event::raise("rating.check",$params);  // content.rating
					if (!\siteConfig::$useMultiVote && $data_voted) {
						echo Text::_("Already voted");
					} else {
						if (User::checkFloodPoint()){
							echo Text::_("Flood found");
						} else {
							Event::raise("rating.vote",$params); // content.rating
							if ($direction=="up"){
								echo Text::_("Good");
							} else {
								echo Text::_("Bad");
							}
						}
					}
				}
		case "comment":
			parent::ajaxVote();
			break;
		default:
			return;
		break;
		}
	}
	/* Список статей */
	public function showList() 	{
		$model=$this->getModel('article');
		$view=$this->getView();
		$model->createPaginator($view,10);
		$psid = Request::getInt('psid',0);
		$list=$model->getArticles(!$this->checkACL("articleEditing", false), $psid);
		$view->assign('articles',$list);
		$art = false;
		if($psid) $art = $model->getArticle($psid);
		$view->assign('main_article',$art);
		
		$view->addBreadcrumb(Text::_("Main page"), Router::_("index.php"));
		$view->addBreadcrumb(Text::_("Articles"),"#");
		
	}
	public function createUpBreadcrumb($view,$psid) {
		$i=0;	$bc=array();
		$model = $this->getModel('article');
		$limitBread=$this->getConfigVal('breadcrumb_lenght');
		$arr=$model->getParentArticle($psid);
		if(count($arr)>0) {
			foreach($arr as $key=>$val) {
				$i++;
				$str=$val["title"];
				if($limitBread>0 && (mb_strlen($str) > $limitBread)) $bc[$i]['text']=mb_substr($str,0,$limitBread)."...";
				else $bc[$i]['text']=$val["title"];
				$bc[$i]['link']=Router::_("index.php?module=article&amp;view=read&amp;psid=".(int)$key."&amp;alias=".$val["alias"]);
			}
		}
		$i++;
		$breadcrumb_start=$this->getConfigVal('breadcrumb_start');
		$breadcrumb_start_link=$this->getConfigVal('breadcrumb_start_link');
		if($breadcrumb_start && $breadcrumb_start_link) {
			$bc[$i]['text']=Text::_($breadcrumb_start); $bc[$i]['link']=Router::_($breadcrumb_start_link);
		} else {
			$bc[$i]['text']=Text::_('Main page'); $bc[$i]['link']=Router::_("index.php");
		}
		$bc=array_reverse($bc);
		foreach($bc as $tkey=>$tval)	{
		  $view->addBreadcrumb($tval['text'],$tval['link']);
		}
	}
}
?>