<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 13da56d6 (2024-11-10 16:07:33)
// 

use ArfaCMS\Core\Date;
use ArfaCMS\Core\Event;
use ArfaCMS\Core\Files;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;
use ArfaCMS\MVC\Module;

defined('_ARFACMS_VALID') or die("Access denied");

echo "<div class=\"article-read article-wrapper\">";
if ($this->art) {
    echo Event::raise("share.article");
    if ($this->canModify) {
        echo "<div class=\"module-shortcuts\">";
        echo "	<a class=\"linkButton btn btn-primary\" rel=\"nofollow\" id=\"newArticle\" href=\"" . Router::_("/" . ADMIN_FOLDER . "/index.php?module=article&amp;view=items&amp;task=modify") . "\">" . Text::_('New article') . "</a>";
        echo "	<a class=\"linkButton btn btn-primary\" rel=\"nofollow\" id=\"newArticle\" target=\"_blank\" href=\"" . Router::_("/" . ADMIN_FOLDER . "/index.php?module=article&amp;view=items&amp;task=modify&amp;psid=" . $this->art->a_id) . "\">" . Text::_('Edit') . "</a>";
        echo "</div>";
    }
    ?>
    <?php if ($this->showTitle == 1) { ?>
        <h1 id="articleTitleRead" class="title main-title"><?php echo $this->articleTitle; ?></h1>
    <?php } ?>
    <?php
    if ($this->art->a_thumb) {
        $filename = ARFACMS_STORAGE_PATH . "article" . DS . "thumbs" . DS . Files::splitAppendix($this->art->a_thumb, true);
        if (Files::isImage($filename)) {
            $filelink = ARFACMS_STORAGE_LINK . "/article/thumbs/" . Files::splitAppendix($this->art->a_thumb);
            echo "<img class=\"article-thumb float-sm-start mb-3 mb-sm-2 me-sm-2\" alt=\"" . $this->art->a_thumb . "\" src=\"" . $filelink . "\" />";
        }
    }
    ?>
    <?php if ($this->articleShowInfo == 1) { ?>
        <div id="articleHeader" class="row mb-3">
            <div class="article-prop article-date col-sm-12 mb-1"><?php echo Date::GetdateRus($this->articleDate); ?></div>
            <div class="article-prop article-author col-sm-12">
                <span><?php echo Text::_('Author'); ?>:&nbsp;&nbsp;</span>
                <a href="<?php echo $this->articleAuthorProfileUrl; ?>"><?php echo $this->articleAuthor; ?></a>
            </div>
            <?php if ($this->use_rating) { ?>
                <div class="article-prop"><b><?php echo Text::_('Rating'); ?>
                        :&nbsp;&nbsp;</b><?php echo $this->articleRating; ?></div>
            <?php } ?>
        </div>
    <?php }
    Event::raise('content.prepare', array("used_in_module" => "article"), $this->articleHTML);
    $strippedHTML = strip_tags($this->articleHTML);
    if (!empty($strippedHTML) || strlen($this->articleHTML) > 15) {
        ?>
        <div id="articleTextRead"><?php echo $this->articleHTML; ?></div><?php
    }
    if ($this->use_rating && $this->canVote && $this->articleShowInfo) {
        echo "<div id=\"articleVote\" class=\"mt-3\">";
        echo Event::raise("rating.rendervotepanel", array("module" => "article", "element" => "object", "psid" => $this->art->a_id, "mess" => Text::_("Vote this article")));  // content.rating
        echo "</div>";
    }
    if (count($this->arrChilds) > 0) {
        /*
        if (!empty($strippedHTML) || strlen($this->articleHTML) > 15) {
            // Util::pre($this->art->a_childs_show_title);
            echo "<h3 class=\"article-childs-title\">" . Text::_("Articles") . "</h3>";
        }
        */
        if ($this->art->a_childs_show_title /* && (!empty($strippedHTML) || strlen($this->articleHTML) > 15)*/) {
            echo "<h3 class=\"article-childs-title\">" . ($this->art->a_childs_title ?: Text::_(Module::getInstance()->getParam("default_childs_list_title"))) . "</h3>";
        }
        echo "<div class=\"article-childs-list\">";
        foreach ($this->arrChilds as $key => $row) {
            echo " <a class=\"article-child\" href=\"" . Router::_("index.php?module=article&amp;view=read&amp;psid=" . $row->a_id . ($row->a_alias ? "&amp;alias=" . $row->a_alias : "")) . "\"><span>" . $row->a_title . "</span></a>";
        }
        echo "</div>";
    }
    if (!$this->notmpl && $this->comm->commentsEnabled() && $this->comm->checkACL("read")) {
        ?>
        <!-- Блок комментариев -->
        <div class="row mb-3">
            <div class="comments-list col-sm-12">
                <?php
                echo "<h4 class=\"comments-list-title\">" . Text::_('Comments') . ":</h4>";
                echo $this->comm->renderComments();
                if ($this->comm->checkACL("write")) {
                    echo $this->comm->renderCommentForm();
                }
                ?>
            </div>
        </div><!-- Конец блока комментариев-->
        <?php
    }
    Event::raise('content.rendered', array(), $this->art);
} else {
    if ($this->canModify) {
        echo "<p class=\"error\">" . Text::_('Article not found') . "</p>";
        echo "<div class=\"module-shortcuts\">";
        echo "	<a class=\"linkButton btn btn-primary\" rel=\"nofollow\" id=\"treeArticle\" href=\"" . Router::_("index.php?module=article&amp;view=tree") . "\">" . Text::_('Tree') . "</a>";
        echo "	<a class=\"linkButton btn btn-primary\" rel=\"nofollow\" id=\"newArticle\" href=\"" . Router::_("/" . ADMIN_FOLDER . "/index.php?module=article&amp;view=items&amp;task=modify") . "\">" . Text::_('New article') . "</a>";
        echo "</div>";
    }
}
echo "</div>";
?>