<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 7a8a46e7 (2024-11-21 18:36:24)
// 

use ArfaCMS\Core\Date;
use ArfaCMS\Core\Event;
use ArfaCMS\Core\Files;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;

defined('_ARFACMS_VALID') or die("Access denied");

echo "<div class=\"postListShare\">" . Event::raise("share.blog") . "</div>";
if ($this->canWrite) {
    echo "<div class=\"module-shortcuts\">";
    echo "	<a class=\"linkButton btn btn-primary\" rel=\"nofollow\" id=\"treeArticle\" href=\"" . Router::_("index.php?module=blog&amp;task=modify&amp;layout=new&amp;psid=" . $this->psid) . "\">" . Text::_('New post') . "</a>";
    echo "</div>";
}
// Если нужно то можно оверрайдить метатеги
// Portal::getInstance()->setTitle( ($this->blog->b_meta_title ? $this->blog->b_meta_title : $this->blog->b_name).($this->page>1 ? " - ".Text::_("Page")." ".$this->page : "") );
echo "<h1 class=\"title main-title\">" . $this->blog->b_name . "</h1>";
if (strlen($this->blog->b_description) > 20) {
    echo "<div class=\"blog-descr mb-3 clearfix\">";
    if ($this->blog->b_thumb) {
        $filename = ARFACMS_STORAGE_PATH . "blog" . DS . "blog_thumbs" . DS . Files::splitAppendix($this->blog->b_thumb, true);
        if (Files::isImage($filename)) {
            $filelink = ARFACMS_STORAGE_LINK . "/blog/blog_thumbs/" . Files::splitAppendix($this->blog->b_thumb);
            echo "<img class=\"blog-thumb float-sm-start mb-3 mb-sm-2 me-sm-2\" title=\"" . $this->blog->b_title_thm . "\" alt=\"" . $this->blog->b_alt_thm . "\" src=\"" . $filelink . "\" />";
        }
    }
    echo $this->blog->b_description;
    echo "</div>";
}
?>
<?php if ((isset($this->postDates["postStartDate"]) && $this->postDates["postStartDate"]) || (isset($this->postDates["postEndDate"]) && $this->postDates["postEndDate"])) { ?>
    <div class="blog-dates"><?php echo Text::_("Filtered") . ": " . Date::fromSQL($this->postDates["postStartDate"], true) . " - " . Date::fromSQL($this->postDates["postStartDate"], true); ?>
        <a href="<?php echo Router::_("index.php?module=blog&view=list&psid=" . $this->blog->b_id . "&alias=" . $this->blog->b_alias . "&mid=" . $_SESSION['active_menu_id'] . "&reset=1", false, false); ?>"><?php echo Text::_("Reset filter"); ?></a>
    </div>
<?php } ?>

<div class="posts-list">
    <?php
    // Добавим мид для подсветки меню - выводим только если есть верхний
    $mid = Request::getInt('mid');
    $addmid = '';
    if ($mid) $addmid = "&mid=" . $mid;
    foreach ($this->posts as $post) {
        $postlink = Router::_("index.php?module=blog&amp;view=post&amp;psid=" . $post->p_id . "&amp;alias=" . $post->p_alias . $addmid);
        ?>
        <div class="row mb-3">
            <div class="col-sm-12" id="post_<?php echo $post->p_id; ?>">
                <div class="post-block<?php echo $post->p_enabled ? "" : " unpublished"; ?>">
                    <div class="post-theme">
                        <span class="post-date"><?php echo Date::fromSQL($post->p_date, false, true); ?></span>
                        <h2 class="title"><a href="<?php echo $postlink; ?>"><?php echo $post->p_theme; ?></a></h2>
                    </div>
                    <div class="post-text mb-2 clearfix">
                        <?php
                        if ($post->p_thumb) {
                            $filename = ARFACMS_STORAGE_PATH . "blog" . DS . "thumbs" . DS . Files::splitAppendix($post->p_thumb, true);
                            if (Files::isImage($filename)) {
                                $filelink = ARFACMS_STORAGE_LINK . "/blog/thumbs/" . Files::splitAppendix($post->p_thumb);
                                echo "<img class=\"post-thumb float-sm-start mb-3 mb-sm-2 me-sm-2\" title=\"" . $post->p_title_thm . "\" alt=\"" . $post->p_alt_thm . "\" src=\"" . $filelink . "\" />";
                            }
                        }
                        if (!$this->blog->b_guieditor) Event::raise("bbcode.parse", array(), $post->p_text);
                        Event::raise('content.prepare', array("used_in_module" => "blog", "clean_all" => 1), $post->p_text);
                        $first_hr = mb_strpos($post->p_text, '<hr id="system-readmore"', 0);
                        if ($first_hr) echo mb_substr($post->p_text, 0, $first_hr);
                        else echo Text::cutHtml($post->p_text, \siteConfig::$shortTextLength);
                        ?>
                    </div>
                    <div class="row">
                        <?php if ($this->blog->b_post_rating) echo "<div class=\"col post-rating\"><span>" . Text::_('Rating') . "</span>: " . $post->p_rating . "</div>"; ?>
                        <div class="col readmore">
                            <?php
                            if (!$this->blog->b_hide_comments) {
                                echo "<a href=\"" . $postlink . "\">" . Text::_('Read post and comments') . ($post->p_comments ? "&nbsp;(" . $post->p_comments . ")" : "") . "</a>";
                            } else {
                                echo "<a href=\"" . $postlink . "\">" . Text::_('Read more') . "</a>";
                            }
                            ?>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    <?php } ?>
</div>
