<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 1c690378 (2024-10-13 03:03:48)
// 

use ArfaCMS\Core\Date;
use ArfaCMS\Core\Event;
use ArfaCMS\Core\Files;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;

defined('_ARFACMS_VALID') or die("Access denied");

if (isset($this->post) && $this->post) {
    if (!$this->post->p_enabled) {
        $published = " unpublished";
        $post_on_text = "Enable";
    } else {
        $published = "";
        $post_on_text = "Disable";
    }
    echo Event::raise("share.blogpost");
    if ($this->canModify) {
        echo "<div class=\"module-shortcuts\">";
        echo "	<a class=\"linkButton btn btn-primary\" rel=\"nofollow\" id=\"editBlogPost\" href=\"" . Router::_("index.php?module=blog&amp;task=modify&amp;psid=" . $this->post->p_id) . "\">" . Text::_('Edit') . "</a>";
        echo "	<a class=\"linkButton btn btn-primary\" rel=\"nofollow\" id=\"togglePostPublished\" onclick=\"javascript: if (confirm('" . Text::_("Toggle post published flag") . "?')) return true; else return false;\" href=\"" . Router::_("index.php?module=blog&amp;task=togglePostPublished&amp;psid=" . $this->post->p_id) . "\">" . Text::_($post_on_text) . "</a>";
        if ($this->post->p_closed) $comm_on_text = "Comms on"; else $comm_on_text = "Comms off";
        echo "	<a class=\"linkButton btn btn-primary\" rel=\"nofollow\" id=\"togglePostComments\" onclick=\"javascript: if (confirm('" . Text::_("Toggle post comments write flag") . "?')) return true; else return false;\" href=\"" . Router::_("index.php?module=blog&amp;task=togglePostComments&amp;psid=" . $this->post->p_id) . "\">" . Text::_($comm_on_text) . "</a>";
        if (!$this->post->p_enabled) echo "	<a class=\"linkButton btn btn-primary\" rel=\"nofollow\" id=\"togglePostDeleted\" onclick=\"javascript: if (confirm('" . Text::_("Toggle post deleted flag") . "?')) return true; else return false;\" href=\"" . Router::_("index.php?module=blog&amp;task=togglePostDeleted&amp;psid=" . $this->post->p_id) . "\">" . Text::_("Toggle deleted") . "</a>";
        echo "</div>";
    }
    ?>
    <div class="post-block single-post" id="post_<?php echo $this->post->p_id; ?>">
        <h1 class="post-theme<?php echo $published; ?>"><?php echo $this->post->p_theme; ?></h1>
        <div class="row mb-3">
            <div class="post-text col-lg-12">
                <?php if (!$this->blog->b_hide_properties) { ?>
                    <div class="post-data">
                        <div class="post-prop">
                            <b><?php echo Text::_('Date'); ?></b>: <?php echo Date::fromSQL($this->post->p_date, false, true); ?>
                        </div>
                        <div class="post-prop">
                            <b><?php echo Text::_('Author'); ?></b>: <a href="<?php echo Router::_("index.php?module=user&amp;view=info&amp;psid=" . $this->post->p_author_id); ?>"><?php echo $this->post->author; ?></a>
                        </div>
                        <?php if ($this->post->tags) { ?>
                            <div class="post-prop">
                                <b><?php echo Text::_('Tags'); ?></b>: <?php echo $this->post->tags; ?>
                            </div>
                        <?php } ?>
                        <?php if ($this->blog->b_post_rating) { ?>
                            <div class="post-prop">
                                <b><?php echo Text::_('Rating'); ?></b>: <?php echo $this->post->p_rating; ?>
                            </div>
                        <?php } ?>
                        <?php if ($this->canVote) { ?>
                            <div class="post-prop">
                                <?php echo Event::raise("rating.rendervotepanel", array("module" => "blog", "element" => "object", "psid" => $this->post->p_id, "mess" => Text::_("Vote this post"))); ?>
                            </div>
                        <?php } ?>
                    </div>
                <?php } ?>
                <?php
                if ($this->post->p_thumb) {
                    $filename = ARFACMS_STORAGE_PATH . "blog" . DS . "thumbs" . DS . Files::splitAppendix($this->post->p_thumb, true);
                    if (Files::isImage($filename)) {
                        $filelink = ARFACMS_STORAGE_LINK . "/blog/thumbs/" . Files::splitAppendix($this->post->p_thumb);
                        echo "<img class=\"post-thumb float-sm-start mb-3 mb-sm-2 me-sm-2\" title=\"" . $this->post->p_title_thm . "\" alt=\"" . $this->post->p_alt_thm . "\" src=\"" . $filelink . "\" />";
                    }
                }
                if (!$this->blog->b_guieditor) Event::raise("bbcode.parse", array(), $this->post->p_text);
                Event::raise('content.prepare', array("used_in_module" => "blog"), $this->post->p_text);
                echo $this->post->p_text;
                ?>
            </div>
        </div>
        <?php if (!$this->blog->b_hide_comments) { ?>
            <div class="row mb-3">
                <div class="comments-list col-sm-12"><!-- Блок комментариев -->
                    <?php if ($this->comm->commentsEnabled() && $this->comm->checkACL("read")) {
                        if ($this->comm_id) $comments = $this->comm->renderComment($this->comm_id, false);
                        else $comments = $this->comm->renderComments();
                        if ($comments) {
                            echo "<h4 class=\"comments-list-title\">" . Text::_('Comments') . ":</h4>";
                            echo $comments;
                        }
                        if (!$this->post->p_closed) {
                            if ($this->comm->checkACL("write")) {
                                echo $this->comm->renderCommentForm();
                            }
                            /*
                            elseif(!User::getInstance()->isLoggedIn()) {
                                echo "<div id=\"sign_in\" class=\"text-center\">";
                                echo "<h5 class=\"blogCommentWarning\">".Text::_("For adding comments it is necessary to become authorized")."</h5>";
                                echo "<h5 class=\"not_authorized\">".Text::_("You are not authorized").".</h5>";
                                echo "<h5 class=\"please_login\">".Text::_(\backofficeConfig::$noRegistration ? "Please log in" : "Please log in or register").".</h5>";
                                echo "<div class=\"buttons\">";
                                echo "<a rel=\"nofollow\" class=\"linkButton rel-popup-text btn btn-primary\" href=\"".Router::_("index.php?module=user&amp;view=login")."\">".Text::_("Log in")."</a>";
                                if (!\backofficeConfig::$noRegistration) echo "	<a class=\"linkButton btn btn-primary\" href=\"".Router::_("index.php?module=user&amp;view=register")."\">".Text::_("Register")."</a>";
                                echo "</div>";
                                echo "</div>";
                            }
                            */
                        }
                    } ?>
                </div><!-- Конец блока комментариев-->
            </div>
        <?php } ?>
    </div>
<?php } ?>