<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 90a4a87f (2024-10-10 03:15:40)
// 

use ArfaCMS\Core\Date;
use ArfaCMS\Core\Files;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Session;
use ArfaCMS\Core\Settings;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Util;
use ArfaCMS\MVC\Module;
use ArfaCMS\Sprav\SpravController;

defined('_ARFACMS_VALID') or die("Access denied");

class catalogControllerDefault extends SpravController
{
    private $memory_limit = 192;
    private $max_execution_time = 120;

    public function __construct($name, $module)
    {
        parent::__construct($name, $module);
        Text::parseCustom("catalog.model.exchange1c");
    }

    public function ajaxgetSKU()
    {
        $psid = $this->getPsid();
        $model = $this->getModel();
        echo $model->getSKUPath($psid);
    }

    public function ajaxgetGoodsInfo()
    {
        $psid = $this->getPsid();
        $model = $this->getModel('goods');
        $result = $model->getElement($psid);
        echo json_encode($result);
    }

    public function showExport()
    {
        if (intval(\adminConfig::$adminMemoryLimit) < $this->memory_limit) ini_set('memory_limit', $this->memory_limit . "M");
        if (intval(\adminConfig::$adminTimeLimit) < $this->max_execution_time) ini_set('max_execution_time', $this->max_execution_time);
        $hidden_fields = array("g_id", "g_deleted", "g_thumb", "g_medium_image", "g_thumb");
        $mdl = Module::getInstance();
        $model = $this->getModel();
        $view = $this->getView();
        $this->checkACL("view" . ucfirst($mdl->getName()) . ucfirst($view->getName()));
        $step = Request::getInt('step', 0);
        if ($step) {
            $res = $model->proceedExport($hidden_fields);
            $view->assign("res", Router::_("index.php?module=catalog&task=downloadResultCSV&filename=" . $res));
        } else {
            $fields = $model->getFields();
            $helper = $this->getHelper("groupsTree");
            $helper->buildTreeArrays("", 0, 1, 0);
            $tree = $helper->getTreeHTML(0, "export_ggr", 0, "", "", "show");
            $view->assign("tree", $tree);
            $view->assign("hidden_fields", $hidden_fields);
            $view->assign("fields", $fields);
            $view->setLayout('settings');
        }
    }

    public function showImport()
    {
        if (intval(\adminConfig::$adminMemoryLimit) < $this->memory_limit) ini_set('memory_limit', $this->memory_limit . "M");
        if (intval(\adminConfig::$adminTimeLimit) < $this->max_execution_time) ini_set('max_execution_time', $this->max_execution_time);
        $mdl = Module::getInstance();
        $model = $this->getModel();
        $view = $this->getView();
        $this->checkACL("view" . ucfirst($mdl->getName()) . ucfirst($view->getName()));
        $hidden_fields = array("g_id", "g_deleted", "g_change_date", "g_change_uid", "g_enabled");
        $def_fields = array(
            "g_type" => 1,
            "g_measure" => \catalogConfig::$default_measure,
            "g_pack_measure" => \catalogConfig::$default_measure,
            "g_pack_koeff" => 1,
            "g_vmeasure" => \catalogConfig::$default_vol_measure,
            "g_size_measure" => \catalogConfig::$default_size_measure,
            "g_wmeasure" => \catalogConfig::$default_wmeasure,
            "g_currency" => \catalogConfig::$default_currency,
            "g_selltype" => 0,
            "g_tax" => \catalogConfig::$default_order_taxes,
            "g_vendor" => \catalogConfig::$default_vendor,
            "g_manufacturer" => \catalogConfig::$default_manufacturer
        );
        $step = Request::getInt('step', 0);
        $codepage = Request::getSafe('codepage', "");
        if ($codepage == "windows-1251") $convert = true; else $convert = false;
        switch ($step) {
            case 1:
                $count_records = 0;
                /* НЕ УБИВАТЬ !!! ОТРЕМИТЬ ПЕРЕД РАБОТОЙ !!! */
                $model->resetSessionVars();
                $model->cleanTempTable();
                $result = Files::uploadDataFile("CSV", "filecsv", "import", PATH_TMP, true, $convert);
                if ($result) $count_records = $model->importTempTable($result["file"]);
                $fields = $model->getFields($hidden_fields);
                $fields_data = $model->getFieldsData($def_fields);
                $add_fields_w_lists = $model->getAddFieldsWLists($hidden_fields);
                $groupsTree = $this->getHelper("groupsTree");
                $view->assign("groupsTree", $groupsTree);
                $view->assign("fields", $fields);
                $view->assign("hidden_fields", $hidden_fields);
                $view->assign("fields_data", $fields_data);
                $view->assign("af_wlists", $add_fields_w_lists);
                $view->assign("count_records", $count_records);
                $view->setLayout('settings');
                break;
            case 2:
                // сложим всё в сессию
                $model->setImportVars(array_keys($def_fields));
                $view->setLayout('confirm');
                break;
            case 3:
                $view->setLayout('results');
                break;
            case 4:
                Portal::getInstance()->disableTemplate();
                $row2start = Request::getInt("row2start");
                $res = $model->proceedImport($row2start, $hidden_fields, array_keys($def_fields));
                echo json_encode($res);
                Util::halt();
                break;
            default:
                $view->setLayout('fileselector');
                break;
        }
    }

    function showGoodsgroup()
    {
        $this->showData();
    }

    public function showGoods()
    {
        $this->showData('ggr_name', 'goodsgroup');
    }

    public function showImages()
    {
        $this->showData('g_name', 'goods');
    }

    public function showVideos()
    {
        $this->showData('g_name', 'goods');
    }

    public function showUsers()
    {
        $this->showData();
    }

    public function showGoodsprices()
    {
        $this->showData('g_name', 'goods');
    }

    public function showVendors()
    {
        $this->showData('vc_name', 'vendor_categories');
    }

    public function showVendor_cats()
    {
        $this->showData();
    }

    public function showManufacturers()
    {
        $this->showData('mfc_name', 'manufacturer_categories');
    }

    public function showManufacturer_cats()
    {
        $this->showData();
    }

    public function modifyElement($ajaxModify = false, $clone_mode = false)
    {
        $mdl = Module::getInstance();
        $reestr = $mdl->get('reestr');
        $view = $this->getView();
        $model = $this->getModel();
        $viewname = $view->getName();
        switch ($viewname) {
            case "goods":
                $multy_code = Request::getInt('multy_code', false);        // ид верхней группы
                $psid = $this->getPsid();
                $links = $model->getLinkArray($psid);
                if (count($links) == 0) {
                    $prow = $model->getParentElement($multy_code);
                    if (isset($prow->{$model->meta->parent_name})) $links = array(0 => array('id' => $multy_code, 'title' => $prow->{$model->meta->parent_name}));
                }
                $groups = $this->getHelper('groupsTree');
                $parents = $groups->getWholeTreeUp($links);
                $fbgList = $model->getGroupsFields(array_keys($parents));
                foreach ($model->meta->is_add as $key => $is_add) {
                    if ($is_add) {
                        if ($model->meta->is_add_custom[$key] == 1) {
                            if (!array_key_exists($is_add, $fbgList)) {
                                $model->meta->input_view[$key] = 0;
                            }
                        }
                    }
                }
                $analogList = $model->getAnalogs($psid);
                $view->assign('analogList', $analogList);

                $additionalList = $model->getAdditionals($psid);
                $view->assign('additionalList', $additionalList);

                $discountsList = $model->getDiscounts($psid);
                $view->assign('discountsList', $discountsList);
                $complectList = $model->getComplectSet($psid);
                $view->assign('complectList', $complectList);
                if (!$psid && $multy_code) {
                    $model->loadMeta();
                    $model->meta->updateArrayField("default_value", "g_main_grp", $multy_code);
                }
                parent::modifyElement($ajaxModify, $clone_mode);
                break;
            case "goodsgroup":
                $multy_code = Request::getInt('multy_code', false);        // ид верхней группы
                $psid = $this->getPsid();
                $group_fields = $model->getGroupFields($psid);
                $view->assign('group_fields', $group_fields);
                parent::modifyElement($ajaxModify, $clone_mode);
                break;
            case "discounts":
                $multy_code = Request::getInt('multy_code', false);        // ид верхней группы
                $psid = $this->getPsid();
                $goodsList = $model->getGoods($psid);
                $view->assign('goodsList', $goodsList);
                parent::modifyElement($ajaxModify, $clone_mode);
                break;
            default:
                parent::modifyElement($ajaxModify, $clone_mode);
                break;
        }
    }

    public function customDataSave($new_psid, &$model)
    {
        if (!$new_psid) return false;
        switch ($this->getView()->getName()) {
            case "goods":
                $links = $model->getLinkArray($new_psid);
                $groups = $this->getHelper('groupsTree');
                $parents = $groups->getWholeTreeUp($links);
                $model->cleanNonGroupsFields($new_psid, array_keys($parents));

                $analogs = Request::getSafe("analogEditor", array());
                $reverse = $this->getConfigVal('reverse_analog_link');
                $model->saveAnalogs($new_psid, $analogs, $reverse);

                $additionals = Request::getSafe("additionalEditor", array());
                $model->saveAdditionals($new_psid, $additionals);

                $discounts = Request::getSafe("discountEditor", array());
                $model->saveDiscounts($new_psid, $discounts);

                $g_type = Request::getInt('g_type', 0);
                if ($g_type == 5) { // это комплект
                    $complectsets = array();
                    $complect_ids = Request::getSafe("complectEditor", array());
                    $complect_quantities = Request::getSafe("complectEditor_quantity", array());
                    if (count($complect_ids)) {
                        foreach ($complect_ids as $complect_gid) {
                            if (array_key_exists($complect_gid, $complect_quantities)) {
                                $cquantity = $complect_quantities[$complect_gid];
                                if ($cquantity) $complectsets[$complect_gid] = $cquantity;
                            }
                        }
                    }
                    if ($model->saveComplectSet($new_psid, $complectsets)) {
                        if (\catalogConfig::$complectPriceAsGoodsSum && $g_type == 5 && count($complectsets)) {
                            $model->updatePricesFromComplect($new_psid);
                        }
                    }
                }
                break;
            case "goodsgroup":
                $fields = Request::getSafe('group_field', array());
                $model->saveGroupFields($new_psid, $fields);
                break;
            case "discounts":
                $goods = Request::getSafe("goodsEditor", array());
                $model->saveGoods($new_psid, $goods);
                break;
            case "goodsprices":
                $model->updatePriceChangerInfo($new_psid);
                break;
            default:
                break;
        }
        return true;
    }

    public function showGoods_stat()
    {
        $this->showData();
    }

    public function showDiscounts()
    {
        $this->showData();
    }

    public function showOptionvals()
    {
        $this->showData();
    }

    public function modifyOptionval()
    {
        Module::getInstance()->get('reestr')->set('task', 'saveOptionval');
        parent::modify();
    }

    public function saveOptionval()
    {
        parent::save();
    }

    public function showOptions()
    {
        $this->showData();
    }

    public function modifyOption()
    {
        Module::getInstance()->get('reestr')->set('task', 'saveOption');
        parent::modify();
    }

    public function saveOption()
    {
        parent::save();
    }

    public function showOptions_data()
    {
        $this->showData('g_name', 'goods');
    }

    public function modifyOption_data()
    {
        Module::getInstance()->get('reestr')->set('task', 'saveOption_data');
        parent::modify();
    }

    public function saveOption_data()
    {
        parent::save();
    }

    public function showOptionvals_data()
    {
        $this->showData();
    }

    public function modifyOptionvals_data()
    {
        $reestr = Module::getInstance()->get('reestr');
        $psid = $this->getPsid();
        $multy_code = Request::getInt('multy_code', 0);
        $opt_vals_parent = $this->getModel()->getOptValsParent($multy_code);
        $reestr->set("opt_vals_parent", $opt_vals_parent);
        $reestr->set('psid', $psid);
        $reestr->set('task', 'saveOptionval_data');
        parent::modify();
    }

    public function saveOptionval_data()
    {
        parent::save();
    }

    public function downloadResultCSV()
    {
        $this->set("view", "export", true);
        $this->checkACL("view" . ucfirst(Module::getInstance()->getName()) . ucfirst($this->getView()->getName()));
        $file = Request::getSafe("filename", "");
        if ($file) {
            Util::download(PATH_TMP, $file);
        } else {
            $this->setRedirect(Router::_("index.php?module=catalog&view=export"), Text::_("File absent"));
        }
    }
    /**
     * полное удаление всей базы товаров и групп
     * используется для ускорения в исключительных случаях
     * копирование системы от другого клиента с другой товарной матрицей
     * очстики демоданных системы
     * возможно, конечно этому место в сервисе
     * явное управление через админку не предусматривается во избежание проблем
     * запускается только ручным набором - имя специально усложнено, чтобы случайно нельяз было набрать
     *
     */
    /*
    // *********************************************************************************
    //@FIXME Пока заремил. Слишком опасно.
    // Решается выделением всех товаров, в фильтре без учета родителей, и их удалением.
    // С группами аналогично.
    // Если делать очистку то через сервис, с проверкой прав и на все модули.
    // *********************************************************************************
    public function clear42Goods42base42() {
        $m_goods=$this->getModel('goods');
        $m_ggr=$this->getModel('goodsgroup');
        $db=Database::getInstance();
        $sql_g="delete from #__goods";
        $sql_ggr="delete from #__goods_group";

        $db->setQuery($sql_g);
        $db->query();

        $db->setQuery($sql_ggr);
        $db->query();

        $result_g=$m_goods->cleanLinkFromGoods();
        $result_ggr=$m_ggr->cleanLinkFromGroup();
        echo 'clear catalog base done';
    }
    */
    /********************************** EXCHANGE WITH 1C START ***********************************/
    public function echoMessage($status = null, $message = null, $halt = true)
    {
        while (ob_get_level()) {
            ob_end_clean();
        }    // clear output buffer
        if (\siteConfig::$debugMode) ob_start();
        if (!is_null($status)) {
            if ($status === true) echo "success\n";
            elseif ($status === false) echo "failure\n";
            elseif ($status === 2) echo "progress\n"; // @TODO For future purposes
        }
        if (!is_null($message)) {
            if (is_array($message) && count($message)) $message = implode("\n", $message);
            if ($message) {
                if ($this->getModule()->getParam("1c_messages_cp1251")) $message = mb_convert_encoding($message, "CP1251", DEF_CP);
                echo $message . "\n";
            }
        }
        if (\siteConfig::$debugMode) {
            $output = ob_get_contents();
            ob_end_clean();
            echo $output;
            Util::logFile($output, "Answer for TS (echoMessage" . ($this->getModule()->getParam("1c_messages_cp1251") ? ", CP1251" : "") . "): ");
        }
        if ($halt) Util::halt();
    }

    public function autoexchange1c()
    {
        if (\siteConfig::$debugMode) Util::logFile("In controller : " . Request::getSafe("type") . ":" . Request::getSafe("mode"));
        if (intval(\adminConfig::$adminMemoryLimit) < $this->memory_limit) ini_set('memory_limit', $this->memory_limit . "M");
        if (intval(\adminConfig::$adminTimeLimit) < $this->max_execution_time) ini_set('max_execution_time', $this->max_execution_time);
        $messages = array();
        if (defined("_ARFACMS_EXCHANGE") && $this->checkACL("viewCatalogExchange1c")) {
            Portal::getInstance()->disableTemplate();
            switch (Request::getSafe("type")) {
                case "sale":
                    switch (Request::getSafe("mode")) {
                        case "checkauth":
                            // Autologin in Portal
                            break;
                        case "init":
                            $result = $this->getModel("exchange1c")->initRemote($messages);
                            $this->echoMessage(null, $messages);
                            break;
                        case "query":
                            // Let's return orders.xml
                            $result = true;
                            // Next check is specially for those alternatively gifted boys and girls, who don't request init (For examole: 1C)
                            if (Util::toBool(Session::getInstance()->getUserVar("exchange1c_init_ok")) !== true) {
                                $result = $this->getModel("exchange1c")->initRemote($messages);
                                $messages = array();
                            }
                            if ($result) {
                                $result = $this->getModel("exchange1c")->processExport($messages, true);
                                if ($result && isset($result["filename"]) && $result["filename"]) {
                                    $this->getModel("exchange1c")->processDownload($result["filename"]);
                                } else {
                                    $this->echoMessage(false, $messages);
                                }
                            } else {
                                $messages[] = "Init failed";
                                $this->echoMessage(false, $messages);
                            }
                            break;
                        case "success":
                            // Let'change order status
                            $result = $this->getModel("exchange1c")->processOrdersChangeStatus($messages, true);
                            $this->echoMessage($result, $messages);
                            break;
                        case "file":
                            // Possible the same as catalog:file
                            $result = $this->getModel("exchange1c")->autoUpload($messages);
                            $this->echoMessage($result, $messages);
                            break;
                        case "import":
                            // Possible the same as catalog:import
                            $filename = Request::getSafe("filename");
                            $result = $this->getModel("exchange1c")->autoUploadUnzip($messages); // Every time
                            if ($result) $result = $this->getModel("exchange1c")->processImport($filename, $messages, true);
                            $this->echoMessage($result, $messages);
                            break;
                        default:
                            $this->echoMessage(true);
                            break;
                    }
                    break;
                case "catalog":
                    switch (Request::getSafe("mode")) {
                        case "checkauth":
                            // Autologin in Portal
                            break;
                        case "init":
                            $result = $this->getModel("exchange1c")->initRemote($messages);
                            $this->echoMessage(null, $messages);
                            break;
                        case "file":
                            $result = $this->getModel("exchange1c")->autoUpload($messages);
                            $this->echoMessage($result, $messages);
                            break;
                        case "import":
                            $filename = Request::getSafe("filename");
                            $result = $this->getModel("exchange1c")->autoUploadUnzip($messages); // Every time
                            if ($result) $result = $this->getModel("exchange1c")->processImport($filename, $messages, true);
                            $this->echoMessage($result, $messages);
                            break;
                        default:
                            $this->echoMessage(true);
                            break;
                    }
                    break;
                default:
                    break;
            }
        } else {
            $this->echoMessage(false, "401 Unauthorized");
        }
        Util::logFile("Response not send on " . Request::getSafe("type") . ":" . Request::getSafe("mode"));
        Util::halt();
    }

    public function ajaxExchange1C_log()
    {
        $this->set("view", "exchange1c", true);
        if (intval(\adminConfig::$adminMemoryLimit) < $this->memory_limit) ini_set('memory_limit', $this->memory_limit . "M");
        if (intval(\adminConfig::$adminTimeLimit) < $this->max_execution_time) ini_set('max_execution_time', $this->max_execution_time);
        if ($this->checkACL("view" . ucfirst(Module::getInstance()->getName()) . ucfirst($this->getView()->getName()), false)) {
            $log = Request::getSafe("log");
            switch ($log) {
                case "catalog":
                case "sale":
                    $filename = PATH_LOG . "exchange1c-" . $log . ".log";
                    $this->set("layout", "log", true);
                    $view = $this->getView("exchange1c");
                    $view->assign("log", @file_get_contents($filename));
                    $view->render();
                    break;
                default:
                    echo Text::_("Access denied");
                    break;
            }
        } else {
            echo Text::_("Access denied");
        }
    }

    public function showExchange1c()
    {
        if (intval(\adminConfig::$adminMemoryLimit) < $this->memory_limit) ini_set('memory_limit', $this->memory_limit . "M");
        if (intval(\adminConfig::$adminTimeLimit) < $this->max_execution_time) ini_set('max_execution_time', $this->max_execution_time);
        $this->checkACL("view" . ucfirst(Module::getInstance()->getName()) . ucfirst($this->getView()->getName())); // May be already done in parent
        $view = $this->getView();
        $success = array(1 => false, 2 => true);
        $messages = array(1 => array(), 2 => array());
        $messages[1][] = Text::_("Make backup first") . "!!!";
        $start_date = Settings::getVar("1c_exchange_orders_success_date");
        if (!$start_date) $start_date = Date::fromSQL(Date::todaySQL() . " 00:00:00");
        $start_date = Date::fromSQL(Date::toSQL($start_date), false, true);
        $end_timestamp = Date::mysqldatetime_to_timestamp(Date::todaySQL()) + 60 * 60 * 24;
        $end_date = Date::formatTimestamp($end_timestamp, true, false);
        $view->assign("end_date", $end_date);
        $view->assign("start_date", $start_date);
        $view->assign("success_1", $success);
        $view->assign("messages_1", $messages);
        $view->assign("activeTab", 1);
    }

    public function ajaxprocessImport1C()
    {
        $this->set("view", "exchange1c", true);
        if (intval(\adminConfig::$adminMemoryLimit) < $this->memory_limit) ini_set('memory_limit', $this->memory_limit . "M");
        if (intval(\adminConfig::$adminTimeLimit) < $this->max_execution_time) ini_set('max_execution_time', $this->max_execution_time);
        $model = $this->getModel("exchange1c");
        $model->setLogfile("catalog");
        $result = array();
        $messages = array();
        $result["field_id"] = "";
        if ($this->checkACL("view" . ucfirst(Module::getInstance()->getName()) . ucfirst($this->getView()->getName()), false)) {
            $filename = Request::getSafe("filename");
            $field_id = Request::getSafe("field_id");
            if ($filename && $field_id) {
                $res = $model->processImport($filename, $messages);
                if ($res === 2) {
                    $result["status"] = "progress";
                } elseif ($res) {
                    $result["status"] = "success";
                    $result["field_id"] = $field_id;
                } else {
                    $result["status"] = "failure";
                }
            } else {
                $result["status"] = "failure";
                $messages[] = Text::_("Wrong file name");
            }
        } else {
            $result["status"] = "failure";
            $messages[] = Text::_("Access denied");
        }
        $result["messages"] = $messages;
        echo json_encode($result);
    }

    public function processImport1C()
    {
        $this->set("view", "exchange1c", true);
        if (intval(\adminConfig::$adminMemoryLimit) < $this->memory_limit) ini_set('memory_limit', $this->memory_limit . "M");
        if (intval(\adminConfig::$adminTimeLimit) < $this->max_execution_time) ini_set('max_execution_time', $this->max_execution_time);
        $this->checkACL("view" . ucfirst(Module::getInstance()->getName()) . ucfirst($this->getView()->getName()));
        $model = $this->getModel();
        $model->setLogfile("catalog");
        $model->resetVars();
        if ($this->getModule()->getParam("1c_log_always_clean")) $model->clearLog();
        $view = $this->getView();
        $view->assign("activeTab", 1);
        $files = array();
        $success = array(1 => true, 2 => true);
        $messages = array(1 => array(), 2 => array());
        $success[1] = $model->manualUpload($messages[1], $files);
        // if($success[1] && count($files)){ }
        $view->assign("files_1", $files);
        $view->assign("success_1", $success);
        $view->assign("messages_1", $messages);

        $start_date = Settings::getVar("1c_exchange_orders_success_date");
        if (!$start_date) $start_date = Date::fromSQL(Date::todaySQL() . " 00:00:00");
        $start_date = Date::fromSQL(Date::toSQL($start_date), false, true);
        $end_timestamp = Date::mysqldatetime_to_timestamp(Date::todaySQL()) + 60 * 60 * 24;
        $end_date = Date::formatTimestamp($end_timestamp, true, false);
        $view->assign("end_date", $end_date);
        $view->assign("start_date", $start_date);

        $view->render();
    }

    public function processExport1C()
    {
        $this->set("view", "exchange1c", true);
        if (intval(\adminConfig::$adminMemoryLimit) < $this->memory_limit) ini_set('memory_limit', $this->memory_limit . "M");
        if (intval(\adminConfig::$adminTimeLimit) < $this->max_execution_time) ini_set('max_execution_time', $this->max_execution_time);
        $this->checkACL("view" . ucfirst(Module::getInstance()->getName()) . ucfirst($this->getView()->getName()));
        $start_date = Request::getDateTime("start_date");
        $end_date = Request::getDateTime("end_date");
        $model = $this->getModel();
        $model->setLogfile("sale");
        $model->resetVars();
        if ($this->getModule()->getParam("1c_log_always_clean")) $model->clearLog();
        $success = array(1 => true, 2 => true);
        $messages = array(1 => array(), 2 => array());
        $success[2] = $model->processExport($messages[2], false, $start_date, $end_date);
        $view = $this->getView();
        if (isset($success[2]["filename"]) && $success[2]["filename"]) {
            $view->assign("filelink", Router::_("index.php?module=catalog&task=downloadResult1C&filename=" . $success[2]["filename"]));
            $success[2] = true;
            $messages[2][] = Text::_("Export finished");
            $messages[2][] = Text::_("Everything OK");
        } else {
            $view->assign("filelink", "");
            $messages[2][] = Text::_("Export failed");
            $success[2] = false;
        }
        $view->assign("end_date", Date::fromSQL($end_date, true));
        $view->assign("start_date", Date::fromSQL($start_date, true));
        $view->assign("activeTab", 2);
        $view->assign("success_2", $success);
        $view->assign("messages_2", $messages);
        $view->render();
    }

    public function downloadResult1C()
    {
        $this->set("view", "exchange1c", true);
        $this->checkACL("view" . ucfirst(Module::getInstance()->getName()) . ucfirst($this->getView()->getName()));
        $file = Request::getSafe("filename", "");
        if ($file) {
            $model = $this->getModel("exchange1c");
            $model->processDownload($file);
        } else {
            $this->setRedirect(Router::_("index.php?module=catalog&view=exchange1c"), Text::_("File absent"));
        }
    }
    /********************************** EXCHANGE WITH 1C STOP ***********************************/
}

?>