<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: c3388405 (2023-09-17 18:08:19)
// 

use ArfaCMS\Core\Basket;
use ArfaCMS\Core\Date;
use ArfaCMS\Core\Files;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Session;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\User;
use ArfaCMS\MVC\Module;
use ArfaCMS\Sprav\SpravModel;

defined('_ARFACMS_VALID') or die("Access denied");

class catalogModelOrders extends SpravModel
{

    /*
    public function getStatuses(){
        $sql="SELECT * FROM #__orders_status WHERE os_enabled=1 AND os_deleted=0 ORDER BY os_id ASC";
        $this->_db->setQuery($sql);
        return $this->_db->loadObjectList('os_id');
    }
    */
    public function getOrders()
    {
        $user_id = User::getInstance()->getId();
        if ($user_id) {
            $sql = "SELECT * FROM #__orders WHERE o_uid=" . $user_id . " ORDER BY o_date DESC,o_id DESC";
            $this->_db->setQuery($sql);
            return $this->_db->loadObjectList();
        } else {
            return array();
        }
    }

    /*
    public function getAbstractOrder($order_id){
        $sql="SELECT * FROM #__orders WHERE o_id=".$order_id;
        $this->_db->setQuery($sql);
        $this->_db->loadObject($res);
        return $res;
    }
    */
    public function getOrder($order_id, $order_hash)
    {
        $user_id = User::getInstance()->getID();
        if ($user_id) {
            $sql = "SELECT * FROM #__orders WHERE o_uid=" . $user_id . " AND o_id=" . $order_id;
            $this->_db->setQuery($sql);
            $this->_db->loadObject($res);
            return $res;
        } elseif ($order_hash) {
            $sql = "SELECT * FROM #__orders WHERE o_hash='" . $order_hash . "' AND o_id=" . $order_id;
            $this->_db->setQuery($sql);
            $this->_db->loadObject($res);
            return $res;
        } else {
            return null;
        }
    }

    public function getOrderItems($order_id)
    {
        $sql = "SELECT * FROM #__orders_items WHERE i_order_id=" . $order_id;
        $this->_db->setQuery($sql);
        return $this->_db->loadObjectList();
    }

    public function saveOrder($pt, $dt, $userdata, $hash)
    {
        $basket = Basket::getInstance();
        $_goods = $basket->calculateOrder($pt, $dt, 0, $userdata);
        $calculation_error = !(!$pt->isError() && !$dt->isError());
        $order_comments = Request::getSafe("order_comments", "");
        if (!$calculation_error && $_goods && count($_goods) > 0) {
            $vendor = \catalogConfig::$multy_vendor ? Session::getVar("basket_vendor") : \catalogConfig::$default_vendor;
            $ordersHelper = Module::getHelper("order", "orders");
            // Update sums to zero for "catalogConfig::$hide_prices"
            $basket_deliverySum = \catalogConfig::$hide_prices ? 0 : $basket->deliverySum;
            $basket_paymentSum = \catalogConfig::$hide_prices ? 0 : $basket->paymentSum;
            $basket_discountSum = \catalogConfig::$hide_prices ? 0 : $basket->discountSum;
            $basket_taxesSum = \catalogConfig::$hide_prices ? 0 : $basket->taxesSum;
            $basket_total = \catalogConfig::$hide_prices ? 0 : $basket->total;
            $order_id = $ordersHelper->addOrder($hash, \catalogConfig::$default_order_status, $vendor, $dt, $pt,
                $userdata, $basket->quantity, \catalogConfig::$default_measure, $basket->weight, \catalogConfig::$default_wmeasure,
                $basket_deliverySum, $basket_paymentSum, $basket_discountSum, $basket_taxesSum, $basket_total,
                $basket->points, $order_comments, Module::getInstance()->getName());
            if ($order_id) {
                foreach ($_goods as $key => $val) {
                    if(\catalogConfig::$hide_prices) {
                        // Update price and sum to zero for "catalogConfig::$hide_prices"
                        $val->g_price = 0;
                        $val->g_sum = 0;
                    }
                    $item_id = $ordersHelper->addOrderItem($order_id, $val);
                    if ($item_id) {
                        $basket->reduceStocks($key);
                        if (count($val->options_files)) {
                            foreach ($val->options_files as $fk => $fv) {
                                $upload_dir = Files::checkUserDir("items" . DS . Files::getAppendix($fv[0]['value']), "orders");
                                if(!$ordersHelper->addOrderItemFile(Basket::getInstance()->getTempFile($fv[0]['value']), $upload_dir, $order_id, $item_id, $val->g_id, $fk, $fv[0]['title'], $fv[0]['value'])){
                                    $basket->order_message = Text::_("Failed adding order item file");
                                    return false;
                                }
                            }
                        }
                    } else {
                        $basket->order_message = Text::_("Order items add failed");
                        return false;
                    }
                }
            } else {
                $basket->order_message = Text::_("Order add failed");
                return false;
            }
            $basket->order_complete = 1;
            $basket->order_id = $order_id;
            $basket->order_vendor = $vendor;
            $basket->order_message = Text::_("Your order accepted");
            return $order_id;
        } elseif ($calculation_error) {
            $basket->order_message = Text::_("Order calculation error");
            return false;
        } else {
            $basket->order_message = Text::_("Basket is empty");
            return false;
        }
    }
}

?>