<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: cc761d91 (2024-10-21 20:19:53)
// 

use ArfaCMS\Core\ConfigTMPL;
use ArfaCMS\Core\Debugger;
use ArfaCMS\Core\Event;
use ArfaCMS\Core\HTMLControls;
use ArfaCMS\Core\Menus;
use ArfaCMS\Core\orderDelivery;
use ArfaCMS\Core\orderPayment;
use ArfaCMS\Core\Params;
use ArfaCMS\Core\Plugin;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Settings;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Translator;
use ArfaCMS\Core\Util;
use ArfaCMS\Core\Widget;
use ArfaCMS\DB\Database;
use ArfaCMS\DB\DBUtil;
use ArfaCMS\MVC\Module;
use ArfaCMS\Sprav\SpravController;
use ArfaCMS\Sprav\SpravMetadata;

defined('_ARFACMS_VALID') or die("Access denied");

class confControllerDefault extends SpravController
{
    public function ajaxcheckSpravField()
    {
        $moduleName = Module::getInstance()->getName();
        $viewname = $this->getView()->getName();
        $layout = $this->getView()->getLayout();

        $model = $this->getModel();
        $model->loadMeta();
        $fld = Request::getSafe('fld');
        $val = Request::getSafe('val');
        $psid = Request::getInt('psid', 0);
        switch ($fld) {
            case "f_name":
                if (strlen($val) > 3) {
                    if (substr($val, 0, 3) != "df_") $val = "df_" . DBUtil::cleanNameForDB($val);
                } else {
                    echo Text::_("Error");
                    return;
                }
                break;
        }
        if ($model->isUniqueFieldValue($fld, $val, $psid)) echo "OK"; else echo Text::_("Occupied");
    }
    /* Управление видимостью полей */
    /* Ajax функции */
    public function ajaxListModules()
    {
        $this->set('view', 'fields', true);
        $view = $this->getView();
        $view->listModules();
    }

    public function ajaxListViews()
    {
        $module_name = Request::getSafe('m_module', "");
        $result = $this->checkFields($module_name, false);
//		Util::logFile($result);
        $this->set('view', 'fields', true);
        $view = $this->getView();
        $view->listViews();
    }

    public function ajaxListLayouts()
    {
        $this->set('view', 'fields', true);
        $view = $this->getView();
        $view->listLayouts();
    }

    // форма выбора видимость каких полей редактировать
    public function selectVisio()
    {
        $this->set('view', 'fields', true);
        $this->set('layout', 'selectvisio', true);
        $view = $this->getView();
        $view->render();
    }

    // форма редактирования видимости полей
    public function prepareVisioForm()
    {
        $m_admin_side = Request::getInt('m_admin_side');
        $m_module = Request::getSafe('m_module', '');
        $m_view = Request::getSafe('m_view', '');
        $m_layout = Request::getSafe('m_layout', '');
        if (($m_admin_side != 0 && $m_admin_side != 1) || !$m_module || !$m_view || !$m_layout) {
            $this->setRedirect("index.php?module=conf&task=selectVisio", Text::_("Data for selected items not found"));
        } else {
            $model = $this->getModel('fields');
            $res = $model->getLayoutInfo($m_admin_side, $m_module, $m_view, $m_layout); // Получаем сохраненные настройки
            Module::getInstance($m_module);
            $meta = new SpravMetadata($m_module, $m_view, $m_layout, true, true, abs($m_admin_side - 1)); // Получаем текущую метадату
            if ($res) {
                $this->set('view', 'fields', true);
                $view = $this->getView();
                $view->assign("m_admin_side", $m_admin_side);
                $view->assign("m_meta_field", array_flip($meta->field));
                $view->assign("m_meta_name", $meta->field_title);
                $view->assign("meta", $meta);
                $view->assign("m_module", $m_module);
                $view->assign("m_view", $m_view);
                $view->assign("m_layout", $m_layout);
                $view->assign("res", $res);
                $this->set('layout', 'visioform', true);
                $error = false;
                foreach ($meta->field as $id => $fld) {
                    if (!array_key_exists($fld, $res)) $error = true;
                }
                if ($error) $this->setRedirect("index.php?module=conf&task=prepareFields&m_module=" . $m_module, Text::_("Metadata in tables is different") . ". " . Text::_("Reread data") . ".");
                else $view->render();
            } else $this->setRedirect("index.php?module=conf&task=selectVisio", Text::_("Data for selected items not found"));
        }
    }

    public function saveVisio()
    {
        $mdl = Module::getInstance();
        $model = $mdl->getModel('fields');
        $is_apply = Request::getSafe('apply');
        $m_admin_side = Request::getSafe('m_admin_side', false);
        $m_module = Request::getSafe('m_module', false);
        $m_view = Request::getSafe('m_view', false);
        $m_layout = Request::getSafe('m_layout', false);
        $res = $model->saveVisioData();
        if ($res) $msg = Text::_("Save successfull"); else $msg = Text::_("Save unsuccessfull");
        if ($is_apply) $url = "index.php?module=conf&task=prepareVisioForm&m_admin_side=" . $m_admin_side . "&m_module=" . $m_module . "&m_view=" . $m_view . "&m_layout=" . $m_layout;
        else $url = "index.php?module=conf&task=selectVisio";
        $this->setRedirect($url, $msg);
    }

    // получаем форму для загрузки полей из метадаты
    public function prepareFields()
    {
        $m_module = Request::getSafe('m_module', "");
        $this->set('view', 'fields', true);
        $view = $this->getView();
        $view->assign("m_module", $m_module);
        $this->set('layout', 'preparefields', true);
        $view->render();
    }

    // загружаем поля из метадаты
    public function checkFields($_module_name = "", $redirect = true)
    {
        $msg_arr = array();
        $model = $this->getModel('fields');
        $module_name = Request::getSafe('m_module', $_module_name);
        $arrMod = Module::getInstalledModules();
        if ($module_name && in_array($module_name, $arrMod)) {
            Debugger::getInstance()->milestone("checkFields for front modules " . $module_name);
            $res = $model->saveFields(0, $module_name);
            if ($res) $msg_arr[] = Text::_("Front") . " : " . Text::_("Save successfull") . " " . $model->getMsg();
            else $msg_arr[] = Text::_("Front") . " :" . Text::_("Save unsuccessfull") . " " . $model->getMsg();
            Debugger::getInstance()->milestone("checkFields for admin modules " . $module_name);
            $res = $model->saveFields(1, $module_name);
            if ($res) $msg_arr[] = Text::_("Admin zone") . " : " . Text::_("Save successfull") . " " . $model->getMsg();
            else $msg_arr[] = Text::_("Admin zone") . " : " . Text::_("Save unsuccessfull") . " " . $model->getMsg();
            $url = "index.php?module=conf&task=selectVisio";
        } else {
            $msg_arr[] = Text::_("Module undefined");
            $url = "index.php?module=conf&task=prepareFields";
        }
        if ($redirect) $this->setRedirect($url, implode("<br />", $msg_arr));
        else return $msg_arr;
    }

    /* заготовка
    public function ajaxRereadMetadata()  {
        $this->set('view','fields',true);
        $model = $this->getModel('fields');
        $res=$model->rereadMetadata();
        Util::halt();
    }
    */
    public function checkFieldsByTable($tables = array())
    {
        $model = $this->getModel('fields');
        if (count($tables)) {
            $modules = $model->getModulesByTables($tables);
            if (count($modules)) {
                foreach ($modules as $module => $ex) {
                    $this->checkFields($module, false);
                }
            }
        }
    }

    public function saveConfig()
    {
        // CSRF. Verify form token field
        $this->verifyFormToken();

        $is_cancel = Request::getSafe('cancel_but');
        if (!$is_cancel) {
            $db = Database::getInstance();
            $activeTab = Request::getSafe('config', 'site');
            $found = 0;
            $up_sql = '';
            foreach (ConfigTMPL::$_tabs as $k => $v) {
                if ($v[0] == $activeTab) $found = 1;
            }
            if ($found) {
                // нашли существующую секцию
                $confTMPL_name = "ArfaCMS\\Core\\" . $activeTab . "ConfigTMPL";
                $confTMPL = new $confTMPL_name;
                foreach ($confTMPL->props as $_key => $_val) {
                    switch ($_val[0]) {
                        case "boolean":
                            $cur_val = Request::getInt($_key);
                            if ($cur_val > 0) $cur_val = 1;
                            break;
                        case "integer":
                            $cur_val = Request::getInt($_key);
                            break;
                        case "float":
                            $cur_val = Request::getFloat($_key);
                            break;
                        case "text":
                            $cur_val = Request::getRealValue($_key); // @TODO CHECK ME !!!
                            break;
                        case "string":
                        case "filenames":
                        case "folder":
                        case "select":
                        case "select_method":
                        case "table_select":
                            $cur_val = Request::getSafe($_key, '');
                            break;
                        case "password":
                            $cur_val = Request::getSafe($_key, '');
                            break;
                        case 'multiselect':
                        case "multiselect_method":
                        case "table_multiselect":
                            $cur_val = Request::getSafe($_key, '');
                            if (is_array($cur_val)) $cur_val = implode(";", $cur_val);
                            break;
                    }
                    if ($_val[0] == "password" && !$cur_val) continue;
                    $up_sql .= "INSERT INTO #__config VALUES ('" . $activeTab . "','" . $_key . "','" . $db->doubleBaks($cur_val) . "')";
                    $up_sql .= " ON DUPLICATE KEY UPDATE `cfg_value`='" . $db->doubleBaks($cur_val) . "'" . $db->getDelimiter() . "\n";
                }
                $db->setQuery($up_sql);
                $sql_result = $db->query_batch(true, true);
                if ($sql_result) {
                    $msg = 'Config saved';
                    Event::raise('configuration.saved');
                } else $msg = 'Config save failed';
                $this->setRedirect("index.php?module=conf&view=config&config=" . $activeTab, Text::_($msg));
            } else {
                $this->setRedirect("index.php");
            }
        } else {
            $this->setRedirect("index.php");
        }
    }

    public function showCladr()
    {
        $this->showData();
    }

    public function showModules()
    {
        $this->showData();
    }

    public function showPlugins()
    {
        $this->showData();
    }

    public function prepareDataForModify(&$elem, &$model, &$view)
    {
        switch ($view->getName()) {
            case "modules":
                $params = Params::parse($elem->m_config);
                $def_params = Module::getInstance($elem->m_name)->getParamsMask();
                $view->assign("params", $params);
                $view->assign("def_params", $def_params);
                break;
            case "plugins":
                $params = Params::parse($elem->p_params);
                $def_params = Plugin::getInstance($elem->p_path . "." . $elem->p_name, 1)->getParamsMask();
                $view->assign("params", $params);
                $view->assign("def_params", $def_params);
                break;
            case "widgets":
                $wp = Params::parse($elem->aw_config);
                if (is_null($wp)) $wp = array();
                if ($elem->aw_access == "all") $w_access = $elem->aw_access;
                else $w_access = explode(";", $elem->aw_access);

                $w_params = $model->getWidgetParams($elem->aw_name);
                $menus = new Menus(true);
                $m_items = $menus->getAllMenusItems();
                if ($elem->aw_visible_in == "all") {
                    $w_visible = $elem->aw_visible_in;
                } else {
                    $w_visible = explode(";", $elem->aw_visible_in);
                }
                if ($model->hideWidgetContentParam($elem->aw_name)) {
                    $elem->aw_content = "";
                    $model->meta->updateArrayField("input_view", "aw_content", 0);
                }
                $view->assign("w_visible", $w_visible);
                $view->assign("w_access", $w_access);
                $view->assign("w_params", $w_params);
                $view->assign("m_items", $m_items);

                $view->assign("requiredDisabledModules", $model->getRequiredDisabledModules($elem->aw_name));
                if ((isset($wp["Widget_ID"]) && !$wp["Widget_ID"]) || !isset($wp["Widget_ID"])) $wp["Widget_ID"] = "widget_" . $elem->aw_id;
                $view->assign("wp", $wp);
                break;
            case "paymenttypes":
                $params = Params::parse($elem->pt_params);
                $def_params = $model->getParamsMask($elem);
                $view->assign("params", $params);
                $view->assign("def_params", $def_params);
                break;
            case "deliverytypes":
                $params = Params::parse($elem->dt_params);
                $def_params = $model->getParamsMask($elem);
                $view->assign("params", $params);
                $view->assign("def_params", $def_params);
                break;
            default:
                break;
        }
        return true;
    }

    public function customDataCheckForSave(&$model)
    {
        switch ($this->getView()->getName()) {
            case "modules":
                $psid = $this->getPsid();
                $_enabled = Request::getInt('m_enabled', 0);
                $elem = $model->getElement($psid);
                if ($elem->m_name == \siteConfig::$defaultModule && !$_enabled) {
                    $model->addSaveError("m_enabled", Text::_("Impossible to disable main module"));
                    return false;
                } elseif (Module::isCoreModule($elem->m_name) && !$_enabled) {
                    $model->addSaveError("m_enabled", Text::_("Impossible to disable core module"));
                    return false;
                }
                break;
            default:
                break;
        }
        return true;
    }

    public function prepareDataForSave(&$model)
    {
        switch ($this->getView()->getName()) {
            case "modules":
                break;
            case "plugins":
                break;
            case "widgets":
                break;
            case "dopfields":
                if (!$this->getPsid()) {
                    $f_name = Request::getSafe("f_name");
                    if (strlen($f_name) > 3) {
                        if (substr($f_name, 0, 3) != "df_") $f_name = "df_" . DBUtil::cleanNameForDB($f_name);
                        Router::getInstance()->setVarsVal("f_name", $f_name);
                    } else return false;
                }
                break;
            default:
                break;
        }
        return true;
    }

    public function customDataSave($new_psid, &$model)
    {
        if (!$new_psid) return false;
        switch ($this->getView()->getName()) {
            case "modules":
                $params = Request::getSafe("mod_param", array());
                $elem = $model->getElement($new_psid);
                $def_params = Module::getInstance($elem->m_name)->getParamsMask();
                if (!$model->saveParams($new_psid, $def_params, $params)) return false;
                break;
            case "plugins":
                $params = Request::getSafe("plg_param", array());
                $elem = $model->getElement($new_psid);
                $def_params = Plugin::getInstance($elem->p_path . "." . $elem->p_name, 1)->getParamsMask();
                if (!$model->saveParams($new_psid, $def_params, $params)) return false;
                break;
            case "widgets":
                $elem = $model->getElement($new_psid);
                // Parameters
                $params = Request::getSafe("aw_config", array());
                $def_params = $model->getWidgetParams($elem->aw_name);
                // Access
                $w_access_all = Request::getSafe("aw_access_all", "off");
                if ($w_access_all == "off") {
                    $w_access_arr = Request::getSafe("aw_access", array());
                    $w_access = "";
                    foreach ($w_access_arr as $role => $on) {
                        $w_access .= "$role;";
                    }
                    $w_access = mb_substr($w_access, 0, mb_strlen($w_access, DEF_CP) - 1, DEF_CP);
                } else {
                    $w_access = "all";
                }
                // Visibility
                $w_visible_all = Request::getSafe("aw_visible_all", "off");
                $w_visibility = "";
                if ($w_visible_all == "off") {
                    $w_visible_except = Request::getInt("visible_except");
                    if ($w_visible_except) $w_visibility = "except;";
                    $w_visibility_arr = Request::getSafe("aw_visible_in", array());
                    foreach ($w_visibility_arr as $w_vis) {
                        $w_visibility .= "$w_vis;";
                    }
                    $w_visibility = mb_substr($w_visibility, 0, mb_strlen($w_visibility, DEF_CP) - 1, DEF_CP);
                } else {
                    $w_visibility = "all";
                }
                if (!$model->saveWidgetParams($new_psid, $params, $def_params, $w_access, $w_visibility)) return false;
                Widget::cleanWidgetCache($new_psid, $elem->aw_name);
                break;
            case "paymenttypes":
                $params = Request::getSafe("pt_params", array());
                $elem = $model->getElement($new_psid);
                $def_params = $model->getParamsMask($elem);
                if (!$model->saveParams($new_psid, $def_params, $params)) return false;
                break;
            case "deliverytypes":
                $params = Request::getSafe("dt_params", array());
                $elem = $model->getElement($new_psid);
                $def_params = $model->getParamsMask($elem);
                if (!$model->saveParams($new_psid, $def_params, $params)) return false;
                break;
            case "dopfields":
                $elem = $model->getElement($new_psid);
                $this->checkFieldsByTable(array($elem->f_table));
                break;
            default:
                break;
        }
        return true;
    }

    public function pi()
    {
        ob_start();
        phpinfo();
        $pinfo = ob_get_contents();
        ob_end_clean();
        $pinfo = preg_replace('%^.*<body>(.*)</body>.*$%ms', '$1', $pinfo);
        echo '<div class="moduleBody confModule"><div id="phpinfo">' . $pinfo . '</div></div>';
    }

    public function createmap()
    {
        $model = $this->getModel('sitemap');
        $msg = $model->createMaps(true, true);
        $url = "index.php?module=conf&view=sitemap";
        $this->setRedirect($url, $msg);
    }

    public function newWidget()
    {
        $this->checkACL("viewConfWidgets");
        $widgets = Widget::getInstalledForFront();
        $view = $this->getView();
        if (count($widgets)) {
            foreach ($widgets as $w) {
                Text::parseWidget($w->w_name);
                $w->w_name_text = Text::_($w->w_name . " widget") . " (" . $w->w_name . ")";
                // $w->w_name_text = $w->w_name . " (" . Text::_($w->w_name . " widget") . ")";
            }
        }
        Util::sortStdClassArray($widgets, "w_name_text");
//		Util::showArray($widgets);
        $view->assign('widget_selector', HTMLControls::renderSelect('w_name', 'w_name', 'w_name', 'w_name_text', $widgets, 0, 0));
        $view->set('layout', 'new', true);
        $view->render();
    }

    public function saveNewWidget()
    {
        $this->checkACL("viewConfWidgets");
        $w_name = Request::getSafe("w_name", "");
        if (!$w_name) $this->newWidget();
        else {
            Text::parseWidget($w_name);
            $model = Module::getInstance()->getModel();
            $psid = $model->saveNewWidget($w_name);
            if ($psid) {
                $reestr = Module::getInstance()->get('reestr');
                $this->setRedirect("index.php?module=conf&view=widgets&task=modify&psid=" . $psid);
            } else {
                $this->setRedirect("index.php?module=conf&view=widgets", "Save unsuccessfull");
            }
        }
    }

    public function showWidgets()
    {
        $this->showData();
    }

    public function showDopfields_groups()
    {
        $this->showData();
    }

    public function showDopfields()
    {
        $this->showData();
    }

    public function showDopfields_choices()
    {
        $this->showData();
    }

    public function delete()
    {
        parent::delete();
        $viewname = $this->getView()->getName();
        $arr_psid = $this->getArrPsid();
        if ($viewname == "dopfields" && count($arr_psid)) {
            $model = $this->getModel("fields");
            $tables = $model->getTablesByFields($arr_psid);
            if (count($tables)) $this->checkFieldsByTable($tables);
        }
    }
    //****// Metadata START //****//
    /*
    public function modifyField($hdr=false)	{
        // @TODO Заготовка
        $model = $this->getModel();
        $view = $this->getView();
        $psid = $this->getPsid();
        $view->setLayout("modify_field");
        $res = $model->getField($psid);
        if (!$res&&$hdr) $res=$hdr;
        $view->assign("res",$res);
        $view->assign("psid",$psid);
        $view->render();
    }
    public function showMetadata()	{
        // @TODO Заготовка
        $model = $this->getModel();
        $view = $this->getView();
        $psid = $this->getPsid();
        switch($this->get('layout')){
            case "fields":
                if($psid){
                    $result = $model->getFields($psid);
                } else {
                    $url="index.php?module=conf&view=metadata";
                    $this->setRedirect($url);
                }
                break;
            default:
                $result = $model->getHeaders();
                break;
        }
        $view->assign("result",$result);
    }
    public function modifyMetadata($hdr=false)	{
        // @TODO Заготовка
        $model = $this->getModel();
        $view = $this->getView();
        $psid = $this->getPsid();
        $view->setLayout("modify");
        $res = $model->getHeader($psid);
        if (!$res && $hdr) $res = $hdr;
        $view->assign("res", $res);
        $view->assign("psid", $psid);
        $view->render();
    }
	public function deleteMetadata(){
        // @TODO Заготовка
		$model = $this->getModel();
		$psid = $this->getPsid();
		if ($psid) $result = $model->deleteMetadata($psid); else $result = false;
		if($result) $msg = Text::_("Delete successfull"); else $msg = Text::_("Delete unsuccessfull");
		$url = "index.php?module=conf&view=metadata";
		$this->setRedirect($url, $msg);
	}
	public function saveMetadata(){
        // @TODO Заготовка
		$model = $this->getModel();
		$hdr=$model->getHeaderFromRequest();
		if (!$hdr->h_module || !$hdr->h_view || !$hdr->h_layout) $this->modifyMetadata($hdr);
		else {
			$result = $model->saveHeader($hdr);
			if($result) {
                $msg = Text::_("Save successfull");
            }
			else {
                $msg = Text::_("Save unsuccessfull");
            }
			$url = "index.php?module=conf&view=metadata";
			$this->setRedirect($url, $msg);
		}
	}
    */
    //****// Metadata STOP //****//
    public function showRedirectLinks()
    {
        $this->showData();
    }

    public function showTmplzones()
    {
        $this->showData();
    }

    public function showMansitemap()
    {
        $this->showData();
    }

    public function checkTranslateList()
    {
        $html = '';
        $modules = Module::getInstalledModules();
        $translator = new Translator();
        foreach ($modules as $module) {
            $html .= $translator->checkModuleForTranslate($module);
        }
        echo $html;
    }

    public function checkTranslate()
    {
        if (!defined("_ARFACMS_TRANSLATE")) return false;
        $psid = $this->getPsid();
        $translator = new Translator();
        // выводим протокол проверки
        $model = $this->getModel();
        $moduleinfo = $model->getElement($psid);
        //Util::showArray($moduleinfo);
        $html = $translator->checkModuleForTranslate($moduleinfo->m_name);
        echo $html;
    }

    // =============================================================================
    function showCurrency()
    {
        $this->showData();
    }

    function showCurrency_rate()
    {
        $this->showData('c_name', 'currency');
    }

    public function showMeasures()
    {
        $this->showData();
    }

    public function showTaxes()
    {
        $this->showData();
    }

    // =============================================================================
    public function showDeliverytypes()
    {
        /*
        // Use $parent_subordination=false in metadata instead
        $reestr = Module::getInstance()->get('reestr');
        $reestr->set('consider_parents',false);
        */
        $this->showData();
    }

    public function showPaymenttypes()
    {
        $this->showData();
    }

    public function newDTS()
    {
        $this->checkACL("viewConfDeliverytypes");
        $model = Module::getInstance()->getModel();
        $view = $this->getView();
        $view->assign('templates_arr', orderDelivery::getInstalled());
        $view->assign('modules_arr', Module::getModules());
        $view->set('layout', 'new', true);
        $view->render();
    }

    public function saveNewDTS()
    {
        $this->checkACL("viewConfDeliverytypes");
        $tmpl_name = Request::getSafe("tmpl_name", "");
        $module_name = Request::getSafe("module_name", "");
        if (!$tmpl_name) $this->newDTS();
        else {
            $model = Module::getInstance()->getModel();
            $psid = $model->saveNewDTS($tmpl_name, $module_name);
            if ($psid) {
                $this->setRedirect("index.php?module=conf&view=deliverytypes&task=modify&psid=" . $psid);
            } else {
                $this->setRedirect("index.php?module=conf&view=deliverytypes", "Save unsuccessfull");
            }
        }
    }

    public function newPTS()
    {
        $this->checkACL("viewConfPaymenttypes");
        // $model = Module::getInstance()->getModel();
        $view = $this->getView();
        $view->assign('templates_arr', orderPayment::getInstalled());
        $view->assign('modules_arr', Module::getModules());
        $view->set('layout', 'new', true);
        $view->render();
    }

    public function saveNewPTS()
    {
        $this->checkACL("viewConfPaymenttypes");
        $tmpl_name = Request::getSafe("tmpl_name", "");
        $module_name = Request::getSafe("module_name", "");
        if (!$tmpl_name) $this->newPTS();
        else {
            $model = Module::getInstance()->getModel();
            $psid = $model->saveNewPTS($tmpl_name, $module_name);
            if ($psid) {
                $this->setRedirect("index.php?module=conf&view=paymenttypes&task=modify&psid=" . $psid);
            } else {
                $this->setRedirect("index.php?module=conf&view=paymenttypes", "Save unsuccessfull");
            }
        }
    }

    public function ajaxgetContlist()
    {
        if ($this->get('view') == "paymenttypes" && $this->get('layout') == "selector") {
            if (Request::checkExists("dt_module")) {
                Settings::setVar("payments_selector_dt_module", Request::getSafe("dt_module"), "string", "conf");
            }
            $dt_module = Settings::getVar("payments_selector_dt_module", "conf");
            $model = $this->getModel();
            $model->loadMeta();
            $model->meta->custom_sql = "AND (c.pt_module='' OR c.pt_module='" . $dt_module . "')";
        }
        parent::ajaxgetContlist();
    }

}

?>