<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 24f3b9b3 (2024-11-13 15:20:21)
// 

use ArfaCMS\Core\ConfigTMPL;
use ArfaCMS\Core\Files;
use ArfaCMS\Core\HTMLControls;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Text;
use ArfaCMS\DB\Database;

defined('_ARFACMS_VALID') or die("Access denied");

$_activeTab = Request::getSafe("config", "site"); ?>
<div class="conf-config rounded-pan">
    <h4 class="title"><?php echo Text::_("System settings") ?></h4>
    <ul class="nav nav-tabs" id="tabs">
        <?php foreach (ConfigTMPL::$_tabs as $_tab_key => $_tab_val) {
            if ($_tab_val[0] == $_activeTab) $_class = ' active'; else $_class = "";
            ?>
            <li class="nav-item">
                <button class="nav-link <?php echo $_class; ?>" data-bs-target="#tab_<?php echo $_tab_key; ?>" data-bs-toggle="tab"><?php echo Text::_($_tab_val[1]); ?></button>
            </li>
        <?php } ?>
    </ul>
    <div class="tab-content pt-3 px-3">
        <?php foreach (ConfigTMPL::$_tabs as $_tab_key => $_tab_val) {
            if ($_tab_val[0] == $_activeTab) $_class = ' active'; else $_class = "";
            ?>
            <div id="tab_<?php echo $_tab_key; ?>" class="tab-pane<?php echo $_class; ?>">
                <form name="<?php echo $_tab_val[0] ?>Form" action="index.php" method="post">
                    <input type="hidden" name="_requestVerificationToken" value="<?php echo Request::getVerificationToken(); ?>"/>
                    <input type="hidden" name="config" value="<?php echo $_tab_val[0] ?>"/>
                    <input type="hidden" name="module" value="conf"/> <input type="hidden" name="task" value="saveConfig"/>
                    <?php
                    $confTMPL_name = "ArfaCMS\\Core\\" . $_tab_val[0] . "ConfigTMPL";
                    $confTMPL = new $confTMPL_name;
                    $conf_name = $_tab_val[0] . "Config";
                    foreach ($confTMPL->props as $_key => $_val) {
                        $_cur_val = $confTMPL->getConfigVar($conf_name, $_key);
                        if (!$_cur_val && in_array($_val[0], array("string")) && (isset($_val[1]) && $_val[1] != false)) $_cur_val = $_val[1];
                        $_source = false;
                        $fld_id = $conf_name . $_key;
                        echo '<div class="row mb-3">';
                        switch ($_val[0]) {
                            case 'boolean':
                                $_css_class_1 = "col col-md-5 mb-0 align-self-center";
                                $_css_class_2 = "col-auto col-md-7 align-self-center";
                                break;
                            default:
                                $_css_class_1 = "col-md-5 mb-2 mb-md-0 align-self-center";
                                $_css_class_2 = "col-md-7 align-self-center";
                                break;
                        }
                        echo '<div class="' . $_css_class_1 . '">';
                        echo HTMLControls::renderLabelField($fld_id, Text::_($_key));
                        if (isset($_val[3]) && $_val[3]) echo HTMLControls::renderBalloonButton($_key . " description");
                        echo '</div>';
                        echo '<div class="' . $_css_class_2 . '">';
                        switch ($_val[0]) {
                            case 'boolean':
                                if ($_cur_val) $checked = "checked=\"checked\""; else $checked = "";
                                echo '<div class="form-check form-switch">';
                                echo '<input class="form-check-input" type="checkbox" id="' . $fld_id . '" value="1" name="' . $_key . '" ' . $checked . ' />';
                                echo '</div>';
                                break;
                            case 'integer':
                            case 'float':
                                echo '<input type="text" id="' . $fld_id . '" class="form-control numeric" size="19" value="' . $_cur_val . '" name="' . $_key . '" />';
                                break;
                            case 'text':
                                echo '<textarea id="' . $fld_id . '" name="' . $_key . '" cols="45" rows="6" class="form-control">' . $_cur_val . '</textarea>';
                                break;
                            case 'string':
                                echo '<input type="text" id="' . $fld_id . '" size="200" class="form-control" value="' . $_cur_val . '" name="' . $_key . '" />';
                                break;
                            case 'password':
                                echo '<div class="password-wrapper">';
                                echo '<div class="show-password" title="' . Text::_("Show") . '/' . Text::_("Hide") . '">';
                                echo '<i class="bi show-password-control" id="show_check' . $fld_id . '" onclick="$(this).toggleClass(\'checked\'); $(\'#' . $fld_id . '\').attr(\'type\', $(this).hasClass(\'checked\') ? \'text\' : \'password\')"></i>';
                                echo '</div>';
                                echo '<input type="password" id="' . $fld_id . '" size="60" class="form-control password-control" value="' . $_cur_val . '" name="' . $_key . '" />';
                                echo '</div>';
                                break;
                            case 'select':
                                if (isset($_val[2])) {
                                    foreach ($_val[2] as $key => $val) {
                                        if (!isset($_val[4]) || $_val[4] != false) {
                                            $_val[2][$key] = Text::_($val);
                                        }
                                    }
                                    echo HTMLControls::renderSelect($_key, $fld_id, "", "", $_val[2], $_cur_val, 0, "", 0);
                                }
                                break;
                            case 'select_method':
                                if (isset($_val[2])) {
                                    $method_array = explode("!!", $_val[2]);
                                    if (is_array($method_array) && count($method_array)) {
                                        $class_func = explode("::", $method_array[0]);
                                        if (is_array($class_func) && count($class_func) > 1) {
                                            if (count($method_array) > 1) {
                                                $arr = call_user_func($class_func, $method_array[1]);
                                            } else {
                                                $arr = call_user_func($class_func);
                                            }
                                            echo HTMLControls::renderSelect($_key, $fld_id, false, false, $arr, $_cur_val, 0, "", 0);
                                        }
                                    }
                                }
                                break;
                            case 'multiselect':
                                if (isset($_val[2])) {
                                    echo HTMLControls::renderSelect($_key, $fld_id, "", "", $_val[2], $_cur_val, 0, "", 5);
                                }
                                break;
                            case 'multiselect_method':
                                if (isset($_val[2])) {
                                    $method_array = explode("!!", $_val[2]);
                                    if (is_array($method_array) && count($method_array)) {
                                        $class_func = explode("::", $method_array[0]);
                                        if (is_array($class_func) && count($class_func) > 1) {
                                            if (count($method_array) > 1) {
                                                $arr = call_user_func($class_func, $method_array[1]);
                                            } else {
                                                $arr = call_user_func($class_func);
                                            }
                                            echo HTMLControls::renderSelect($_key, $fld_id, false, false, $arr, $_cur_val, 0, "", 5);
                                        }
                                    }
                                }
                                break;
                            case 'table_multiselect':
                                if (isset($_val[2])) {
                                    $db = Database::getInstance();
                                    $db->setQuery($_val[2]);
                                    $_source = $db->loadObjectList();
                                    echo HTMLControls::renderSelect($_key, $fld_id, "fld_id", "fld_name", $_source, $_cur_val, 0, "", 5);
                                }
                                break;
                            case 'table_select':
                                if (isset($_val[2])) {
                                    $db = Database::getInstance();
                                    $db->setQuery($_val[2]);
                                    $_source = $db->loadObjectList();
                                    echo HTMLControls::renderSelect($_key, $fld_id, "fld_id", "fld_name", $_source, $_cur_val, 0);
                                }
                                break;
                            case 'folder':
                                if (isset($_val[2])) {
                                    $_source = Files::getFolders(PATH_SITE . $_val[2]);
                                    echo HTMLControls::renderSelect($_key, $fld_id, 0, 'filename', $_source, $_cur_val, 0);
                                }
                                break;
                            case 'files':
                                if (isset($_val[2])) {
                                    $_source = Files::getFiles(PATH_SITE . $_val[2], false, false);
                                    echo HTMLControls::renderSelect($_key, $fld_id, 0, 'filename', $_source, $_cur_val, 1);
                                }
                                break;
                            case 'filenames':
                                if (isset($_val[2])) {
                                    $_source = Files::getFiles(PATH_SITE . $_val[2], false, false);
                                    foreach ($_source as $_src_key => $_src_val) {
                                        $_source[$_src_key]["filename"] = str_replace(".php", "", $_src_val["filename"]);
                                    }
                                    echo HTMLControls::renderSelect($_key, $fld_id, 'filename', 'filename', $_source, $_cur_val, 1);
                                }
                                break;
                            default:
                                echo '&nbsp;';
                        }
                        echo '</div>';
                        echo '</div>';
                    }
                    ?>
                    <div class="buttons">
                        <input type="submit" class="commonButtonbtn btn btn-primary" value="<?php echo Text::_('Apply tab'); ?>"/><input type="submit" name="cancel_but" class="commonButton btn btn-primary" value="<?php echo Text::_('Cancel'); ?>"/>
                    </div>
                </form>
            </div>
        <?php } ?>
    </div>
</div>