<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 4558d275 (2024-07-22 23:13:06)
// 

use ArfaCMS\Core\Event;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\User;
use ArfaCMS\Core\Util;
use ArfaCMS\MVC\Module;
use ArfaCMS\MVC\Controller;
use ArfaCMS\Tools\Mailer;

defined('_ARFACMS_VALID') or die("Access denied");

class feedbackControllerDefault extends Controller {
    public function ajaxShowForm(){
        $this->showForm(Request::getSafe('option') == "ajax");
    }
    public function showForm($is_ajax = false){
        // "/feedback/form/primer-formi.html"
        // "/feedback/form/napishite-nam.html?fld_3=%D0%97%D0%B0%D0%BF%D1%80%D0%BE%D1%81%20%D0%BE%20%D1%81%D0%BE%D1%82%D1%80%D1%83%D0%B4%D0%BD%D0%B8%D1%87%D0%B5%D1%81%D1%82%D0%B2%D0%B5"
        // "/feedback/form/vopros-po-tovaru.html?fld_13=64&fld_11=%D0%9F%D1%80%D0%BE%D1%88%D1%83%20%D1%81%D0%BE%D0%BE%D0%B1%D1%89%D0%B8%D1%82%D1%8C%20%D1%86%D0%B5%D0%BD%D1%83"
        if($is_ajax) Portal::getInstance()->disableTemplate();
        $psid = $this->getPsid();
        $model = $this->getModel();
        $form = $model->getForm($psid);
        $reestr = Module::getInstance()->get('reestr');
        $reestr->set('ajaxModify', $is_ajax == "ajax" ? 1 : 0);
        $reestr->set('controller', Request::getSafe("controller"));
        $reestr->set('layout', Request::getSafe('layout'));
        if(is_object($form)) {
            $form->frm_captcha = !$this->checkACL("feedbackDisableCaptcha", false);
            $fields = $model->getFormFields($psid);
            $view = $this->getView();
            $view->assign("form", $form);
            $view->assign("fields", $fields);
            $view->setMeta("title", $form->frm_name);
            $view->setMeta("description", $form->frm_name);
            $view->prepareForm();
            $view->setLayout($form->frm_template);
            // @TODO Temporary render for test in base Controller
            // if($is_ajax) $view->render();
        } else {
            $this->setRedirect("index.php", Text::_("Page not found"), 404);
        }
    }
    public function save(){
        // CSRF. Verify form token field
        $this->verifyFormToken(true);

        $errors = array(); $data = array();
        Portal::getInstance()->disableTemplate();
        $psid = $this->getPsid();
        $model = $this->getModel();
        $form = $model->getForm($psid);
        $answer["status"] = "FAILED";
        if (!$this->checkACL("feedbackDisableCaptcha", false) && User::checkFloodPoint()) {
            $answer["message"] = Text::_('Flood is found');
        } else {
            if(is_object($form)) {
                $form->frm_captcha = !$this->checkACL("feedbackDisableCaptcha", false);
                if($form->frm_captcha) Event::raise("captcha.checkResult",array("module"=>"feedback"));
                if($form->frm_captcha && isset($_SESSION['captcha_string']) && $_SESSION['captcha_string']!="OK" ) {
                    $answer["message"] = $_SESSION["captcha_string"];
                } else {
                    $fields = $model->getFormFields($psid);
                    foreach ($fields as &$field){
                        $error_found = false; $_error_text = "";
                        $default_error_text = Text::_("Please fill") . " &quot;".$field->fld_label."&quot;";
                        switch($field->t_val_type){
                            case "boolean":
                                $val = Request::getBool("fld_" . $field->fld_id);
                                $_error_text = Text::_("Please select") . " &quot;".$field->fld_label."&quot;";
                                break;
                            case "date":
                                $val = Request::getDate("fld_" . $field->fld_id);
                                $_error_text = $default_error_text;
                                break;
                            case "datetime":
                                $val = Request::getDateTime("fld_" . $field->fld_id);
                                $_error_text = $default_error_text;
                                break;
                            case "email":
                                $val = Request::getSafe("fld_" . $field->fld_id);
                                if($val && !Mailer::checkEmail($val)){
                                    $_error_text = Text::_("Please enter") . " &quot;".$field->fld_label."&quot;";
                                    $error_found = true;
                                } else {
                                    $_error_text = $default_error_text;
                                }
                                break;
                            case "float":
                                $val = Request::getFloat("fld_" . $field->fld_id);
                                $_error_text = $default_error_text;
                                break;
                            case "goods":
                                $val = Module::getHelper("goods", "feedback")->getGoodsName(Request::getInt("fld_" . $field->fld_id));
                                $_error_text = Text::_("Form error for") . " &quot;".$field->fld_label."&quot;";
                                break;
                            case "int":
                                $val = Request::getInt("fld_" . $field->fld_id);
                                $_error_text = $default_error_text;
                                break;
                            case "phone":
                                $val = Request::getSafe("fld_" . $field->fld_id);
                                $_error_text = $default_error_text;
                                break;
                            case "string":
                            case "text":
                                if ($field->t_input_type == 'select') {
                                    $val = Request::getInt("fld_" . $field->fld_id);
                                    $_error_text = Text::_("Please select") . " &quot;".$field->fld_label."&quot;";
                                    if($val && isset($field->field_values[$val])) {
                                        $val = $field->field_values[$val];
                                    } elseif($val) {
                                        $error_found = true;
                                    }
                                } elseif ($field->t_input_type == 'multiselect') {
                                    $val = Request::getSafe("fld_" . $field->fld_id);
                                    $_error_text = Text::_("Please select") . " &quot;".$field->fld_label."&quot;";
                                    if (is_array($val)) {
                                        $data_vals = array();
                                        foreach ($val as $m_val){
                                            if($m_val && isset($field->field_values[$m_val])) {
                                                $data_vals[] = $field->field_values[$m_val];
                                            } elseif($m_val) {
                                                $error_found = true;
                                            }
                                        }
                                        $val = implode(", ", $data_vals);
                                    }
                                } else {
                                    $val = Request::getSafe("fld_" . $field->fld_id);
                                    $_error_text = $default_error_text;
                                }
                                break;
                        }
                        if ($error_found || ($field->fld_required && !$val)){
                            $errors[$field->fld_id] = $_error_text;
                        } else {
                            if($val) $data[$field->fld_id] = $field->fld_name . ": " . $val;
                        }
                    }
                    if(count($errors)){
                        $answer["message"] = implode(". ", $errors).".";
                    } else {
                        $msgid = $model->saveMessage($form, $data);
                        if ($msgid) {
                            $res = true;
                            if (\siteConfig::$sendFeedbacksByMail) {
                                $res = $model->sendMessage($msgid, $form, $data);
                                Event::raise("feedback.send", array('msgid'=>$msgid, "form"=>$form, "data"=>$data, "result"=>$res));
                            } else {
                                Event::raise("feedback.saved", array('msgid'=>$msgid, "form"=>$form, "data"=>$data, "result"=>$res));
                            }
                            if($res) {
                                $message = Text::_('Feedback sent');
                            } else {
                                $message = Text::_('Feedback saved but not sent');
                            }
                            $answer["status"] = "OK"; // It is for both cases
                        } else {
                            $message = Text::_('Feedback not saved');
                        }
                        $answer["message"] = $message;
                    }
                }
            } else {
                $answer["message"] = Text::_('Form not found');
            }
        }

        echo json_encode($answer);
        Util::halt();
    }
    /********************************************/
    public function showMessages() {
        $model = $this->getModel();
        $view = $this->getView();
        $view->addBreadcrumb(Text::_('Cabinet'),"index.php?module=user&amp;view=panel");
        $view->addBreadcrumb(Text::_("My feedbacks"),"#");
        $view->assign('list', $model->getMyMessages());
    }
}
?>