<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: c9f9d167 (2024-10-09 22:05:16)
// 

use ArfaCMS\Core\Debugger;
use ArfaCMS\Core\Event;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Util;
use ArfaCMS\MVC\Module;
use ArfaCMS\Core\HTMLControls;

defined('_ARFACMS_VALID') or die("Access denied");
$reestr = Module::getInstance()->get('reestr');
// Util::traceStack();
if(isset($title_h)) $title_h++; else $title_h = 1;
$wrapper_class = $reestr->get("ajaxModify") ? "feedback-ajax-content-wrapper" : "feedback-content-wrapper";
?>
<?php if (is_object($this->form)) { ?>
    <div class="<?php echo $wrapper_class; ?> feedback-form-wrapper-<?php echo $this->form->frm_id; ?>">
        <h<?php echo $title_h; ?> class="title main-title"><?php echo $this->form->frm_name; ?></h<?php echo $title_h; ?>>
        <?php if (count($this->fields)) { ?>
            <div class="feedback-form">
                <div class="sendform-response"></div>
                <?php
                Debugger::getInstance()->milestone('Before form StartLayout');
                $this->frm->startLayout();
                ?>
                <?php if ($this->form->frm_prefix) { ?>
                    <div class="feedback-form-prefix row mb-3">
                        <div class="col-md-12"><?php echo $this->form->frm_prefix; ?></div>
                    </div>
                <?php } ?>
                <?php
                foreach ($this->fields as $field) {
                    switch ($field->t_input_type) {
                        case "hidden":
                            continue 2;
                            break;
                        case "checkbox":
                            echo "<div id=\"row_for_fld_" . $field->fld_id . "\" class=\"row mb-3 row-checkbox\"><div class=\"col-lg-12\"><div class=\"form-check\">";
                            $this->frm->renderInputPart("fld_" . $field->fld_id);
                            echo "&nbsp;";
                            $this->frm->renderLabelFor("fld_" . $field->fld_id);
                            echo "&nbsp;";
                            $this->frm->renderBalloonFor("fld_" . $field->fld_id, false);
                            echo "</div></div></div>";
                            break;
                        default:
                            if($field->t_val_type == "goods"){
                                echo "<div id=\"row_for_fld_" . $field->fld_id . "\" class=\"row mb-3 row-goods\"><div class=\"col-lg-12\">";
                                $this->frm->renderLabelFor("fld_" . $field->fld_id);
                                echo ":<span class=\"feedback-field-title feedback-field-goods-title\">";
                                echo Module::getHelper("goods", "feedback")->getGoodsName($this->frm->getInputValue("fld_" . $field->fld_id));
                                echo "</span></div></div>";
                            } else {
                                echo "<div id=\"row_for_fld_" . $field->fld_id . "\" class=\"row mb-2 row-label\"><div class=\"col-lg-12\">";
                                $this->frm->renderLabelFor("fld_" . $field->fld_id);
                                echo ":&nbsp;";
                                $this->frm->renderBalloonFor("fld_" . $field->fld_id, false);
                                echo "</div></div><div class=\"row mb-3\"><div class=\"col-lg-12\">";
                                $this->frm->renderInputPart("fld_" . $field->fld_id);
                                echo "</div></div>";
                            }
                            break;
                    }
                }
                if ($this->form->frm_captcha) {
                    echo "<div class=\"row mb-3 row-for-captcha\"><div class=\"col-lg-12\">";
                    echo Event::raise("captcha.renderForm", array("module" => "feedback"));
                    echo "</div></div>";
                }
                if ($this->form->frm_agreed) {
                    echo "<div class=\"privacy_policy_block row mb-3\"><div class=\"col-auto mx-auto\"><div class=\"form-check mx-auto\">";
                    $pp_art = Module::getHelper('article', 'article')->getArticle(intval(\siteConfig::$privacy_policy_article));
                    echo HTMLControls::renderCheckbox("privacy_policy_agree", 0,1,"privacy_policy_agree", "required", "", true);
                    echo "&nbsp;" . Text::_('I have read and agreed with')
                        . " <a" . ($reestr->get("ajaxModify") ? " target=\"_blank\"" : " rel=\"nofollow\" class=\"rel-popup-text\"") . " href=\"" . Router::_("index.php?module=article&amp;view=read&amp;psid=" . $pp_art->a_id . "&amp;alias=" . $pp_art->a_alias . "&amp;notmpl=1") . "\">" . Text::_('privacy policy') . "</a>";
                    echo "</div></div></div>";
                }
                if ($this->form->frm_postfix) {
                ?>
                <div class="feedback-form-postfix row mb-3">
                    <div class="col-md-12"><?php echo $this->form->frm_postfix; ?></div>
                </div>
                <?php
                }
                echo "<div class=\"buttons row\"><div class=\"col-lg-12\">";
                echo "<input type=\"submit\" class=\"commonButton btn btn-primary\" name=\"apply\" value=\"" . Text::_("Send") . "\" id=\"apply\">";
                echo "</div></div>";
                Debugger::getInstance()->milestone('After form DisplayOutput');
                $this->frm->endLayout();
                ?>
            </div>
        <?php } ?>
    </div>
<?php } ?>
