<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 2ba520e3 (2024-10-31 16:22:54)
// 

use ArfaCMS\Core\Request;
use ArfaCMS\Core\Response;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\User;
use ArfaCMS\MVC\Module;
use ArfaCMS\Sprav\SpravController;

defined('_ARFACMS_VALID') or die("Access denied");

class forumControllerDefault extends SpravController
{
    public $actions = array("read", "write", "moderate", "postvote");

    public function showsections()
    {
        $this->showData();
    }

    public function showthemes()
    {
        $this->showData("f_name", "forum_sections");
    }

    public function showposts()
    {
        $this->showData("t_name", "forum_themes");
    }

    public function ajaxupdateRoleRule()
    {
        // CSRF. Verify form token field
        $this->verifyFormToken(true);

        $forum_id = Request::getInt("psid");
        $role_id = Request::getInt("role");
        $action = Request::getSafe("act");
        $state = Request::getInt("flag");
        if (!$forum_id || !$role_id || !in_array($action, $this->actions)) {
            $response = new Response("FAILED", Text::_('Wrong data'), 400);
        } else {
            $rmodel = $this->getModel("rights");
            if ($rmodel->setRuleForRole($forum_id, $role_id, $action, $state)) {
                $response = new Response("OK");
            } else {
                $response = new Response("FAILED", Text::_('Wrong data'), 400);
            }
        }
        $response->send();
    }

    public function ajaxupdateUserRule()
    {
        // CSRF. Verify form token field
        $this->verifyFormToken(true);

        $forum_id = Request::getInt("psid");
        $user_id = Request::getInt("user");
        $action = Request::getSafe("act");
        $state = Request::getInt("flag");
        if (!$forum_id || !$user_id || !in_array($action, $this->actions)) {
            $response = new Response("FAILED", Text::_('Wrong data'), 400);
        } else {
            $rmodel = $this->getModel("rights");
            if ($rmodel->setRuleForUser($forum_id, $user_id, $action, $state)) {
                $response = new Response("OK");
            } else {
                $response = new Response("FAILED", Text::_('Wrong data'), 400);
            }
        }
        $response->send();
    }

    public function viewRolesMap()
    {
        $this->checkACL("viewSetForumRights");
        $mdl = Module::getInstance();
        $view = $this->getView();
        $fmodel = $this->getModel("sections");
        $rmodel = $this->getModel();
        $forums = $fmodel->getOrderedForums();
        $roles = $rmodel->getRoles();
        $rights = $rmodel->getAllRulesForAllRoles();
        if (!count($forums)) $this->setRedirect('index.php?module=forum', Text::_('Forum not exists'));
        else {
            $view->assign("forums", $forums);
            $view->assign("roles", $roles);
            $view->assign("rights", $rights);
            $view->assign("actions", $this->actions);
        }
    }

    public function viewUsersMap()
    {
        $this->checkACL("viewSetForumRights");
        $mdl = Module::getInstance();
        $view = $this->getView();
        $fmodel = $this->getModel("sections");
        $rmodel = $this->getModel();
        $forums = $fmodel->getOrderedForums();
        $users = $rmodel->getUsers();
        $rights = $rmodel->getAllRulesForAllUsers();
        if (!count($forums)) $this->setRedirect('index.php?module=forum', Text::_('Forum not exists'));
        else {
            $view->assign("forums", $forums);
            $view->assign("users", $users);
            $view->assign("rights", $rights);
            $view->assign("actions", $this->actions);
        }
    }

    public function showRights()
    {
        $this->checkACL("viewSetForumRights");
        $view = $this->getView();
        $model = $this->getModel("sections");
        switch ($this->get('layout')) {
            case "rolesmap":
                $this->viewRolesMap();
                break;
            case "usersmap":
                $this->viewUsersMap();
                break;
            case "modify":
                $this->modifyRights();
                break;
            default:
                $rModel = $this->getModel();
                $return = Request::getSafe("return", "");
                $psid = $this->getPsid();
                $arr_psid = $this->getArrPsid();
                if (!$psid && $arr_psid && is_array($arr_psid) && count($arr_psid) > 0) $psid = $arr_psid[0]; // @TODO remove unnecessary check
                $forum = $model->getForum($psid);
                if ($forum) {
                    $roles = $rModel->getRoles();
                    $view->assign('forum_id', $psid);
                    $view->assign('forum_name', $forum->f_name);
                    $view->assign('roles', $roles);
                    $view->assign('return', $return);
                } else $this->setRedirect('index.php?module=forum', Text::_('Forum not exists'));
                break;
        }
    }

    public function modifyRights()
    {
        $this->checkACL("viewSetForumRights");
        $psid = Request::getInt('psid', 0);
        $subject = Request::getSafe('subject', 'user');

        $view = $this->getView();
        $view->assign("actions", $this->actions);

        $view->assign('subject', $subject);
        $subjectName = "";

        $rModel = $this->getModel();
        $model = $this->getModel("sections");
        $forum = $model->getForum($psid);
        if ($forum) {
            $view->assign('psid', $psid);
            $view->assign('forum_name', $forum->f_name);

            $rules = array();
            if ($subject == "role") {
                $roleId = Request::getInt('roleid', 0);
                if ($roleId != 0) {
                    $view->assign('roleId', $roleId);
                    $subjectName = $rModel->getRoleTitle($roleId);
                    $rules = $rModel->getRulesForRole($psid, $roleId);
                } else $this->warning(Text::_('Invalid role') . ": " . $roleId, __FUNCTION__);
            } else {
                $login = Request::getSafe('userlogin', '');
                if ($login) {
                    $uid = User::getUserId($login);
                    if ($uid) {
                        $subjectName = $login;
                        $view->assign('userId', $uid);
                        $rules = $rModel->getRulesForUser($psid, $uid);
                    } else $this->setRedirect('index.php?module=forum&view=rights&psid=' . $psid, Text::_('User not found'));
                } else $this->setRedirect('index.php?module=forum&view=rights&psid=' . $psid, Text::_('Empty username'));
            }

            $view->assign('rules', $rules);
            $view->assign('subjectName', $subjectName);
        } else $this->setRedirect('index.php?module=forum', Text::_('Forum not found'));
    }

    public function updateRights()
    {
        // CSRF. Verify form token field
        $this->verifyFormToken();

        $this->checkACL("viewSetForumRights");

        $psid = Request::getInt('psid', 0);
        $roleId = Request::getInt('roleId', 0);
        $userId = Request::getInt('userId', 0);
        $subject = Request::getSafe('subject', 'role');
        $access = Request::getSafe('access', array());
        $userLogin = User::getLoginFor($userId);
        $is_apply = Request::getSafe('apply');
        $redirectUrl = "index.php?module=forum&view=rights&psid=" . $psid;
        if ($is_apply) {
            $redirectUrl.= '&layout=modify';
            if ($subject == 'user') {
                $redirectUrl.= '&subject=user&userlogin=' . $userLogin;
            } else {
                $redirectUrl.= '&subject=role&roleid=' . $roleId;
            }
        }
        $model = $this->getModel("rights");

        if ($subject == "user") $model->cleanRulesForUser($psid, $userId);
        else $model->cleanRulesForRole($psid, $roleId);

        foreach ($access as $action => $state) {
            if (intval($state) == 1) {
                if ($subject == "role") $model->setRuleForRole($psid, $roleId, $action, 1);
                else $model->setRuleForUser($psid, $userId, $action, 1);
            }
        }
        $this->setRedirect($redirectUrl, Text::_('Rights updated'));
    }

}

?>