<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: fbff1702 (2024-10-01 01:30:00)
// 

use ArfaCMS\Core\Event;
use ArfaCMS\Core\HTMLControls;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\User;

defined('_ARFACMS_VALID') or die("Access denied");

if ($this->errMessage) { echo "<h4 class=\"invalid\">".$this->errMessage."</h4>"; }
if ($this->section) { // выводим информацию о родительском форуме
    echo "<h1 class=\"title main-title\">".$this->section->f_name."</h1>";
}
if($this->premoderated) echo "<p class=\"red-warning\">".Text::_("The message will be published after consideration by moderator")."</p>";
?>
<form action="<?php echo Router::_("index.php"); ?>" method="post" id="ThemeForm">
    <input type="hidden" name="module" value="forum" />
    <input type="hidden" name="task" value="saveTheme" />
    <input type="hidden" name="psid" value="<?php echo $this->section->f_id; ?>" />
    <input type="hidden" name="tid" value="<?php echo $this->theme->t_id; ?>" />
    <input type="hidden" name="page" value="<?php echo $this->page; ?>" />
    <div id="forumTheme" class="row mb-3">
        <div class="col-lg-12 mb-2">
            <?php echo HTMLControls::renderLabelField("themeTitle", Text::_('Theme')); ?>
        </div><div class="col-lg-12">
            <?php echo HTMLControls::renderInputText("themeTitle",$this->theme->t_theme, 100, 100, "themeTitle", "commonEdit", false, true); ?>
        </div>
    </div>
    <div class="themeText row mb-3">
        <div class="col-lg-12 mb-2">
            <?php echo HTMLControls::renderLabelField("themeText", Text::_('Text')); ?>
        </div><div class="col-lg-12">
            <?php
            Event::raise("bbcode.editor",array("element_id"=>"themeText"));
            echo HTMLControls::renderBBCodeEditor("themeText","themeText",$this->theme->t_text,80,15,"editorArea required");
            ?>
        </div>
    </div>
    <?php
        if($this->canModerate){
            echo "<div class=\"themeRow row mb-3\"><div class=\"col-lg-12\"><div class=\"form-check\">";
            echo HTMLControls::renderCheckbox("t_fixed",$this->theme->t_fixed,1,"","checkbox").HTMLControls::renderLabelField("t_fixed", Text::_('Fixed theme'));
            echo "</div></div></div>";
        }
        echo "<div class=\"themeRow row mb-3\"><div class=\"col-lg-12\"><div class=\"form-check\">";
        echo HTMLControls::renderCheckbox("t_closed",$this->theme->t_closed,1,"","checkbox").HTMLControls::renderLabelField("t_closed", Text::_('Closed theme'));
        echo "</div></div></div>";
        if(!$this->theme->t_id || ($this->theme->t_author_id && User::getInstance()->getID()==$this->theme->t_author_id)) {
            echo "<div class=\"themeRow row mb-3\"><div class=\"col-lg-12\"><div class=\"form-check\">";
            echo HTMLControls::renderCheckbox("t_subscribe",$this->userSubscribed,1,"","checkbox").HTMLControls::renderLabelField("t_subscribe", Text::_('Subscribe to theme'));
            echo "</div></div></div>";
        }
    ?>
    <div class="postTagsEditor row mb-3" id="postTagsBlock">
        <div class="col-lg-12 mb-2">
            <?php echo HTMLControls::renderLabelField("themeTags", Text::_('Tags')." (".Text::_('comma separated').")"); ?>
        </div><div class="col-lg-12">
            <?php echo HTMLControls::renderInputText("themeTags",$this->theme->t_tags, 80, 80, "themeTags", "commonEdit"); ?>
        </div>
    </div>
    <?php if(!$this->disableCaptcha) { ?>
        <div id="forumCaptcha" class="row mb-3"><div class="col-lg-12">
            <?php echo Event::raise("captcha.renderForm",array("module"=>"forum"))?>
        </div></div>
    <?php } ?>
    <div id="themeFooter" class="buttons">
        <input type="submit" class="commonButton btn btn-primary" value="<?php echo Text::_('Send'); ?>" />
    </div>
</form>
