<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 220257aa (2024-10-17 21:41:43)
// 

use ArfaCMS\Core\Files;
use ArfaCMS\Core\HTMLControls;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\simpleTreeTable;
use ArfaCMS\Core\Text;
use ArfaCMS\MVC\Module;
use ArfaCMS\Sprav\SpravView;

defined('_ARFACMS_VALID') or die("Access denied");

class menusViewItems extends SpravView
{
    // @TODO Add more fields to show/hide
    private $flds = array(
        0 => array('mi_module', 'mi_view', 'mi_layout', 'mi_alias', 'mi_psid', 'mi_controller', 'mi_task'),
        1 => array('mi_link'),
        2 => array('mi_canonical_id'),
        3 => array()
    );

    public function modify($row)
    {
//      Util::pre("var flds = ". json_encode($this->flds) . ";\n");
        $func_name = "updateFieldsVisibilityOnModify_" . time();
        $script = "";
        $script .= "function " . $func_name . "(id){";
        $script .= "var flds = " . json_encode($this->flds) . ";";
//      $script.= "console.dir(flds);";
        $script .= "
    flds.forEach(function callback(curVal, ind, arr) {
        curVal.forEach(function callback(curVal2, ind2, arr2) {
            $('#wrapper-' + curVal2).hide();
        });
    });
    flds.forEach(function callback(curVal, ind, arr) {
        if(ind != id){
            curVal.forEach(function callback(curVal2, ind2, arr2) {
                // console.log(curVal2);
                $('#' + curVal2).val('');
            });
        }
    });
    flds[id].forEach(function callback(curVal, ind, arr) {
        $('#wrapper-' + curVal).show();
    })
    ";
        $script .= "}";
        $script .= "
            $(document).ready(function(){
                $('#mi_type').change(function() {
                " . $func_name . "($(this).val());
                });
                " . $func_name . "($('#mi_type').val());
            });";

        if ($script) Portal::getInstance()->addScriptDeclaration($script);
        parent::modify($row);
    }

    public function renderViewsSelector()
    {
        $arr = array();
        $arr[""] = "[ " . Text::_("Not selected") . " ]";
        if ($this->modname) {
            $path = PATH_MODULES . $this->modname . DS . "views"; // We need front views
            $files_arr = Files::getFiles($path, false, false);
        } else $files_arr = array();
        if (count($files_arr)) {
            foreach ($files_arr as $key => $val) {
                $item = substr($key, 0, strpos($key, "."));
                $arr[$item] = $item;
            }
        }
        return HTMLControls::renderSelect("mi_view", "mi_view", "", "", $arr, $this->vname, false);
    }

    public function renderControllersSelector()
    {
        $arr = array();
        $arr[""] = "[ " . Text::_("Default value") . " ]";
        $files_arr = Module::getControllers($this->modname, true);
        if (count($files_arr)) {
            foreach ($files_arr as $key => $val) {
                if ($key == "default.php") continue;
                $item = substr($key, 0, strpos($key, "."));
                $arr[$item] = $item;
            }
        }
        return HTMLControls::renderSelect("mi_controller", "mi_controller", "", "", $arr, $this->vcontroller, false);
    }

    public function renderMenuSelector()
    {
        $tree = new simpleTreeTable();
        $tree->table = "menus";
        $tree->fld_id = "mi_id";
        $tree->fld_parent_id = "mi_parent_id";
        $tree->fld_title = "mi_name";
        $tree->fld_deleted = "mi_deleted";
        $tree->fld_enabled = "mi_enabled";
        $tree->select_top_level_text = "[ " . Text::_("Not selected") . " ]";
        $tree->buildTreeArrays($this->mi_id, 0, 1, 0);
        return $tree->getSelectHTML(0, "mi_canonical_id", "mi_canonical_id", $this->mi_canonical_id, "singleSelect");
    }

    public function getViewsList()
    {
        // Only options, for ajax
        $html = "<option value=\"\">[ " . Text::_("Not selected") . " ]</option>";
        if ($this->modname) {
            $path = PATH_MODULES . $this->modname . DS . "views"; // We need front views
            $files_arr = Files::getFiles($path, false, false);
        } else $files_arr = array();
        if (count($files_arr)) {
            foreach ($files_arr as $key => $val) {
                $item = substr($key, 0, strpos($key, "."));
                $html .= "<option value=\"" . $item . "\">" . $item . "</option>";

            }
        }
        return $html;
    }

    public function getControllersList()
    {
        // Only options, for ajax
        $html = "<option value=\"\">[ " . Text::_("Default value") . " ]</option>";
        $files_arr = Module::getControllers($this->modname, true);
        if (count($files_arr)) {
            foreach ($files_arr as $val) {
                if ($val == "default") continue;
                $html .= "<option value=\"" . $val . "\">" . $val . "</option>";
            }
        }
        return $html;
    }

    public function renderModulesSelector($_cur_val = "")
    {
        $_mdls[""] = "[ " . Text::_("Not selected") . " ]";
        $modules = Module::getModules(true);
        foreach ($modules as $_module) {
            $_mdls[$_module] = $_module;
        }
        $js = "getViewsForModule(this, 'mi_view');";
        $js .= "getControllersForModule(this, 'mi_controller');";
        return HTMLControls::renderSelect("mi_module", "mi_module", "", "", $_mdls, $_cur_val, false, $js);
    }
}

?>