<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 4e1f9281 (2024-07-29 09:57:38)
// 

use ArfaCMS\Core\orderDelivery;
use ArfaCMS\Core\orderPayment;
use ArfaCMS\Core\Event;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\User;
use ArfaCMS\Sprav\SpravController;

defined('_ARFACMS_VALID') or die("Access denied");

class ordersControllerDefault extends SpravController
{
    /*
     * @TODO Роутер. Надо ли ?
     */

    public function showOrder()
    {
        switch ($this->get('layout')) {
            case "print": // Печать одного заказа
                $this->set('layout', "default", true);
                $order_id = Request::getInt('order_id', 0);
                $order_hash = Request::getSafe('order_hash', '');
                // $this->printOrder($order_id, $order_hash);
                Portal::getInstance()->changeTemplateFile('print');
                $this->renderOrder($order_id, $order_hash, 1, 0);
                break;
            default: // Просмотр заказа
                $order_id = Request::getInt('order_id', 0);
                $order_hash = Request::getSafe('order_hash', '');
                $view = $this->getView();
                if (!Portal::getInstance()->inPrintMode()) {
                    $view->addBreadcrumb(Text::_('Cabinet'), Router::_("index.php?module=user&view=panel"));
                    if (User::getInstance()->isLoggedIn()) $view->addBreadcrumb(Text::_('Orders'), Router::_("index.php?module=orders"));
                    $view->addBreadcrumb(Text::_('Order'), "#");
                }
                $this->renderOrder($order_id, $order_hash);
                break;
        }

    }

    private function renderOrder($order_id, $order_hash = "", $override_css = 0, $controls = 1, $_echo = 1)
    {
        $view = $this->getView();
        $model = $this->getModel("orders");
        if ($order_id) {
            $order = $model->getOrder($order_id, $order_hash);
            if ($order) $order_items = $model->getOrderItems($order_id);
            else {
                $order = false;
                $order_items = false;
            }
        } else {
            $order = false;
            $order_items = false;
        }
        if ($order) {
            $payment = orderPayment::getPaymentClass($order->o_pt_id);
            $delivery = orderDelivery::getDeliveryClass($order->o_dt_id);
            $params = array("order" => $order);
            Event::raise("content.orders_order_render", $params);
        } else {
            $delivery = false;
            $payment = false;
        }
        $view->assign("override_css", $override_css);
        $view->assign("dt_class", $delivery);
        $view->assign("pt_class", $payment);
        $view->assign("order", $order);
        $view->assign("controls", $controls);
        $view->assign("order_items", $order_items);
        if (!$_echo) {
            ob_start();
            $view->render();
            $html = ob_get_contents();
            ob_end_clean();
            $this->haltView();
            return $html;
        }
    }

    public function showOrders()
    {
        $view = $this->getView();
        $view->addBreadcrumb(Text::_('Cabinet'), Router::_("index.php?module=user&view=panel"));
        $view->addBreadcrumb(Text::_('Orders'), "#");
        $model = $this->getModel();
        $orders = $model->getOrders();
        $status_arr = $model->getStatuses();
        $view->assign("orders", $orders);
        $view->assign("status_arr", $status_arr);
    }

    public function showPayment()
    {
        $order_id = Request::getInt('order_id', 0);
        $order_hash = Request::getSafe('order_hash', '');
        if (!$order_id) {
            // Для систем которые думают что они самые умные и не дают нормальных возвратных url
            // Например отдается постом orderId, тогда добавляем в ссылку &order_id_alias=orderId
            $order_id_alias = Request::getSafe('order_id_alias', "");
            if ($order_id_alias) $order_id = Request::getInt($order_id_alias, 0);
            if (!$order_id) {
                /*
                // Для систем которые думают что они самые умные, да еще и самые хитрые и присылают вообще неизвестно что.
                // Например отдается поток в котором json
                // Для них callback ссылки будут вида :
                // index.php?module=orders&view=payment&payment=robokassa&mode=recieve
                // или
                // /orders/payment/robokassa/recieve.html
                */
                $pt_name = Request::getSafe("payment");
                if ($pt_name) {
                    $mode = Request::getSafe("mode");
                    $order_id = orderPayment::getOrderIdByAbstractPaymentClass($pt_name, "", $mode);
                }
            }
        }
        $model = $this->getModel("orders");
        // $user_id=Session::restoreToken(Request::getSafe("ARFACMS_TOKEN","")); // @FIXME FOR WHAT IS IT ???
        if (Request::getSafe('mode', 'show') == "show" && !User::getInstance()->isLoggedIn()) {
            $order = $model->getOrder($order_id, $order_hash);
        } else {
            $order = $model->getAbstractOrder($order_id);
        }
        if ($order) {
            $order_items = $model->getOrderItems($order_id);
            $pt_class = orderPayment::getPaymentClass($order->o_pt_id);
            if ($pt_class) $pt_class->assignOrder($order, $order_items);
        } else {
            $pt_class = false;
        }
        $view = $this->getView();
        if (!Portal::getInstance()->inPrintMode()) {
            $view->addBreadcrumb(Text::_('Cabinet'), Router::_("index.php?module=user&view=panel"));
            if (User::getInstance()->isLoggedIn()) $view->addBreadcrumb(Text::_('Orders'), Router::_("index.php?module=orders"));
            $view->addBreadcrumb(Text::_('Order payment'), "#");
        }
        $view->assign("pt_class", $pt_class);
    }

    public function showPayments()
    {
        // @TODO Under construction
        $view = $this->getView();
        $view->addBreadcrumb(Text::_('Cabinet'), "index.php?module=user&amp;view=panel");
        $view->addBreadcrumb(Text::_('My payments'), "#");
    }

    public function showReports()
    {
        // @TODO Under construction
        $view = $this->getView();
        $view->addBreadcrumb(Text::_('Cabinet'), "index.php?module=user&amp;view=panel");
        $view->addBreadcrumb(Text::_('My reports'), "#");
    }

    public function showSales()
    {
        // @TODO Under construction
        $view = $this->getView();
        $view->addBreadcrumb(Text::_('Cabinet'), Router::_("index.php?module=user&view=panel"));
        $view->addBreadcrumb(Text::_('My sales'), "#");
        $model = $this->getModel();
        $orders = $model->getOrders();
        $status_arr = $model->getStatuses();
        $view->assign("orders", $orders);
        $view->assign("status_arr", $status_arr);
    }
}

?>