<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: dc974189 (2024-10-13 02:45:16)
// 

use ArfaCMS\Core\Currency;
use ArfaCMS\Core\Date;
use ArfaCMS\Core\orderUserdata;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Util;
use ArfaCMS\Core\Vendor;
use ArfaCMS\Core\Measure;
use ArfaCMS\MVC\Module;

defined('_ARFACMS_VALID') or die("Access denied");

class ordersHelperEmail
{
    public function renderOrderEmail($data, $template)
    {
        $html = "";
        $lPath = PATH_TEMPLATES . Portal::getInstance()->getTemplate() . DS . 'html' . DS . 'emails' . DS . 'orders' . DS . $template . '.php';
        if (is_file($lPath)) {
            ob_start();
            require $lPath;
            $html .= ob_get_contents();
            ob_end_clean();
        } else {
            if (method_exists($this, $template)) {
                ob_start();
                $this->{$template}($data);
                $html .= ob_get_contents();
                ob_end_clean();
            } else {
                Util::halt("Template " . $template . " absent.");
            }
        }
        return $html;
    }

    /*********************************************************************************/

    private function new_order_admin($data)
    {
        $taxesInOrder = array();
        $payment = $data["payment"];
        $delivery = $data["delivery"];
        $order = $data["order"];
        $items = $data["order_items"];
        $summa = 0;
        $total = 0;
        $counter = 0;
        $vendor = Vendor::getInstance()->getVendor($order->o_vendor);
        $price_digits = Module::getInstance("orders")->getParam("price_digits");
        $quantity_digits = Module::getInstance("orders")->getParam("quantity_digits");
        // ###ВЕРСТКА НАЧАТА###
        ?>
        <div style="border:1px solid #ddd;padding:8px;">
            <h1><?php echo Text::_("Order") . " № " . $order->o_id . " " . Text::_("from") . " " . Date::GetdateRus($order->o_date); ?></h1>
            <div style="margin-bottom: 12px;"><b><?php echo Text::_("Vendor") . ":</b> " . $vendor->v_name; ?></div>
            <div style="margin-bottom: 12px;">
                <b><?php echo Text::_("Delivery type"); ?>: </b><?php echo $order->o_dt_title; ?>
                <?php
                if ($delivery) {
                    $delivery_data = $delivery->getDecodedData($order->o_dt_data);
                    echo $delivery->renderInfo($delivery_data);
                }
                ?>
            </div>
            <div style="margin-bottom: 12px;">
                <b><?php echo Text::_("Payment type"); ?>: </b><?php echo $order->o_pt_title; ?>
                <?php
                if ($payment) {
                    $payment_data = $payment->getDecodedData($order->o_pt_data);
                    echo $payment->renderInfo($payment_data);
                }
                ?>
            </div>
            <?php
            $ud = new orderUserdata(0);
            $userdata = $ud->getDecodedData($order->o_userdata);
            if (is_array($userdata)) {
                $_userdata_arr = array();
                if (array_key_exists("userdata_person", $userdata) && $userdata["userdata_person"]) $_userdata_arr[] = "<b>" . Text::_("Contact person") . ":</b> " . $userdata["userdata_person"];
                if (array_key_exists("userdata_email", $userdata) && $userdata["userdata_email"]) $_userdata_arr[] = "<b>" . Text::_("Contact email") . ":</b> " . $userdata["userdata_email"];
                if (array_key_exists("userdata_phone", $userdata) && $userdata["userdata_phone"]) $_userdata_arr[] = "<b>" . Text::_("Contact phone") . ":</b> " . $userdata["userdata_phone"];
                if (count($_userdata_arr)) { ?>
                    <div class="order_row"><?php echo implode("<br />", $_userdata_arr); ?></div><?php }
            }
            ?>
            <?php if (trim(strip_tags($order->o_comments))) { ?>
                <div style="margin-bottom: 12px;">
                    <b><?php echo Text::_("Comments"); ?>: </b><?php echo htmlspecialchars($order->o_comments); ?>
                </div>
            <?php } ?>
            <div style="display:table;border-top:1px solid #ddd;border-left:1px solid #ddd; margin-bottom: 12px;width:100%;">
                <div style="display:table-row;background-color: #f2f2f2;">
                    <div style="display:table-cell;white-space:nowrap;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                        <?php echo Text::_('NPP'); ?>
                    </div>
                    <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                        <?php echo Text::_('Articul'); ?>
                    </div>
                    <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                        <?php echo Text::_('Goods title'); ?>
                    </div>
                    <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                        <?php echo Text::_('Price'); ?>
                    </div>
                    <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                        <?php echo Text::_('Quantity'); ?>
                    </div>
                    <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                        <?php echo Text::_('Sum'); ?>
                    </div>
                </div>
                <?php foreach ($items as $g) { ?>
                    <?php $counter++; ?>
                    <?php if ($g->i_g_quantity) { ?>
                        <div style="display:table-row;">
                            <?php
                            $current_price = $g->i_g_price;
                            $current_quantity = $g->i_g_quantity;
                            $current_sum = $g->i_g_sum;
                            $summa = $summa + $current_sum;
                            if ($g->i_g_tax > 0) {
                                $tax_name = base64_encode($g->i_g_tax_name);
                                if (!isset($taxesInOrder[$tax_name])) $taxesInOrder[$tax_name] = 0;
                                $taxesInOrder[$tax_name] += $g->i_g_tax;
                            }
                            ?>
                            <div style="display:table-cell;text-align:center;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                                <?php echo $counter; ?>
                            </div>
                            <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                                <?php echo $g->i_g_sku; ?>
                            </div>
                            <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                                <?php
                                echo $g->i_g_name;
                                if ($g->i_g_options_text) {
                                    if (Portal::getInstance()->inPrintMode()) {
                                        $i_g_options_title = "";
                                        $i_g_options_text = $g->i_g_options_text;
                                    } else {
                                        $i_g_options_title = $g->i_g_options_text;
                                        $i_g_options_text = mb_substr($i_g_options_title, 0, \siteConfig::$shortTextLength) . "...";
                                    }
                                    echo "<span title=\"" . $i_g_options_title . "\" class=\"basket-row-options\">" . $i_g_options_text . "</span>";
                                }
                                ?>
                            </div>
                            <div style="display:table-cell;white-space:nowrap;text-align:right;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                                <?php echo number_format($current_price, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR); ?>
                            </div>
                            <div style="display:table-cell;white-space:nowrap;text-align:right;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                                <?php echo number_format($current_quantity, $quantity_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . " " . Measure::getInstance()->getShortName($g->i_g_measure); ?>
                            </div>
                            <div id="sum_<?php echo $g->i_g_id; ?>"
                                 style="display:table-cell;white-space: nowrap;text-align:right;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                                <?php echo number_format($current_sum, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR); ?>
                            </div>
                        </div>
                    <?php } ?>
                <?php } ?>

            </div>
            <div style="display:table;border-top:1px solid #ddd;border-left:1px solid #ddd; margin-bottom: 12px;width:100%;">
                <div style="display:table-row;background-color: #f2f2f2;">
                    <?php if ($order->o_points > 0) { ?>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo Text::_('Total points'); ?>
                        </div>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo $order->o_points; ?>
                        </div>
                    <?php } else { ?>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;"></div>
                    <?php } ?>
                    <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                        <?php echo Text::_('Sum'); ?>
                    </div>
                    <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                        <?php echo number_format($summa, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                    </div>
                </div>
            </div>
            <?php $total = $summa; ?>
            <div style="display:table;border-top:1px solid #ddd;border-left:1px solid #ddd; margin-bottom: 12px;width:100%;">
                <?php if (($order->o_dt_sum) && (($order->o_dt_sum) > 0)) {
                    if ($order->o_dt_tax_val) {
                        $tax_name = base64_encode($order->o_dt_tax_name);
                        if (!isset($taxesInOrder[$tax_name])) $taxesInOrder[$tax_name] = 0;
                        $taxesInOrder[$tax_name] += $order->o_dt_tax_sum;
                    } ?>
                    <div style="display:table-row;">
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo Text::_("Delivery sum"); ?>
                        </div>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo number_format($order->o_dt_sum, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                        </div>
                    </div>
                    <?php $total = $total + $order->o_dt_sum; ?>
                <?php } ?>
                <?php if (($order->o_pt_sum) && ($order->o_pt_sum > 0)) { ?>
                    <div style="display:table-row;">
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo Text::_("Payment commission"); ?>
                        </div>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo number_format($order->o_pt_sum, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                        </div>
                    </div>
                    <?php $total = $total + $order->o_pt_sum; ?>
                <?php } ?>
                <?php if (floatval($order->o_discount_sum)) { ?>
                    <div style="display:table-row;">
                        <?php if ($order->o_discount_sum > 0) { ?>
                            <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                                <?php echo Text::_("Discount"); ?>
                            </div>
                        <?php } else { ?>
                            <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                                <?php echo Text::_("Fee"); ?>
                            </div>
                        <?php } ?>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo number_format($order->o_discount_sum, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                        </div>
                    </div>
                    <?php $total = $total - $order->o_discount_sum; ?>
                <?php } ?>
                <?php if ($total != $summa) { ?>
                    <div style="display:table-row;">
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo Text::_("Total"); ?>
                        </div>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo number_format($total, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                        </div>
                    </div>
                <?php } ?>
                <?php if (floatval($order->o_taxes_sum)) { ?>
                    <div style="display:table-row;">
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo Text::_("Taxes in order"); ?>
                        </div>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo number_format($order->o_taxes_sum, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                        </div>
                    </div>
                    <?php if (count($taxesInOrder)) { ?>
                        <?php foreach ($taxesInOrder as $kt => $vt) { ?>
                            <div style="display:table-row;">
                                <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                                    <?php print base64_decode($kt); ?>
                                </div>
                                <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                                    <?php echo number_format($vt, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                                </div>
                            </div>
                        <?php } ?>
                    <?php } ?>
                <?php } ?>
                <div style="display:table-row;">
                    <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                        <?php echo Text::_("Grand total"); ?>
                    </div>
                    <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                        <?php echo number_format($order->o_total_sum, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                    </div>
                </div>
            </div>
        </div>
        <?php
        // ###ВЕРСТКА ЗАКОНЧЕНА###
    }

    private function new_order_user($data)
    {
        $taxesInOrder = array();
        $payment = $data["payment"];
        $delivery = $data["delivery"];
        $order = $data["order"];
        $items = $data["order_items"];
        $summa = 0;
        $total = 0;
        $counter = 0;
        $vendor = Vendor::getInstance()->getVendor($order->o_vendor);
        $price_digits = Module::getInstance("orders")->getParam("price_digits");
        $quantity_digits = Module::getInstance("orders")->getParam("quantity_digits");
        // ###ВЕРСТКА НАЧАТА###
        ?>
        <div style="border:1px solid #ddd;padding:8px;">
            <h1><?php echo Text::_("Order") . " № " . $order->o_id . " " . Text::_("from") . " " . Date::GetdateRus($order->o_date); ?></h1>
            <div style="margin-bottom: 12px;"><b><?php echo Text::_("Vendor") . ":</b> " . $vendor->v_name; ?></div>
            <div style="margin-bottom: 12px;">
                <b><?php echo Text::_("Delivery type"); ?>: </b><?php echo $order->o_dt_title; ?>
                <?php
                if ($delivery) {
                    $delivery_data = $delivery->getDecodedData($order->o_dt_data);
                    echo $delivery->renderInfo($delivery_data);
                }
                ?>
            </div>
            <div style="margin-bottom: 12px;">
                <b><?php echo Text::_("Payment type"); ?>: </b><?php echo $order->o_pt_title; ?>
                <?php
                if ($payment) {
                    $payment_data = $payment->getDecodedData($order->o_pt_data);
                    echo $payment->renderInfo($payment_data);
                }
                ?>
            </div>
            <?php
            $ud = new orderUserdata(0);
            $userdata = $ud->getDecodedData($order->o_userdata);
            if (is_array($userdata)) {
                $_userdata_arr = array();
                if (array_key_exists("userdata_person", $userdata) && $userdata["userdata_person"]) $_userdata_arr[] = "<b>" . Text::_("Contact person") . ":</b> " . $userdata["userdata_person"];
                if (array_key_exists("userdata_email", $userdata) && $userdata["userdata_email"]) $_userdata_arr[] = "<b>" . Text::_("Contact email") . ":</b> " . $userdata["userdata_email"];
                if (array_key_exists("userdata_phone", $userdata) && $userdata["userdata_phone"]) $_userdata_arr[] = "<b>" . Text::_("Contact phone") . ":</b> " . $userdata["userdata_phone"];
                if (count($_userdata_arr)) { ?>
                    <div class="order_row"><?php echo implode("<br />", $_userdata_arr); ?></div><?php }
            }
            /* hide prices check start */
            $cols4name = 3;
            $cols4price = 2;
            $cols4sum = 2;
            $show_prices = true;
            $test_sum = 0;
            foreach ($items as $g) {
                $test_sum = $test_sum + $g->i_g_sum;
            }
            if (!$test_sum) {
                $cols4name = $cols4name + $cols4price + $cols4sum;
                $show_prices = false;
            }
            /* hide prices check stop */
            ?>
            <?php if (trim(strip_tags($order->o_comments))) { ?>
                <div style="margin-bottom: 12px;">
                    <b><?php echo Text::_("Comments"); ?>: </b><?php echo htmlspecialchars($order->o_comments); ?>
                </div>
            <?php } ?>
            <div style="display:table;border-top:1px solid #ddd;border-left:1px solid #ddd; margin-bottom: 12px;width:100%;">
                <div style="display:table-row;background-color: #f2f2f2;">
                    <div style="display:table-cell;white-space:nowrap;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                        <?php echo Text::_('NPP'); ?>
                    </div>
                    <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                        <?php echo Text::_('Articul'); ?>
                    </div>
                    <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                        <?php echo Text::_('Goods title'); ?>
                    </div>
                    <?php if ($show_prices) { ?>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                            <?php echo Text::_('Price'); ?>
                        </div>
                    <?php } ?>
                    <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                        <?php echo Text::_('Quantity'); ?>
                    </div>
                    <?php if ($show_prices) { ?>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                            <?php echo Text::_('Sum'); ?>
                        </div>
                    <?php } ?>
                </div>
                <?php foreach ($items as $g) { ?>
                    <?php $counter++; ?>
                    <?php if ($g->i_g_quantity) { ?>
                        <div style="display:table-row;">
                            <?php
                            $current_price = $g->i_g_price;
                            $current_quantity = $g->i_g_quantity;
                            $current_sum = $g->i_g_sum;
                            $summa = $summa + $current_sum;
                            if ($g->i_g_tax > 0) {
                                $tax_name = base64_encode($g->i_g_tax_name);
                                if (!isset($taxesInOrder[$tax_name])) $taxesInOrder[$tax_name] = 0;
                                $taxesInOrder[$tax_name] += $g->i_g_tax;
                            }
                            ?>
                            <div style="display:table-cell;text-align:center;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                                <?php echo $counter; ?>
                            </div>
                            <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                                <?php echo $g->i_g_sku; ?>
                            </div>
                            <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                                <?php
                                echo $g->i_g_name;
                                if ($g->i_g_options_text) {
                                    if (Portal::getInstance()->inPrintMode()) {
                                        $i_g_options_title = "";
                                        $i_g_options_text = $g->i_g_options_text;
                                    } else {
                                        $i_g_options_title = $g->i_g_options_text;
                                        $i_g_options_text = mb_substr($i_g_options_title, 0, \siteConfig::$shortTextLength) . "...";
                                    }
                                    echo "<span title=\"" . $i_g_options_title . "\" class=\"basket-row-options\">" . $i_g_options_text . "</span>";
                                }
                                ?>
                            </div>
                            <?php if ($show_prices) { ?>
                                <div style="display:table-cell;white-space:nowrap;text-align:right;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                                    <?php echo number_format($current_price, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR); ?>
                                </div>
                            <?php } ?>
                            <div style="display:table-cell;white-space:nowrap;text-align:right;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                                <?php echo number_format($current_quantity, $quantity_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . " " . Measure::getInstance()->getShortName($g->i_g_measure); ?>
                            </div>
                            <?php if ($show_prices) { ?>
                                <div id="sum_<?php echo $g->i_g_id; ?>"
                                     style="display:table-cell;text-align:right;white-space:nowrap;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;">
                                    <?php echo number_format($current_sum, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR); ?>
                                </div>
                            <?php } ?>
                        </div>
                    <?php } ?>
                <?php } ?>
            </div>
            <?php if ($order->o_points > 0 || $summa) { ?>
                <div style="display:table;border-top:1px solid #ddd;border-left:1px solid #ddd; margin-bottom: 12px;width:100%;">
                    <div style="display:table-row;background-color: #f2f2f2;">
                        <?php if ($order->o_points > 0) { ?>
                            <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                                <?php echo Text::_('Total points'); ?>
                            </div>
                            <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                                <?php echo $order->o_points; ?>
                            </div>
                        <?php } else { ?>
                            <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;"></div>
                        <?php } ?>
                        <?php if ($summa) { ?>
                            <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                                <?php echo Text::_('Sum'); ?>
                            </div>
                            <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                                <?php echo number_format($summa, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                            </div>
                        <?php } else { ?>
                            <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;"></div>
                        <?php } ?>
                    </div>
                </div>
            <?php } ?>
            <?php $total = $summa; ?>
            <div style="display:table;border-top:1px solid #ddd;border-left:1px solid #ddd; margin-bottom: 12px;width:100%;">
                <?php if (($order->o_dt_sum) && (($order->o_dt_sum) > 0)) {
                    if ($order->o_dt_tax_val) {
                        $tax_name = base64_encode($order->o_dt_tax_name);
                        if (!isset($taxesInOrder[$tax_name])) $taxesInOrder[$tax_name] = 0;
                        $taxesInOrder[$tax_name] += $order->o_dt_tax_sum;
                    } ?>
                    <div style="display:table-row;">
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo Text::_("Delivery sum"); ?>
                        </div>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo number_format($order->o_dt_sum, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                        </div>
                    </div>
                    <?php $total = $total + $order->o_dt_sum; ?>
                <?php } ?>
                <?php if (($order->o_pt_sum) && ($order->o_pt_sum > 0)) { ?>
                    <div style="display:table-row;">
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo Text::_("Payment commission"); ?>
                        </div>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo number_format($order->o_pt_sum, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                        </div>
                    </div>
                    <?php $total = $total + $order->o_pt_sum; ?>
                <?php } ?>
                <?php if (floatval($order->o_discount_sum)) { ?>
                    <div style="display:table-row;">
                        <?php if ($order->o_discount_sum > 0) { ?>
                            <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                                <?php echo Text::_("Discount"); ?>
                            </div>
                        <?php } else { ?>
                            <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                                <?php echo Text::_("Fee"); ?>
                            </div>
                        <?php } ?>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo number_format($order->o_discount_sum, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                        </div>
                    </div>
                    <?php $total = $total - $order->o_discount_sum; ?>
                <?php } ?>
                <?php if ($total != $summa) { ?>
                    <div style="display:table-row;">
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo Text::_("Total"); ?>
                        </div>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo number_format($total, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                        </div>
                    </div>
                <?php } ?>
                <?php if (floatval($order->o_taxes_sum)) { ?>
                    <div style="display:table-row;">
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo Text::_("Taxes in order"); ?>
                        </div>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo number_format($order->o_taxes_sum, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                        </div>
                    </div>
                    <?php if (count($taxesInOrder)) { ?>
                        <?php foreach ($taxesInOrder as $kt => $vt) { ?>
                            <div style="display:table-row;">
                                <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                                    <?php print base64_decode($kt); ?>
                                </div>
                                <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                                    <?php echo number_format($vt, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                                </div>
                            </div>
                        <?php } ?>
                    <?php } ?>
                <?php } ?>
                <?php if (floatval($order->o_total_sum) > 0) { ?>
                    <div style="display:table-row;">
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo Text::_("Grand total"); ?>
                        </div>
                        <div style="display:table-cell;padding:6px 4px;border-bottom:1px solid #ddd;border-right:1px solid #ddd;text-align:right;">
                            <?php echo number_format($order->o_total_sum, $price_digits, DEFAULT_DECIMAL_SEPARATOR, DEFAULT_THOUSAND_SEPARATOR) . "&nbsp;" . Currency::getShortName(DEFAULT_CURRENCY); ?>
                        </div>
                    </div>
                <?php } ?>
            </div>
        </div>

        <!-- SPECIALLY FOR USER :) -->
        <h4><?php echo Text::_("Thank you for your order"); ?></h4>
        <br/><br/><a id="print_but" target="_blank"
                     href="<?php echo Router::_("index.php?module=orders&view=order&order_id=" . $order->o_id . ($order->o_hash ? "&order_hash=" . $order->o_hash : ""), false, true, 1, 2); ?>"><?php echo Text::_('Link to order'); ?></a>
        <?php
        // ###ВЕРСТКА ЗАКОНЧЕНА###
    }

    private function order_paid_admin($order){
        // ###ВЕРСТКА НАЧАТА###
        echo "<h2 class=\"title\">";
        echo sprintf(Text::_("Payment for the order has been successfully completed"), "№ ".$order->o_id." ".Text::_("from")." ".Date::GetdateRus($order->o_date));
        echo "</h2>";
        // ###ВЕРСТКА ЗАКОНЧЕНА###
    }

}

?>
