<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: a4ab2987 (2023-09-16 13:56:14)
// 

use ArfaCMS\Core\Date;
use ArfaCMS\Core\Userdata;
use ArfaCMS\MVC\Module;
use ArfaCMS\Sprav\SpravModel;

defined('_ARFACMS_VALID') or die("Access denied");

class ordersModelOrders extends SpravModel
{
    public function decodeOrdersData($orders)
    {
        if (count($orders)) {
            foreach ($orders as $order) {
                $datafields = array("o_dt_data", "o_pt_data", "o_userdata", "o_pt_result");
                foreach ($datafields as $fld) {
                    if (\backofficeConfig::$cryptoUserData) $order->{$fld} = Userdata::getInstance($order->o_uid, true)->decode($order->{$fld});
                    else $order->{$fld} = json_decode(base64_decode($order->{$fld}), true);
                }
            }
        }
        return $orders;
    }

    public function getOrders()
    {
        $reestr = Module::getInstance()->get('reestr');
        $psids = $reestr->get('arr_psid');
        if (is_array($psids)) $arr_psid = "(" . implode(",", $psids) . ")"; else $arr_psid = "(" . $psids . ")";
        $sql = "SELECT * FROM #__orders WHERE o_id IN " . $arr_psid . " ORDER BY o_date DESC";
        $this->_db->setQuery($sql);
        return $this->_db->loadObjectList();
    }

    public function getOrdersItems()
    {
        $reestr = Module::getInstance()->get('reestr');
        $arr_psid = $reestr->get('arr_psid');
        if (is_array($arr_psid)) $psids = "(" . implode(",", $arr_psid) . ")"; else $psids = "(" . $arr_psid . ")";
        $sql = "SELECT * FROM #__orders_items WHERE i_order_id IN " . $psids . " ORDER BY i_order_id DESC";
        $this->_db->setQuery($sql);
        return $this->_db->loadObjectList();
    }

    // =============================================
    // For 1C:exchange
    // =============================================
    public function getOrdersByDate($start_date = false, $end_date = false, $skip_deleted = false, $statuses = array())
    {
        $sql = "SELECT * FROM #__orders";
        $where = array();
        if ($start_date && Date::isSQLDate($start_date)) $where[] = " o_date>='" . $start_date . "'";
        if ($end_date && Date::isSQLDate($end_date)) $where[] = " o_date<='" . $end_date . "'";
        if (count($statuses)) $where[] = " o_status IN (" . implode(",", $statuses) . ")";
        if ($skip_deleted) $where[] = " o_deleted=0";
        if (count($where)) $sql .= " WHERE " . implode(" AND ", $where);
        $sql .= " ORDER BY o_date";
        $this->_db->setQuery($sql);
        return $this->_db->loadObjectList();
    }

    public function getOrderItems($psid)
    {
        $sql = "SELECT * FROM #__orders_items WHERE i_order_id =" . $psid;
        $this->_db->setQuery($sql);
        return $this->_db->loadObjectList();
    }

    public function getStatusName($psid)
    {
        $status = $this->getStatus($psid);
        if (isset($status->os_name)) return $status->os_name;
        else return "";
    }

    public function setStatus($psid, $status)
    {
        if (!$status || !$psid) return false;
        $sql = "UPDATE `#__orders` SET `o_status`=" . $status . " WHERE `o_id`=" . $psid;
        $this->_db->setQuery($sql);
        return $this->_db->query();
    }
    /*
	// @ TODO May be delete ???
	public function getStatus($psid){
		$object = null;
		$sql="SELECT * FROM #__orders_status WHERE os_enabled=1 AND os_deleted=0 AND os_id=".$psid;
		$this->_db->setQuery($sql);
		$this->_db->loadObject($object);
		return $object;
	}
	public function getStatuses(){
		$sql="SELECT * FROM #__orders_status WHERE os_enabled=1 AND os_deleted=0 ORDER BY os_id ASC";
		$this->_db->setQuery($sql);
		return $this->_db->loadObjectList('os_id');
	}
    */
    /*
	// @ TODO May be delete ???
    // список картинок для набора товаров
	public function getImageForItems($id_array) {
		$sql="select g_id,g_thumb,g_image,g_name from #__goods WHERE g_id IN ('".implode("','",$id_array)."')";
		$this->_db->setQuery($sql);
		return $this->_db->loadObjectList('g_id');
	}
    */
}

?>