<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: e98f078f (2024-10-06 21:04:05)
// 

use ArfaCMS\Core\Files;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Settings;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Util;
use ArfaCMS\MVC\Model;

defined('_ARFACMS_VALID') or die("Access denied");

class serviceModelUpdater extends Model
{
    private $_update_log = array();
    private $_url = "https://arfa-cms.ru/customer/";
    private $_delimiter = null;
    private $_query_params = array();

    public function __construct($module)
    {
        parent::__construct($module);
        $this->_delimiter = $this->_db->getDelimiter();
        $this->_query_params["site_addr"] = \siteConfig::$siteDomain;
        $this->_query_params["site_auth_key"] = \backofficeConfig::$regKey;
        $this->_query_params["version_major"] = Portal::getVersionMajor();
        $this->_query_params["version_minor"] = Portal::getVersionMinor();
        /*
         // For updates to new major or minor version set them directly
         $this->_query_params["version_major"] = 100;
         $this->_query_params["version_minor"] = 200;
         */
        $this->_query_params["version_beta"] = \backofficeConfig::$updatesBetaChannel ? "1" : "0";
    }

    private function logOK($text)
    {
        $this->_update_log[] = $text . " - <span class=\"log_ok\">OK</span>";
    }

    private function logError($text)
    {
        $this->_update_log[] = $text . " - <span class=\"log_error\">" . Text::_("Error") . "</span>";
    }

    private function logWarning($text)
    {
        $this->_update_log[] = $text . " - <span class=\"log_warning\">" . Text::_("Warning") . "</span>";
    }

    private function logMessage($text)
    {
        $this->_update_log[] = $text;
    }

    public function getLog()
    {
        return $this->_update_log;
    }

    public function getNewMajorVersion()
    {
        return $this->_query_params["version_major"];
    }

    public function getNewMinorVersion()
    {
        return $this->_query_params["version_minor"];
    }

    public function getFullCurrentVersion($current_revision)
    {
        return $this->getNewMajorVersion() . "." . $this->getNewMinorVersion() . "." . $current_revision;
    }

    public function getVersionRevision()
    {
        return Portal::getInstance()->getVersionRevision();
    }

    public function changedMajorMinor()
    {
        if ($this->getNewMajorVersion() != Portal::getVersionMajor() || $this->getNewMinorVersion() != Portal::getVersionMinor())
            return $this->getNewMajorVersion() . "." . $this->getNewMinorVersion();
        else return "";
    }

    public function checkUpdates()
    {
        $query_params = $this->_query_params;
        $query_params["task"] = "checkVersion";
        $url = $this->_url . '?' . http_build_query($query_params);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url); // set url to post to
        curl_setopt($ch, CURLOPT_FAILONERROR, 1);
        curl_setopt($ch, CURLOPT_FRESH_CONNECT, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 0);// allow redirects
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); // return into a variable
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($ch, CURLOPT_TIMEOUT, 5);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
        curl_setopt($ch, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
        // curl_setopt($ch, CURLOPT_POST, 1); 	// set POST method
        // curl_setopt($ch, CURLOPT_POSTFIELDS, ""); // add POST fields
        $result = curl_exec($ch); // run the whole process
        curl_close($ch);
        try {
            // Util::pre($result);
            $res = json_decode($result, true);
            // Util::pre($res);
            return $res;
        } catch (Exception $e) {

        }
        return null;
    }

    public function getUpdateCode($res)
    {
        if (isset($res["code"])) return intval($res["code"]);
        return 0;
    }

    public function getUpdateStatus($res)
    {
        if (isset($res["status"])) return $res["status"];
        return "failed";
    }

    public function getUpdateRevision($res)
    {
        if (isset($res["version"])) return intval($res["version"]);
        return 0;
    }

    public function getUpdateMessage($res)
    {
        if (isset($res["message"])) return $res["message"];
        return "";
    }

    /*
    public function checkNewVersion(){
        $query_params = $this->_query_params;
        $query_params["task"] = "checkVersion";
        $url = $this->_url . '?' . http_build_query($query_params);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL,$url); // set url to post to
        curl_setopt($ch, CURLOPT_FAILONERROR, 1);
        curl_setopt($ch, CURLOPT_FRESH_CONNECT,1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 0);// allow redirects
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); // return into a variable
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($ch, CURLOPT_TIMEOUT, 5);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
        curl_setopt($ch, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
        // curl_setopt($ch, CURLOPT_POST, 1); 	// set POST method
        // curl_setopt($ch, CURLOPT_POSTFIELDS, ""); // add POST fields
        $result = curl_exec($ch); // run the whole process
        curl_close($ch);
        $new_version = 0;
        try{
            //			Util::pre($result);
            $res = json_decode($result, true);
            //			Util::pre($res);
            if(isset($res["status"]) && $res["status"] == "success"){
                $new_version = isset($res["version"]) ? $res["version"] : "0";
            }
        } catch (Exception $e) {

        }
        return $new_version;
    }
    */
    public function proceedPackage()
    {
        $cur_update = "update" . time();
        $filename = PATH_TMP . DS . $cur_update . ".zip";
        $_SESSION['update_folder'] = $cur_update;
        if ($this->getUpdatePackage($cur_update, $filename)) {
            if ($this->unzipPackage($cur_update, $filename)) {
                if ($this->checkFiles($cur_update)) return true;
            }
        }
        return false;
    }

    private function unzipPackage($cur_update, $filename)
    {
        $zip = new ZipArchive;
        if (!$zip) {
            $this->logError(Text::_("ZipArchive object error"));
            return false;
        }
        $res = $zip->open($filename);
        if ($res === TRUE) {
            $zip->extractTo(PATH_TMP . $cur_update);
            $zip->close();
            unlink($filename);
            $this->logOK(Text::_("Extracting package contents to temporary folder"));
        } else {
            $this->logError(Text::_("Package open error") . " : " . $res);
            return false;
        }
        return true;
    }

    private function getUpdatePackage($cur_update, $filename)
    {
        if (mkdir(PATH_TMP . $cur_update, 0755, false)) $this->logOK(Text::_("Creating temporary folder"));
        else {
            $this->logError(Text::_("Directory is not writable") . " : " . PATH_TMP);
            return false;
        }
        $package = fopen($filename, 'wb');
        $query_params = $this->_query_params;
        $query_params["task"] = "getUpdatePackage";
        $url = $this->_url . '?' . http_build_query($query_params);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_FAILONERROR, 1);
        curl_setopt($ch, CURLOPT_FRESH_CONNECT, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 0);// allow redirects
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($ch, CURLOPT_FILE, $package);
        curl_setopt($ch, CURLOPT_HEADER, 0); // allow redirects
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 30);
        curl_setopt($ch, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
        if (curl_exec($ch)) {
            $this->logOK(Text::_("File loading"));
        } else {
            $this->logError(Text::_("Error at file loading with code") . " : " . curl_error($ch));
            curl_close($ch);
            fclose($package);
            return false;
        }
        curl_close($ch);
        fclose($package);
        return true;
    }

    private function deleteUpdateFolder($src)
    {
        if (!Files::removeFolder($src, 1)) {
            $this->logWarning(Text::_("Temporary folder can not be deleted"));
            $this->logWarning(Text::_("Remove it manually") . " : " . $src);
        }
        if (isset($_SESSION['update_folder'])) unset($_SESSION['update_folder']);
    }

    public function proceedUpdates($new_version)
    {
        if (isset($_SESSION['update_folder']) && $_SESSION['update_folder']) {
            $cur_update = $_SESSION['update_folder'];
            unset($_SESSION['update_folder']);
            $src = PATH_TMP . $cur_update;
            if ($this->copyAdminFolder($cur_update)) {
                if ($this->copyFiles($cur_update)) {
                    $new_rev = floor($new_version);
                    // $my_rev=Portal::getInstance()->getVersionRevision();
                    if ($this->proceedDbRestructure($new_rev)) {
                        $this->logOK(Text::_("Database restructuring finished")); // "<br />".
                    } else {
                        $this->logWarning(Text::_("Database restructuring failed")); // "<br />".
                        $this->logWarning(Text::_("Apply restructuring manually"));
                    }
                    $this->logOK(Text::_("Update finished")); // "<br />".
                    $this->deleteUpdateFolder($src);
                }
            }
        } else Util::redirect("index.php", Text::_("Updates folder not found"));
        return $this->_update_log;
    }

    private function copyAdminFolder($cur_update)
    {
        $result = true;
        if (!defined("ADMIN_FOLDER") || ADMIN_FOLDER == "") {
            $this->logError(Text::_("Admin folder not found"));
            $result = false;
            return $result;
        }
        $src = PATH_TMP . $cur_update . DS . "sources" . DS . "admin_folder" . DS;
        if (!is_dir($src)) {
            Util::redirect("index.php", Text::_("Admin updates folder not found"));
        } else {
            $dest = PATH_SITE . ADMIN_FOLDER . DS;;
            $this->logMessage(Text::_("Processing admin folder"));

            if (is_writable($dest)) $this->logOK(Text::_("Is writeble") . " : " . $dest);
            else {
                $this->logError(Text::_("Is not writeble") . " : " . $dest);
                $result = false;
            }
            if (Files::copyFolder($src, $dest, true)) $this->logOK(Text::_("All files are copied"));
            else {
                $this->logError(Text::_("Copying of files failed"));
                $this->logWarning(Text::_("It is strongly recommended to perform manual update"));
                $result = false;
            }
            if ($result) {
                if (Files::removeFolder($src, 1)) $this->logOK(Text::_("Temporary folder deleted"));
                else {
                    $this->logError(Text::_("Temporary folder can not be deleted") . " : " . $src);
                    $result = false;
                }
            }
        }
        return $result;
    }

    private function copyFiles($cur_update)
    {
        $result = true;
        $src = PATH_TMP . $cur_update . DS . "sources" . DS;
        if (!is_dir($src)) {
            Util::redirect("index.php", Text::_("Updates folder not found"));
        } else {
            $dest = PATH_SITE;
            $this->logMessage(Text::_("Processing package"));
            if (is_writable($dest)) $this->logOK(Text::_("Is writeble") . " : " . $dest);
            else {
                $this->logError(Text::_("Is not writeble") . " : " . $dest);
                $result = false;
            }
            if (Files::copyFolder($src, $dest, true)) $this->logOK(Text::_("All files are copied"));
            else {
                $this->logError(Text::_("Copying of files failed"));
                $this->logWarning(Text::_("It is strongly recommended to perform manual update"));
                $result = false;
            }
        }
        return $result;
    }

    private function checkFiles($cur_update)
    {
        $result = true;
        $src = PATH_TMP . $cur_update . DS . "sources" . DS;
        $hiddenfiles = array(".", "..");
        if (!is_dir($src)) {
            if (isset($_SESSION['update_folder'])) unset($_SESSION['update_folder']);
            Util::redirect("index.php", Text::_("Updates folder not found"));
        } else {
            $dest = PATH_SITE;
            $fc = array();
            $counter = 0;
            $this->logMessage(Text::_("Processing package"));
            Files::getFolderContent($src, "", $hiddenfiles, 1, $fc, $counter);
            if (($fc) && (count($fc))) {
                foreach ($fc as $f) {
                    if ($f["subpath"]) $tmppath = $dest . $f["subpath"] . DS . $f["filename"];
                    else $tmppath = $dest . $f["filename"];
                    if (!Files::canWrite($tmppath, $f["folder"])) {
                        $this->logError(Text::_("Is not writeble") . " : " . $tmppath);
                        $result = false;
                        // Lets delete updates
                        $src = PATH_TMP . $cur_update;
                        $this->deleteUpdateFolder($src);
                    }
                }
            }
        }
        if ($result) $this->logOK(Text::_("All files and folders are writeble"));
        else {
            $this->logError(Text::_("Some files and folders are not writeble")); // "<br />".
        }
        return $result;
    }

    public function proceedDbRestructure($new_rev)
    {
        $result = true;
        $my_rev = intval(Settings::getVar("restruct_version"));
        if ($my_rev < $new_rev) {
            for ($rev = $my_rev + 1; $rev <= $new_rev; $rev++) {
                $file = PATH_MODULES . "service" . DS . "restructure" . DS . "restructure_" . $rev . ".php";
                $className = "Restructure_" . $rev;
                if (is_file($file)) {
                    require_once $file;
                    if (!class_exists($className, false)) {
                        $this->logError(Text::_("Error applying restructure file") . ": " . $className);
                        return false;
                    }
                    $messages = array();
                    $errors = array();
                    $r_class = new $className;
                    if (!$r_class->proceed($messages, $errors)) {
                        if (count($errors)) {
                            foreach ($errors as $error) $this->logError($error);
                        }
                        return false;
                    }
                    if (count($messages)) {
                        foreach ($messages as $message) $this->logOK($message);
                    }
                    Settings::setVar("restruct_version", $rev);
                }
            }
        }
        Settings::setVar("restruct_version", $new_rev);
        return $result;
    }
}

?>