<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: cfd81958 (2024-10-09 21:59:11)
// 

use ArfaCMS\Core\Files;
use ArfaCMS\Core\HTMLControls;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Text;

defined('_ARFACMS_VALID') or die("Access denied");

?>
<div id="mediamanager" class="media-manager rounded-pan">
    <h4 class="title main-title"><?php echo Text::_("Media manager"); ?></h4>
    <?php if ($this->info_message) echo "<div class=\"info_message " . $this->error_class . "\">" . $this->info_message . "</div>"; ?>
    <input type="hidden" id="mm_requestVerificationToken" value="<?php echo Request::getVerificationToken(); ?>"/>
    <div class="row mb-3">
        <div class="col-xl-6 mb-2 mb-xl-0">
            <fieldset class="grouped-fields uploader px-3 pb-3">
                <legend><?php echo Text::_("Create folder"); ?></legend>
                <div class="row mb-3">
                    <div class="col-lg-12"><input class="form-control" type="text" id="newfolder"/></div>
                </div>
                <div class="buttons">
                    <input class="commonButton btn btn-primary" type="button" onclick="mm_createFolder('<?php echo $this->folder; ?>')" value="<?php echo Text::_("Create"); ?>"/>
                </div>
            </fieldset>
        </div>
        <div class="col-xl-6">
            <fieldset class="grouped-fields uploader px-3 pb-3">
                <legend><?php echo Text::_("Upload file"); ?></legend>
                <form id="mm_single_upload" method="post" action="<?php echo $_SERVER['PHP_SELF']; ?>" enctype="multipart/form-data" onsubmit="mm_uploadFile(event)">
                    <div class="fileselector-no-buttons">
                        <?php echo HTMLControls::renderInputFile("up_file", "", 38, "up_file"); ?>
                    </div>
                    <p>
                        <span class="warning"><?php echo Text::_("Max file size") . " " . ini_get("upload_max_filesize"); ?></span>
                    </p>
                    <input type="hidden" name="_requestVerificationToken" value="<?php echo Request::getVerificationToken(); ?>"/>
                    <input type="hidden" name="option" value="ajax"/>
                    <input type="hidden" name="task" value="createMedia"/>
                    <input type="hidden" name="module" value="service"/>
                    <input type="hidden" name="folder" value="<?php echo $this->folder; ?>"/>
                    <div class="upload-progress">
                        <div class="bar"></div>
                        <div class="percent">0%</div>
                    </div>
                    <div id="status"></div>
                    <div class="buttons mt-3">
                        <input class="commonButton btn btn-primary" type="submit" value="<?php echo Text::_("Upload"); ?>"/>
                    </div>
                </form>
            </fieldset>
        </div>
    </div>
    <div class="filemanager-wrapper">
        <?php
        echo "<table id=\"filemanager\" class=\"table table-bordered table-hover table-condensed align-middle sprav-table\">";
        if ($this->folder) {
            echo "<tr><td class=\"currentfolder\" colspan=\"6\">";
            echo "<a class=\"item item-folder\" title=\"" . Text::_("Folder up") . "\" onclick=\"mm_getContent('" . $this->folder . "','',1)\"><i class=\"bi bi-arrow-up-left-square\"></i></a>";
            echo "<b>" . Text::_("Current folder") . ":</b> " . $this->folder;
            echo "</td></tr>";
        }
        echo "<tr class=\"text-center\">";
        echo "<th width=\"5%\">&nbsp;</th>";
        echo "<th>" . Text::_("Name") . "</th>";
        echo "<th width=\"15%\">" . Text::_("File size") . "</th>";
        echo "<th width=\"15%\">" . Text::_("File date") . "</th>";
        echo "<th width=\"15%\">" . Text::_("Actions") . "</th>";
        echo "</tr>";
        $files = $this->files;
        if ($files && count($files)) {
            foreach ($files as $file) {
                $ext = strtolower(substr($file['filename'], strrpos($file['filename'], '.') + 1));
                $filename = stripslashes($file['filename']);
                if ($file['folder'] == 1 && $filename == "..") continue;
                if (mb_strlen($filename, DEF_CP) > 43) {
                    $short_filename = mb_substr($file['filename'], 0, 40, DEF_CP) . '...';
                } else $short_filename = $filename;
                echo "<tr class=\"text-center\">";
                $delete_js = "javascript:if(confirm('" . Text::_("Do you want to delete") . " " . $filename . " ?'))return true; else return false;";
                if ($file['folder'] == 1) {
                    echo "<td><a class=\"mmButton btn btn-primary\" title=\"" . Text::_("Delete") . "\" onclick=\"mm_deleteFolder('" . $this->folder . "','" . $filename . "','" . Text::mapjsAddSlashes(Text::_("Do you want to delete") . "?") . "')\">";
                    echo "<i class=\"bi bi-trash\"></i>";
                    echo "</a></td>";
                    echo "<td class=\"text-start\"><a class=\"item item-folder\" title=\"" . Text::_("Open folder") . "\" onclick=\"mm_getContent('" . $this->folder . "','" . $filename . "',0)\"><i class=\"bi bi-folder\"></i>" . $short_filename . "</a></td>";
                    echo "<td>" . ($filename == ".." ? "" : $file['filesize'] . " KB") . "</td>";
                    echo "<td>" . ($filename == ".." ? "" : $file['filedate']) . "</td>";
                    echo "<td>&nbsp;</td>";
                } elseif ($file['folder'] == 0) {
                    $href = "index.php?module=service&amp;task=downloadFile&amp;folder=" . rawurlencode($this->folder) . "&amp;file=" . rawurlencode($filename);
                    echo "<td><a class=\"mmButton btn btn-primary\" title=\"" . Text::_("Delete") . "\"  onclick=\"mm_deleteFile('" . $this->folder . "','" . $filename . "','" . Text::mapjsAddSlashes(Text::_("Do you want to delete") . "?") . "')\">";
                    echo "<i class=\"bi bi-trash\"></i>";
                    echo "</a></td>";
                    echo "<td class=\"text-start\"><a class=\"item item-file\" title=\"" . Text::_("Get link") . "\" class=\"getlink\" onclick=\"mm_setLink(this, '" . $this->folder . "','" . $filename . "')\"><i class=\"bi bi-file-earmark\"></i>" . $short_filename . "</a></td>";
                    echo "<td>" . $file['filesize'] . " KB</td>";
                    echo "<td>" . $file['filedate'] . "</td>";
                    echo "<td class=\"text-nowrap\">";
                    echo "<a class=\"mmButton btn btn-primary\" title=\"" . Text::_("Download") . "\" target=\"_blank\" class=\"download\" href=\"" . $href . "\"><i class=\"bi bi-download\"></i></a>";
                    if (Files::isImage($this->filepath . DS . $filename)) {
                        $imagehref = ARFACMS_STORAGE_LINK . "/" . Files::pathURLEncode($this->folder) . "/" . rawurlencode($filename);
                        echo "&nbsp;<a class=\"rel-popup-image btn btn-primary\" target1=\"_blank\" title=\"" . Text::_("Preview") . "\" class=\"preview rel-popup-image\" href=\"" . $imagehref . "\"><i class=\"bi bi-camera\"></i></a>";
                    } else echo "&nbsp;";
                    echo "</td>";
                }
                echo "</tr>";
            }
        }
        echo "</table>";
        ?>
        <div id="medialink">...</div>
    </div>
</div>
