<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: dbee5e70 (2024-10-10 20:33:04)
// 

use ArfaCMS\Core\Event;
use ArfaCMS\Core\HTMLControls;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\User;
use ArfaCMS\Core\Util;

defined('_ARFACMS_VALID') or die("Access denied");

$data_sn = array();
$suffix = "";
if (\backofficeConfig::$allowSNLogin) {
    Event::raise("user.login_form", array("module" => "user"), $data_sn);
    if (count($data_sn)) $suffix = "-sn";
}
?>
<h4 class="title main-title"><?php echo Text::_("Authorization"); ?></h4>
<div class="auth-block<?php echo $suffix; ?>">
    <div id="loginForm" class="commonPopup">
        <form action="<?php echo Router::_("index.php"); ?>" method="post">
            <input type="hidden" name="option" value="login"/>
            <input type="hidden" name="return_url" value="<?php echo((isset($this->return_url) && $this->return_url) ? base64_encode($this->return_url) : Util::getRefererUrl()); ?>"/>
            <div class="row mb-3">
                <div class="login-label col-sm-12 mb-2 align-self-center">
                    <?php echo(\backofficeConfig::$allowEmailLogin ? HTMLControls::renderLabelField(false, Text::_("Email") . ":") : HTMLControls::renderLabelField(false, Text::_("Login name") . ":")); ?>
                </div>
                <div class="login-input col-sm-12">
                    <input type="text" required="required" class="commonEdit loginEdit form-control" name="username" value=""/>
                </div>
            </div>
            <div class="row mb-3">
                <div class="pwd-label col-sm-12 mb-2 align-self-center">
                    <?php echo HTMLControls::renderLabelField(false, Text::_("Password") . ":"); ?>
                </div>
                <div class="pwd-input col-sm-12">
                    <input type="password" required="required" class="commonEdit loginEdit form-control" name="userpass" value=""/>
                </div>
            </div>
            <div class="row mb-3">
                <div class="pwd-input col-sm-12">
                    <div class="form-check">
                        <?php echo HTMLControls::renderCheckbox("remember", 0, 1, "remember_me"); ?>
                        <?php echo HTMLControls::renderLabelField('remember_me', 'Remember me', true); ?>
                    </div>
                </div>
            </div>
            <div id="loginButtons" class="buttons">
                <?php echo HTMLControls::renderButton("login", Text::_('Log in'), "submit", "", "btn btn-primary"); ?>
                <?php echo(!\backofficeConfig::$noRegistration && !User::getInstance()->isLoggedIn() ? "<a rel=\"nofollow\" class=\"regButton linkButton btn btn-primary\" href=\"" . Router::_("index.php?module=user&amp;view=register") . "\">" . Text::_("Register") . "</a>" : ""); ?>
            </div>
        </form>
    </div>
    <?php if (count($data_sn)) { ?>
        <div id="snBlock" class="mt-3 text-center">
            <p class="title"><?php echo Text::_("You may login as"); ?></p>
            <div id="snButtons" class="sn-buttons mt-0">
                <?php echo implode($data_sn); ?>
            </div>
        </div>
    <?php } ?>
</div>