<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 1714fcae (2024-10-17 22:21:56)
// 

use ArfaCMS\Core\aForm;
use ArfaCMS\Core\Event;
use ArfaCMS\Core\HTMLControls;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\User;
use ArfaCMS\MVC\Module;

defined('_ARFACMS_VALID') or die("Access denied");

$js = "$(document).ready(function() {
    $('#rLogin').bind('blur', function() { checkFieldUnique('user',this,0); });
    $('#rNickname').bind('blur', function() { checkFieldUnique('user',this,0); });
    $('#rEmail').bind('blur', function() { checkFieldUnique('user',this,0); });
});
";
Portal::getInstance()->addScriptDeclaration($js); 
$rules_art = Module::getHelper('article','article')->getArticle(intval(\siteConfig::$site_rules_article));
$pp_art = Module::getHelper('article','article')->getArticle(intval(\siteConfig::$privacy_policy_article));
if(is_object($rules_art) && is_object($pp_art)) {
    $frm = new aForm("regForm");
    $frm->addInput(array("ID"=>"task", "TYPE"=>"hidden", "NAME"=>"task", "VAL"=>"registerUser"));
    $frm->addInput(array("ID"=>"module", "TYPE"=>"hidden", "NAME"=>"module", "VAL"=>"user"));
    $frm->addInput(array("ID"=>"return_url", "TYPE"=>"hidden", "NAME"=>"return_url", "VAL"=>$this->return_url));
    $frm->addInput(array("ID"=>"rNickname", "TYPE"=>"text", "NAME"=>"rNickname", "VAL"=>"", "LABEL"=>Text::_("Nickname"), "REQUIRED"=>array("FLAG"=>true), "CLASS"=>"commonEdit form-control"));
    $frm->addInput(array("ID"=>"rLogin", "TYPE"=>"text", "NAME"=>"rLogin", "VAL"=>"", "LABEL"=>Text::_("Login name"), "REQUIRED"=>array("FLAG"=>true), "CLASS"=>"commonEdit form-control"));
    $frm->addInput(array("ID"=>"rEmail", "TYPE"=>"text", "NAME"=>"rEmail", "VAL"=>"", "LABEL"=>Text::_("E-mail"), "REQUIRED"=>array("FLAG"=>true), "CLASS"=>"commonEdit form-control email"));
    $frm->addInput(array("ID"=>"rPassword", "TYPE"=>"password", "NAME"=>"rPassword", "VAL"=>"", "LABEL"=>Text::_("Password"), "REQUIRED"=>array("FLAG"=>true), "CLASS"=>"commonEdit form-control"));
    $frm->addInput(array("ID"=>"rPasswordRetype", "TYPE"=>"password", "NAME"=>"rPasswordRetype", "VAL"=>"", "LABEL"=>Text::_("Confirm password"), "REQUIRED"=>array("FLAG"=>true), "CLASS"=>"commonEdit form-control"));
    if (\siteConfig::$use_referral_system) {
        if (isset($_COOKIE['referral'])) {
            $frm->addInput(array("ID"=>"codeassign", "TYPE"=>"hidden", "NAME"=>"codeassign", "VAL"=>$_COOKIE['referral']));
            $codeassign_message = Text::_('You were invited by')." ".User::getNicknameByAffCode($_COOKIE['referral']);
        } else {
            $frm->addInput(array("ID"=>"codeassign", "TYPE"=>"text", "NAME"=>"codeassign", "VAL"=>"", "LABEL"=>Text::_("assign code"), "CLASS"=>"commonEdit form-control"));
        }
    }
    $frm->addInput(array("CLASS"=>"commonButton btn btn-primary","TYPE"=>"submit", "VAL"=>Text::_("Register"), "ID"=>"submit_button", "NAME"=>"submit_button"));
?>
    <div id="registrationForm">
        <?php $frm->startLayout(); ?>
            <h1 class="title"><?php echo Text::_("New user registration")?></h1>
            <b><?php echo Text::_('If you are already registered'); ?>, <a href="<?php echo Router::_("index.php?module=user&view=login"); ?>" rel="nofollow" class="rel-popup-text" data-modal-size="medium" data-modal-backdrop="0"><?php echo Text::_('then sign in'); ?>.</a></b><br /><br />
            <?php if (\backofficeConfig::$regConfirmation==1) { ?>
                <b><?php echo Text::_('If you do not recieved confirmation letter'); ?>, <a href="<?php echo Router::_("index.php?module=user&view=confirm"); ?>" rel="nofollow" class="rel-popup-text" data-modal-size="medium" data-modal-backdrop="0"><?php echo Text::_('click here'); ?>.</a></b><br /><br />
            <?php } ?>
            <div class="registrationFormField row mb-3">
                <div class="col-sm-12 mb-2"><?php $frm->renderLabelFor("rNickname"); ?></div>
                <div class="col-sm-12"><?php $frm->renderInputPart("rNickname"); ?></div>
            </div>
            <?php if(!\backofficeConfig::$allowEmailLogin) { ?>
                <div class="registrationFormField row mb-3">
                    <div class="col-sm-12 mb-2"><?php $frm->renderLabelFor("rLogin"); ?></div>
                    <div class="col-sm-12"><?php $frm->renderInputPart("rLogin"); ?></div>
                </div>
            <?php } ?>
            <div class="registrationFormField row mb-3">
                <div class="col-sm-12 mb-2"><?php $frm->renderLabelFor("rEmail"); ?></div>
                <div class="col-sm-12"><?php $frm->renderInputPart("rEmail"); ?></div>
            </div>
            <div class="registrationFormField row mb-3">
                <div class="col-sm-12 mb-2"><?php $frm->renderLabelFor("rPassword"); ?></div>
                <div class="col-sm-12"><?php $frm->renderInputPart("rPassword"); ?></div>
            </div>
            <div class="registrationFormField row mb-3">
                <div class="col-sm-12 mb-2"><?php $frm->renderLabelFor("rPasswordRetype"); ?></div>
                <div class="col-sm-12"><?php $frm->renderInputPart("rPasswordRetype"); ?></div>
            </div>

            <?php if (\siteConfig::$use_referral_system) { ?>
                <?php if(isset($codeassign_message)) { ?>
                    <h4><?php echo $codeassign_message; ?></h4>
                <?php } else { ?>
                    <div class="registrationFormField row mb-3">
                        <div class="col-sm-12 mb-2"><?php $frm->renderLabelFor("codeassign"); ?></div>
                        <div class="col-sm-12"><?php $frm->renderInputPart("codeassign"); ?></div>
                    </div>
                <?php } ?>
            <?php } ?>

            <div class="privacy_policy_block row mb-3">
                <div class="col-lg-12">
                    <?php echo HTMLControls::renderCheckbox("privacy_policy_agree", "", 1, "privacy_policy_agree", "commonEdit", "", true); ?>
                    <?php echo Text::_('I have read and agreed with'); ?>
                    <a rel="nofollow" class="rel-popup-text" href="<?php echo Router::_("index.php?module=article&amp;view=read&amp;psid=".$pp_art->a_id."&amp;alias=".$pp_art->a_alias."&amp;notmpl=1"); ?>">
                        <?php echo Text::_('privacy policy'); ?>
                    </a>
                </div>
            </div>
            <div class="site_policy_block row mb-3">
                <div class="col-lg-12">
                    <?php echo HTMLControls::renderCheckbox("rAgree", "", 1, "rAgree", "", "", true); ?>
                    <?php echo Text::_('I have read and agreed with'); ?>
                    <a rel="nofollow" class="rel-popup-text" href="<?php echo Router::_("index.php?module=article&amp;view=read&amp;psid=".$rules_art->a_id."&amp;alias=".$rules_art->a_alias."&amp;notmpl=1"); ?>.">
                        <?php echo Text::_('site rules'); ?>
                    </a>
                </div>
            </div>
            <?php echo Event::raise("register.renderForm",array("module"=>"user"))?>
            <div class="row-for-captcha row mb-3"><div class="col-lg-12"><?php echo Event::raise("captcha.renderForm",array("module"=>"user"))?></div></div>
            <div id="registrationFormFooter" class="buttons">
                <?php $frm->renderInputPart("submit_button"); ?>
            </div>
        <?php $frm->endLayout(); ?>
    </div>
<?php } else { ?>
    <?php echo Text::_('Registration disabled'); ?>
<?php }?>