<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 39d62b36 (2023-03-27 08:33:36)
// 

namespace ArfaCMS\Core;

defined('_ARFACMS_VALID') or die("Access denied");

final class CachingMachine extends BaseObject
{
    private static $_pattern = "<?php die(); ?>";

    public static function getHTMLCache($type, $widgetName, $cacheId, $backtime)
    {
        if (defined("_ADMIN_MODE") || !intval(\siteConfig::$cacheLife)) return;
        $backtime = $backtime * 60; // in seconds
        switch ($type) {
            case "widgets":
                if (!$backtime) return;
                $path = PATH_CACHE . $type . DS;
                $role = User::getInstance()->getRole();
                $lang = Text::getLanguage();
                $file = $widgetName . "_" . $cacheId . "_" . $role . "_" . $lang . ".php";
                break;
            default:
                if (!$backtime) $backtime = intval(\siteConfig::$cacheLife);
                $path = false;
                break;
        }
        if ($path && self::actualFile($path . $file, $backtime)) return self::readFile($path . $file);
        else return false;
    }

    public static function setHTMLCache($type, $name, $cacheId, $data, $backtime = 0)
    {
        if (defined("_ADMIN_MODE") || !intval(\siteConfig::$cacheLife)) return;
        $backtime = $backtime * 60; // in seconds
        switch ($type) {
            case "widgets":
                if (!$backtime) return;
                $path = PATH_CACHE . $type . DS;
                $role = User::getInstance()->getRole();
                $lang = Text::getLanguage();
                $file = $name . "_" . $cacheId . "_" . $role . "_" . $lang . ".php";
                break;
            default:
                if (!$backtime) $backtime = intval(\siteConfig::$cacheLife);
                $path = false;
                break;
        }
        if ($path && (!self::actualFile($path . $file, $backtime))) self::writeFile($path, $file, $data);
    }

    private static function actualFile($file, $backtime)
    {
        if (is_file($file)) {
            if (filemtime($file) < (time() - $backtime)) {
                unlink($file);
                return false;
            } else return true;
        } else return false;
    }

    private static function writeFile($path, $file, $data)
    {
        Files::checkFolder($path, true);
        $handle = fopen($path . $file, "w");
        if ($handle) {
            fwrite($handle, self::$_pattern . $data);
            fclose($handle);
            Debugger::getInstance()->message("Write cache for " . $path . $file);
        }
    }

    private static function readFile($file)
    {//return false;
        if (is_file($file)) {
            $data = file_get_contents($file, false, null, mb_strlen(self::$_pattern, DEF_CP));
            Debugger::getInstance()->message("Read cache for " . $file);
            return $data;
        } else return false;
    }
}

?>