<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: d44d7e6c (2024-10-20 19:11:26)
// 

namespace ArfaCMS\Core;

defined('_ARFACMS_VALID') or die("Access denied");

final class ErrorPages
{
    public static $_err_codes_4xx = array(403 => "Forbidden", 404 => "Not Found");

    public static function renderDump()
    {
        return Debugger::getInstance()->dump();
    }

    private static function getCustomTemplatePath($filename = "")
    {
        // Here we get default template to avoid cycling in debug mode
        // Don't use: $custom_path=PATH_TEMPLATES.Portal::getInstance()->getTemplate().DS.$filename;
        if(defined("_ADMIN_MODE")) $custom_path = PATH_TEMPLATES . \adminConfig::$adminTemplate . DS . $filename;
        else $custom_path = PATH_TEMPLATES . \siteConfig::$siteTemplate . DS . $filename;
        if (file_exists($custom_path) && is_file($custom_path)) {
            return $custom_path;
        } else {
            $custom_path = PATH_SITE . $filename;
            if (file_exists($custom_path) && is_file($custom_path)) {
                return $custom_path;
            } else {
                return false;
            }
        }
    }

    public static function renderRedirect($url, $message, $code)
    {
        if($message) Portal::addSystemMessage($message, $code);
        $system_messages = Portal::getSystemMessages();
        // Util::pre($message, $system_messages, array_column($system_messages, 'text'));
        if(count($system_messages)){
            $message = (count($system_messages) > 1 ? "<br>" : "") . implode(",<br>", array_column($system_messages, "text"));
        }
        /* ------------------------------------------- */
        Portal::resetSystemMessages();
        Util::traceStackToDebugger();
        $custom_path = self::getCustomTemplatePath("redirect_page.php");
        if ($custom_path) {
            require_once $custom_path;
            exit();
        } else {
            $_stylesheets = array(LINK_CSS."/debug.css", LINK_CSS."/errors.css");
            $errorHTML = HTMLControls::renderStaticHeader(Text::_("Debug mode"), "", "", $_stylesheets);
            $errorHTML .= "<div id=\"error_header\"><div id=\"logo\" class=\"redirect\"></div><div id=\"headLine\"></div></div>";
            $errorHTML .= "<div class=\"fatal-error-text\">" . Text::_("Redirect") . " " . $code . ": " . $message . "</div>";
            $errorHTML .= "<div class=\"fatal-error-link\"><a class=\"go_back\" href=\"" . ($url ? $url : "/") . "\">" . ($url ? Text::_('Continue') : Text::_('Go main page')) . "</a></div>";
            $errorHTML .= self::renderDump();
            $errorHTML .= HTMLControls::renderStaticFooter();
            exit($errorHTML);
        }
    }

    public static function render404($url, $message = "")
    {
        if($message) Portal::addSystemMessage($message, 404);
        $system_messages = Portal::getSystemMessages();
        // Util::pre($message, $system_messages, array_column($system_messages, 'text'));
        if(count($system_messages)){
            $message = (count($system_messages) > 1 ? "<br>" : "") . implode(",<br>", array_column($system_messages, "text"));
        }
        /* ------------------------------------------- */
        Portal::resetSystemMessages();
        Util::traceStackToDebugger();
        $custom_path = self::getCustomTemplatePath("404.php");
        if ($custom_path) {
            require_once $custom_path;
            exit();
        } else {
            $_stylesheets = array(LINK_CSS."/debug.css", LINK_CSS."/errors.css");
            $errorHTML = HTMLControls::renderStaticHeader(Text::_("Page not found"), "", "", $_stylesheets);
            $errorHTML .= "<div id=\"error_header\"><div id=\"logo\" class=\"logo404\"></div><div id=\"headLine\"></div></div>";
            $errorHTML .= "<div class=\"fatal-error-text\">" . Text::_("Error 404") . ": " . (\siteConfig::$debugMode ? $message : Text::_("Page not found")) . "</div>";
            $errorHTML .= "<div class=\"fatal-error-link\"><a class=\"go_back\" href=\"" . ($url ? $url : "/") . "\">" . ($url ? Text::_('Continue') : Text::_('Go main page')) . "</a></div>";
            $errorHTML .= self::renderDump();
            $errorHTML .= HTMLControls::renderStaticFooter();
            exit($errorHTML);
        }
    }

    public static function render503($message = "")
    {
        Portal::addSystemMessage($message, 503);
        $system_messages = Portal::getSystemMessages();
        // Util::pre($message, $system_messages, array_column($system_messages, 'text'));
        if(count($system_messages)){
            $message = (count($system_messages) > 1 ? "<br>" : "") . implode(",<br>", array_column($system_messages, "text"));
        }
        /* ------------------------------------------- */
        Portal::resetSystemMessages();
        Util::traceStackToDebugger();
        $custom_path = self::getCustomTemplatePath("503.php");
        if ($custom_path) {
            require_once $custom_path;
            exit();
        } else {
            $_stylesheets = array(LINK_CSS."/debug.css", LINK_CSS."/errors.css");
            $errorHTML = HTMLControls::renderStaticHeader(Text::_("Service Temporarily Unavailable"), "", "", $_stylesheets);
            $errorHTML .= "<div id=\"error_header\"><div id=\"logo\" class=\"logo503\"></div><div id=\"headLine\"></div></div>";
            $errorHTML .= "<div class=\"fatal-error-text\">" . Text::_("Service Temporarily Unavailable") . (\siteConfig::$debugMode ? ": " . $message : "") . "</div>";
            $errorHTML .= self::renderDump();
            $errorHTML .= HTMLControls::renderStaticFooter();
            exit($errorHTML);
        }
    }
}

?>