<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 0fc0f7b0 (2022-08-18 18:57:04)
// 

namespace ArfaCMS\Core;

defined('_ARFACMS_VALID') or die("Access denied");

class Event
{
    private static $_events = array();

    public static function register($eventName, $pluginName)
    {
        if (array_key_exists($eventName, self::$_events) && array_key_exists($pluginName, self::$_events[$eventName])) {
            Debugger::getInstance()->warning("Event reregistration");
        } else {
            self::$_events[$eventName][$pluginName] = $pluginName;
            Debugger::getInstance()->message("Event registered: " . $pluginName . "=&gt;" . $eventName);
        }
    }

    public static function raise($eventName, $params = array(), &$data = null)
    {
        $result = null;
        if (array_key_exists($eventName, self::$_events)) {
            Debugger::getInstance()->message("Event raised : " . $eventName . " [Handler found]");
            foreach (self::$_events[$eventName] as $plugin => $val) {
                $res = Plugin::getInstance($plugin)->executeEvent($eventName, $params, $data);
                // возващаем пока только текст
                if (!is_bool($res) && !is_null($res)) $result .= $res;
            }
        } else {
            Debugger::getInstance()->warning("Event raised : " . $eventName . " [Handler absent]");
        }
        return $result;
    }

    public static function getEvents()
    {
        return self::$_events;
    }
}

?>