<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: cdee9efb (2024-10-20 16:16:20)
// 

namespace ArfaCMS\Core;

use ArfaCMS\DB\Database;

defined('_ARFACMS_VALID') or die("Access denied");

class HTMLControls extends BaseObject
{
    private static $unificator = 0;
    private static $elid = 0;

    public static function renderBalloonButton($text, $translate = true)
    {
        if ($translate) $text = Text::_($text);
        $html = '<div class="balloon_button" data-bs-custom-class="custom-tooltip" data-bs-html="true" data-bs-toggle="tooltip" data-bs-placement="top" data-bs-title="' . $text . '"></div>';
        return $html;
    }

    public static function renderPopupMultySelect($name, $data, $key_fld = "", $name_fld = "", $link_add = "", $js_add = "", $js_inf = '', $linksAreNew = false)
    {
        return self::renderPopupMultySelectWQ($name, $data, false, $key_fld, $name_fld, "", $link_add, $js_add, $js_inf, $linksAreNew);
    }

    public static function renderPopupMultySelectWQ($name, $data, $withQuantity = false, $key_fld = "", $name_fld = "", $quant_fld = "", $link_add = "", $js_add = "", $js_inf = '', $linksAreNew = false)
    {
        Portal::getInstance()->addScriptDeclaration("$(window).on('load', function() { addAfterContentLoadHandler('modifySelector'); });");
        if (!$key_fld) $key_fld = "id";
        if (!$name_fld) $name_fld = "title";
        if (!$quant_fld) $quant_fld = "quantity";
        $html = "<div id=\"" . $name . "\" class=\"row multyselector\">";

        $html .= "<input type=\"hidden\"  id=\"" . $name . "_delete\" name=\"" . $name . "_delete\" value=\"" . Text::_("Delete") . "\" />";
        $html .= "<input type=\"hidden\" id=\"" . $name . "_exists\" name=\"" . $name . "_exists\" value=\"" . Text::_("Already added") . "\" />";
        $html .= "<input type=\"hidden\" id=\"" . $name . "_wq\" name=\"" . $name . "_wq\" value=\"" . ($withQuantity ? 1 : 0) . "\" />";
        if ($link_add) {
            $html .= "<div class=\"col-sm-12 text-md-end mb-0\"><a class=\"button-sprav-selector\" href=\"" . $link_add . "\" title=\"" . Text::_("Add") . "\"><i class=\"bi bi-plus-circle\"></i>" . Text::_("Add") . "</a></div>";
        } elseif ($js_add) {
            $html .= "<div class=\"col-sm-12 text-end mb-0\"><a onclick=\"" . $js_add . "\" title=\"" . Text::_("Add") . "\"><i class=\"bi bi-plus-circle\"></i>" . Text::_("Add") . "</a></div>";
        }
        if (is_array($data) && count($data)) {
            foreach ($data as $elem) {
                $current_id = $name . "_" . htmlspecialchars($elem[$key_fld]);
                //if ($linksAreNew) $html .= "<p class=\"newline float-fix\">"; else $html .= "<p class=\"float-fix\">";
                $html .= "<div class=\"col-sm-12 mt-2" . ($linksAreNew ? "newline" : "") . "\">";
                $html .= "<span class=\"button-delete-link\" onclick=\"deleteMultyRow('" . $current_id . "','" . Text::_("Delete") . "')\" title=\"" . Text::_("Delete") . "\"><i class=\"bi bi-dash-circle\"></i></span>";
                $html .= self::renderHiddenField($name . "[]", $elem[$key_fld], $current_id);
                $cur_js_inf = sprintf($js_inf, $elem[$key_fld]);
                $html .= self::renderLabelField($current_id, $elem[$name_fld], 0, $cur_js_inf);
                if ($withQuantity) $html .= self::renderInputText($name . "_quantity[" . $elem[$key_fld] . "]", $elem[$quant_fld], "", "", $name . "_quantity_" . $elem[$key_fld], 'quantity numeric form-control');
                $html .= "</div>";
            }
        }
        $html .= "</div>";
        return $html;
    }

    public static function renderAddressPanel($name, $data, $id, $class, $readonly, $module)
    {
        $html = "<div class=\"addressselector\">";
        $html .= self::renderPictoButton('', 'btn-primary', '', Text::_('Modify address'), "showAddressEditor('" . $id . "'" . ($module ? ",'" . $module . "'" : "") . ")");
        $html .= self::renderHiddenField($name, base64_encode(json_encode($data)), $id);
        $html .= self::renderLabelField($id, $data["fullinfo"]);
        $html .= "</div>";
        return $html;
    }

    public static function renderSwitchCheckbox($name, $checked_val = "", $val = "1", $id = "", $class = "", $js = "", $required = false)
    {
        return "<div class=\"form-check form-switch\">" . self::renderCheckbox($name, $checked_val, $val, $id, $class, $js, $required) . "</div>";
    }

    public static function renderCheckbox($name, $checked_val = "", $val = "1", $id = "", $class = "", $js = "", $required = false)
    {
        if (!$id) $id = $name;
        $html = "<input type=\"checkbox\" id=\"" . $id . "\" name=\"" . $name . "\" value=\"" . $val . "\"";
        if ($class !== false) $html .= " class=\"form-check-input" . ($class ? " " : "") . $class . "\"";
        if ($checked_val == $val) $html .= " checked=\"checked\"";
        if ($required) $html .= " required=\"required\"";
        if ($js) $html .= " onclick=\"javascript:" . $js . ";\"";
        $html .= " autocomplete=\"off\" />";
        return $html;
    }

    public static function renderInputText($name, $val = "", $size = "", $maxlength = "", $id = "", $class = '', $readonly = false, $required = false, $title = '', $js_arr = array(), $placeholder = '', $type = "text", $data_attr_arr = array())
    {
        if ($id === false)
            $id_text = '';
        else {
            if (!$id) $id = $name;
            $id_text = "id=\"" . $id . "\"";
        }
        if ($readonly) $readonly = " readonly=\"readonly\""; else $readonly = "";
        $html = "<input type=\"" . $type . "\" " . $id_text . $readonly . " value=\"" . htmlspecialchars(htmlspecialchars_decode($val)) . "\" name=\"" . $name . "\"";
        if ($size) $html .= " size=\"" . $size . "\"";
        if ($class !== false) $html .= " class=\"form-control" . ($class ? " " . $class : "") . "\"";
        if ($maxlength) $html .= " maxlength=\"" . $maxlength . "\"";
        if ($title) $html .= " title=\"" . $title . "\"";
        if ($required) $html .= " required=\"required\"";
        if ($placeholder) $html .= " placeholder=\"" . $placeholder . "\"";
        if (is_array($data_attr_arr) && count($data_attr_arr)) {
            foreach ($data_attr_arr as $data_attr_key => $data_attr_val) {
                $html .= " data-" . $data_attr_key . "=\"" . $data_attr_val . "\"";
            }
        }
        if (count($js_arr)) {
            foreach ($js_arr as $act => $func) $html .= " " . $act . "=\"" . $func . "\"";
        }
        $html .= " />";
        return $html;
    }

    public static function renderFileSelector($name, $val = "", $size = "", $maxlength = "", $id = "", $class = '', $readonly = false, $required = false, $title = '')
    {
        if ($id === false)
            $id_text = '';
        else {
            if (!$id) $id = $name;
            $id_text = "id=\"" . $id . "\"";
        }
        if ($readonly) $readonly = " readonly=\"readonly\""; else $readonly = "";
        $html = "<div class=\"wrapper-filepath\"><input type=\"text\" " . $id_text . $readonly . " value=\"" . htmlspecialchars(htmlspecialchars_decode($val)) . "\" name=\"" . $name . "\"";
        if ($size) $html .= " size=\"" . $size . "\"";
        $html .= " class=\"form-control" . ($class ? " " . $class : "") . "\"";
        if ($maxlength) $html .= " maxlength=\"" . $maxlength . "\"";
        if ($title) $html .= " title=\"" . $title . "\"";
        if ($required) $html .= " required=\"required\"";
        $html .= " />";
        $html .= "<a class=\"rel-popup-text btn mediamanager-selector\" href=\"index.php?module=service&amp;task=startMediamanager&amp;nfl=-1&amp;ret_elem=" . $id . "\"><i class=\"bi bi-inboxes\"></i></a></div>";
        return $html;
    }

    public static function renderInputFile($name, $val = "", $size = 20, $id = "", $class = '', $required = false)
    {
        if ($id === false)
            $id_text = '';
        else {
            if (!$id) $id = $name;
            $id_text = "id=\"" . $id . "\"";
        }
        $html = "<input type=\"file\" " . $id_text . " size=\"" . $size . "\" value=\"" . $val . "\" name=\"" . $name . "\"";
        if ($required) $html .= " required=\"required\"";
        $html .= " class=\"form-control" . ($class ? " " . $class : "") . "\"";
        $html .= " />";
        return $html;
    }

    public static function renderDateTimeSelector($name, $val = "", $with_time = false, $with_date = false, $required = false, $readonly = true, $id = "", $class = 'form-control', $data_attr_arr = array())
    {
        /*
        if ($id === false) $id_text = '';
        else {
            if (!$id) $id = $name;
            $id_text = "id=\"" . $id . "\"";
        }
        */
        if ($with_date && $with_time) $class .= ($class ? " " : "") . "datetimeselector";
        elseif ($with_date && !$with_time) $class .= ($class ? " " : "") . "dateselector";
        elseif (!$with_date && $with_time) $class .= ($class ? " " : "") . "timeselector";
        $html = "<div class=\"wrapper-datetimeselector" . ($required ? "" : " wrapper-datetimeselector-with-clear-button") . "\">";
        // if(!$required) $html.= "<img width=\"1\" height=\"1\" class=\"clear_datetime_input\" src=\"/assets/images/blank___gif\" title=\"" . Text::_("Clear") . "\" alt=\"D\" />";
        if (!$required) $html .= HTMLControls::renderPictoButton('', 'date-selector-clear', 'bi-eraser-fill', Text::_("Clear"));
        // $html.= "<input type=\"text\" " . $id_text . " maxlength=\"" . ($with_time ? "16" : "10") . "\" size=\"" . ($with_time ? "16" : "10") . "\" value=\"" . $val . "\" name=\"" . $name . "\" class=\"" . $class . "\"" . ($required ? " required=\"required\"" : "") . " />";
        $html .= HTMLControls::renderInputText($name, $val, $with_time ? "16" : "10", $with_time ? "16" : "10", $id, $class, $readonly, $required, "", array(), "", "text", $data_attr_arr);
        // $html.= "<img width=\"1\" height=\"1\" class=\"date_selector\" src=\"/assets/images/blank___gif\" title=\"" . Text::_("Select date") . "\" alt=\"D\" />";
        $html .= HTMLControls::renderPictoButton('', 'date-selector', 'bi-calendar3', Text::_("Select date"));
        $html .= "</div>";
        return $html;
    }

    public static function renderPictoButton($id = '', $class = 'btn-primary', $class_i = '', $title = '', $js = '')
    {
        if ($id) $id_text = "id=\"" . $id . "\""; else $id_text = '';
        if ($js) $js = " onclick=\"javascript:" . $js . ";\"";
        if ($title) $title = " title=\"" . $title . "\"";
        $html = "<button type=\"button\" " . $id_text . " class=\"btn" . ($class ? " " : "") . $class . "\"" . $js . $title . ">";
        $html .= "<i class=\"bi" . ($class_i ? " " : "") . $class_i . "\"></i>";
        $html .= "</button>";
        return $html;
    }

    public static function renderButton($name, $val = "", $type = "button", $id = "", $class = 'commonButton btn btn-primary', $js = '', $title = "")
    {
        if ($id === false) $id_text = '';
        else {
            if (!$id) $id = $name;
            $id_text = "id=\"" . $id . "\"";
        }
        if ($js) $js = " onclick=\"javascript:" . $js . ";\"";
        if ($title) $title = " title=\"" . $title . "\"";
        $html = "<input type=\"" . $type . "\" " . $id_text . " value=\"" . htmlspecialchars(htmlspecialchars_decode($val)) . "\" name=\"" . $name . "\"";
        if ($class) $html .= " class=\"" . $class . "\"";
        $html .= " " . $js . $title . " />";
        return $html;
    }

    /*
        public static function renderImageButton($name, $title = "", $id = "", $class = 'button', $js = '')
        {
            if ($id === false) $id = "";
            else {
                if (!$id) {
                    if ($name === false) $id = ""; else $id = $name;
                }
            }
            $html = "<img width=\"10\" height=\"10\" style=\"background:red\" src=\"/assets/images/blank.gif\""; // This blank image is OK !!!
            if ($name) $html .= " name=\"" . $name . "\"";
            if ($id) $html .= " id=\"" . $id . "\"";
            if ($class) $html .= " class=\"" . $class . "\"";
            if ($js) $html .= " onclick=\"javascript:" . $js . ";\"";
            if ($title) $html .= " title =\"" . $title . "\"";
            $html .= " alt=\"\"  />";
            return $html;
        }
        public static function renderIcon($class, $size = 16)
        {
            if(!intval($size)) $size = 16;
            $html = "<img width=\"" . $size . "\" height=\"" . $size . "\" style=\"background:red\" src=\"/assets/images/blank.gif\" class=\"" . $class . "\" alt=\"\" />"; // This blank image is OK !!!
            return $html;
        }
    */

    public static function renderRadio($name, $id, $val = "", $label = "", $class = "", $checked = true, $js = '', $required = 0)
    {
        $html = "<input type=\"radio\" value=\"" . $val . "\" name=\"" . $name . "\"";
        if ($id) $html .= " id=\"" . $id . "\"";
        if ($checked) $html .= " checked=\"checked\"";
        if ($required) $required_txt = " required=\"required\""; else $required_txt = "";
        if ($class !== false) $html .= " class=\"form-check-input" . ($class ? " " : "") . $class . "\"";
        if ($js) $html .= " onchange=\"" . $js . "\"";
        $html .= $required_txt . " />";
        if ($label) $html .= self::renderLabelField($id, $label);
        return $html;
    }

    public static function renderYesNoRadioButton($name, $id, $value = 1, $label_yes = '', $label_no = '', $class = '', $js = "")
    {

        if (!$label_yes) $label_yes = Text::_("Yes");
        if (!$label_no) $label_no = Text::_("No");
        $checked_yes = $value;
        $checked_no = !$value;
        $html = self::renderRadio($name, $name . "_yes", 1, $label_yes, $class, $checked_yes, $js);
        $html .= self::renderRadio($name, $name . "_no", 0, $label_no, $class, $checked_no, $js);
        return $html;
    }

    public static function renderLabelField($id = "", $value = "", $forceTranslation = 0, $js = "", $class = "", $title = "")
    {
        if ($js) {
            if (!$title) $title = Text::_("Update");
            $js = " style=\"cursor:pointer;\" onclick=\"javascript:" . $js . ";\"";
        }
        if ($title) $title = " title=\"" . $title . "\"";
        if ($forceTranslation) $value = Text::_($value);
        if ($id) return "<label class=\"label" . ($class ? " " . $class : "") . "\"" . $title . $js . " for=\"" . $id . "\">" . $value . "</label>";
        else return "<span class=\"fake-label" . ($class ? " " . $class : "") . "\"" . $title . $js . ">" . $value . "</span>";
    }

    public static function renderHiddenField($name = "", $val = "", $id = "", $class = "")
    {
        if ($id === false) $id_text = '';
        else {
            if (!$id) $id = $name;
            $id_text = "id=\"" . $id . "\"";
        }
        if ($class) $class = "class=\"" . $class . "\"";
        return "<input type=\"hidden\" " . $id_text . " value=\"" . htmlspecialchars(htmlspecialchars_decode($val)) . "\" name=\"" . $name . "\" " . $class . " />";
    }

    // микроразметка добавлена
    public static function renderImage($link = "", $noimage = false, $width = 0, $height = 0, $title = "", $alt = "", $class = "", $default_dims = false, $props = false)
    {
        if ($link) {
            $src = " src=\"" . $link . "\"";
        } else {
            if ($noimage === false) $src = "";
            elseif ($noimage) $src = " src=\"" . $noimage . "\"";
            else $src = " src=\"" . Files::getNoImageLink("no-photo.png") . "\"";
        }
        if ($class) $class = " class=\"" . $class . "\""; else $class = "";
        if ($width) $width = " width=\"" . $width . "\""; else $width = "";
        if ($height) $height = " height=\"" . $height . "\""; else $height = "";
        if ($title) $title = " title=\"" . htmlspecialchars(htmlspecialchars_decode($title)) . "\"";
        if ($props) $props = " itemprop=\"image\"";
        $_html = "<img" . $class . $title . $width . $height . $src . $props . " alt=\"" . $alt . "\" />";
        return $_html;
    }

    public static function renderFlash($link = "", $width = 0, $height = 0)
    {
        if ($link && $width && $height) {
            $_html = '<object type="application/x-shockwave-flash" data="' . $link . '"';
            if ($width) $_html .= ' width ="' . $width . '"';
            if ($height) $_html .= ' height ="' . $height . '"';
            $_html .= '>';
            $_html .= '<param name="movie" value="' . $link . '" />';
            $_html .= '<param name="quality" value="high" />';
            $_html .= '<param name="wmode" value="transparent" />';
            $_html .= '</object>';
        } else $_html = "";
        return $_html;
    }

    public static function renderRating($rating = 0, $max_raiting = 5)
    {
        $html = "<div class=\"stars\">";
        for ($i = 1; $i <= $max_raiting; $i++) {
            if (($rating < $i) && ($rating > ($i - 1))) {
                $img = "rating_half.gif";
            } elseif ($rating < $i) {
                $img = "rating_off.gif";
            } else {
                $img = "rating_on.gif";
            }
            $imgpath = '/templates/' . Portal::getInstance()->getTemplate() . '/images/' . $img;
            $html .= "<img src=\"" . $imgpath . "\" alt=\"\" />";
        }
        $html .= "</div>";
        return $html;
    }

    /**
     *
     * Формирование списка по массивам разных типов. Определяется передаваемыми данными.
     * @param _name - имя поля
     * @param _id - ид поля
     * @param _key_fld - имя поля (value option) в элементе массива( в объекте) если пусто - работает по обычному варианту ассоциативного массива
     * @param _val_fld - имя поля (title option) в элементе массива( в объекте) если пусто - работает по обычному варианту ассоциативного массива
     * @param _arr - массив данных формата массив объектов
     * @param _sel_val - выбранное значение(ключ)
     * @param _zero_fill -  добавлять базовое (пустое значение)
     * @param _js - функиця обработки
     * @param multiple - ждет значения одновременно видимых элементов, 0 - отключен
     * @param class - класс css
     */
    public static function renderSelectColored($_name, $_id, $_key_fld, $_val_fld, $_arr, $_arr_classes, $_sel_val = "", $_zero_fill = true, $_js = "", $multiple = 0, $class = "", $required = false)
    {
        if ($_id === false) $id_text = '';
        else {
            if (!$_id) $_id = $_name;
            $id_text = "id=\"" . $_id . "\"";
        }
        $selected = " selected=\"selected\"";
        if ($multiple) {
            if (!is_array($_sel_val)) $_sel_val = preg_split('/[\;]/', $_sel_val);
            $multiple = " multiple=\"multiple\" size=\"" . $multiple . "\"";
            $_name = $_name . "[]";
            $_class = "multiSelect form-select";
        } else {
            if (!is_array($_sel_val)) $_sel_val = array((string)$_sel_val);
            $multiple = "";
            $_class = "singleSelect form-select";
        }
        if ($class) $_class .= " " . $class;
        if ($required) $required_txt = " required=\"required\""; else $required_txt = "";

        $_html = "<select class=\"" . $_class . "\" " . $multiple . " name=\"" . $_name . "\" " . $id_text . " " . $required_txt;
        if (is_array($_js) && count($_js)) {
            foreach ($_js as $act => $func) $_html .= " " . $act . "=\"" . $func . "\"";
        } else if ($_js) {
            $_html .= ($_js ? "onchange=\"" . $_js . "\"" : "");
        }
        $_html .= ">";
        $_html_options = "";
        if (count($_arr) > 0) {
            foreach ($_arr as $key => $val) {
                if (is_object($val)) {
                    if (strval($val->{$_key_fld}) === "0") $_zero_fill = false;
                    $_html_options .= "<option" . (isset($_arr_classes[$key]) ? " class=\"" . $_arr_classes[$key] . "\"" : "") . " value=\"" . $val->{$_key_fld} . "\"" . (in_array($val->{$_key_fld}, $_sel_val) ? $selected : "") . ">" . $val->{$_val_fld} . "</option>";
                } else {
                    if ($_key_fld && $_val_fld) {
                        if (strval($val[$_key_fld]) === "0") $_zero_fill = false;
                        $_html_options .= "<option" . (isset($_arr_classes[$key]) ? " class=\"" . $_arr_classes[$key] . "\"" : "") . " value=\"" . $val[$_key_fld] . "\"" . (in_array($val[$_key_fld], $_sel_val) ? $selected : "") . ">" . $val[$_val_fld] . "</option>";
                    } elseif ($_val_fld) {
                        if (strval($key) === "0") $_zero_fill = false;
                        $_html_options .= "<option" . (isset($_arr_classes[$key]) ? " class=\"" . $_arr_classes[$key] . "\"" : "") . " value=\"" . $key . "\"" . (in_array($key, $_sel_val) ? $selected : "") . ">" . $val[$_val_fld] . "</option>";
                    } else {
                        if (strval($key) === "0") $_zero_fill = false;
                        $_html_options .= "<option" . (isset($_arr_classes[$key]) ? " class=\"" . $_arr_classes[$key] . "\"" : "") . " value=\"" . $key . "\"" . (in_array($key, $_sel_val) ? $selected : "") . ">" . $val . "</option>";
                    }
                }
            }
        }
        if ($_zero_fill === true || $_zero_fill === '') {
            $_html .= "<option class=\"no_selection\" value=\"" . ($_zero_fill === '' ? "" : "0") . "\"" . ($_sel_val == 0 ? $selected : "") . "> --- " . Text::_("Not selected") . " --- </option>";
        }
        $_html .= $_html_options;
        $_html .= "</select>";
        return $_html;
    }

    public static function renderSelect($_name, $_id, $_key_fld, $_val_fld, $_arr, $_sel_val = "", $_zero_fill = true, $_js = "", $multiple = 0, $class = "", $required = false)
    {
        return self::renderSelectColored($_name, $_id, $_key_fld, $_val_fld, $_arr, array(), $_sel_val, $_zero_fill, $_js, $multiple, $class, $required);
    }

    public static function renderRadioGroup($_name, $_id, $_key_fld, $_val_fld, $_arr, $_sel_val = 0, $_js = "", $class = "radio", $required = false)
    {
        if ($_id === false) $id_text = '';
        else {
            if (!$_id) $_id = $_name;
            $id_text = "id=\"" . $_id . "\"";
        }
        $_selection = preg_split('/[\;]/', $_sel_val);
        $_class = "radio";
        if ($class) $_class .= " " . $class;
        $_html = "";
        if (count($_arr) > 0) {
            $i = 0;
            foreach ($_arr as $key => $val) {
                $i++;
                if (is_object($val)) {
                    $_html .= "<div class=\"form-check\">" . self::renderRadio($_name, $_name . "_" . $i, $val->{$_key_fld}, $val->{$_val_fld}, $_class, (in_array($val->{$_key_fld}, $_selection) ? true : false), $_js, $required) . "</div>";
                } else {
                    if ($_key_fld && $_val_fld) {
                        $_html .= "<div class=\"form-check\">" . self::renderRadio($_name, $_name . "_" . $i, $val[$_key_fld], $val[$_val_fld], $_class, (in_array($val[$_key_fld], $_selection) ? true : false), $_js, $required) . "</div>";
                    } elseif ($_val_fld) {
                        $_html .= "<div class=\"form-check\">" . self::renderRadio($_name, $_name . "_" . $i, $key, $val[$_val_fld], $_class, (in_array($key, $_selection) ? true : false), $_js, $required) . "</div>";
                    } else {
                        $_html .= "<div class=\"form-check\">" . self::renderRadio($_name, $_name . "_" . $i, $key, $val, $_class, (in_array($key, $_selection) ? true : false), $_js, $required) . "</div>";
                    }
                }
            }
        }
//		$_html.="</select>";
        return $_html;
    }

    public static function renderStaticHeader($title = '', $keywords = '', $description = '', $stylesheets = null, $scripts = null, $metanames = null, $lang = null)
    {
        if (!$lang) $lang = Text::getLanguage();
        $_html = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">';
            $_html .= '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="' . $lang . '-' . $lang . '" lang="' . $lang . '-' . $lang . '">';
            $_html .= '<head>';
            $_html .= '<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />';
            $_html .= '<meta http-equiv="Pragma" content="no-cache" />';
            $_html .= '<meta http-equiv="cache-control" content="no-cache" />';
            $_html .= '<meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0" />';
            $_html .= '<meta name="description" content="' . $description . '" />';
            $_html .= '<meta name="keywords" content="' . $keywords . '" />' . CR_LF;
        $array_default = array("viewport" => "width=device-width, initial-scale=1.0, maximum-scale=1.0",
            "description" => "'.$description.'",
            "keywords" => "'.$keywords.'"
        );
        if (is_array($metanames)) {
            foreach ($metanames as $key_meta => $val_meta) {
                if (!array_key_exists($key_meta, $array_default))
                    $_html .= '<meta name="' . $key_meta . '" content="' . $val_meta . '" />' . CR_LF;
            }
        }
        if (\siteConfig::$enableGeneratorMetaTag) $_html .= '<meta name="generator" content="Arfa-CMS" />' . CR_LF;
        $_html .= '<title>' . $title . '</title>' . CR_LF;
        if (is_array($stylesheets)) {
            foreach ($stylesheets as $style) {
                $_html .= '<link rel="stylesheet" href="' . $style . '" type="text/css" />' . CR_LF;
            }
        }
        if (is_array($scripts)) {
            foreach ($scripts as $script) {
                $_html .= '<script  src="' . $script . '"></script>' . CR_LF;
            }
        }
        $_html .= '</head><body class="static-page">';
        return $_html;
    }

    public static function renderStaticFooter()
    {
        $_html = '</body></html>';
        return $_html;
    }

    public static function renderMessage($message = "", $type = "info")
    {
        if ($message) {
            $html = "<div class=\"alert alert-" . $type . " alert-dismissible fade show\" role=\"alert\">";
            $html .= "<div class=\"msg-body\">" . $message . "</div>";
            $html .= "<button type=\"button\" class=\"btn-close\" data-bs-dismiss=\"alert\" aria-label=\"Close\"></button>";
            $html .= "</div>";
        }
        return $html;
    }

    public static function renderEditor($id, $taName = '', $editorText = '', $toolbar = 'Default', $height = 0, $width = 0, $params = array(), $class = 'editorArea', $taTitle = '', $placeholder = '')
    {
        if (!$taName) $taName = $id;
        if (array_key_exists("cols", $params)) $cols = $params["cols"]; else $cols = 80;
        if (array_key_exists("rows", $params)) $rows = $params["rows"]; else $rows = 10;
        $params["edName"] = $taName;
        if ($height) $params["height"] = $height;
        if ($width) $params["width"] = $width;
        $params["toolbar"] = $toolbar;
        Event::raise("editor.render", $params);
        $editorHTML = "<textarea name=\"" . $taName . "\" title=\"" . $taTitle . "\"  placeholder=\"" . $placeholder . "\"   cols=\"" . $cols . "\" class=\"" . $class . "\" rows=\"" . $rows . "\" id=\"" . $id . "\">" . htmlentities($editorText, ENT_COMPAT, DEF_CP) . "</textarea>";
        return $editorHTML;
    }

    public static function renderBBCodeEditor($id, $name = '', $val = '', $cols = 35, $rows = 7, $class = 'form-control', $readonly = false, $required = false, $title = '', $placeholder = '')
    {
        if (!$name) $name = $id;
        $html = "<textarea name=\"" . $name . "\" cols=\"" . $cols . "\"";
        if ($class) $html .= " class=\"" . $class . "\"";
        if ($required) $html .= " required=\"required\"";
        if ($title) $html .= " title=\"" . $title . "\"";
        if ($readonly) $html .= " readonly=\"readonly\"";
        if ($placeholder) $html .= " placeholder=\"" . $placeholder . "\"";
        $html .= "rows=\"" . $rows . "\" id=\"" . $id . "\">" . $val . "</textarea>";
        return $html;
    }

    public static function hideEmail($email, $class = "", $replace_with_link = true)
    {
        if ($class) $class = " class=\"" . $class . "\"";
        if (rand(0, 1)) $el = "span"; else $el = "p";
        self::$unificator = self::$unificator + 1;
        if (!self::$elid) self::$elid = md5(time());
        $elid = self::$elid . self::$unificator;
        $rand = rand(0, 3);
        switch ($rand) {
            case 0:
                $dot = " tutadot ";
                $at = " predostavlen ";
                break;
            case 1:
                $dot = " zdesenot ";
                $at = " na servere ";
                break;
            case 2:
                $dot = " chtoto ";
                $at = " sponsor ";
                break;
            case 3:
                $dot = " tochka ";
                $at = " sobaka ";
                break;
        }

        $email = str_replace(".", $dot, $email);
        $email = str_replace("@", $at, $email);
        $newmail = "<" . $el . " id=\"" . $elid . "\">" . $email . "</" . $el . ">";
        $js = "$(document).ready(function(){
				var chego = $('" . $el . "#" . $elid . "');
						var gdemy = /" . $at . "/;
								var razdelili = /" . $dot . "/g;
										var zamenit = $(chego).text().replace(gdemy,'@').replace(razdelili,'.');";
        if ($replace_with_link) $js .= "$(chego).after('<a" . $class . " href=\"mailto:'+zamenit+'\">'+ zamenit +'</a>');";
        else    $js .= "$(chego).after(zamenit);";
        $js .= "$(chego).remove();
	});";
        Portal::getInstance()->addScriptDeclaration($js);
        return $newmail;
    }

    public static function renderParamsPanel($name, $title, $mask, $params, $param_prefix = "", $active_tab = 1)
    {
        $tabs_count = 1;
        $tab = 1;
        $new_tab = 1;
        $html = array();
        $html_tabs = array();
        $_html = "";
        foreach ($mask as $prmName => $prmType) {
            if ($prmType["vtype"] == "tab") $tabs_count++;
        }
        if ($tabs_count < $active_tab) $active_tab = 1;
        $html[$tab] = "";
        if (!$title) $title = "Main settings";
        $html_tabs[$tab] = Text::_($title);
        if (count($mask)) {
            foreach ($mask as $prmName => $prmType) {
                if ($new_tab != $tab) {
                    $tab = $new_tab;
                    $html[$tab] = "";
                }
                if ($prmName) {
                    if ($prmType["vtype"] == "title") {
                        $html[$tab] .= "<div class=\"row mb-3 params-" . $prmType["vtype"] . "\"><div class=\"col-md-12\">";
                    } elseif ($prmType["vtype"] != "tab") {
                        if ($prmType["vtype"] == "boolean") {
                            $_css_class_1 = "col-md-5 col mb-0 align-self-center";
                            $_css_class_2 = "col-md-7 col-auto align-self-center";
                        } else {
                            $_css_class_1 = "col-md-5 mb-2 mb-md-0 align-self-center";
                            $_css_class_2 = "col-md-7";
                        }
                        $html[$tab] .= "<div class=\"row mb-3 params-" . $prmType["vtype"] . "\"><div class=\"" . $_css_class_1 . "\">";
                        $html[$tab] .= "<label for=\"" . $prmName . "\">" . Text::_(str_replace("_", " ", $param_prefix . $prmName)) . "</label>";
                        if (isset($prmType["descr"]) && $prmType["descr"]) $html[$tab] .= HTMLControls::renderBalloonButton($prmType["descr"], false);
                        $html[$tab] .= "</div><div class=\"" . $_css_class_2 . "\">";
                    }
                    if ((is_array($params)) && (array_key_exists($prmName, $params))) {
                        $val = $params[$prmName];
                        if (!$val && isset($prmType["fill_default"]) && $prmType["fill_default"]) $val = $prmType["vdefault"];
                    } else $val = $prmType["vdefault"];
                    switch ($prmType["vtype"]) {
                        case "boolean":
                            if ($val) $checked = "checked=\"checked\" ";
                            else $checked = "";
                            $html[$tab] .= "<div class=\"form-check form-switch\"><input id=\"" . $prmName . "\" name=\"" . $name . "[" . $prmName . "]\" type=\"checkbox\" class=\"form-check-input\" " . $checked . " value=\"1\" autocomplete=\"off\" /></div>";
                            break;
                        case "table_select":
                        case "table_multiselect":
                            if (isset($prmType["source"])) {
                                Database::getInstance()->setQuery($prmType["source"]);
                                $_source = Database::getInstance()->loadObjectList();
                                $zero_fill = !($prmType["vdefault"] || $prmType["vtype"] == "table_multiselect");
                                if (isset($prmType["fill_default"]) && ($prmType["fill_default"] === true || $prmType["fill_default"] === "")) {
                                    if ($prmType["vdefault"] === "") $zero_fill = "";
                                }
                                $html[$tab] .= HTMLControls::renderSelect($name . "[" . $prmName . "]", $prmName, "fld_id", "fld_name", $_source, $val, $zero_fill, (isset($prmType["vjs"]) && $prmType["vjs"] ? $prmType["vjs"] : ""), ($prmType["vtype"] == "table_multiselect" ? 10 : 0));
                            } else $html[$tab] .= "<input class=\"form-control\" id=\"" . $prmName . "\" name=\"" . $name . "[" . $prmName . "]\" type=\"text\" value=\"" . $val . "\" />";
                            break;
                        case "select":
                        case "multiselect":
                        case "select_method":
                        case "multiselect_method":
                            if (isset($prmType["source"]) && is_array($prmType["source"])) {
                                reset($prmType["source"]);
                                $zero_fill = !($prmType["vdefault"] || $prmType["vtype"] == "multiselect" || $prmType["vtype"] == "multiselect_method");
                                if (isset($prmType["fill_default"]) && ($prmType["fill_default"] === true || $prmType["fill_default"] === "")) {
                                    if ($prmType["vdefault"] === "") $zero_fill = "";
                                }
                                if (is_array(current($prmType["source"])) || is_object(current($prmType["source"]))) $html[$tab] .= HTMLControls::renderSelect($name . "[" . $prmName . "]", $prmName, "id", "name", $prmType["source"], $val, $zero_fill, (isset($prmType["vjs"]) && $prmType["vjs"] ? $prmType["vjs"] : ""), ($prmType["vtype"] == "multiselect" ? 10 : 0));
                                else $html[$tab] .= HTMLControls::renderSelect($name . "[" . $prmName . "]", $prmName, false, false, $prmType["source"], $val, $zero_fill, (isset($prmType["vjs"]) && $prmType["vjs"] ? $prmType["vjs"] : ""), ($prmType["vtype"] == "multiselect" ? 10 : 0));
                            } else $html[$tab] .= "<input class=\"form-control\" id=\"" . $prmName . "\" name=\"" . $name . "[" . $prmName . "]\" type=\"text\" value=\"" . $val . "\" />";
                            break;
                        case "integer":
                            $html[$tab] .= "<input class=\"form-control numeric\" id=\"" . $prmName . "\" name=\"" . $name . "[" . $prmName . "]\" type=\"text\" value=\"" . intval($val) . "\" />";
                            break;
                        case "float":
                            $html[$tab] .= "<input class=\"form-control decimal\" id=\"" . $prmName . "\" name=\"" . $name . "[" . $prmName . "]\" type=\"text\" value=\"" . floatval($val) . "\" />";
                            break;
                        case "string":
                            $html[$tab] .= "<input class=\"form-control\" id=\"" . $prmName . "\" name=\"" . $name . "[" . $prmName . "]\" type=\"text\" value=\"" . $val . "\" />";
                            break;
                        case "text":
                            $html[$tab] .= "<textarea class=\"form-control\" id=\"" . $prmName . "\" name=\"" . $name . "[" . $prmName . "]\">" . $val . "</textarea>";
                            break;
                        case "ro_string":
                            $html[$tab] .= "<p class=\"ro_string\">" . $val . "</p>";
                            break;
                        case "title":
                            $html[$tab] .= "<h5 class=\"prm-title\">" . $prmType["vdefault"] . "</h5>";
                            break;
                        case "filepath":
                            $html[$tab] .= HTMLControls::renderFileSelector($name . "[" . $prmName . "]", $val, "", "", $name . "_" . $prmName);
                            break;
                        case "date":
                            if (!Date::isEmpty($val)) $val = Date::fromSQL($val, true);
                            $html[$tab] .= HTMLControls::renderDateTimeSelector($name . "[" . $prmName . "]", $val, false, true, "", true, $name . "_" . $prmName);
                            break;
                        case "tab":
                            $new_tab++;
                            $html_tabs[$new_tab] = $prmType["vdefault"];
                            break;
                        default:
                            $html[$tab] .= Text::_("Wrong parameter type") . ": " . $prmType["vtype"];
                            break;
                    }
                    switch ($prmType["vtype"]) {
                        case "table_select":
                            // case "table_multiselect":
                        case "select":
                            // case "multiselect":
                        case "select_method":
                            // case "multiselect_method":
                            if (isset($prmType["js_descr"])) {
                                if (is_array($prmType["js_descr"]) && count($prmType["js_descr"])) {
                                    $html[$tab] .= "<div class=\"prm_descr\" id=\"" . $prmName . "_descr\">";
                                    foreach ($prmType["js_descr"] as $kd => $vd) {
                                        $html[$tab] .= "<div" . ($kd != $val || !trim($vd) ? " style=\"display:none;\"" : "") . " class=\"prm_descr_elem\" id=\"" . $kd . "_descr_elem\">" . $vd . "</div>";
                                    }
                                    $html[$tab] .= "</div>";
                                }
                            }
                            break;
                        default:
                            break;
                    }
                    if ($prmType["vtype"] != "tab") $html[$tab] .= "</div></div>";
                }
            }
        }
        $_html .= "<div class=\"params-wrapper\">";
        if (count($html) > 1) {
            $_html .= "<ul class=\"nav nav-tabs\" id=\"" . $name . "_tabs\">";
            foreach ($html_tabs as $ind => $html_tab_header) {
                $_html .= "<li class=\"nav-item\"><a class=\"nav-link" . ($active_tab == $ind ? " active" : "") . "\" data-bs-target=\"#" . $name . "_tab_" . $ind . "\" data-key=\"" . $ind . "\" data-bs-toggle=\"tab\">" . $html_tab_header . "</a></li>";
            }
            $_html .= "</ul>";
            $_html .= "<div class=\"tab-content pt-3 px-3\">";
            foreach ($html as $ind => $html_tab_body) {
                $_html .= "<div class=\"tab-pane" . ($active_tab == $ind ? " active" : "") . "\" id=\"" . $name . "_tab_" . $ind . "\">";
                $_html .= "<div class=\"params-body\">" . $html_tab_body . "</div>";
                $_html .= "</div>";
            }
            $_html .= "</div>";
            $_html .= HTMLControls::renderHiddenField("activeTab", $active_tab);
        } else {
            $_html .= "<fieldset class=\"params px-3\">";
            $_html .= "<legend>";
            $_html .= self::renderLabelField(false, Text::_($title));
            $_html .= "</legend>";
            $_html .= "<div class=\"params-body\">" . $html[1] . "</div>";
            $_html .= "</fieldset>";
        }
        $_html .= "</div>";
        return $_html;
    }
}

?>