<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 5bc55e59 (2023-05-18 13:05:22)
// 

namespace ArfaCMS\Core;

defined('_ARFACMS_VALID') or die("Access denied");

class InstallerXML extends BaseObject
{
    private $_path = "";               // путь до файла установщика
    private $_isLoaded = false;        // флаг удачной загрузки
    private $_xml = null;              // собстовенно сам объект xml

    public $type = "";                 // тип (module, plugin, widget, package, payment, delivery)
    public $name = "";                 // имя модуля, виджета, плагина и т.д. в нижнем регистре
    public $nameAsIs = "";             // имя модуля, виджета, плагина и т.д. как есть
    public $subname = "";              // категория плагина (обязательно) в нижнем регистре
    public $wside = 0;                 // сторона виджета
    public $description = "";          // описание
    public $version = "";              // версия
    public $author = "";               // автор
    public $email = "";                // email автора
    public $site = "";                 // сайт автора
    public $license = "";              // лицензия

    public $dependsOn = array();       // depends on modules

    public $mainRoot = "files";        // main root
    public $mainDirs = array();        // main dirs
    public $mainFiles = array();       // main files

    public $frontMenus = array();      // front menus
    public $adminMenus = array();      // admin menus

    public $frontACL = array();        // front ACL
    public $adminACL = array();        // admin ACL

    public $language = array();        // language files

    public $cssRoot = "css";           // css root
    public $cssFiles = array();        // css files
    public $cssDirs = array();         // css dirs

    public $jsRoot = "js";             // javascript root
    public $jsFiles = array();         // javascript files
    public $jsDirs = array();          // javascript dirs

    public $assetsRoot = "assets";     // custom assets root
    public $assetsDirs = array();      // custom assets dirs
    public $assetsFiles = array();     // custom assets files

    public $packagesRoot = "packages"; // custom packages root
    public $packagesDirs = array();    // custom packages dirs
    public $packagesFiles = array();   // custom packages files

    public $params = array();          // параметры
    public $queries = array();         // SQL queries for install
    public $unqueries = array();       // SQL queries for uninstall

    public function __construct($path)
    {
        $this->initObj();

        $this->_path = $path;
        $this->_isLoaded = $this->load();
    }

    private function getValueXML($node, $subnodeName, $defaultValue = "")
    {
        if (isset($node->{$subnodeName})) {
            return $node->{$subnodeName};
        } else return $defaultValue;
    }

    private function getAttrXML($node, $attrName, $defaultValue = "")
    {
        if (isset($node[$attrName])) {
            return $node[$attrName];
        } else return $defaultValue;
    }

    private function getDependences()
    {
        $mArr = array();
        if (isset($this->_xml->dependences)) {
            if (isset($this->_xml->dependences->module)) {
                foreach ($this->_xml->dependences->module as $module) {
                    if (strlen($module) > 1) {
                        $mArr[] = strval($module);
                    }
                }
            }
        }
        return $mArr;
    }

    private function getQueries()
    {
        $qArr = array();
        if (isset($this->_xml->queries)) {
            if (isset($this->_xml->queries->query)) {
                foreach ($this->_xml->queries->query as $query) {
                    if (strlen($query) > 1) {
                        $queryObj = new \stdClass();
                        $queryObj->max_version = strval($this->getAttrXML($query, "max_version"));
                        $queryObj->sql = strval($query);
                        $qArr[] = $queryObj;
                    }
                }
            }
        }
        return $qArr;
    }

    private function getUninstallQueries()
    {
        $qArr = array();
        if (isset($this->_xml->unqueries)) {
            if (isset($this->_xml->unqueries->query)) {
                foreach ($this->_xml->unqueries->query as $query) {
                    if (strlen($query) > 1) {
                        $qArr[] = strval($query);
                    }
                }
            }
        }
        return $qArr;
    }

    private function getMenus($node)
    {
        $mArr = array();
        if (isset($node->menu)) {
            foreach ($node->menu as $grp) {
                $cvarName = strval($grp['name']);
                if (strlen($cvarName) > 1) {
                    $mArr[$cvarName] = $this->getMenusItems($grp);
                }
            }
        }
        return $mArr;
    }

    private function getMenusItems($node)
    {
        $mArr = array();
        if (isset($node->item)) {
            foreach ($node->item as $item) {
                $cvarName = strval($item['name']);
                if (strlen($cvarName) > 1) {
                    $mArr[$cvarName] = strval($item);
                }
            }
        }
        return $mArr;
    }

    private function getNodeDirs($node)
    {
        $dirArr = array();
        if (isset($node->directory)) {
            foreach ($node->directory as $node_dir) {
                $subdirs = $this->getNodeDirs($node_dir);
                $dirName = strval($this->getAttrXML($node_dir, "name", ""));
                if (strlen($dirName) > 0) {
                    $dirData = new \stdClass();
                    $dirData->dirs = $subdirs;
                    $dirData->files = $this->getNodeFiles($node_dir);
                    $dirArr[$dirName] = $dirData;
                }
            }
        }

        return $dirArr;
    }

    /**
     * Читает теги &lt;file&gt; на заданном уровне, без рекурсии.
     * @param $node
     * @return array
     */
    private function getNodeFiles($node)
    {
        $filesArr = array();
        if (isset($node->file)) {
            foreach ($node->file as $_fileName) {
                $fileSrc = strval($_fileName);
                $fileDst = $this->getAttrXML($_fileName, "dst", $fileSrc);
                if (strlen($fileSrc) > 3) {
                    $filesArr[$fileSrc] = $fileDst;
                }
            }
        }
        return $filesArr;
    }

    private function getACL($node)
    {
        $aclData = array();
        if (isset($node->acl)) {
            if (isset($node->acl->object)) {
                foreach ($node->acl->object as $aclobj) {
                    $aclName = strval($aclobj['name']);
                    $aclDescr = strval($aclobj);
                    if (strlen($aclName) > 1) {
                        $aclData[$aclName] = $aclDescr;
                    }
                }
            }
        }
        return $aclData;
    }

    private function load()
    {
        if (file_exists($this->_path)) {
            // Load
            $this->_xml = simplexml_load_file($this->_path);
            if ($this->_xml) {
                $this->type = $this->getAttrXML($this->_xml, "type");
                $this->update = ((string)$this->getAttrXML($this->_xml, "update", "false") === "true" ? true : false);
                $this->subname = strtolower($this->getValueXML($this->_xml, "subname"));
                $this->name = strtolower($this->getValueXML($this->_xml, "name"));
                $this->nameAsIs = $this->getValueXML($this->_xml, "name");
                $this->wside =  $this->getValueXML($this->_xml, "wside"); // @TODO Пока так.
                $this->version = (string)$this->getValueXML($this->_xml, "version");
                $this->description = (string)$this->getValueXML($this->_xml, "description");
                $this->author = (string)$this->getValueXML($this->_xml, "author");
                $this->email = (string)$this->getValueXML($this->_xml, "email");
                $this->site = (string)$this->getValueXML($this->_xml, "site");
                $this->license = (string)$this->getValueXML($this->_xml, "license");

                $this->dependsOn = $this->getDependences();

                // Check for descriptors
                if (
                    !isset($this->_xml->files)
                    &&
                    !isset($this->_xml->frontend)
                    &&
                    !isset($this->_xml->backend)
                    &&
                    !isset($this->_xml->packages)
                    &&
                    !isset($this->_xml->assets)
                ) {
                    $this->error(Text::_('Nothing to install'));
                    return false;
                }

                // get root files dir
                $this->mainRoot = $this->getAttrXML($this->_xml->files, "root", "files");

                // get style files
                if (isset($this->_xml->css)) {
                    $this->cssRoot = $this->getAttrXML($this->_xml->css, "root", "css");
                    $this->cssFiles = $this->getNodeFiles($this->_xml->css);
                    $this->cssDirs = $this->getNodeDirs($this->_xml->css);
                }
                // get js files
                if (isset($this->_xml->js)) {
                    $this->jsRoot = $this->getAttrXML($this->_xml->js, "root", "js");
                    $this->jsFiles = $this->getNodeFiles($this->_xml->js);
                    $this->jsDirs = $this->getNodeDirs($this->_xml->js);
                }

                if (isset($this->_xml->assets)) {
                    $this->assetsRoot = $this->getAttrXML($this->_xml->assets, "root", "assets");
                    $this->assetsFiles = $this->getNodeFiles($this->_xml->assets);
                    $this->assetsDirs = $this->getNodeDirs($this->_xml->assets);
                }
                if (isset($this->_xml->packages)) {
                    $this->packagesRoot = $this->getAttrXML($this->_xml->packages, "root", "packages");
                    $this->packagesFiles = $this->getNodeFiles($this->_xml->packages);
                    $this->packagesDirs = $this->getNodeDirs($this->_xml->packages);
                }

                // Get ACL
                $this->frontACL = $this->getACL($this->_xml->frontend);
                $this->adminACL = $this->getACL($this->_xml->backend);


                // Get files
                $this->mainFiles = $this->getNodeFiles($this->_xml->files);

                // Get directories
                $this->mainDirs = $this->getNodeDirs($this->_xml->files);

                // Get menus
                $this->frontMenus = $this->getMenus($this->_xml->frontend);
                $this->adminMenus = $this->getMenus($this->_xml->backend);

                // Get config
                if (isset($this->_xml->params)) {
                    if (isset($this->_xml->params->param)) {
                        foreach ($this->_xml->params->param as $cvar) {
                            $cvarName = strval($cvar['name']);
                            if (strlen($cvarName) > 1) {
                                $this->params[$cvarName] = strval($cvar);
                            }
                        }
                    }
                }
                $this->queries = $this->getQueries();
                $this->unqueries = $this->getUninstallQueries();

                // Get language
                if (isset($this->_xml->language)) {
                    foreach ($this->_xml->language as $language) {
                        $langName = strval($language['name']);
                        if (strlen($langName) == 2) {
                            $this->language[$langName] = strval($language);
                        }
                    }
                }
                return true;
            } else return false;
        } else return false;
    }

    //-------------------------------------------------------

    public function isLoaded()
    {
        return $this->_isLoaded;
    }

    public function xmlData()
    {
        return $this->_xml->asXML();
    }

}

?>