<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: bb01f27c (2024-10-30 23:39:04)
// 

namespace ArfaCMS\Core;

defined('_ARFACMS_VALID') or die("Access denied");

class Response extends BaseObject
{
    private $code = 200;
    private $status = "";
    private $message = "";
    private $data = array();

    public function __construct($status = "", $message = "", $code = 0)
    {
        if ($code) $this->code = $code;
        if ($status) $this->status = $status;
        if ($message) $this->message = $message;
    }

    /**
     * @return int|mixed
     */
    public function getCode()
    {
        return $this->code;
    }

    /**
     * @param int|mixed $code
     */
    public function setCode($code)
    {
        $this->code = $code;
    }

    /**
     * @return mixed|string
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @param mixed|string $status
     */
    public function setStatus($status)
    {
        $this->status = strtoupper($status);
    }

    /**
     * @return mixed|string
     */
    public function getMessage()
    {
        return $this->message;
    }

    /**
     * @param mixed|string $message
     */
    public function setMessage($message)
    {
        $this->message = $message;
    }

    /**
     * @return array
     */
    public function getData()
    {
        return $this->data;
    }

    /**
     * @param array $data
     */
    public function setData($data)
    {
        if(!is_array($data)) return false;
        $this->data = $data;
        return true;
    }

    public function addData($key, $val, $override = true)
    {
        if(!$key) return false;
        if(array_key_exists($key, $this->data) && !$override) return false;

        $this->data[$key] = $val;
        return true;
    }

    public function send($header_code = 0)
    {
        if ($header_code) $this->code = $header_code;

        $answer = array();
        $answer["code"] = $this->code;
        $answer["status"] = $this->status;
        $answer["message"] = $this->message;
        $answer["data"] = $this->data;

        if ($header_code !== false) Util::sendHeader($this->code);
        echo json_encode($answer);
        Util::halt();
    }
}

?>