<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 0c782758 (2023-09-17 17:28:42)
// 

namespace ArfaCMS\Core;

use ArfaCMS\DB\Database;

defined('_ARFACMS_VALID') or die("Access denied");

class Taxes extends BaseObject
{
    private $_precision = 2;
    private $_taxes = null;

    private static $_instance = null;

    public static function createInstance()
    {
        if (self::$_instance == null) {
            self::$_instance = new self();
        }
    }

    public static function getInstance()
    {
        self::createInstance();
        return self::$_instance;
    }
    private function __construct()
    {
        $this->initObj();
        $this->reloadTaxes();
    }
    private function reloadTaxes()
    {
        $db = Database::getInstance();
        $sql = "SELECT * FROM #__taxes WHERE t_enabled=1 AND t_deleted=0";
        $db->setQuery($sql);
        $this->_taxes = $db->loadObjectList("t_id");
    }

    /*****************************************************************/

    public function getTax($tax_id, $enabled_only = true)
    {
        if (!array_key_exists($tax_id, $this->_taxes)) return null;
        if($enabled_only && !$this->_taxes[$tax_id]->t_enabled) return null;
        return $this->_taxes[$tax_id];
    }

    public function getTaxName($tax_id)
    {
        return array_key_exists($tax_id, $this->_taxes) ? $this->_taxes[$tax_id]->t_name : "";
    }

    public function getTaxValue($tax_id)
    {
        return array_key_exists($tax_id, $this->_taxes) ? $this->_taxes[$tax_id]->t_value : 0;
    }

    public function calculateTaxSum($tax_id, $current_sum)
    {
        if (!$current_sum || !array_key_exists($tax_id, $this->_taxes)) return 0;
        return round($current_sum / (100 + $this->_taxes[$tax_id]->t_value) * $this->_taxes[$tax_id]->t_value, $this->_precision);
    }

}

?>