<?php

namespace ArfaCMS\Core;

use ArfaCMS\MVC\Module;

final class Text
{
    private static $_language = 'ru';
    private static $_languageE = 'ru_RU';
    private static $_defs = array();
    private static $_parsedIni = array();
    private static $_available_langs = array();

    public static function setLanguage($lang = 'ru')
    {
        $list = Files::getFolders(PATH_SITE . "language" . DS . "common", false, false);
        foreach ($list as $l) self::$_available_langs[$l['filename']] = 1;
        if (defined('_ADMIN_MODE')) {
            self::$_language = $lang;
        } else {
            $new_lang = Request::getSafe('lang', '');
            if ($new_lang) {
                Session::getInstance()->setcookie('arfacms_language', $new_lang, time() + 60 * 60 * 24 * 365, "/");
            }
            if ((!$new_lang) && ((!isset($_COOKIE['arfacms_language']) || (!$_COOKIE['arfacms_language'])))) {
                self::$_language = $lang;
            } else {
                if ($new_lang) $stored_lang = $new_lang; else    $stored_lang = $_COOKIE['arfacms_language'];
                if (array_key_exists($stored_lang, self::$_available_langs)) self::$_language = $stored_lang;
                else self::$_language = $lang;
            }
        }
        self::$_languageE = self::$_language . "_" . strtoupper(self::$_language);
    }

    public static function getAllLanguages()
    {
        return self::$_available_langs;
    }

    public static function getLanguage($short = true)
    {
        if ($short) return self::$_language;
        else return self::$_languageE;
    }

    public function checkIfKeyExists($key)
    {
        return isset(self::$_defs[$key]);
    }

    public static function dump()
    {
        foreach (self::$_defs as $key => $def) {
            echo $key . '=' . $def . '<br />';
        }
    }

    public static function _($key)
    {
        $value = self::get($key);
        return $value;
    }

    private static function get($key)
    {
        // Empty key
        if (!$key) {
            if (Debugger::getInstance()->debugTranslations()) {
                Debugger::getInstance()->translation("Empty value");
                return "Empty value";
            } else {
                return $key;
            }
        }

        // Normal key
        if (isset(self::$_defs[$key])) return self::$_defs[$key];

        // Uppercase key
        $ukey = strtoupper($key);
        if (isset(self::$_defs[$ukey])) return self::$_defs[$ukey];

        // Nothing found before
        if (Debugger::getInstance()->debugTranslations()) Debugger::getInstance()->translation($key);
        return $key;
    }

    public static function parseCustom($_name)
    {
        self::parse($_name, "custom");
    }

    public static function parseAllModules()
    {
        if (defined("_ADMIN_MODE")) {
            $modules = Module::getInstalledModules();
            foreach ($modules as $moduleName) self::parseModule($moduleName);
        }
    }

    public static function parseModule($moduleName)
    {
        self::parse($moduleName, "modules");
    }

    public static function parseWidget($widgetName)
    {
        self::parse($widgetName, "widgets");
    }

    public static function parsePlugin($_path, $_name)
    {
        self::parse($_name, "plugins" . DS . $_path);
    }

    public static function parse($iniName, $folder = "")
    {
        self::parseBaseIni($iniName, $folder);
        self::parseTemplateIni($iniName, $folder);
    }

    public static function parseBaseIni($iniName, $folder = "")
    {
        $iniPath = PATH_LANGUAGE . 'common' . DS . self::$_language . DS . $folder . ($folder ? DS : "") . $iniName . '.ini';
        self::parseIni($iniPath);
    }

    public static function parseTemplateIni($iniName, $folder = "")
    {
        $iniPath = PATH_LANGUAGE . Portal::getTemplateName() . DS . self::$_language . DS . $folder . ($folder ? DS : "") . $iniName . '.ini';
        self::parseIni($iniPath, 1);
    }

    public static function showTranslations()
    {
        Util::showArray(self::$_defs, 'List translate value');
    }

    private static function parseIni($iniPath, $tmpl_file = 0)
    {
        if (isset(self::$_parsedIni[$iniPath]) && self::$_parsedIni[$iniPath] == $iniPath) {
            // Nothing to do
        } else {
            if (is_file($iniPath)) {
                $newDefs = parse_ini_file($iniPath, false);
                foreach ($newDefs as $key => $value) {
                    self::$_defs[$key] = $value;
                }
                self::$_parsedIni[$iniPath] = $iniPath;
                // if (\siteConfig::$debugMode > 1) Debugger::getInstance()->milestone("Loaded language : " . $iniPath);
                if (Debugger::getInstance()->debugTranslations()) Debugger::getInstance()->milestone("Loaded language : " . $iniPath);
            } else {
                // if (\siteConfig::$debugMode > 1 && !$tmpl_file) Debugger::getInstance()->warning("Language file absent : " . $iniPath);
                if (Debugger::getInstance()->debugTranslations() && !$tmpl_file) Debugger::getInstance()->warning("Language file absent : " . $iniPath);
            }
        }
    }

    public static function cutHtml($text, $cutLength = 0, $cutTextSuffix = "...")
    {
        $result = self::toHtml(self::fromHtml($text, $cutLength, $cutTextSuffix));
        // $result = preg_replace("/(<br \/>){2,}/", "<br />", $result);
        $result = preg_replace("/(<br\s*\/?>){2,}/", "<br />", $result);
        $result = preg_replace('/^\s*(?:<br\s*\/?>\s*)*/i', '', $result);
        $result = preg_replace('/\s*(?:<br\s*\/?>\s*)*$/i', '', $result);
        return $result;
    }

    public static function toHtml($text)
    {
        // Newlines ONLY !!! Don't put anything else !!! 
        return str_replace(array("\r\n", "\n", "\r"), "<br />", $text);
    }

    public static function fromHtml($text, $cutLength = 0, $cutTextSuffix = "")
    {
        $text = preg_replace("/&nbsp;/", " ", $text);
        $text = preg_replace('/\s+/', " ", $text);

        $pattern = array("/(<\/tr>)/", "/(<\/p>)/", "/(<br\s*\/?>\s*)/", "/(<p\s*>\s*)/");
        $replace = array("\n", "\n", "\n", "\n");
        $text = trim(strip_tags(preg_replace($pattern, $replace, $text)), "\n");

        $text = preg_replace("/(^[\r\n]*|[\r\n]+)[\s\t]*[\r\n]+/", "\n", $text);

        if ($cutLength) $text = mb_substr($text, 0, $cutLength, DEF_CP) . $cutTextSuffix;
        return $text;

        /*
        $pattern = array("/\n/", "/(<p\s*>\s*)/", "/(<\/p>)/", "/(<br\s*\/?>\s*)/");
        $replace = array("", "", "\n", "\n");
        $text = trim(strip_tags(preg_replace($pattern, $replace, $text)), "\n");
        if ($cutLength) $text = mb_substr($text, 0, $cutLength, DEF_CP) . $cutTextSuffix;
        return $text;
        */
    }

    public static function cr_lf_replace($text)
    {
        $text_arr = preg_split('#([\n\r]+)#Usi', $text);
        if (is_array($text_arr)) {
            $text_arr_1 = array_map('trim', $text_arr);
            $text_arr_2 = array_diff($text_arr_1, array(''));
            $text = implode(CR_LF, $text_arr_2);
        } else $text = trim($text);
        $text = preg_replace('/ {2,}/', ' ', $text);
        return $text;
    }

    public static function mapjsAddSlashes($str)
    {
        $pattern = array("/\\\\/", "/\n/", "/\r/", "/\"/", "/\'/", "/&/", "/</", "/>/");
        $replace = array("\\\\\\\\", "\\n", "\\r", "\\\"", "\\'", "\\x26", "\\x3C", "\\x3E");
        return preg_replace($pattern, $replace, $str);
    }

    public static function replaceUnicodeMatches($matches)
    {
        return html_entity_decode("&#x" . (is_array($matches) && count($matches) ? $matches[1] : $matches) . ";", ENT_COMPAT, "UTF-8");
    }

    public static function utf8_unicode($str, $as_arr = false)
    {
        $uni_arr = array();
        $values = array();
        $lookingFor = 1;
        for ($i = 0; $i < strlen($str); $i++) {
            $thisValue = ord($str[$i]);
            if ($thisValue < 128) $uni_arr[] = $thisValue;
            else {
                if (count($values) == 0) $lookingFor = ($thisValue < 224) ? 2 : 3;
                $values[] = $thisValue;
                if (count($values) == $lookingFor) {
                    $number = ($lookingFor == 3)
                        ? (($values[0] % 16) * 4096) + (($values[1] % 64) * 64) + ($values[2] % 64)
                        : (($values[0] % 32) * 64) + ($values[1] % 64);
                    $uni_arr[] = $number;
                    $values = array();
                    $lookingFor = 1;
                }
            }
        }
        if ($as_arr) return $uni_arr;
        return '&#' . implode(";&#", $uni_arr) . ';';
    }

    public static function ucfirst($string)
    {
        $string = trim($string);
        $strlen = mb_strlen($string, DEF_CP);
        $firstChar = mb_substr($string, 0, 1, DEF_CP);
        $then = mb_substr($string, 1, $strlen - 1, DEF_CP);
        return mb_strtoupper($firstChar, DEF_CP) . $then;
    }

    public static function trim_htmlentities($str, $entities)
    {
        if (!count($entities)) return $str;
        $pattern = '~^(' . implode('|', $entities) . ')*(.*?)(' . implode('|', $entities) . ')*$~';
        // Util::pre($pattern);
        return preg_replace($pattern, '$2', $str);;
    }
}