<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 5865c579 (2023-09-27 01:08:56)
// 

namespace ArfaCMS\Core;

use ArfaCMS\DB\Database;
use ArfaCMS\MVC\Module;

defined('_ARFACMS_VALID') or die("Access denied");

class orderDelivery extends orderUserdata
{
    protected $delivery = null;
    protected $tax_name = "";
    protected $tax_sum = 0;
    protected $need_recalc = 1;

    public function needRecalc()
    {
        return $this->need_recalc;
    }

    public function isLoaded()
    {
        return is_object($this->delivery);
    }

    public function __construct($delivery, $_data = false)
    {
        parent::__construct($delivery);
        $this->delivery = $delivery;
        if ($_data) $this->data = $_data;
        if ($this->delivery) {
            Text::parse(strtolower($this->delivery->dt_name), "deliveries");
            $this->params = Params::parse($this->delivery->dt_params);
        }
    }

    public static function getOrderIdByAbstractDeliveryClass($dt_name = "", $module = "", $mode)
    {
        if(!$dt_name) return 0;
        $db = Database::getInstance();
        $sql = "SELECT count(dt_id) FROM #__dts_active WHERE dt_deleted=0 AND dt_enabled=1 AND dt_name='" . $dt_name . "'";
        if($module)  $sql.= " AND (dt_module='' OR dt_module='".$module."')";
        // else $sql.= " AND dt_module=''";
        $db->setQuery($sql);
        if ($db->loadResult() > 0) {
            try {
                $dt_class = "ArfaCMS\\Delivery\\" . ucfirst($dt_name);
                return $dt_class::getOrderId($mode);
            } catch (\Exception $e) {
                return 0;
            }
        }
        return 0;
    }

    public static function getDeliveryClass($dt_id = false, $delivery = false)
    {
        if(!$delivery) $delivery = self::getDeliveryType($dt_id);
        if($delivery){
            $dt_class = "ArfaCMS\\Delivery\\" . ucfirst($delivery->dt_name);
            return new $dt_class($delivery);
        } else {
            // Util::halt("Delivery class not found");
            return false;
        }
    }

    public static function getInstalled(){
        $sql = "SELECT * FROM #__dts";
        $db = Database::getInstance();
        $db->setQuery($sql);
        return $db->loadAssocList();
    }

    public static function isInstalled($name = ""){
        if (!$name) return 0;
        $sql = "SELECT COUNT(*) FROM #__dts WHERE LOWER(d_name)='".strtolower($name)."'";
        $db = Database::getInstance();
        $db->setQuery($sql);
        return $db->loadResult();
    }

    public static function getDeliveryType($dt_id)
    {
        if(!$dt_id) return false;
        $db = Database::getInstance();
        $sql = "SELECT * FROM #__dts_active WHERE dt_deleted=0 AND dt_id=" . $dt_id;
        if (!defined("_ADMIN_MODE")) $sql.= " AND  dt_enabled=1";
        $db->setQuery($sql);
        if ($db->loadObject($res)) return $res;
        else return false;
    }

    public static function getDeliveryTypes($pt_id = 0, $weight_limit = 0, $summa = -1, $is_front = false, $module = "") {
        $db = Database::getInstance();
        if($pt_id){
            $sql = "SELECT d.* FROM #__dts_active AS d, #__dts_active_links as l
					WHERE l.dt_id=d.dt_id AND d.dt_deleted=0 AND l.parent_id=".(int)$pt_id;
            /*
            if (!defined("_ADMIN_MODE")) $sql.= " AND  d.dt_enabled=1";
            if($module) $sql.= " AND (d.dt_module='' OR d.dt_module='".$module."')";
            else $sql.= " AND d.dt_module=''";
            if($is_front && !User::getInstance()->isAdmin()) $sql.= " AND d.dt_admin_only=0";
            if ($weight_limit) $sql.= " AND (d.dt_weight_limit=0 || d.dt_weight_limit>".$weight_limit.")";
            $sql.= " ORDER BY d.dt_ordering, d.dt_title";
            */
        } else {
            $sql = "SELECT d.* FROM #__dts_active AS d WHERE dt_deleted=0";
            /*
            if (!defined("_ADMIN_MODE")) $sql.= " AND  d.dt_enabled=1";
            if($module) $sql.= " AND (d.dt_module='' OR d.dt_module='".$module."')";
            else $sql.= " AND d.dt_module=''";
            if($is_front && !User::getInstance()->isAdmin()) $sql.=" AND d.dt_admin_only=0";
            $sql.= " ORDER BY d.dt_ordering, d.dt_title";
            */
        }
        if (!defined("_ADMIN_MODE")) $sql.= " AND  d.dt_enabled=1";
        if($module) $sql.= " AND (d.dt_module='' OR d.dt_module='".$module."')";
        else $sql.= " AND d.dt_module=''";
        if($is_front && !User::getInstance()->isAdmin()) $sql.= " AND d.dt_admin_only=0";
        if ($weight_limit) $sql.= " AND (d.dt_weight_limit=0 || d.dt_weight_limit>".$weight_limit.")";
        $sql.= " ORDER BY d.dt_ordering, d.dt_title";

        $db->setQuery($sql);
        $dt_list = $db->loadObjectList();
        if($summa != -1){
            $new_dts=array();
            foreach ($dt_list as $a=>$dt){
                $dt_min_sum = Currency::getInstance()->convert($dt->dt_min_sum, $dt->dt_currency, DEFAULT_CURRENCY);
                $dt_max_sum = Currency::getInstance()->convert($dt->dt_max_sum, $dt->dt_currency, DEFAULT_CURRENCY);
                if($dt_min_sum == 0 && $dt_max_sum == 0) $new_dts[] = $dt;
                elseif($dt_min_sum > 0 && $dt_max_sum > 0){
                    if($summa > $dt_min_sum && $summa <= $dt_max_sum) $new_dts[] = $dt;
                } elseif($dt_min_sum > 0){
                    if($summa > $dt_min_sum) $new_dts[] = $dt;
                } elseif($dt_max_sum>0){
                    if($summa <= $dt_max_sum) $new_dts[] = $dt;
                }
            }
            $dt_list = $new_dts;
        }
        return $dt_list;
    }

    public static function getDeliveryTypesForModule($pt_id = 0, $weight_limit = 0, $summa = -1, $is_front = false){
        return self::getDeliveryTypes($pt_id, $weight_limit, $summa, $is_front, Module::getInstance()->getName());
    }

    // -------------------------------------------------------------------

    /**
     * @param integer $mode : 0-controller (list in new order); 1-ajax; 2-basket (calculate order on save)
     * @param array $userdata : from order->save()
     * @return float
     */
    public function calculate($mode, $userdata = array())
    {
        $this->calculateTaxSum($this->delivery->dt_price);
        return $this->delivery->dt_price;
    }

    public function getId()
    {
        return $this->delivery->dt_id;
    }

    public function getTitle()
    {
        return $this->delivery->dt_title;
    }

    public function getTaxId()
    {
        return $this->delivery->dt_tax;
    }

    public function getCurrency()
    {
        return $this->delivery->dt_currency;
    }

    public function getTaxSum()
    {
        return $this->tax_sum;
    }

    public function getTaxName()
    {
        return $this->tax_name;
    }

    public function calculateTaxSum($current_sum = 0)
    {
        if (!$current_sum) return 0;
        if (!$this->getTaxId()) return 0;
        $tax = Taxes::getInstance()->getTax($this->getTaxId());
        if (is_object($tax)) {
            $this->tax_sum = round($current_sum / (100 + $tax->t_value) * $tax->t_value, 2);
            $this->tax_name = $tax->t_name;
        } else {
            $this->is_error = 1;
            $this->error_text = Text::_("Failed initializing delivery tax");
            $this->tax_sum = 0;
            $this->tax_name = "Error tax";
        }
    }

    public function executeDataQuery($query)
    {
        return "";
    }

    public function prepare($is_ajax)
    {
        /* Just for override */
    }

    public function finished()
    {
        /* Just for override */
    }
}

?>