<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: b6b1944a (2023-09-27 15:32:39)
// 

namespace ArfaCMS\Core;

use ArfaCMS\DB\Database;
use ArfaCMS\MVC\Module;

defined('_ARFACMS_VALID') or die("Access denied");

class orderPayment extends orderUserdata
{
    // @FIXME Проверить и убрать или заполнять таблицу "c_orders_payments"
    protected $payment = null;

    public function __construct($payment, $_data = false)
    {
        parent::__construct($payment);
        $this->payment = $payment;
        if ($_data) $this->data = $_data;
        if ($this->payment) {
            Text::parse(strtolower($this->payment->pt_name), "payments");
            $this->params = Params::parse($this->payment->pt_params);
        }
    }

    public static function getOrderIdByAbstractPaymentClass($pt_name = "", $module = "", $mode)
    {
        if(!$pt_name) return 0;
        $db = Database::getInstance();
        $sql = "SELECT count(pt_id) FROM #__pts_active WHERE pt_deleted=0 AND pt_enabled=1 AND pt_name='" . $pt_name . "'";
        if($module) $sql.= " AND (pt_module='' OR pt_module='".$module."')";
        // else $sql.= " AND pt_module=''";
        $db->setQuery($sql);
        if ($db->loadResult() > 0) {
            try {
                $pt_class = "ArfaCMS\\Payment\\" . ucfirst($pt_name);
                return $pt_class::getOrderId($mode);
            } catch (\Exception $e) {
                return 0;
            }
        }
        return 0;
    }

    public static function getPaymentClass($pt_id= false, $payment = false)
    {
        if(!$payment) $payment = self::getPaymentType($pt_id);
        if($payment) {
            $pt_class = "ArfaCMS\\Payment\\" . ucfirst($payment->pt_name);
            return new $pt_class($payment);
        } else {
            // Util::halt("Payment class not found");
            return false;
        }
    }

    public static function getInstalled(){
        $sql = "SELECT * FROM #__pts";
        $db = Database::getInstance();
        $db->setQuery($sql);
        return $db->loadAssocList();
    }

    public static function isInstalled($name = ""){
        if (!$name) return 0;
        $sql = "SELECT COUNT(*) FROM #__pts WHERE LOWER(p_name)='".strtolower($name)."'";
        $db = Database::getInstance();
        $db->setQuery($sql);
        return $db->loadResult();
    }

    public static function getPaymentType($pt_id)
    {
        $res = null;
        if(!$pt_id) return false;
        $db = Database::getInstance();
        $sql = "SELECT * FROM #__pts_active WHERE pt_deleted=0 AND pt_id=" . $pt_id;
        if (!defined("_ADMIN_MODE")) $sql.= " AND  pt_enabled=1";
        $db->setQuery($sql);
        if ($db->loadObject($res)) return $res;
        else return false;
    }

    public static function getPaymentTypes($dt_id = 0, $is_front = false, $module = "") {
        $db = Database::getInstance();
        if($dt_id){
            $sql = "SELECT p.* FROM #__pts_active AS p, #__dts_active_links as l
					WHERE l.parent_id=p.pt_id AND p.pt_deleted=0 AND l.dt_id=".(int)$dt_id;
        } else {
            $sql = "SELECT p.* FROM #__pts_active AS p WHERE p.pt_deleted=0";
        }
        if (!defined("_ADMIN_MODE")) $sql.= " AND p.pt_enabled=1";
        if($module) $sql.= " AND (p.pt_module='' OR p.pt_module='".$module."')";
        else $sql.= " AND p.pt_module=''";
        if($is_front && !User::getInstance()->isAdmin()) $sql.= " AND p.pt_admin_only=0";
        $sql.= " ORDER BY p.pt_ordering, p.pt_title";
        $db->setQuery($sql);
        return $db->loadObjectList();
    }
    public static function getPaymentTypesForModule($dt_id = 0, $is_front = false){
        return self::getPaymentTypes($dt_id, $is_front, Module::getInstance()->getName());
    }
    // -------------------------------------------------------------------

    public function setPaid()
    {
        $sql = "UPDATE #__orders SET o_paid=1";
        if ($this->payment->pt_set_status) {
            $sql .= ", o_status=" . $this->payment->pt_set_status;
        }
        $sql .= " WHERE o_id=" . $this->order->o_id;
        Database::getInstance()->setQuery($sql);
        if (Database::getInstance()->query()) {
            Event::raise("order.paid", array("module" => Module::getInstance()->getName(), "order" => $this->order, "payment" => $this->payment));
            return true;
        }
        return false;
    }

    public function setPaymentResultData($data)
    {
        $enc_data = $this->getEncodedArray($data);
        $sql = "UPDATE #__orders SET o_pt_result='" . $enc_data . "' WHERE o_id=" . $this->order->o_id;
        Database::getInstance()->setQuery($sql);
        if (Database::getInstance()->query()) return true;
        else return false;
    }

    // -------------------------------------------------------------------

    public function prepare($is_ajax)
    {
        /* Just for override */
    }

    /**
     * Render payment finished
     * @return void
     */
    public function finished()
    {
        /* Just for override */
    }

    /**
     * Main function
     * @return void
     */
    public function show()
    {
        /* Just for override */
    }

    /**
     * Sending data to payment service
     * @return true
     */
    public function send()
    {
        return true;
    }

    /**
     * Callback function
     * @return true
     */
    public function recieve()
    {
        return true;
    }

    public function calculate($userdata = array())
    {
        return $this->payment->pt_price;
    }

    public function getId()
    {
        return $this->payment->pt_id;
    }

    public function getTitle()
    {
        return $this->payment->pt_title;
    }

    public function getCurrency()
    {
        return $this->payment->pt_currency;
    }

    public function getOrderTotalSumForPayment($currency_code = "")
    {
        if (!$currency_code) $currency_code = $this->getCurrency();
        $order_summ = Currency::getInstance()->convert($this->order->o_total_sum, $this->order->o_currency, $currency_code);
        return $order_summ;
    }

    public function executeDataQuery($query)
    {
        return "";
    }

    protected function getPageHead($h_tag_num = 3)
    {
        $html = "<h".$h_tag_num." class=\"title\">" . Text::_("Order payment") . " № " . $this->order->o_id . " " . Text::_("from") . " " . Date::GetdateRus($this->order->o_date) . "</h".$h_tag_num.">";
        $html .= "<h".($h_tag_num + 1)." class=\"sub-title\">" . Text::_("Your payment type") . " : " . $this->payment->pt_title . "</h".($h_tag_num + 1).">";
        return $html;
    }

    /**
     * Renders payment data
     * @param $data
     * @return string
     */
    public function renderPaymentData($data = "")
    {
        /* JUST FOR OVERRIDE */
        return "";
    }

    public function notifyAdmin(){
        $orders = Module::getInstance("orders");
        if($orders->getParam("notify_about_payment_email")){
            $ordersEmailHelper = Module::getHelper("email", "orders");
            $text = $ordersEmailHelper->renderOrderEmail($this->order, "order_paid_admin");

            // $to = $orders->getParam("payment_notification_email", true, \soConfig::$siteEmail); // Not working now
            $to = $orders->getParam("payment_notification_email");
            if(!$to) $to = \soConfig::$siteEmail;

            if($orders->getParam('sms2admin')) $admin_phone=\soConfig::$Phone; else $admin_phone="";

            $theme = Text::_("Payment by order"). " № ".$this->order->o_id;
            // Util::pre($to, $theme, $text, "html", $admin_phone);
            aNotifier::addToQueue($to, $theme, $text, "html", $admin_phone);
        }
    }
}
?>