<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 8edfb304 (2024-10-05 23:38:32)
// 

namespace ArfaCMS\DB;

use ArfaCMS\Core\BaseObject;

defined('_ARFACMS_VALID') or die("Access denied");

class Exporter extends BaseObject
{

    //---------- Singleton implementation ------------
    private static $_instance = null;
    private $replace_prefix = true;

    public static function createInstance($replace_prefix = true)
    {
        if (self::$_instance == null) {
            self::$_instance = new self($replace_prefix);
        }
    }

    public static function getInstance($replace_prefix = true)
    {
        self::createInstance($replace_prefix);
        return self::$_instance;
    }

    //------------------------------------------------

    private $_db = null;

    private function __construct($replace_prefix)
    {
        $this->replace_prefix = $replace_prefix;
        $this->_db = Database::getInstance();
    }

    public function getTablesList($like = '')
    {
        $sql = "SHOW FULL TABLES";
        if ($like) $sql = "SHOW FULL TABLES LIKE '%" . $like . "%'";
        $this->_db->setQuery($sql);
        $tbd = $this->_db->loadResultArray();
        return $tbd;
    }

    public function getCreateTable($table, $delimiter = '')
    {
        if (!$delimiter) $delimiter = $this->_db->getDelimiter();
        $this->_db->setQuery('SHOW CREATE TABLE ' . $table);
        $tbd = $this->_db->loadAssocList();
        $tbq = $tbd[0]['Create Table'] . $delimiter . "\n";
        if ($this->replace_prefix) return str_replace('CREATE TABLE `' . $this->_db->getPrefix(), 'CREATE TABLE `#__', $tbq);
        else return $tbq;
    }

    public function getInserts($table, $delimiter = '')
    {
        if (!$delimiter) $delimiter = $this->_db->getDelimiter();
        $this->_db->setQuery('SELECT * FROM `' . $table . '`');
        if ($this->replace_prefix) $table = "#__" . substr($table, strlen($this->_db->getPrefix()));
        $data = $this->_db->loadAssocList();
        $query = "";
        foreach ($data as $row) {
            $fields = "";
            $vals = "";
            foreach ($row as $f => $v) {
                if (is_null($v)) $v = "NULL";
                else $v = "'" . str_replace(CR_LF, '\r\n', addslashes($v)) . "'";
                if ($fields != "") $fields .= ",";
                $fields .= "`" . $f . "`";
                if ($vals != "") $vals .= ",";
                $vals .= $v;
            }
            $query .= "INSERT INTO `" . $table . "`(" . $fields . ") VALUES(" . $vals . ")" . $delimiter . "\n";
        }
        return $query;
    }

}

?>