<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: a94d9578 (2024-12-18 17:57:08)
// 

namespace ArfaCMS\Delivery;

use ArfaCMS\Core\Address;
use ArfaCMS\Core\HTMLControls;
use ArfaCMS\Core\orderDelivery;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\User;
use ArfaCMS\Core\Userdata;

defined('_ARFACMS_VALID') or die("Access denied");

class Simple extends orderDelivery
{
    protected $exc_required = array("psid", "type_id", "use_as_default", "fullinfo", "apartment");
    protected $need_recalc = 0;

    public function getParamsMask()
    {
        $params = parent::getParamsMask();
        $params['require_address']["vtype"] = "boolean";
        $params['require_address']["vdefault"] = 0;
        return $params;
    }

    public function prepare($is_ajax)
    {
        $func_name = "afterAjaxUpdate_Delivery_" . $this->delivery->dt_id;
        if ($is_ajax) {
            // echo "<script>".$func_name."();</script>";
        } else {
            $script = "function " . $func_name . "(is_ajax) {";
            $script .= "$('#listAddr').trigger('change');"; // Заполнение формы по выбранному адресу. Вариант №1, Часть №1
            $script .= "console.log('" . $func_name . "(is_ajax=' + is_ajax + ')');";
            $script .= "}";
//            $script.= "$(document).ready(function() { ".$func_name."(0); });"; // Это не надо
            Portal::getInstance()->addScriptDeclaration($script);
        }
    }

    public function save()
    {
        if (User::getInstance()->isLoggedIn()) {
            $savedAddrId = Request::getInt('listAddr');
            $userdata = Userdata::getInstance(User::getInstance()->u_id);
            if (Request::getBool('save_adress')) $userdata->saveAddress($savedAddrId);
        }
        $address = Address::getTmpl();
        foreach ($address as $key => $val) $address[$key] = stripslashes(Request::getSafe($key, ""));
        $this->data = $address;
        return $this->data;
    }

    public function renderForm()
    {
        if (User::getInstance()->isLoggedIn()) {
            $list_address = Userdata::getInstance(User::getInstance()->u_id)->getAddresses();
        } else {
            $list_address = array();
        }
        $selected_addr = Request::getInt("listAddr", 0);

        if (is_array($list_address) && count($list_address)) {
            foreach ($list_address as $key => $val) {
                // if(!$selected_addr) $selected_addr = $key; // Заполнение формы по выбранному адресу. Вариант №2
                $list_addr[$key] = $val['fullinfo'];
            }
        } else $list_addr = false;
        if ($selected_addr) {
            $address = Userdata::getInstance(User::getInstance()->u_id)->getAddress($selected_addr);
        } else {
            $address = Address::getTmpl();
        }
        if (!isset($address['country_id'])) $address['country_id'] = 0;
        if (!isset($address['region_id'])) $address['region_id'] = 0;
        if (!isset($address['district_id'])) $address['district_id'] = 0;
        if (!isset($address['locality_id'])) $address['locality_id'] = 0;

        $html = "<div class=\"order-userdata-form\">";
        if ($this->getConfigValue("require_address")) {
            $html .= "<fieldset class=\"grouped-fields px-3 mb-3\"><legend>" . Text::_("Delivery address") . "</legend>";
            $html .= "<div id=\"useraddrselector\" class=\"user-address-selector\">";
            if ($list_addr) {
                $html .= "<div class=\"row mb-3\"><div class=\"col-md-5 mb-2 mb-md-0 align-self-center\">" . HTMLControls::renderLabelField("listAddr", Text::_("Saved addresses")) . "</div><div class=\"col-md-7\">" . HTMLControls::renderSelect('listAddr', 'listAddr', '', '', $list_addr, $selected_addr, 1, 'fillAddressPanel(this); orderUserdataChanged(this);') . "</div></div>";
                // -----------------------------------------------------------
                // Заполнение формы по выбранному адресу. Вариант №1, Часть №2
                // Portal::getInstance()->addScriptDeclaration("$(document).ready(function() { $('#listAddr').trigger('change'); });");
                $func_name = "afterAjaxUpdate_Delivery_" . $this->delivery->dt_id;
                // Portal::getInstance()->addScriptDeclaration("$(document).ready(function() { ".$func_name."(0); });");
                Portal::getInstance()->addScriptDeclaration("$(document).ready(function() { if ($.isFunction(window['" . $func_name . "'])) " . $func_name . "(0); });");
            }
            $html .= "</div>";
            $isLoggedIn = User::getInstance()->isLoggedIn();
            $html .= Address::renderEditor($address, (!\siteConfig::$useTextAddress && $isLoggedIn), $isLoggedIn);
            $html .= "<div class=\"addressPanel addressSubPanel\">";
            if ($isLoggedIn) $html .= "<div class=\"row mb-3 addr-save_adress\"><div class=\"col-md-5 col-sm-9\">" . HTMLControls::renderLabelField("save_adress", Text::_("Save address in list")) . "</div><div class=\"col-md-7 col-sm-3\">" . HTMLControls::renderCheckbox("save_adress", 1) . "</div></div>";
            $html .= "</div>";
            $html .= "</fieldset>";
        }
        $html .= "</div>";
        return $html;
    }

    public function renderInfo($data = "")
    {
        $html = "";
        if ($this->getConfigValue("require_address")) {
            $html .= "<br/>" . HTMLControls::renderLabelField(false, "Delivery address", true) . ": ";
            if (isset($data["zipcode"])) $html .= $data["zipcode"] . ", ";
            if (\siteConfig::$useTextAddress) {
                if (isset($data["country"])) $html .= $data["country"] . ", ";
                if (isset($data["region"])) $html .= $data["region"] . ", ";
                if (isset($data["district"])) $html .= $data["district"] . ", ";
                if (isset($data["locality"])) $html .= $data["locality"] . ", ";
            } else {
                if (isset($data["country_id"])) $html .= Address::getCountryName($data["country_id"]) . ", ";
                if (isset($data["region_id"])) $html .= Address::getRegionName($data["region_id"]) . ", ";
                if (isset($data["district_id"])) $html .= Address::getDistrictName($data["district_id"]) . ", ";
                if (isset($data["locality_id"])) $html .= Address::getLocalityName($data["locality_id"]) . ", ";
            }
            if (isset($data["street"]) && $data["street"]) $html .= $data["street"] . ", ";
            if (isset($data["house"]) && $data["house"]) $html .= $data["house"] . ", ";
            if (isset($data["apartment"]) && $data["apartment"]) $html .= $data["apartment"];
        }
        return $html;
    }

    public function checkData(&$err_message)
    {
        $no_errors = true;
        $address = Address::getTmpl();
        if ($this->getConfigValue("require_address")) {
            $required = $this->intersectRequired(Address::getRequiredTmpl());
            foreach ($required as $key) {
                if (!Request::getSafe($key, "") || Request::getSafe($key, "") == "0") {
                    $err_message[] = Text::_("Some fields not filled");
                    $no_errors = false;
                    break;
                }
            }
        }
        return $no_errors;
    }
}

?>