<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: dbee5e70 (2024-10-10 20:33:04)
// 

namespace ArfaCMS\MVC;

use ArfaCMS\Core\HTMLRenderer;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\User;
use ArfaCMS\Core\Util;

defined('_ARFACMS_VALID') or die("Access denied");

class View extends HTMLRenderer
{
    private $_showPagePanel = false;
    private $_breadCrumb = array();
    protected $_isCanonicalToMainpage = false;
    protected $_rendered = false;
    private $_layoutFile = "";
    protected $_adm_folder = "";

    public function __construct($name)
    {
        $this->initObj($name);
        $this->_adm_folder = defined("_ADMIN_MODE") ? "admin". DS : "";
        // @TODO Canonical links
        /*
        echo "<pre>";
        echo "<br />Module=".Module::getInstance()->getName();
        echo "<br />Viewname=".$this->getName();
        echo "<br />Default Viewname=".Module::getInstance()->getDefaultView();
        echo "<br />Layout=".$this->getLayout();
        echo "<br />Psid=".Module::getInstance()->getController()->getPsid();
        echo "</pre>";
        */
        if (!defined("_ADMIN_MODE")) {
            if (Module::getInstance()->getName() == \siteConfig::$defaultModule) {
                if ($this->getName() == Module::getInstance()->getDefaultView()) {
                    if ($this->getLayout() == "default") {
                        if (Module::getInstance()->getController()->getPsid() == Module::getInstance()->getParam("Default_item_ID") && !Portal::getInstance()->isMainpage()) {
                            $this->_isCanonicalToMainpage = true;
//							Portal::getInstance()->addCustomTag("<link rel=\"canonical\" href=\"".Portal::getInstance()->getURI()."\" />");
                        }
                    }
                }
            }
        }
    }

    public function setMeta($metatag, $metavalue)
    {
        if (($metatag) && ($metavalue)) {
            switch ($metatag) {
                case "title":
                    $metavalue = mb_substr($metavalue, 0, 250, DEF_CP);
                    Portal::getInstance()->setTitle($metavalue);
                    break;
                case "keywords":
                    $metavalue = mb_substr($metavalue, 0, 250, DEF_CP);
                    Portal::getInstance()->setMeta("keywords", $metavalue);
                    break;
                case "description":
                    $metavalue = mb_substr($metavalue, 0, 250, DEF_CP);
                    Portal::getInstance()->setDescription($metavalue);
                    break;
            }
        }
    }

    public function getLayout()
    {
        return $this->get('layout');
    }

    public function setLayout($layout)
    {
        $_layout = $this->get('layout');
        if (defined('_ADMIN_MODE')) $lPath = PATH_TEMPLATES . \adminConfig::$adminTemplate . DS . 'html' . DS . 'modules' . DS . $this->get('module') . DS . $this->getName() . DS . $layout . '.php';
        else $lPath = PATH_TEMPLATES . Portal::getInstance()->getTemplate() . DS . 'html' . DS . 'modules' . DS . $this->get('module') . DS . $this->getName() . DS . $layout . '.php';
        if (!is_file($lPath)) {
            $lPath = PATH_MODULES . $this->get('module') . DS . 'views' . DS . $this->_adm_folder . 'layouts' . DS . $this->getName() . DS . $layout . '.php';
        }
        if (is_file($lPath)) {
            $this->set('layout', $layout, true);
            $this->_layoutFile = $lPath;
        } else {
            if ($_layout != $layout) {
                $this->error(Text::_("Layout change error") . " : |" . $_layout . "| => |" . $layout . "|");
            } else {
                if ($layout == "default") {
                    $this->fatalError(Text::_("Layout not found") . " : " . $lPath, ($this->_debugger == 0 ? "404" : "503")); // Realy fatal if default, else redirect already set in next lines.
                } else {
                    if ((\siteConfig::$debugMode && User::getInstance()->isAdmin()) || \siteConfig::$debugMode > 100) $this->fatalError(Text::_("Layout not found") . " : " . $lPath, defined("_ADMIN_MODE") ? "503" : "404"); // Render fatal for admin debug.
                    else Util::redirect(Router::_("index.php"), Text::_("Layout not found") . " : " . $lPath, "404");
                }
            }
        }
    }

    public function getCustomLayoutPath($layout)
    {
        if (defined('_ADMIN_MODE')) $lPath = PATH_TEMPLATES . \adminConfig::$adminTemplate . DS . 'html' . DS . 'modules' . DS . $this->get('module') . DS . $this->getName() . DS . $layout . '.php';
        else $lPath = PATH_TEMPLATES . Portal::getInstance()->getTemplate() . DS . 'html' . DS . 'modules' . DS . $this->get('module') . DS . $this->getName() . DS . $layout . '.php';
        if (!is_file($lPath)) {
            $lPath = PATH_MODULES . $this->get('module') . DS . 'views' . DS . $this->_adm_folder . 'layouts' . DS . $this->getName() . DS . $layout . '.php';
        }
        if (is_file($lPath)) return $lPath;
        return false;
    }

    // Alias for call from other layout
    // Don't try to include in function (vars become inaccessible).
    public function getLayoutForInclude($layout)
    {
        return $this->getCustomLayoutPath($layout);
    }

    public function resetRenderFlag()
    {
        $this->_rendered = false;
    }

    public function render($wrapper = true, $return_html = false)
    {
        $_html = "";
        $this->prepare();
        $layout = $this->get('layout');
        $this->setLayout($layout);

        if (!is_file($this->_layoutFile)) {
            $this->fatalError(Text::_("Layout not found") . " : " . $this->_layoutFile, "404"); // Realy fatal. This is main view render.
        } else {
            if (!$this->_rendered || $return_html) {
                ob_start();
                if (Portal::getInstance()->noTemplate()) {
                    // @FIXME Check if this working correct
                    if($this->get('option') == "ajax"){
                        $strSrc = "modules/".$this->get("module").".css";
                        $source_path = PATH_CSS . Util::dsPath($strSrc);
                        $source_path_tmpl = PATH_TEMPLATES . Portal::getInstance()->getTemplate() . DS . "css" . DS . Util::dsPath($strSrc);
                        echo "<style>";
                        if (file_exists($source_path)) echo file_get_contents($source_path);
                        if (file_exists($source_path_tmpl)) echo file_get_contents($source_path_tmpl);
                        echo "</style>";
                    }
                    $class_postfix = "-nt";
                } else {
                    $class_postfix = "";
                }
                if($wrapper) echo "<div class=\"moduleBody" . $class_postfix . " " . $this->get("module") . "Module\">";
                if($wrapper) echo "<div class=\"content" . $class_postfix . "\">";
                require_once $this->_layoutFile;
                if($wrapper) echo "</div>";
                echo $this->renderPagePanel();
                if($wrapper) echo "</div>";
                $_html = ob_get_contents();
                ob_end_clean();
            }
        }
        $this->milestone("View rendered => " . $this->get('module') . "." . $this->getName() . "." . $layout, __FUNCTION__);

        if($return_html) return $_html;
        echo $_html;
        return "";
    }
    private function renderShortcut($shortcut)
    {
        $scHTML = "";
        if ($shortcut["text"]) {
            $scHTML .= "<a href=\"#\" class=\"linkButton btn btn-primary \" rel=\"nofollow\"";
            if ($shortcut["id"]) $scHTML .= " id=\"" . $shortcut['id'] . "\"";
            $buttonWidth = intval($shortcut['width']);
            if ($buttonWidth) $scHTML .= " style=\"width:" . $buttonWidth . "px\"";
            $scHTML .= " onclick=\"" . $shortcut["onclick"] . "\">" . $shortcut['text'] . "</a>";

        }
        return $scHTML;
    }

    public function getBreadCrumbArray()
    {
        return $this->_breadCrumb;
    }

    public function setBreadCrumbArray($array)
    {
        $this->_breadCrumb = $array;
    }

    public function renderBreadCrumb()
    {
        if (Portal::getInstance()->inPrintMode()) return "";
        $headerHTML = "<div class=\"moduleBreadcrumb\">";
        foreach ($this->_breadCrumb as $bc) {
            $headerHTML .= "<span class=\"breadcrumb-divider\"></span>";
            if ($bc->link == "#") $headerHTML .= "<a class=\"breadcrumb\">" . $bc->text . "</a>";
            else $headerHTML .= "<a class=\"breadcrumb\" href=\"" . $bc->link . "\">" . $bc->text . "</a>";
        }
        $headerHTML .= "</div>";
        return $headerHTML;
    }

    public function renderPagePanel()
    {
        if ($this->_showPagePanel == true) {
            if (defined('_ADMIN_MODE')) $lPath = PATH_TEMPLATES . \adminConfig::$adminTemplate . DS . 'html' . DS . 'modules' . DS . 'paginator.php';
            else $lPath = PATH_TEMPLATES . Portal::getInstance()->getTemplate() . DS . 'html' . DS . 'modules' . DS . 'paginator.php';
            if (is_file($lPath)) {
                ob_start();
                require $lPath;
                $_html = ob_get_contents();
                ob_end_clean();
            } else {
                $_html = "<div class=\"navigator\">";
                $_html .= "<div class=\"navigator_pages\">";
                $_html .= "<ul class=\"pagination justify-content-center\">";
                if ($this->firstPageLink) $_html .= "<li class=\"page-item\"><a class=\"page-link firstPageLink\" href=\"" . $this->firstPageLink . "\">" . Text::_("First page") . "</a></li>";
                else $_html .= "<li class=\"page-item disabled\"><span class=\"page-link firstPageLink\">" . Text::_("First page") . "</span></li>";
                if ($this->prevPageLink) $_html .= "<li class=\"page-item\"><a class=\"page-link prevPageLink\" href=\"" . $this->prevPageLink . "\">" . Text::_("Prev.page") . "</a></li>";
                else $_html .= "<li class=\"page-item disabled\"><span class=\"page-link prevPageLink\">" . Text::_("Prev.page") . "</span></li>";
                echo $this->pageLinks;
                if ($this->nextPageLink) $_html .= "<li class=\"page-item\"><a class=\"page-link nextPageLink\" href=\"" . $this->nextPageLink . "\">" . Text::_("Next page") . "</a></li>";
                else $_html .= "<li class=\"page-item disabled\"><span class=\"page-link nextPageLink\">" . Text::_("Next page") . "</span></li>";
                if ($this->lastPageLink) echo "<li class=\"page-item\"><a class=\"page-link lastPageLink\" href=\"" . $this->lastPageLink . "\">" . Text::_("Last page") . "</a></li>";
                else $_html .= "<li class=\"page-item disabled\"><span class=\"page-link lastPageLink\">" . Text::_("Last page") . "</span></li>";
                $_html .= "</ul></div>";
                $_html .= "<div class=\"navigator_records text-center\">";
                $_html .= Text::_("Records") . "&nbsp;" . $this->pageRange . "&nbsp;" . Text::_("of") . "&nbsp;" . $this->recordsTotal;
                $_html .= "</div>";
                $_html .= "</div>";
            }
        } else $_html = "";
        return $_html;
    }

    public function showPagePanel($show = true)
    {
        $this->_showPagePanel = $show;
    }

    public function addBreadcrumb($text, $link)
    {
        $bc = new \stdClass();
        $bc->text = $text;
        if ($link == "#") $bc->link = $link;
        else $bc->link = Router::_($link);
        $this->_breadCrumb [] = $bc;
    }

    public function setBreadcrumb($text, $link)
    {
        $bc = new \stdClass();
        $bc->text = $text;
        if ($link == "#") $bc->link = $link;
        else $bc->link = Router::_($link);
        $this->_breadCrumb = array($bc);
    }

    public function prepare()
    {
    } // For overriding
}

?>