<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: fbff1702 (2024-10-01 01:30:00)
// 

namespace ArfaCMS\Payment;
use ArfaCMS\Core\Measure;
use ArfaCMS\Core\orderPayment;
use ArfaCMS\Core\Date;
use ArfaCMS\Core\Files;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\TextRus;
use ArfaCMS\Core\User;
use ArfaCMS\Core\Userdata;
use ArfaCMS\Core\Vendor;

defined('_ARFACMS_VALID') or die("Access denied");

class Simple extends orderPayment {
	public function getParamsMask(){
		$params = parent::getParamsMask();
		return $params;
	}
	private function initConstants(){
		$vendor = Vendor::getInstance()->getVendor($this->order->o_vendor);
		$vendor_addr=Vendor::getInstance()->getPostAddress($this->order->o_vendor);
		$stamp_path=ARFACMS_STORAGE_PATH.DS.'catalog'.DS.'vendors'.DS;
		$stamp_path.='stamp'.DS.Files::splitAppendix($vendor->v_pechat,true);
		if(file_exists($stamp_path) && is_file($stamp_path)) {
			$vendor_pechat=ARFACMS_STORAGE_LINK.'/catalog/vendors/';
			$vendor_pechat.='stamp/'.Files::splitAppendix($vendor->v_pechat);
		} else $vendor_pechat="/assets/images/blank.gif"; // This blank image is OK !!!
		define ('PAYMENT_DEFAULT_FIRM_NAME', $vendor->v_name);
		define ('PAYMENT_DEFAULT_FIRM_ADDRESS', $vendor_addr);
		define ('PAYMENT_DEFAULT_FIRM_PHONE', $vendor->v_phone.($vendor->v_phone && $vendor->v_fax ? "/" : "").$vendor->v_fax);
		define ('PAYMENT_DEFAULT_OGRN', $vendor->v_ogrn);
		define ('PAYMENT_DEFAULT_INN', $vendor->v_inn);
		define ('PAYMENT_DEFAULT_KPP', $vendor->v_kpp);
		define ('PAYMENT_DEFAULT_DIRECTOR', $vendor->v_boss);
		define ('PAYMENT_DEFAULT_GLAVBUH', $vendor->v_ca_name);
		define ('PAYMENT_DEFAULT_BANK_NAME', $vendor->v_bank);
		define ('PAYMENT_DEFAULT_ACCOUNT_NUMBER', $vendor->v_sett_acc);
		define ('PAYMENT_DEFAULT_BIK', $vendor->v_bik);
		define ('PAYMENT_DEFAULT_BANK_NUMBER', $vendor->v_bank_acc);
		define ('PAYMENT_PECHAT_IMAGE', $vendor_pechat);
	}
    public function prepare($is_ajax)
    {
        $func_name = "afterAjaxUpdate_Payment_".$this->payment->pt_id;

        if($is_ajax){
            // echo "<script>".$func_name."();</script>";
        } else {
            $script = "function ".$func_name."(is_ajax) {";
            $script.= "console.log('".$func_name."(is_ajax=' + is_ajax + ')');";
            $script.= "}";
            // $script.= "$(document).ready(function() { ".$func_name."(0); });";  // Это не надо
            Portal::getInstance()->addScriptDeclaration($script);
        }
    }
	/* Собственные функции*/
	private function showPanel() {
		if(User::getInstance()->isLoggedIn()) $CompanyInfo=Userdata::getInstance(User::getInstance()->getID())->getCompany();
		else $CompanyInfo=false;
		if($CompanyInfo) {
			if($CompanyInfo['org_type']){
				$address=Userdata::getInstance(User::getInstance()->getID())->getDefaultAddress(2); // Адрес регистрации
			} else {
				$address=Userdata::getInstance(User::getInstance()->getID())->getDefaultAddress(2); // Адрес регистрации
			}
			$payment_user = $CompanyInfo["surname"]." ".$CompanyInfo["firstname"]." ".$CompanyInfo["patronymic"];
			$payment_user_address = $address['fullinfo'];
			$payment_user_inn = $CompanyInfo["inn"];
		} else {
			$payment_user = "";
			$payment_user_address = "";
			$payment_user_inn = "";
		}
		
		echo "<form name=\"simple_payment\" target=\"_blank\" action=\"/index.php\" method=\"post\">";
		echo "<h3 class=\"title\">".Text::_("Order payment")." № ".$this->order->o_id." ".Text::_("from")." ".Date::GetdateRus($this->order->o_date)."</h3>";
		echo "<h4 class=\"subtitle\">".Text::_("Your payment type")." : ".$this->payment->pt_title."</h4>";
		echo "<div class=\"row mb-3\"><div class=\"col-md-4 mb-2 mb-md-0 align-self-center\">";
		echo "<label for\"payment_user\">".Text::_("Payer")."</label>";
		echo "</div><div class=\"col-md-8\">";
		echo "<input type=\"text\" class=\"form-control\" name=\"payment_user\" value=\"".$payment_user."\" />";
		echo "</div></div>";
		echo "<div class=\"row mb-3\"><div class=\"col-md-4 mb-2 mb-md-0 align-self-center\">";
		echo "<label for\"payment_user_inn\">".Text::_("INN")."</label>";
		echo "</div><div class=\"col-md-8\">";
		echo "<input type=\"text\" class=\"form-control\" name=\"payment_user_inn\" value=\"".$payment_user_inn."\" />";
		echo "</div></div>";
		echo "<div class=\"row mb-3\"><div class=\"col-md-4 mb-2 mb-md-0 align-self-center\">";
		echo "<label for\"payment_user\">".Text::_("Address")."</label>";
		echo "</div><div class=\"col-md-8\">";
		echo "<input type=\"text\" class=\"form-control\" name=\"payment_user_address\" value=\"".$payment_user_address."\" />";
		echo "</div></div>";
		echo "<div class=\"row mb-3\"><div class=\"col-md-4 mb-2 mb-md-0 align-self-center\">";
		echo "<label for\"payment_user\">".Text::_("What to print")."</label>";
		echo "</div><div class=\"col-md-8\">";
		echo "<select name=\"docname\" class=\"form-select\">";
		echo "	<option selected=\"selected\" value=\"receipt\">".Text::_('Receipt')."</option>";
		echo "	<option value=\"account\">".Text::_('Account')."</option>";
		echo "</select>";
		echo "</div></div>";
		echo "<div class=\"buttons\"><div class=\"col-md-12\">";
		echo "<input type=\"hidden\" name=\"option\" value=\"print\" />";
		echo "<input type=\"hidden\" name=\"module\" value=\"orders\" />";
		echo "<input type=\"hidden\" name=\"view\" value=\"payment\" />";
//		echo "<input type=\"hidden\" name=\"layout\" value=\"\" />";
		echo "<input type=\"hidden\" name=\"order_id\" value=\"".$this->order->o_id."\" />";
		echo "<input type=\"submit\" class=\"btn btn-primary\" name=\"submit\" value=\"".Text::_("Print")."\" />";
		echo "</div></div></form>";
	}
	public function show() {
		$docname = Request::getSafe('docname','');
        if($docname) $templPath = PATH_TEMPLATES.Portal::getInstance()->getTemplate().DS.'html'.DS."payments".DS.strtolower($this->payment->pt_name).DS.$docname.".php";
        else $templPath = "";
        switch($docname){
            case "receipt":
                $this->initConstants();
                if (file_exists($templPath)) require_once $templPath;
                else $this->templateReceipt();
                break;
            case "account":
                $this->initConstants();
                if (file_exists($templPath)) require_once $templPath;
                else $this->templateAccount();
                break;
            default:
                $this->showPanel();
                break;
        }
	}

    private function templateReceipt(){

        $payment_user = Request::getSafe("payment_user");
        $payment_user_address = Request::getSafe("payment_user_address");
        $payment_user_inn = Request::getSafe("payment_user_inn");
        if($payment_user_inn) $payment_user_inn = "(ИНН ".$payment_user_inn.")";
?>
        <style type="text/css">
            body, table { font-size:12px; font-family:Tahoma, Arial, sans-serif; }
            td, th { padding:2px 3px 2px 3px; }
            /*
            .b_t, b_t2 {border-collapse:collapse;}
            .b_t td, .b_t th {border:solid 1px #000000;}
            .b_t2 {border:solid 1px #000000;}
             */
            table.big-t { width:630px; margin-left: auto; margin-right: auto; }
            /*
            .big {font-size:14px; font-weight:bold;}
            .middle {font-weight:bold;}
            .head1 {border-top-width: 2px; border-top-style: solid; border-top-color: #000000; font-size:9px;}
            .head1 a {font-size:9px;}
            .n_btm_b {border-bottom: none 0px #FFFFFF;}
             */
            .t-m { width:630px; margin: 10px auto 0; }
            /*
            .w_12 {width:12%;}
            .w_150 {width:150px;}
            .w_120 {width:120px;}
            .ErrMsg {font-size: 18px;color: #d71e1e;}
             */
            .kv {font-size:11px; font-family:Tahoma, Arial, sans-serif; }
            .kv td {padding:1px 2px 0px 2px;}
            .inner {padding: 5px;}
            .small {font-size:5pt; border-top:solid 0.5pt #000000; text-align:center; vertical-align:top; padding:0px !important;}
            .big-t {border: dotted 1px #000000; margin-top:70px; margin-bottom:70px;}
            .btm {border-bottom:solid 0.5pt #000000;}
            .ital {font-size:9pt; font-style:oblique; font-family:'Times New Roman', serif, Tahoma; font-weight:bold;}
            .lc_size {width:180px;}
            @media print {
                .printhidden, #debugger { visibility:hidden; display:none;}
            }
        </style>
        <!-- Шаблон квитанции - начало -->
        <table border="0" cellpadding="0" cellspacing="0" class="kv big-t">
            <tr>
                <td class="inner lc_size" style="border-right:solid 1px #000000; border-bottom: dotted 1px #000000;" align="center" valign="top">
                    <div style="padding:5px 0 0 0;">Извещение</div>
                    <div style="padding:210px 0 0 0;">Кассир</div>
                </td>
                <td class="inner" style="border-bottom: dotted 1px #000000;">
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td align="center" width="30%" style="font-size:6pt;">СБЕРБАНК РОССИИ ОАО<br /><span style="font-size:5pt;">Основан в 1841 году</span></td>
                            <td align="right" style="padding-right:15px; font-size:6pt;">Форма №ПД-4</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td class="ital"><?php echo PAYMENT_DEFAULT_FIRM_NAME ?>&nbsp;</td>
                        </tr>
                        <tr>
                            <td class="small">(наименование получателя платежа)</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td class="ital">ИНН: <?php echo PAYMENT_DEFAULT_INN ?>&nbsp;</td>
                            <td>&nbsp;</td>
                            <td class="ital"><?php echo PAYMENT_DEFAULT_ACCOUNT_NUMBER ?>&nbsp;</td>
                        </tr>
                        <tr>
                            <td class="small">ИНН налогового органа</td>
                            <td>&nbsp;</td>
                            <td class="small">(номер счета получателя платежа)</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td width="5%">в</td>
                            <td class="ital"><?php echo PAYMENT_DEFAULT_BANK_NAME ?>&nbsp;</td>
                            <td width="5%">БИК</td>
                            <td class="btm ital"><?php echo PAYMENT_DEFAULT_BIK ?>&nbsp;</td>
                        </tr>
                        <tr>
                            <td>&nbsp;</td>
                            <td class="small">(наименование банка)</td>
                            <td>&nbsp;</td>
                            <td>&nbsp;</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td width="50%">Номер кор/сч банка получателя платежа</td>
                            <td class="btm ital"><?php echo PAYMENT_DEFAULT_BANK_NUMBER ?>&nbsp;</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td class="ital">по счету №<?php printf("%08d", $this->order->o_id)?></td>
                            <td>&nbsp;</td>
                            <td>&nbsp;</td>
                        </tr>
                        <tr>
                            <td class="small">(наименование платежа)</td>
                            <td>&nbsp;</td>
                            <td class="small">(номер лицевого счета (код) плательщика)</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td width="20%">Ф.И.О.&nbsp;плательщика</td>
                            <td class="btm ital"><?php echo $payment_user." ".$payment_user_inn;?>&nbsp;</td>
                        </tr>
                        <tr>
                            <td width="20%">Адрес&nbsp;плательщика</td>
                            <td class="btm ital"><?php echo $payment_user_address;?>&nbsp;</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td width="20%">Сумма платежа</td>
                            <td width="30%" class="btm ital"><?php printf("%.2f", $this->order->o_total_sum)?> p.</td>
                            <td width="30%">Сумма платы за услуги</td>
                            <td width="20%" class="btm">&nbsp;</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td width="15%">Итого</td>
                            <td width="30%" class="btm ital"><?php printf("%.2f", $this->order->o_total_sum)?> p.</td>
                            <td width="5%">&nbsp;</td>
                            <td width="25%" class="btm ital">'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'</td>
                            <td width="5%">20</td>
                            <td width="5%" class="btm ital">&nbsp;&nbsp;&nbsp;</td>
                            <td width="15%">г.</td>
                        </tr>
                        <tr>
                            <td colspan="7" style="font-size:6pt;">С условиями приема указанной в платежном документе суммы в т.ч. с суммой взымаемой платы за услуги банка ознакомлен и согласен</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv" style="margin-bottom:5px;">
                        <tr>
                            <td align="right" width="65%">Подпись плательщика</td>
                            <td  class="btm">&nbsp;</td>
                        </tr>
                    </table>
                </td>
            </tr>
            <tr>
                <td class="inner lc_size" style="border-right:solid 1px #000000;" align="center" valign="top">
                    <div style="padding:200px 0 0 0;">Квитанция</div>
                    <div style="padding:10px 0 0 0;">Кассир</div>
                </td>
                <td class="inner" style="padding-top:5px;">
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td class="ital"><?php echo PAYMENT_DEFAULT_FIRM_NAME ?>&nbsp;</td>
                        </tr>
                        <tr>
                            <td class="small">(наименование получателя платежа)</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td class="ital">ИНН: <?php echo PAYMENT_DEFAULT_INN ?>&nbsp;</td>
                            <td>&nbsp;</td>
                            <td class="ital"><?php echo PAYMENT_DEFAULT_ACCOUNT_NUMBER ?>&nbsp;</td>
                        </tr>
                        <tr>
                            <td class="small">ИНН налогового органа</td>
                            <td>&nbsp;</td>
                            <td class="small">(номер счета получателя платежа)</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td width="5%">в</td>
                            <td class="ital"><?php echo PAYMENT_DEFAULT_BANK_NAME ?>&nbsp;</td>
                            <td width="5%">БИК</td>
                            <td class="btm ital"><?php echo PAYMENT_DEFAULT_BIK ?>&nbsp;</td>
                        </tr>
                        <tr>
                            <td>&nbsp;</td>
                            <td class="small">(наименование банка)</td>
                            <td>&nbsp;</td>
                            <td>&nbsp;</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td width="50%">Номер кор/сч банка получателя платежа</td>
                            <td class="btm ital"><?php echo PAYMENT_DEFAULT_BANK_NUMBER ?>&nbsp;</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td class="ital">по счету №<?php printf("%08d", $this->order->o_id)?></td>
                            <td>&nbsp;</td>
                            <td>&nbsp;</td>
                        </tr>
                        <tr>
                            <td class="small">(наименование платежа)</td>
                            <td>&nbsp;</td>
                            <td class="small">(номер лицевого счета (код) плательщика)</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td width="20%">Ф.И.О.&nbsp;плательщика</td>
                            <td class="btm ital"><?php echo $payment_user." ".$payment_user_inn ;?>&nbsp;</td>
                        </tr>
                        <tr>
                            <td width="20%">Адрес&nbsp;плательщика</td>
                            <td class="btm ital"><?php echo $payment_user_address;?></td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td width="20%">Сумма платежа</td>
                            <td width="30%" class="btm ital"><?php printf("%.2f", $this->order->o_total_sum)?> p.</td>
                            <td width="30%">Сумма платы за услуги</td>
                            <td width="20%" class="btm">&nbsp;</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv">
                        <tr>
                            <td width="15%">Итого</td>
                            <td width="30%" class="btm ital"><?php printf("%.2f", $this->order->o_total_sum)?> p.</td>
                            <td width="5%">&nbsp;</td>
                            <td width="25%" class="btm ital">'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'</td>
                            <td width="5%">20</td>
                            <td width="5%" class="btm ital">&nbsp;&nbsp;&nbsp;</td>
                            <td width="15%">г.</td>
                        </tr>
                        <tr>
                            <td colspan="7" style="font-size:6pt;">С условиями приема указанной в платежном документе суммы в т.ч. с суммой взымаемой платы за услуги банка ознакомлен и согласен</td>
                        </tr>
                    </table>
                    <table width="100%" border="0" cellspacing="0" cellpadding="0" class="kv" style="margin-bottom:5px;">
                        <tr>
                            <td align="right" width="65%">Подпись плательщика</td>
                            <td class="btm">&nbsp;</td>
                        </tr>
                    </table>
                </td>
            </tr>
        </table>
        <table width="630" border="0" cellspacing="0" cellpadding="0" class="t-m">
            <tr>
                <td style="padding: 10px;">Внимание ! Оплата квитанции третьими лицами не допустима. Зачисление денег на наш расчетный счет осуществляется в течение 2-3 банковских дней, для ускорения зачисления вышлите по факсу копию или по e-mail скан оплаченной квитанции.</td>
            </tr>
        </table>
<?php
    }

    private function templateAccount(){

        $taxesInOrder=array();
        $payment_user = Request::getSafe("payment_user");
        $payment_user_address = Request::getSafe("payment_user_address");
        $payment_user_inn = Request::getSafe("payment_user_inn");
        if($payment_user_inn) $payment_user_inn = "ИНН ".$payment_user_inn;
?>
        <style type="text/css">
            #schet{ width:700px; margin-left: auto; margin-right: auto; font-size:12px; font-family:Tahoma, Arial, sans-serif;}
            td, th { padding: 2px 3px 2px 3px; }
            /*
            .b_t, b_t2 {border-collapse:collapse;}
            .b_t td, .b_t th {border:solid 1px #000000;}
            .b_t2 {border:solid 1px #000000;}
            .big {font-size:14px; font-weight:bold;}
            .middle {font-weight:bold;}
            .head1 {border-top-width: 2px; border-top-style: solid; border-top-color: #000000; font-size:9px;}
            .head1 a {font-size:9px;}
            .n_btm_b {border-bottom: none 0px #FFFFFF;}
            .t-m {margin:10px 0 0 0;}
            .w_12 {width:12%;}
            .w_150 {width:150px;}
            .w_120 {width:120px;}
            .ErrMsg {font-size: 18px;color: #d71e1e;}
            .kv {font-size:11px; font-family:Tahoma, Arial, sans-serif;}
            .kv td {padding:1px 2px 0px 2px;}
            .inner {padding: 5px;}
            .small {font-size:5pt; border-top:solid 0.5pt #000000; text-align:center; vertical-align:top; padding:0px !important;}
            .big-t {border: dotted 1px #000000; margin-top:70px; margin-bottom:70px;}
            .btm {border-bottom:solid 0.5pt #000000;}
            .ital {font-size:9pt; font-style:oblique; font-family:'Times New Roman', serif, Tahoma; font-weight:bold;}
            .lc_size {width:180px;}
             */
            #AutoNumber4{ width:100%; }
            #pechat	{display: block; }
            #pechat1 {display: none; }
            @media print {
                .printhidden, #debugger { visibility:hidden; display:none; }
            }
        </style>
        <div id="schet" align="center">
            <table border="0" cellpadding="8" cellspacing="0" width="100%" id="AutoNumber1" style="border-collapse: collapse;">
                <tr>
                    <td>
                        <table width="100%">
                            <tr>
                                <td><b><?php echo PAYMENT_DEFAULT_FIRM_NAME; ?>, ИНН: <?php echo PAYMENT_DEFAULT_INN; ?>, ОГРН: <?php echo PAYMENT_DEFAULT_OGRN; ?></b><br />
                                    Адрес: <?php echo PAYMENT_DEFAULT_FIRM_ADDRESS; ?><br />
                                    тел/факс: <?php echo PAYMENT_DEFAULT_FIRM_PHONE; ?>
                                </td>
                                <td valign="middle" align="right">
                                    &nbsp;  <!--	<img hspace=10 src="/assets/images/logo.jpg"> -->
                                </td>
                            </tr>
                        </table>
                        <b>Образец заполнения платежного поручения:</b><br /><br />
                        <div align="center">
                            <table cellpadding="2" cellspacing="0" id="AutoNumber2" width="100%" style="border-collapse: collapse; border:1px solid #111111;">
                                <tr>
                                    <td>ИНН &nbsp;<?php echo PAYMENT_DEFAULT_INN; ?></td>
                                    <td>КПП </td>
                                    <td>&nbsp;<?php echo PAYMENT_DEFAULT_KPP; ?></td>
                                </tr>
                                <tr>
                                    <td>Получатель:
                                        <br /><?php echo PAYMENT_DEFAULT_FIRM_NAME; ?>&quot;</td>
                                    <td>Сч. N</td>
                                    <td>&nbsp;<?php echo PAYMENT_DEFAULT_ACCOUNT_NUMBER; ?></td>
                                </tr>
                                <tr>
                                    <td rowspan="2">Банк получателя:<br /><?php echo PAYMENT_DEFAULT_BANK_NAME; ?><br /></td>
                                    <td>БИК</td>
                                    <td>&nbsp;<?php echo PAYMENT_DEFAULT_BIK; ?></td>
                                </tr>
                                <tr>
                                    <td>Сч. N</td>
                                    <td>&nbsp;<?php echo PAYMENT_DEFAULT_BANK_NUMBER; ?></td>
                                </tr>
                            </table>
                        </div>

                        <h2 align="center">СЧЕТ N <?php printf("%08d", $this->order->o_id); ?> от <?php echo Date::GetdateRus($this->order->o_date); ?></h2>
                        <p align="left">Плательщик: <?php echo $payment_user.($payment_user_inn ? ",&nbsp;".$payment_user_inn : "")?>
                            <?php echo ($payment_user_address ? "<br />Адрес пдлательщика: ".$payment_user_address : "");?>
                        </p>
                        <div align="center">
                            <table cellpadding="2" cellspacing="0" width="100%" id="AutoNumber3" style="border-collapse: collapse; border:1px solid #000000;" >
                                <tr>
                                    <td style="border:1px solid #000000;">&nbsp;&nbsp;&nbsp;N&nbsp;&nbsp;&nbsp;</td>
                                    <td style="border:1px solid #000000;">Наименование</td>
                                    <td style="border:1px solid #000000;" align="center">Единица<br />изме-<br />рения</td>
                                    <td style="border:1px solid #000000;" align="center">Кол-во</td>
                                    <td style="border:1px solid #000000;" align="center">Цена</td>
                                    <td style="border:1px solid #000000;" align="center">Сумма</td>
                                </tr>
                                <?php
                                $nppp=0;  $total_summa=0;
                                if (count($this->items)>0) {
                                    foreach	($this->items as $grow) {
                                        $nppp=$nppp+1;
                                        $cur_summa=$grow->i_g_quantity*$grow->i_g_price;
                                        $total_summa=$total_summa+$cur_summa;
                                        if ($grow->i_g_tax>0){
                                            $tax_name=base64_encode($grow->i_g_tax_name);
                                            if (!isset($taxesInOrder[$tax_name]))$taxesInOrder[$tax_name]=0;
                                            $taxesInOrder[$tax_name]+=$grow->i_g_tax;
                                        }

                                        ?>
                                        <tr>
                                            <td style="border:1px solid #000000;" align="center"><?php echo $nppp; ?></td>
                                            <td style="border:1px solid #000000;"><?php echo $grow->i_g_name; ?></td>
                                            <td style="border:1px solid #000000;" align="center"><?php echo Measure::getInstance()->getShortName($grow->i_g_measure); ?></td>
                                            <td style="border:1px solid #000000;" align="center"><?php echo $grow->i_g_quantity; ?></td>
                                            <td style="border:1px solid #000000; white-space:nowrap;" align="center"><?php printf("%01.2f", $grow->i_g_price); ?></td>
                                            <td style="border:1px solid #000000; white-space:nowrap;" align="right"><?php printf("%01.2f", $cur_summa); ?></td>
                                        </tr>
                                        <?php
                                    }
                                }
                                ?>
                                <tr>
                                    <td colspan="5" style="border:1px solid #000000; white-space:nowrap;"><p align="right">Итого:</p></td>
                                    <td style="border:1px solid #000000; white-space:nowrap;" align="right"><?php printf("%01.2f", $total_summa); ?></td>
                                </tr>
                                <!-- Стоимость доставки и плата за отгрузку -->
                                <?php	      $shipping_total = $this->order->o_dt_sum;
                                if ($shipping_total>0) {
                                    if ($this->order->o_dt_tax_val){
                                        $tax_name=base64_encode($this->order->o_dt_tax_name);
                                        if (!isset($taxesInOrder[$tax_name])) $taxesInOrder[$tax_name]=0;
                                        $taxesInOrder[$tax_name]+=$this->order->o_dt_tax_sum;
                                    }
                                    ?>

                                    <tr>
                                        <td colspan="5" align="right"><?php echo $this->order->o_dt_title ?> :</td>
                                        <td align="right">
                                            <?php	printf("%01.2f", $shipping_total); ?>
                                        </td>
                                    </tr>

                                    <!-- Налог на стоимость заказа -->
                                <?php       }
                                if ($this->order->o_discount_sum != 0) {
                                    ?>
                                    <tr>
                                        <td colspan="5" align="right">
                                            <?php
                                            if( $this->order->o_discount_sum > 0) echo Text::_('Discount');
                                            else echo Text::_('Fee');
                                            ?>
                                        </td>
                                        <td align="right">
                                            <?php                printf("%01.2f", abs($this->order->o_discount_sum)); ?>
                                        </td>
                                    </tr>
                                <?php				}
                                if ($this->order->o_taxes_sum>0) {
                                    echo "<tr><td colspan=\"5\" align=\"right\">".Text::_("Taxes in order")." : "."</td>";
                                    echo "<td align=\"right\">".number_format($this->order->o_taxes_sum, 2, DEFAULT_DECIMAL_SEPARATOR , DEFAULT_THOUSAND_SEPARATOR)."</td></tr> \n";
                                    if (count($taxesInOrder)) {
                                        foreach($taxesInOrder as $kt=>$vt){
                                            echo "<tr><td colspan=\"5\" align=\"right\">".base64_decode($kt)." : "."</td>";
                                            echo "<td align=\"right\">".number_format($vt, 2, DEFAULT_DECIMAL_SEPARATOR , DEFAULT_THOUSAND_SEPARATOR)."</td></tr> \n";
                                        }
                                    }
                                }

                                $total_summa = $this->order->o_total_sum;
                                ?>
                                <!-- ИТОГО -->
                                <tr>
                                    <td colspan="5" style="border:1px solid #000000; white-space:nowrap;"><p align="right">Всего к оплате:</p></td>
                                    <td style="border:1px solid #000000; white-space:nowrap;" align="right"><?php printf("%01.2f", $total_summa) ; ?></td>
                                </tr>
                            </table>
                        </div>
                        <p><b>Всего на сумму: &nbsp; <?php printf("%01.2f", $total_summa) ; ?></b>
                            <br />(<?php echo TextRus::SumInWords($total_summa); ?>.)
                        </p>
                        <p>&nbsp;</p>
                        <table border="0" cellpadding="0" cellspacing="1" id="AutoNumber4">
                            <tr>
                                <td valign="top" style="white-space:nowrap;"><br />&nbsp;Генеральный директор&nbsp;&nbsp;&nbsp;&nbsp;<br />&nbsp;<br />&nbsp;<br />&nbsp;<br />&nbsp;<br />&nbsp;Главный бухгалтер&nbsp;</td>
                                <td><img alt="Место печати" id="pechat" src="<?php echo PAYMENT_PECHAT_IMAGE; ?>" border="0" /><img alt="pechat1" id="pechat1" src="/assets/images/pechat1.gif" border="0" /></td>
                                <td valign="top" style="white-space:nowrap;"><br />&nbsp;&nbsp;&nbsp;&nbsp;<?php echo PAYMENT_DEFAULT_DIRECTOR; ?><br />&nbsp;<br />&nbsp;<br />&nbsp;<br />&nbsp;<br />&nbsp;&nbsp;&nbsp;&nbsp;<?php echo PAYMENT_DEFAULT_GLAVBUH; ?></td>
                            </tr>
                        </table>
                    </td>
                </tr>
            </table>
        </div>

<?php
    }
}
?>