<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 60230277 (2024-11-14 01:05:45)
// 

namespace ArfaCMS\Sprav;

use ArfaCMS\Core\BaseObject;
use ArfaCMS\Core\Debugger;
use ArfaCMS\Core\Event;
use ArfaCMS\Core\Files;
use ArfaCMS\Core\User;
use ArfaCMS\Core\Util;
use ArfaCMS\DB\Database;
use ArfaCMS\MVC\Module;

defined('_ARFACMS_VALID') or die("Access denied");

class SpravMetadata extends BaseObject
{
    protected $store_type = 1;    // Тип хранения данных в файлах
    protected $_prepView = true;    // апдейт полей из таблицы
    public $success = false;    // Флаг удачной загрузки
    public $constants = array();    // Константы
    public $classname = "";    // Имя класса
    public $module = "";        // Имя модуля (совпадает с именем каталога модуля)
    public $tablename = "";        // Основная таблица БД без префикса
    public $linktable = "";        // Линковочная таблица БД без префикса
    public $link_with_childs = false;    // При линковке если родитель дерево взять всех детей
    public $parent_table = '';  // родительская  таблица, актуальна при линковке, а так равна ch_table поля multy_field
    public $parent_name = '';        // поле наименования родительской таблицы, актуально при линковке
    public $parent_title = '';        // заголовок списка линковки
    public $use_view_rights = '';    // имя view права которой можно использовать
    public $title = "";        // Заголовок таблицы
    public $namestring = "";        // Имя поля с названием в таблице
    public $selector_string = "";        // Список полей разделенных запятыми для подстановки из селектора
    public $alias_field = "";        // Имя поля с псевдонимом в таблице для
    public $parent_subordination = true;        // учитывать подчиненность в списках элементов, влияет на сортировку при линковке без подчинения
    public $parent_code = '';        // ключевое поле родительского представления
    public $parent_view = '';       // родительское представление, куда перейдем по go_up
    public $parent_layout = '';     // родительское отображение, куда перейдем по go_up
    public $keystring = "";        // Основное Ключевое поле таблицы
    public $nofilter = false;   // Принудительное отключение фильтрации
    public $show_nopages = true; //  Показать флажок "Не разбивать на страницы"
    public $show_woparents = true; //  Показать флажок "Без учета родителей"
    public $enabled = "";        // Поле пометки вкл/выкл
    public $deleted = "";        // Поле пометки на удаление
    public $keysort = "";        // Поле сортировки по умолчанию при показе
    public $keysort_dest = "ASC";        // Направление сортировки по умолчанию при показе
    public $ordering_field = "";        // Поле порядка отображения
    public $keycurrency = "";        // Поле в котором содержится валюта хранения(опц.)
    public $checkbox = true;        // Надо ли выводить первой колонкой checkbox для выбора элементов
    public $selector = false;        // Надо ли выводить первой колонкой кнопку для подстановки элементов
    public $tree_index = 0;        // если является деревом, тогда здесь индекс поля
    public $tree_skip_deleted = 0;    // если является деревом, тогда можно скрыть удаленные в дереве
    public $tree_skip_disabled = 0;    // если является деревом, тогда можно скрыть отключенные в дереве
    public $multy_field = "";        // Поле таблицы для связи с родительской таблицей
    public $custom_sql = "";        // Дополнительный текст SQL запроса в условии WHERE
    public $pre_sorting = array();        // Предварительная сортировка в условии ORDER BY
    public $post_sorting = array();        // Дополнительная сортировка в условии ORDER BY
    public $pans_titles = array(1 => "Main data", 2 => "Additional"); // Названия вкладок при редактировании
    public $templates = array();    // Шаблоны
    public $buttons = array();    // Общие для всех кнопки
    public $uni_buttons = array();    // Кнопки уникальные для данного справочника
    public $field = array();    // Поля таблицы
    public $filter = array();    // Наличие фильтрации по полю
    public $filter_ext = array();    // Наличие фильтрации по полю
    public $filter_strict = array();    // Строгая фильтрация по полю
    public $filter_ch_list = array();    // Фильтрация ch поля по списку
    public $field_orderby = array();    // Направление сортировки поля таблицы
    public $field_title = array();    // Заголовки полей таблицы
    public $field_descr = array();    // Описание (всплывающий balloon) полей таблицы
    public $field_on_change = array();    // Javascript на изменение поля при редактировании
    public $field_is_method = array();    // Значение в поле является результатом вычислений и не полем таблицы
    public $size = array();    // Ширина колонки при просмотре
    public $view = array();    // Показывать колонку
    public $link = array();    // Ссылка для значения ячейки
    public $link_vars = array();    // Параметры для подстановки в link
    public $link_types = array();    // Параметры для подстановки в link ( например popup)
    public $link_titles = array();    // Подсказки для подстановки в link ( например popup)
    public $link_picture = array();    // Картинка для  подстановки в ячейку
    public $img_module = array();    // модуль из которого брать картинки( например в статьи из галереи, в блоги из товаров)
    public $upload_path = array();    // хвост пути, куда сохранять этот файл (папка storage/имя модуля/$upload_path)
    public $val_type = array();    // Тип данных поля
    public $val_size = array();    // Длина данных поля (с учетом getSafe длина хранимых данных больше)
    public $input_size = array();    // Размер поля при редактировании
    public $input_type = array();    // Тип поля при редактировании
    public $input_data = array();    // Массив data атрибутов для поля
    public $input_view = array();    // Показывать поле при редактировании
    public $input_page = array();    // Номер вкладки для поля при редактировании
    public $input_last_page = 1;    // Номер последней вкладки при редактировании (только расчитывается !!!)
    public $field_no_update = array();    // Не изменять при редактировании
    public $check_value = array();    // Проверять значение при редактировании элемента (0 - не проверяет, 1 - проверяет, 2 - проверяет на уникальность все, 3 - проверяет на уникальность непустые)
    public $translate_value = array();    // Переводить ли полученное значение
    public $default_value = array();    // Значение по умолчанию
    public $ck_reestr = array();    // Подстановка фиксированного набора значений
    public $ch_table = array();    // Таблицы подстановки для зависимых полей
    public $ch_field = array();    // Поле подстановки из таблицы подстановки
    public $ch_parent_field = array();    // Поле родителя таблицы подстановки (если таблица подстановки дерево и не равна текущей таблице)
    public $ch_id = array();    // Ключевое поле таблицы подстановки
    public $ch_deleted = array();    // Поле пометки на удаление таблицы подстановки
    public $ch_enabled = array();    // Поле пометки на включенность в таблице подстановки
    public $ch_skip_deleted = array();    // Поле исключения удаленных для таблицы подстановки
    public $ch_skip_disabled = array();    // Поле исключения отключенных для таблицы подстановки
    public $ch_sort = array();    // Поле сортировки таблицы подстановки
    public $ch_sp_field = array();    // Дополнительное условие для выборки из таблицы подстановки (Приравнивается к psid редактируемого элемента если не указан ch_sp_field_val)
    public $ch_sp_field_val = array();    // Значение дополнительного условия для выборки из таблицы подстановки
    public $is_add = array();    // поле является дополнительным и живет в другой таблице [tablename]_data
    public $is_add_custom = array();    // если (is_add), то 1 - используется для объекта от группы объектов, 2- является опцией (множественна и подчинена объекту)
    public $classTable = '';
    public $viewName = '';
    public $layoutName = '';
    public $field_order = array();    // Номер по порядку в массиве полей. Для пересортировки массива метаданных
    public $force_front_metadata = 0;                        // принудительная указание что метадата на фронтенде (при вызове из админки)
    protected $_admin_sub_folder = "";

    public function __construct($moduleName = '', $viewName = '', $layoutName = '', $critical = true, $autoPrepView = true, $force_front_metadata = 0)
    {
        $this->_prepView = $autoPrepView;
        $this->initObj($moduleName . 'Metadata');
        $this->module = $moduleName ?: $this->get('module');
        $this->viewName = $viewName ?: $this->get('view');
        $this->layoutName = $layoutName ?: $this->get('layout', 'default', true);
        $this->force_front_metadata = $force_front_metadata;
        if (defined("_ADMIN_MODE")) {
            $this->_admin_sub_folder = $this->force_front_metadata ? "" : "admin" . DS;
        }
        $this->defaultButtons();
        $this->defaultConstants();
        $this->read($critical);
        if ($this->_prepView) $this->prepareView();
    }

    private function prepareView()
    {
        // вызов простановки из базы
        $this->updateViewFromTable();
    }

    private function defaultButtons()
    {
        $this->buttons["go_up"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["new"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["clone"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["new_string"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["modify"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["info"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["filter"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["refresh"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["reorder"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["print"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["datagramm"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["modify_links"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["delete"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["undelete"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["trash"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["list"] = array("module" => '', "show" => 1, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
        $this->buttons["clean_trash"] = array("module" => '', "show" => 0, "view" => "", "task" => "", "link" => false, "layout" => '', "controller" => '', "title" => '');
    }

    private function defaultConstants()
    {
        $this->constants["AUTHOR"] = User::getInstance()->getId();
        $this->constants["AUTHOR_MAIL"] = User::getInstance()->getEmail();
        $this->constants["AUTHOR_NAME"] = User::getInstance()->getNickname();
        $this->constants["AUTHOR_IP"] = User::getInstance()->getIP();
        $this->constants["NOW"] = date("d.m.Y H:i:s");
        $this->constants["TODAY"] = date("d.m.Y");
    }

    public function getDefaultConstant($name)
    {
        if (array_key_exists($name, $this->constants)) return $this->constants[$name];
        return null;
    }

    public function read($critical)
    {
        $_meta = $this->viewName . "." . ($this->layoutName == 'default' ? "defl" : $this->layoutName);
        $metaPath = PATH_MODULES . $this->module . DS . 'metadata' . DS . $this->_admin_sub_folder . $_meta . '.php';
        $this->milestone('Meta: &quot;' . $_meta . '&quot; Metadata file: ' . $metaPath, __FUNCTION__);
        if (file_exists($metaPath)) {
            $_curr_md = null;
            require $metaPath;
            if (!is_null($_curr_md)) {
                if (isset($_curr_md['header']['tablename'])) $this->tablename = $_curr_md['header']['tablename'];
                if (isset($_curr_md['header']['parent_table'])) $this->parent_table = $_curr_md['header']['parent_table'];
                if (isset($_curr_md['header']['parent_code'])) $this->parent_code = $_curr_md['header']['parent_code'];
                if (isset($_curr_md['header']['parent_subordination'])) $this->parent_subordination = $_curr_md['header']['parent_subordination'];
                if (isset($_curr_md['header']['parent_name'])) $this->parent_name = $_curr_md['header']['parent_name'];
                if (isset($_curr_md['header']['parent_title'])) $this->parent_title = $_curr_md['header']['parent_title'];
                if (isset($_curr_md['header']['parent_view'])) {
                    $parent_meta_name = preg_split("/[.]/", $_curr_md['header']['parent_view']);
                    if (count($parent_meta_name) == 2) { // указана мета вместе с layout пробуем загрузить метадату родителя
                        $this->parent_view = $parent_meta_name[0];
                        $this->parent_layout = $parent_meta_name[1];
                    } else {// указана без layout, грузим дефолтную !!!
                        $this->parent_view = $_curr_md['header']['parent_view'];
                        $this->parent_layout = "default";
                    }
                }
                if (isset($_curr_md['header']['title'])) $this->title = $_curr_md['header']['title'];
                if (isset($_curr_md['header']['classname'])) $this->classname = $_curr_md['header']['classname'];
                if (isset($_curr_md['header']['classTable'])) $this->classTable = $_curr_md['header']['classTable'];
                if (isset($_curr_md['header']['keystring'])) $this->keystring = $_curr_md['header']['keystring'];
                if (isset($_curr_md['header']['nofilter'])) $this->nofilter = $_curr_md['header']['nofilter'];
                if (isset($_curr_md['header']['show_nopages'])) $this->show_nopages = $_curr_md['header']['show_nopages'];
                if (isset($_curr_md['header']['show_woparents'])) $this->show_woparents = $_curr_md['header']['show_woparents'];
                if (isset($_curr_md['header']['dopkeyid'])) $this->dopkeyid = $_curr_md['header']['dopkeyid'];
                if (isset($_curr_md['header']['enabled'])) $this->enabled = $_curr_md['header']['enabled'];
                if (isset($_curr_md['header']['deleted'])) $this->deleted = $_curr_md['header']['deleted'];
                if (isset($_curr_md['header']['templates'])) $this->templates = $_curr_md['header']['templates'];
                if (isset($_curr_md['header']['multy_field'])) $this->multy_field = $_curr_md['header']['multy_field'];
                if (isset($_curr_md['header']['keysort'])) $this->keysort = $_curr_md['header']['keysort'];
                if (isset($_curr_md['header']['keysort_dest'])) $this->keysort_dest = $_curr_md['header']['keysort_dest'];
                if (isset($_curr_md['header']['keycurrency'])) $this->keycurrency = $_curr_md['header']['keycurrency'];
                if (isset($_curr_md['header']['ordering_field'])) $this->ordering_field = $_curr_md['header']['ordering_field'];
                if (isset($_curr_md['header']['checkbox'])) $this->checkbox = $_curr_md['header']['checkbox'];
                if (isset($_curr_md['header']['selector'])) $this->selector = $_curr_md['header']['selector'];
                if (isset($_curr_md['header']['custom_sql'])) $this->custom_sql = $_curr_md['header']['custom_sql'];
                if (isset($_curr_md['header']['pre_sorting'])) $this->pre_sorting = $_curr_md['header']['pre_sorting'];
                if (isset($_curr_md['header']['post_sorting'])) $this->post_sorting = $_curr_md['header']['post_sorting'];
                if (isset($_curr_md['header']['pans_titles'])) $this->pans_titles = $_curr_md['header']['pans_titles'];
                $this->namestring = $_curr_md['header']['namestring'] ?? $this->keystring;
                $this->selector_string = $_curr_md['header']['selector_string'] ?? $this->namestring;
                if (isset($_curr_md['header']['alias_field'])) $this->alias_field = $_curr_md['header']['alias_field'];
                if (isset($_curr_md['header']['use_view_rights']) && $_curr_md['header']['use_view_rights']) $this->use_view_rights = $_curr_md['header']['use_view_rights'];
                $_curr_md["fields"] = $this->appendBaseAddFields($this->tablename, $_curr_md["fields"]);
                // значение линковочной таблицы имеет смысл только тогда когда выполняется ряд условий
                if ($this->multy_field == $this->keystring) {
                    if (($this->parent_table) && ($this->parent_code)) {
                        $this->linktable = $_curr_md['header']['linktable'] ?? $this->tablename . "_links";
                        if (isset($_curr_md['header']['link_with_childs'])) $this->link_with_childs = $_curr_md['header']['link_with_childs'];
                    }
                }
                foreach ($this->buttons as $b_key => $b_vals) {
                    $this->buttons[$b_key]['module'] = $_curr_md['buttons'][$b_key]['module'] ?? $this->module;
                    if (isset($_curr_md['buttons'][$b_key]['show'])) $this->buttons[$b_key]['show'] = $_curr_md['buttons'][$b_key]['show'];
                    if (!$this->buttons[$b_key]['show']) continue;
                    // if (isset($_curr_md['buttons'][$b_key]['view'])) $this->buttons[$b_key]['view'] = $_curr_md['buttons'][$b_key]['view'];
                    $this->buttons[$b_key]['view'] = $_curr_md['buttons'][$b_key]['view'] ?? $this->viewName;
                    if (isset($_curr_md['buttons'][$b_key]['task'])) $this->buttons[$b_key]['task'] = $_curr_md['buttons'][$b_key]['task'];
                    $this->buttons[$b_key]['layout'] = $_curr_md['buttons'][$b_key]['layout'] ?? $this->layoutName;
                    if (isset($_curr_md['buttons'][$b_key]['link'])) $this->buttons[$b_key]['link'] = $_curr_md['buttons'][$b_key]['link'];
                    if (isset($_curr_md['buttons'][$b_key]['controller'])) $this->buttons[$b_key]['controller'] = $_curr_md['buttons'][$b_key]['controller'];
                    if (isset($_curr_md['buttons'][$b_key]['title'])) $this->buttons[$b_key]['title'] = $_curr_md['buttons'][$b_key]['title'];
                }
                if (isset($_curr_md['uni_buttons'])) $this->uni_buttons = $_curr_md['uni_buttons'];
                // Тут можно обработать метадату, например добавить поля
                Event::raise('system.metadata.prepare_fields', array("metadata" => $this, "metadata_module" => $this->module, "metadata_file" => $_meta, "metadata_side" => ((defined("_ADMIN_MODE") && $this->force_front_metadata) || !defined("_ADMIN_MODE")) ? 1 : 0), $_curr_md);
                if (isset($_curr_md['fields']['field'])) $this->field = $_curr_md['fields']['field'];
                if (count($this->field) > 0) {
                    foreach ($this->field as $key => $data) {
                        $this->filter[$key] = $_curr_md['fields']['filter'][$key] ?? (defined("_ADMIN_MODE") && !$this->force_front_metadata ? 1 : 0);
                        $this->filter_ext[$key] = $_curr_md['fields']['filter_ext'][$key] ?? (defined("_ADMIN_MODE") && !$this->force_front_metadata ? 1 : 0);
                        $this->filter_strict[$key] = $_curr_md['fields']['filter_strict'][$key] ?? 0;
                        $this->filter_ch_list[$key] = isset($_curr_md['fields']['filter_ch_list'][$key]) && $_curr_md['fields']['filter_ch_list'][$key] ? 1 : 0;
                        $this->field_orderby[$key] = $_curr_md['fields']['sort_order'][$key] ?? ""; // different
                        $this->field_is_method[$key] = $_curr_md['fields']['fim'][$key] ?? false; // different
                        if ($this->field_is_method[$key]) $this->field_orderby[$key] = "NONE";
                        $this->field_on_change[$key] = $_curr_md['fields']['field_on_change'][$key] ?? "";
                        $this->field_no_update[$key] = isset($_curr_md['fields']['update_type'][$key]) && $_curr_md['fields']['update_type'][$key] == 0 ? 1 : 0; // different
                        $this->field_title[$key] = $_curr_md['fields']['name'][$key] ?? ""; // different
                        $this->field_descr[$key] = $_curr_md['fields']['descr'][$key] ?? ""; // different
                        // для совместимости используем str_replace
                        $this->size[$key] = isset($_curr_md['fields']['size'][$key]) ? str_replace('"', "", $_curr_md['fields']['size'][$key]) : "";
                        $this->view[$key] = $_curr_md['fields']['view'][$key] ?? 0;
                        $this->link[$key] = $_curr_md['fields']['link'][$key] ?? "";
                        $this->is_add[$key] = $_curr_md['fields']['is_add'][$key] ?? false;
                        $this->is_add_custom[$key] = $_curr_md['fields']['is_add_custom'][$key] ?? false;
                        // для совместимости используем str_replace
                        $this->link_vars[$key] = isset($_curr_md['fields']['link_key'][$key]) ? str_replace("$", "", $_curr_md['fields']['link_key'][$key]) : ""; // different
                        $this->link_types[$key] = $_curr_md['fields']['link_type'][$key] ?? ""; // different
                        $this->link_titles[$key] = $_curr_md['fields']['link_title'][$key] ?? ""; // different
                        $this->link_picture[$key] = $_curr_md['fields']['link_picture'][$key] ?? "";
                        $this->img_module[$key] = $_curr_md['fields']['img_module'][$key] ?? "";
                        $this->val_type[$key] = $_curr_md['fields']['val_type'][$key] ?? "";
                        $this->val_size[$key] = $_curr_md['fields']['val_size'][$key] ?? ($this->_prepView ? -1 : 0);
                        if (isset($_curr_md['fields']['val_const'][$key])) $this->val_type[$key] = "constanta";
                        $this->input_view[$key] = $_curr_md['fields']['input_view'][$key] ?? (defined("_ADMIN_MODE") && !$this->force_front_metadata ? 1 : 0);
                        // для совместимости используем str_replace
                        $this->input_size[$key] = isset($_curr_md['fields']['input_size'][$key]) ? str_replace('"', "", $_curr_md['fields']['input_size'][$key]) : "";
                        $this->input_type[$key] = $_curr_md['fields']['input_type'][$key] ?? "";
                        $this->input_data[$key] = isset($_curr_md['fields']['input_data'][$key]) && is_array($_curr_md['fields']['input_data'][$key]) ? $_curr_md['fields']['input_data'][$key] : array();
                        $this->input_page[$key] = isset($_curr_md['fields']['input_page'][$key]) ? intval($_curr_md['fields']['input_page'][$key]) : 1;
                        if ($this->input_page[$key] > $this->input_last_page) $this->input_last_page = $this->input_page[$key];
                        $this->field_order[$key] = $_curr_md['fields']['field_order'][$key] ?? $key;
                        $this->upload_path[$key] = $_curr_md['fields']['upload_path'][$key] ?? "";
                        $this->check_value[$key] = $_curr_md['fields']['check_value'][$key] ?? "";
                        $this->translate_value[$key] = $_curr_md['fields']['translate_value'][$key] ?? false;
                        $this->default_value[$key] = $_curr_md['fields']['default_value'][$key] ?? "";
                        $this->ch_table[$key] = $_curr_md['fields']['ch_table'][$key] ?? "";
                        if ($this->ch_table[$key] == $this->tablename && $this->field[$key] == $this->multy_field) {
                            $this->tree_index = $key;
                        }
                        $this->ch_field[$key] = $_curr_md['fields']['ch_field'][$key] ?? "";
                        $this->ch_parent_field[$key] = $_curr_md['fields']['ch_parent_field'][$key] ?? "";
                        if(!$this->ch_parent_field[$key] && $this->ch_table[$key] == $this->tablename && $this->field[$key] != $this->multy_field) {
                            $this->ch_parent_field[$key] = $this->field[$key];
                        }
                        $this->ch_id[$key] = $_curr_md['fields']['ch_id'][$key] ?? "";
                        $this->ch_sort[$key] = $_curr_md['fields']['ch_sort'][$key] ?? "";
                        $this->ch_sp_field[$key] = $_curr_md['fields']['ch_sp_field'][$key] ?? "";
                        $this->ch_sp_field_val[$key] = $_curr_md['fields']['ch_sp_field_val'][$key] ?? "";
                        $this->ch_deleted[$key] = $_curr_md['fields']['ch_deleted'][$key] ?? "";
                        $this->ch_enabled[$key] = $_curr_md['fields']['ch_enabled'][$key] ?? "";
                        $this->ch_skip_deleted[$key] = $_curr_md['fields']['ch_skip_deleted'][$key] ?? 1;
                        $this->ch_skip_disabled[$key] = $_curr_md['fields']['ch_skip_disabled'][$key] ?? 1;
                        $this->ck_reestr[$key] = $_curr_md['fields']['ck_reestr'][$key] ?? "";
                    }
                    $this->success = true;
                    if ($this->tree_index) {
                        if (isset($_curr_md['header']['tree_skip_deleted'])) $this->tree_skip_deleted = $_curr_md['header']['tree_skip_deleted'];
                        if (isset($_curr_md['header']['tree_skip_disabled'])) $this->tree_skip_disabled = $_curr_md['header']['tree_skip_disabled'];
                    }
                }
                unset($_curr_md);
            }
        } else {
            if ($critical) $this->fatalError('Metadata file not found: ' . $metaPath); // Realy fatal, if critical flag true.
        }
    }

    /**
     * дополнием описатели таблиц динамической частью
     * @param string $table имя таблицы к кторой подбираем дополнительные поля
     * @param array $fields_arr предварительно собранный массив который будем наращивать
     */
    private function appendBaseAddFields($table, $fields)
    {
        $add_list = $this->getListAdditionalField($table);
        $add_list_values = $this->getAdditionalFieldsValues($table);
        $_tps = $this->getArrayFieldsType();
        if (is_array($add_list) && count($add_list) > 0) {
            $count = count($fields["field"]);
            $fields["is_add"] = array();
            $fields["is_add_custom"] = array();
            foreach ($add_list as $list) {
                $count++;
                $fields["field"][$count] = $list->f_name;
                $fields["name"][$count] = $list->f_descr;
                $fields["check_value"][$count] = $list->f_required;
                if (!isset($_tps[$list->f_type])) {
                    $fields["val_type"][$count] = 'string';
                    $fields["input_type"][$count] = 'text';
                } else {
                    $fields["val_type"][$count] = $_tps[$list->f_type]->t_val_type;
                    $fields["input_type"][$count] = $_tps[$list->f_type]->t_input_type;
                    if ($fields["input_type"][$count] == "select" || $fields["input_type"][$count] == "multiselect") {
                        if (isset($add_list_values[$list->f_name]) && count($add_list_values[$list->f_name])) {
                            $ck_array = array();
                            foreach ($add_list_values[$list->f_name] as $field_choice) {
                                $ck_array[$field_choice->fc_id] = $field_choice->fc_value;
                            }
                            $fields["ck_reestr"][$count] = $ck_array;
                        } else { // если массив значений пустой проверим может по имени поля у нас есть предустановленный список
                            $fields["ck_reestr"][$count] = SpravStatic::getCKArray($list->f_name);
                        }
                    }
                }
                $fields["is_add"][$count] = $list->f_id;
                $fields["default_value"][$count] = $list->f_default;
                $fields["is_add_custom"][$count] = $list->f_custom;
                $fields["input_view"][$count] = 0;
                $fields["view"][$count] = 0;
                if ($fields["input_type"][$count] == 'image' || $fields["input_type"][$count] == 'file') {
                    $fields["upload_path"][$count] = "_add/" . $this->viewName;
                }
            }
        }
        return $fields;
    }

    /**
     * Список дополнительных полей к таблице,
     * @param string $table имя таблицы к кторой подбираем дополнительные поля
     * return массив объектов с ключом по ид поля
     */
    public function getListAdditionalField($table = '')
    {
        $db = Database::getInstance();
        $query = "SELECT ff.* FROM #__fields_list AS ff WHERE ff.f_deleted=0 AND ff.f_table='" . $table . "' AND ff.f_custom<2";
        $db->setQuery($query);
        return $db->loadObjectList('f_name');
    }

    private function getAdditionalFieldsValues($table = '', $enabled_only = true)
    {
        $this->milestone('Metadata started loading additional fields values: ', __FUNCTION__);
        $result = array();
        $db = Database::getInstance();
        // $query="SELECT fc.* FROM `#__fields_choices` AS fc WHERE fc.fc_deleted=0 AND fc.fc_field_id IN (SELECT ff.f_id FROM #__fields_list AS ff WHERE ff.f_deleted=0 AND ff.f_table='".$table."' AND ff.f_custom<2)";
        $query = "SELECT fc.*, fl.f_name, fl.f_table FROM `#__fields_choices` AS fc, `#__fields_list` as fl";
        $query .= " WHERE fc.fc_field_id=fl.f_id AND fc.fc_deleted=0";
        if ($enabled_only) $query .= " AND fc.fc_enabled=1";
        $query .= " AND fl.f_deleted=0 AND fl.f_table='" . $table . "'";
        $query .= " ORDER BY fl.f_name, fc.fc_ordering";
        $db->setQuery($query);
        $choices = $db->loadObjectList();
        if (count($choices)) {
            foreach ($choices as $choice) {
                $result[$choice->f_name][$choice->fc_id] = $choice;
            }
        }
        $this->milestone('Metadata finished loading additional fields values: ', __FUNCTION__);
        return $result;
    }

    private function getArrayFieldsType()
    {
        $db = Database::getInstance();
        $sql = "SELECT * FROM #__fields_types";
        $db->setQuery($sql);
        return $db->loadObjectList("t_id");
    }

    private function updateViewFromTable()
    {
        $field = $this->field;
        $field_obr = array_flip($field);
        $db = Database::getInstance();
        if (defined('_ADMIN_MODE')) {
            if ($this->force_front_metadata) $m_admin_side = 0; else $m_admin_side = 1;
        } else {
            $m_admin_side = 0;
        }
        $query = " SELECT * FROM #__metadata WHERE `m_module`='" . $this->module . "' AND `m_view`='" . $this->viewName . "' AND `m_layout`='" . ($this->layoutName == "default" ? "defl" : $this->layoutName) . "' AND `m_admin_side`=" . $m_admin_side;
        $db->setQuery($query);
        $cur = $db->loadAssocList();
        if (is_null($cur)) return false;
        foreach ($cur as $fld_dop) {
            $f_key = $fld_dop["m_field"];
            if (isset($field_obr[$f_key])) {
                $key = $field_obr[$f_key];
                $this->size[$key] = $fld_dop["m_width"];
                $this->view[$key] = $fld_dop["m_show"];
                $this->input_size[$key] = $fld_dop["m_input_size"];
                $this->input_view[$key] = $fld_dop["m_input_view"];
                $this->input_page[$key] = $fld_dop["m_input_page"];
                if ($this->input_page[$key] > $this->input_last_page) $this->input_last_page = $this->input_page[$key];
                if ($this->val_size[$key] < 0) $this->val_size[$key] = intval($this->input_size[$key]);
                $this->filter[$key] = $fld_dop["m_show_in_filter"];
                $this->filter_ext[$key] = $fld_dop["m_show_in_filter_ext"];
                $this->filter_strict[$key] = $fld_dop["m_strict_filter"];
                if (defined("_ARFACMS_TRANSLATE")) {
                    $this->translate_value[$key] = $fld_dop["m_translate_value"];
                }
                if ($fld_dop["m_field_order"]) $this->field_order[$key] = $fld_dop["m_field_order"];
                else $this->field_order[$key] = $this->field_order[$key] > 0 ? $this->field_order[$key] : $key;
            }
        }
        $this->reorderFields();
        return true;
    }

    public function reorderFields()
    {
        Debugger::getInstance()->milestone("Start Metadata sorting");

        $array_for_sort = array(
            "field", "filter", "filter_ext", "filter_strict", "filter_ch_list",
            "field_orderby", "field_title", "field_descr", "field_on_change", "field_is_method",
            "size", "view", "link", "link_vars", "link_types",
            "link_titles", "link_picture", "img_module", "upload_path", "val_type",
            "val_size", "input_size", "input_type", "input_view", "input_page",
            "field_no_update", "check_value", "translate_value", "default_value", "ck_reestr",
            "ch_table", "ch_field", "ch_parent_field", "ch_id", "ch_deleted",
            "ch_enabled", "ch_skip_deleted", "ch_skip_disabled", "ch_sort", "ch_sp_field",
            "ch_sp_field_val", "is_add", "is_add_custom", "field_order", "input_data"
        );

        // формируем порядковые номера
        $counter = 1;
        asort($this->field_order);
        foreach ($this->field_order as $fok => $fov) {
            $this->field_order[$fok] = $counter;
            $counter++;
        }
        // меняем индекс поля дерева
        if ($this->tree_index) {
            $this->tree_index = $this->field_order[$this->tree_index];
        }
        // пересортировываем остальные элементы
        $new_order = $this->field_order;
        foreach ($array_for_sort as $array_name) {
            $new_array = array();
            foreach ($this->{$array_name} as $key => $val) {
                $new_array[$new_order[$key]] = $val;
            }
            ksort($new_array);
            $this->{$array_name} = $new_array;
        }
        Debugger::getInstance()->milestone("Finish Metadata sorting");
    }

    public function getFieldIndex($fieldname)
    {
        if (count($this->field) > 0) {
            foreach ($this->field as $key => $val) {
                if ($val == $fieldname) return $key;
            }
        }
        return 0;
    }

    public function updateArrayField($arrayname, $fieldname, $value)
    {
        $key = $this->getFieldIndex($fieldname);
        if ($key && isset($this->{$arrayname}) && array_key_exists($key, $this->{$arrayname})) {
            $this->{$arrayname}[$key] = $value;
            return true;
        } else return false;
    }

    public function cleanAddFields($fields)
    {
        if ($this->success && count($this->field)) {
            foreach ($this->field as $key => $data) {
                if ($this->is_add_custom[$key] && !array_key_exists($this->field[$key], $fields)) {
                    unset($this->field[$key]);
                    unset($this->filter[$key]);
                    unset($this->filter_ext[$key]);
                    unset($this->filter_strict[$key]);
                    unset($this->filter_ch_list[$key]);
                    unset($this->field_orderby[$key]);
                    unset($this->field_is_method[$key]);
                    unset($this->field_on_change[$key]);
                    unset($this->field_no_update[$key]);
                    unset($this->field_title[$key]);
                    unset($this->field_descr[$key]);
                    unset($this->size[$key]);
                    unset($this->view[$key]);
                    unset($this->link[$key]);
                    unset($this->is_add[$key]);
                    unset($this->is_add_custom[$key]);
                    unset($this->link_vars[$key]);
                    unset($this->link_types[$key]);
                    unset($this->link_picture[$key]);
                    unset($this->img_module[$key]);
                    unset($this->val_type[$key]);
                    unset($this->val_size[$key]);
                    unset($this->val_type[$key]);
                    unset($this->input_view[$key]);
                    unset($this->input_size[$key]);
                    unset($this->input_type[$key]);
                    unset($this->input_page[$key]);
                    unset($this->field_order[$key]);
                    unset($this->upload_path[$key]);
                    unset($this->check_value[$key]);
                    unset($this->translate_value[$key]);
                    unset($this->default_value[$key]);
                    unset($this->ch_table[$key]);
                    unset($this->ch_field[$key]);
                    unset($this->ch_parent_field[$key]);
                    unset($this->ch_id[$key]);
                    unset($this->ch_sort[$key]);
                    unset($this->ch_sp_field[$key]);
                    unset($this->ch_sp_field_val[$key]);
                    unset($this->ch_deleted[$key]);
                    unset($this->ch_enabled[$key]);
                    unset($this->ch_skip_deleted[$key]);
                    unset($this->ch_skip_disabled[$key]);
                    unset($this->ck_reestr[$key]);
                }
            }
        }
    }

    /**
     * @param $short_format boolean
     * @param $side int Frontend side is 1, Admin side is 0, Both is null
     * @param $modules array
     * @param $_hiddenfiles array
     * @return array
     */
    public static function getFiles($short_format = false, $side = null, $modules = array(), $_hiddenfiles = null)
    {
        if (is_null($_hiddenfiles)) $_hiddenfiles = Files::getHiddenFilesTemplate();
        $result = array();
        $all_modules = Module::getInstalledModules();
        asort($all_modules);
        foreach ($all_modules as $module_name) {
            if (count($modules) && !in_array($module_name, $modules)) continue;
            // Front meta
            if (is_null($side) || $side == 1) {
                $meta_path = PATH_MODULES . $module_name . DS . 'metadata';
                $metafiles = Files::getFiles($meta_path, $_hiddenfiles, false);
                if ($metafiles && count($metafiles)) {
                    foreach ($metafiles as $metafile => $metainfo) {
                        if (is_file($meta_path . DS . $metafile)) {
                            if ($short_format) $result[] = $meta_path . DS . $metafile;
                            else $result[] = array("module" => $module_name, "path" => $meta_path, "file" => $metafile, "is_admin" => 0, "fullpath" => $meta_path . DS . $metafile);
                        }
                    }
                }
            }
            // Admin meta
            if (is_null($side) || $side == 0) {
                $meta_path = PATH_MODULES . $module_name . DS . 'metadata' . DS . "admin";
                $metafiles = Files::getFiles($meta_path, $_hiddenfiles, false);
                if ($metafiles && count($metafiles)) {
                    foreach ($metafiles as $metafile => $metainfo) {
                        if (is_file($meta_path . DS . $metafile)) {
                            if ($short_format) $result[] = $meta_path . DS . $metafile;
                            else $result[] = array("module" => $module_name, "path" => $meta_path, "file" => $metafile, "is_admin" => 1, "fullpath" => $meta_path . DS . $metafile);
                        }
                    }
                }
            }
        }
        return $result;
    }

    public static function getTableLinks($table = "")
    {
        $result = array("linktable" => array(), "parent_table" => array(), "ch_table" => array());
        if ($table) {
            $metafiles = self::getFiles();
            foreach ($metafiles as $key => $metafile) {
                $names = explode('.', $metafile['file']);
                $meta = null;
                if (count($names) == 3) {
                    $curr_view = $names[0];
                    $curr_layout = $names[1];
                    $meta = new SpravMetadata($metafile['module'], $curr_view, $curr_layout, true, false, !$metafile['is_admin']);
                    // Util::showCollapsedArray($meta);
                    if ($meta->linktable) {
                        // Util::showCollapsedArray($meta, $meta->linktable);
                        if ($table == $meta->tablename) {
                            if (!isset($result['linktable'][$meta->linktable])) {
                                $result['linktable'][$meta->linktable] = array();
                            }
                            $result['linktable'][$meta->linktable][] = $metafile['fullpath'];
                        } elseif ($table == $meta->parent_table) {
                            if (!isset($result['parent_table'])) {
                                $result['parent_table'] = array();
                            }
                            $result['parent_table'][] = $metafile['fullpath'];
                        }
                    }
                    foreach ($meta->ch_table as $key => $ch_table) {
                        if ($table == $ch_table) {
                            if (!isset($result['ch_table'])) {
                                $result['ch_table'] = array();
                            }
                            $result['ch_table'][] = $metafile['fullpath'];
                        }
                    }
                } else {
                    Util::halt('Incorrect filename: ' . $metafile['file']); // @FIXME FOR DELETE
                }
            }
        }
        return $result;
    }
}

?>