<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: b595526d (2024-10-06 16:41:35)
// 

namespace ArfaCMS\Tools;

use ArfaCMS\Core\BaseObject;
use ArfaCMS\Core\Files;
use ArfaCMS\Core\Text;
use ArfaCMS\DB\Database;

defined('_ARFACMS_VALID') or die("Access denied");

class Restructure extends BaseObject
{
    protected $sql_arr = array();
    protected $folders_for_move = array();
    protected $folders4delete = array();
    protected $files4delete = array();

    public function __construct()
    {
        $this->sql_arr = array();
        $this->folders_for_move = array();
        $this->folders4delete = array();
        $this->files4delete = array();
        $this->initObj();
        $this->prepare();
    }

    protected function addSQLQuery($sql)
    {
        $this->sql_arr[] = $sql;
    }

    protected function addFolderForMove($folder)
    {
        $this->folders_for_move[] = $folder;
    }

    protected function addFolder4Delete($folder)
    {
        $this->folders4delete[] = $folder;
    }

    protected function addFile4Delete($file)
    {
        $this->files4delete[] = $file;
    }

    protected function prepare()
    {
        // Just for override
    }

    public function proceed(&$messages, &$errors)
    {
        // =================================================
        // Let's apply queries
        // =================================================
        foreach ($this->sql_arr as $k => $sql) {
            Database::getInstance()->setQuery($sql . Database::getInstance()->getDelimiter());
            if (!Database::getInstance()->query()) {
                // Returning error if failed
                $errors[] = Text::_("Error applying restructure") . ": " . __CLASS__ . " [SQL-query]";
                return false;
            }
        }

        // =================================================
        // Let's move folders
        // =================================================
        foreach ($this->folders_for_move as $k => $folder) {
            if (is_dir($folder["src"]) && $folder["dest"]) {
                if (!Files::copyFolder($folder["src"], $folder["dest"], true)) {
                    $errors[] = Text::_("Error applying restructure") . ": " . __CLASS__ . " [Copy folders]";
                    return false;
                } else {
                    $this->addFolder4Delete($folder["src"]);
                }
            }
        }

        // =================================================
        // Let's delete folders
        // =================================================
        foreach ($this->folders4delete as $k => $folder) {
            if (is_dir($folder) && !Files::removeFolder($folder, 1)) {
                // Returning error if failed
                $errors[] = Text::_("Error applying restructure") . ": " . __CLASS__ . " [Delete folders]";
                return false;
            }
        }

        // =================================================
        // Let's delete files
        // =================================================
        foreach ($this->files4delete as $k => $file) {
            if (is_file($file) && !Files::delete($file, 1)) {
                // Returning error if failed
                $errors[] = Text::_("Error applying restructure") . ": " . __CLASS__ . " [Delete files]";
                return false;
            }
        }

        // =================================================
        // Returning message if successed
        // =================================================
        $messages[] = $this->getName();
        return true;
    }
}

?>