<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 60c8db9c (2024-10-20 00:48:29)
// 

use ArfaCMS\Core\Plugin;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\User;
use ArfaCMS\DB\Database;

defined('_ARFACMS_VALID') or die("Access denied");

class contentPluginrating extends Plugin {

	private $_vote_types=array("comment"=>1, "article.object"=>2, "blog.object"=>3, "user.object"=>4);	
	protected $_events=array("rating.check", "rating.vote","rating.rendervotepanel");	
	
	protected function setParamsMask(){
		parent::setParamsMask();
		$this->addParam("vote_article_cost", "integer", 1);
		$this->addParam("vote_blog_cost", "integer", 1);
		$this->addParam("vote_user_cost", "integer", 1);
		$this->addParam("vote_comment_cost", "integer", 1);
	}
	protected function onRaise($event, &$data) {
		$result = "";
		$psid = $this->getParam("psid", false);
		if (! $psid) return $result;
		$module = $this->getParam("module", false);
		if (! $module) return $result;
		$view = $this->getParam("view", false);
		$element = $this->getParam("element", false);
		$znak = ($this->getParam("direction", false) == "up" ? "+" : "-" );
		switch ($event) {
			case "rating.rendervotepanel" :
				$mess = $this->getParam("mess", false, Text::_ ( "Vote this" ));
				$result = $this->renderVotePanel( $module, $view, $element, $psid, $mess );
				break;
			case "rating.check" :
				$result = $this->checkVoted($module, $element, $psid);
				break;
			case "rating.vote" :
				$author = $this->getParam("author", false, 0);
				$uid = User::getInstance ()->getID ();
				// if ($author!=$uid)
				$result = $this->updateObjectRating ( $module, $view, $element, $psid, $author, $znak );
				break;
			default :
				break;
		}
		return $result;
	}
	public function checkVoted($module,$element,$psid){
		if ($element=="object"){
			$place=$module.".object";
		} elseif ($element=="comment") {
			$place=$element;
		} else return true;
		if (!array_key_exists($place, $this->_vote_types)) return false;
		$sql="SELECT COUNT(v_uid) FROM #__votes WHERE v_uid=".User::getInstance()->getID()." AND v_type=".$this->_vote_types[$place]." AND v_eid=".$psid;
		Database::getInstance()->setQuery($sql); 
		return intval(Database::getInstance()->LoadResult());
	}
	// Эта функция меняет рейтинг пользователя (автора или самого пользователя, смотря как вызвать)
	private function updateUserRating($module, $view, $element,$uid, $znak){ 
		if (!$uid) return false;
		if ($element=="object"){ // это объект
			$delta=$this->getParam("vote_".$module."_cost");
		} elseif ($element=="comment"){ // это обычный комментарий
			$delta=$this->getParam("vote_comment_cost");
		} else return false;
		if (!$delta) return false;
		$sql="UPDATE #__users SET u_rating=u_rating".$znak.$delta." WHERE u_id=".$uid;
		Database::getInstance()->setQuery($sql); 
		return Database::getInstance()->query();
	}
	private function updateObjectRating($module, $view, $element, $psid, $uid, $znak){ 
		if (!$this->updateUserRating($module, $view, $element,$uid, $znak)) return false;
		if ($element=="object"){ // это объект
			$delta=$this->getParam("vote_".$module."_cost");
			if (!$delta) return false;
			$place=$module.".object";
			if ($module=="article"){
				$sql="UPDATE #__articles SET a_rating=a_rating".$znak.$delta." WHERE a_id=".$psid;
			}	elseif ($module=="blog"){
				$sql="UPDATE #__blogs_posts SET p_rating=p_rating".$znak.$delta." WHERE p_id=".$psid;
			} else return false;
		} elseif ($element=="comment"){ // это обычный комментарий
			$place=$element;
			$delta=$this->getParam("vote_comment_cost");
			if (!$delta) return false;
			//@FIXME Now only base tables
			$sql="UPDATE #__comms SET cm_rating=cm_rating".$znak.$delta." WHERE cm_id=".$psid." AND cm_uid=".$uid;
		} else return false;
		Database::getInstance()->setQuery($sql); 
		if (!Database::getInstance()->query()) return false;
		$votesql="INSERT INTO #__votes VALUES (".User::getInstance()->getID().",".$this->_vote_types[$place].",".$psid.")";
		Database::getInstance()->setQuery($votesql);
		return Database::getInstance()->query();
	}
	private function renderVotePanel($module, $view, $element, $psid, $mess){
		$html = "<div class=\"vote-panel\">";
		$html.= "<a class=\"vote-down\" onclick=\"voteRating(this,'".$module."','".$view."','".$element."','".$psid."','down');\"><img alt=\"\" title=\"".Text::_("Bad")."\" width=\"1\" height=\"1\" src=\"/assets/images/blank.gif\" /></a>"; // This blank image is OK !!!
		$html .= "<span>".$mess."</span>";
		$html.= "<a class=\"vote-up\" onclick=\"voteRating(this,'".$module."','".$view."','".$element."','".$psid."','up');\"><img alt=\"\" title=\"".Text::_("Good")."\" width=\"1\" height=\"1\" src=\"/assets/images/blank.gif\" /></a>"; // This blank image is OK !!!
		$html.= "</div>";
		return $html;
	}
}
?>