<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: ec1e92af (2023-09-25 00:45:08)
// 

use ArfaCMS\Core\Debugger;
use ArfaCMS\Core\Menus;
use ArfaCMS\Core\Plugin;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Util;

defined('_ARFACMS_VALID') or die("Access denied");
class systemPluginhtaccess extends Plugin {
	protected $_events = array("system.startingPortalBefore");
	protected function setParamsMask(){
		parent::setParamsMask();
        $this->addParam("using_plugin", "ro_string", Text::_("htaccess_plugin_description"));
        $this->addParam("correct_www_prefix", "boolean", 0, false);
        $this->addParam("correct_protocol_to", "select", "", true, array("http"=>"HTTP", "https"=>"HTTPS"));
        $this->addParam("trim_index_php", "boolean", 0, false);
        $this->addParam("trim_trailing question", "boolean", 0, false);
    }
	protected function onRaise($event, &$data) {
        if(defined("_ADMIN_MODE")) return;
		if(!\seoConfig::$sefMode) return;
		switch($event){
			case "system.startingPortalBefore":
                $server_name = Request::getSafe("SERVER_NAME", "", "server");
                $_request_uri = $_SERVER['REQUEST_URI'];
                $_new_request_uri = $this->processRequestUri($_request_uri);
                $correct_www_prefix = $this->getParam("correct_www_prefix");
                $correct_protocol_to = $this->getParam("correct_protocol_to");
                $current_protocol = Util::getCurrentProtocol();
                $protocol = $correct_protocol_to ? : $current_protocol;
                if($protocol == 'https'){
                    $port = \siteConfig::$siteSSLPort && \siteConfig::$siteSSLPort != 443 ? ":" . \siteConfig::$siteSSLPort : "";
                } else {
                    $port = \siteConfig::$sitePort && \siteConfig::$sitePort != 80 ? ":" . \siteConfig::$sitePort : "";
                }
                // Util::pre($_request_uri);
                // ---------------------------------------------------------------------------------------
                // Correct www prefix
                // ---------------------------------------------------------------------------------------
                if($correct_www_prefix && $server_name != \siteConfig::$siteDomain){
                    $pageURL = $protocol."://";
                    $pageURL.= \siteConfig::$siteDomain;
                    $pageURL.= $port;
                    $pageURL.= $_new_request_uri;
                    // Util::pre($pageURL); Util::halt();
                    // while (ob_get_level()) ob_end_clean();
                    // header('Location: '. $pageURL,true, 0);
                    Util::redirect($pageURL);
                }
                // ---------------------------------------------------------------------------------------
                // Correct protocol
                // ---------------------------------------------------------------------------------------
                if($correct_protocol_to && $current_protocol != $correct_protocol_to){
                    $pageURL = $protocol."://";
                    $pageURL.= $server_name;
                    $pageURL.= $port;
                    $pageURL.= $_new_request_uri;
                    // Util::pre($pageURL);
                    Util::redirect($pageURL);
                }
                // ---------------------------------------------------------------------------------------
                // Correct request_uri
                // ---------------------------------------------------------------------------------------
                if($_request_uri != $_new_request_uri && Request::getMethod() == 'GET'){
                    $pageURL = $protocol."://";
                    $pageURL.= $server_name;
                    $pageURL.= $port;
                    $pageURL.= $_new_request_uri;
                    // Util::pre($pageURL);
                    Util::redirect($pageURL);
                }
                // ---------------------------------------------------------------------------------------
                $request = preg_split('/[\?]/', $_new_request_uri);
                // ---------------------------------------------------------------------------------------
                // Search form
                // ---------------------------------------------------------------------------------------
                $matches = array();
                preg_match('/^\/(search).html$/', $request[0], $matches);
                if(count($matches)==2){
                    // Router::getInstance()->setVarsVal("task", Request::makeCast($matches[1]));
                    $_REQUEST["task"] = Request::makeCast($matches[1]);
                    // Util::pre($matches);
                    $this->initActiveMenuItem();
                    return;
                }
                // ---------------------------------------------------------------------------------------
                // Search link
                // ---------------------------------------------------------------------------------------
                $matches = array();
                preg_match('/^\/(search)\/(any|exact|all)\/(.*).html$/', $request[0], $matches);
                if(count($matches)==4){
                    // Router::getInstance()->setVarsVal("task", Request::makeCast($matches[1]));
                    $_REQUEST["task"] = Request::makeCast($matches[1]);
                    // Router::getInstance()->setVarsVal("searchtype", Request::makeCast($matches[2]));
                    $_REQUEST["searchtype"] = Request::makeCast($matches[2]);
                    // Router::getInstance()->setVarsVal("where_search", Request::makeCast($matches[3]));
                    $_REQUEST["where_search"] = Request::makeCast($matches[3]);
                    // Util::pre($matches);
                    $this->initActiveMenuItem();
                    return;
                }
                // ---------------------------------------------------------------------------------------
                // Referrals
                // ---------------------------------------------------------------------------------------
                if (preg_match("/ref_(.*)\.html/", $request[0], $matches)) {
                    $_REQUEST["referral"] = Request::makeCast($matches[1]);
                    // Util::pre($_REQUEST);
                    // $this->initActiveMenuItem();
                    return;
                }
                // ---------------------------------------------------------------------------------------
                // Images
                // ---------------------------------------------------------------------------------------
                if (preg_match("/.*\.(ico|gif|png|jpg|jpeg)$/", $request[0], $matches)) {
                    while (ob_get_level()) ob_end_clean();
                    Util::sendHeader(404);
                    // Util::pre($matches);
                    Util::halt();
                    return;
                }
			break;
		}
	}
    private function initActiveMenuItem()
    {
        if (!defined('_ADMIN_MODE')) {
            $mid = Request::getInt("mid", 0);
            if ($mid) $_SESSION['active_menu_id'] = $mid;
            else {
                if (Request::getSafe('task') == "search" && \siteConfig::$searchMenuID) {
                    $_SESSION['active_menu_id'] = \siteConfig::$searchMenuID;
                } elseif (\seoConfig::$activeMidFromTable) {
                    if (Request::getSafe('option') != "ajax") {
                        $mid = Menus::searchSuitableMID();
                        if ($mid) $_SESSION['active_menu_id'] = $mid;
                    }
                } else $_SESSION['active_menu_id'] = \siteConfig::$defaultMenuID;
            }
            if ((!isset($_SESSION['active_menu_id']) || (!$_SESSION['active_menu_id']))) {
                $_SESSION['active_menu_id'] = intval(\siteConfig::$defaultMenuID);
            }
            Debugger::getInstance()->marker(__CLASS__."->".__FUNCTION__." : found ".$_SESSION['active_menu_id']);
        }
    }
    private function processRequestUri($_request_uri=""){
        if(!$_request_uri) return $_request_uri;
        if($_request_uri == "/") return $_request_uri;
        // Util::pre($_request_uri);
        if($this->getParam("trim_trailing question")){
            $_request_uri = trim($_request_uri, "?");
        }
        // Util::pre($_request_uri);
        if($this->getParam("trim_index_php")){
            if($_request_uri == "index.php") $_request_uri = "";
            if($_request_uri == "/index.php") $_request_uri = "/";
        }
        // Util::pre($_request_uri);
        return $_request_uri;
    }
}
?>