<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 89853b30 (2024-10-01 18:28:15)
// 

use ArfaCMS\Core\HTMLControls;
use ArfaCMS\Core\Plugin;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Session;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Util;
use Extra\KCAPTCHA\KCAPTCHA;

defined('_ARFACMS_VALID') or die("Access denied");

class systemPluginkcaptcha extends Plugin {
	protected $_events=array("captcha.renderPicture","captcha.renderForm","captcha.checkResult");	
	protected function setParamsMask(){
		parent::setParamsMask();
		$this->addParam("length", "integer", 7, true);
		$this->addParam("height", "integer", 60, true);
		$this->addParam("width", "integer", 120, true);
        $this->addParam("timeout", "integer", 20, true);
	}
	protected function onRaise($event, &$data) {
        Session::getInstance();
        switch($event){
			case "captcha.renderPicture":
				$params=array();
				if(($this->getParam("length")) && ($this->getParam("length")>2) && ($this->getParam("length")<8)) $params["length"]=$this->getParam("length");
				if(($this->getParam("height")) && ($this->getParam("height")>0)) $params["height"]=$this->getParam("height");
				if(($this->getParam("width")) && ($this->getParam("width")>0)) $params["width"]=$this->getParam("width");
				$captcha = new KCAPTCHA($params);
                $this->initSessionVars(Request::getSafe('captcha_id'), $captcha->getKeyString());
			break;
			case "captcha.renderForm":
                $captcha_id = Util::generateRandomString(25, '', 1);
                $html = "<div class=\"captcha\"><div class=\"row\">";
                $html.= "<div class=\"col-auto align-self-center\">";
                $html.= "<img src=\"".Portal::getInstance()->getURI()."index.php?option=captcha&amp;captcha_id=".$captcha_id."\" alt=\"\" />";
                $html.= "<input type=\"hidden\" name=\"captcha_id\" value=\"".$captcha_id."\" />";
                $html.= "</div>";
                $html.= "<div class=\"col align-self-center\">";
                $html.= "    <div class=\"row\">";
                $html.= "        <div class=\"captcha-label col-sm-12 col-lg-auto mb-2 mb-lg-0 align-self-center\">".Text::_("Enter code")."</div>";
                $html.= "        <div class=\"captcha-input col-sm-12 col-md\">";
                $html.= HTMLControls::renderInputText("rCaptchaText", "", "", "", "rCaptchaText", "", false, true);
                $html.= "        </div>";
                $html.= "    </div>";
                $html.= "</div>";
                $html.= "</div></div>";
				return $html;
			break;
			case "captcha.checkResult":
				$captcha	= Request::getRealValue('rCaptchaText',''); // Safety check not need
                $captcha_id = Request::getSafe('captcha_id');
                $captchaValid = $this->getCodeFromSession($captcha_id);
                if ( $captchaValid && $captcha == $captchaValid) $msg="OK"; else $msg =Text::_("Wrong captcha code");
				$_SESSION['captcha_string']=$msg;
			break;
		}
	}
    private function getCodeFromSession($captcha_id=""){
        $captcha_vars = $this->initSessionVars($captcha_id);
        return isset($captcha_vars[$captcha_id]['key']) ? $captcha_vars[$captcha_id]['key'] : "";
    }
    private function initSessionVars($captcha_id="", $new_key = ""){
        $new_captcha_vars = array();
        if($captcha_id){
            $captcha_vars = Session::getVar('kcaptcha_vars');
            if(is_null($captcha_vars) || !is_array($captcha_vars)) {
                Session::setVar('kcaptcha_vars', array());
                $captcha_vars = Session::getVar('kcaptcha_vars');
            }
            foreach ($captcha_vars as $_key=>$_var){
                if(time() - $_var['time'] < $this->getParam("timeout")*60){
                    $new_captcha_vars[$_key] = $_var;
                }
            }
            if($new_key){
                $new_captcha_var = array('time'=>time(), 'key'=>$new_key);
                $new_captcha_vars[$captcha_id] = $new_captcha_var;
            }
            Session::setVar('kcaptcha_vars', $new_captcha_vars);
        }
        return $new_captcha_vars;
    }
}

?>