<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 836b2459 (2024-10-08 22:01:03)
// 

use ArfaCMS\Core\ACRM;
use ArfaCMS\Core\Files;
use ArfaCMS\Core\HTMLControls;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Widget;

defined('_ARFACMS_VALID') or die("Access denied");

class acrmWidget extends Widget
{
    protected $_requiredModules = array("acrm");

    protected function setParamsMask()
    {
        parent::setParamsMask();
        $this->addParam("Client_ID", "table_select", 0, false, "SELECT bcl_id AS fld_id, bcl_name AS fld_name FROM #__banners_clients ORDER BY fld_name");
        $this->addParam("Category_ID", "table_select", 0, false, "SELECT bc_id AS fld_id, bc_name AS fld_name FROM #__banners_categories ORDER BY fld_name");
        $this->addParam("Quantity", "integer", 1);
        $this->addParam("Item_CSS_class", "string", "", false, null, Text::_("For example").": itms-split-2, itms-split-3, itms-split-4, align-self-center ".Text::_("etc."));
        $this->addParam("Show_titles", "boolean", 0);
        $this->addParam("Show_descriptions", "boolean", 0);
        $this->addParam("Randomize", "boolean", 0);
    }

    public function render()
    {
        $widgetHTML = "";
        $client = intval($this->getParam('Client_ID'));
        $cat = intval($this->getParam('Category_ID'));
        $quantity = intval($this->getParam('Quantity'));
        $randomize = intval($this->getParam('Randomize'));
        $item_class = $this->getParam('Item_CSS_class');

        $model = new ACRM();
        $items = $model->getItems($client, $cat, $quantity, $randomize);
        if (is_array($items) && (count($items))) {
            foreach ($items as $item) {
                $widgetHTML .= $this->renderItem($item, $item_class);
                $arr_ids[] = $item->b_id;
            }
            $items_ids = implode(",", $arr_ids);
            $widgetHTML .= HTMLControls::renderHiddenField("itms_ident", $items_ids, false, "itms_ident");
            $widgetHTML = "<div class=\"row\">" . $widgetHTML . "</div>";
        }
        return $widgetHTML;
    }

    public function renderItem($item, $item_class = "")
    {
        $_html = "<div class=\"itms".($item_class ? " " : "").$item_class."\">";
        $width = intval($item->b_width);
        $height = intval($item->b_height);
        $name = $item->b_name;
        $target = $item->b_target;
        if ($target) {
            if (Router::isFullLink($target)) $_blank = " target=\"_blank\""; else $_blank = "";
            $target = Router::_($target);
            $_html .= "<a rel=\"nofollow\" onclick=\"javascript:clickACRM('" . $item->b_id . "')\" " . $_blank . " href=\"" . $target . "\">";
        }
        if ($item->b_image) {
            $imageurl = ARFACMS_STORAGE_LINK . "/acrm/i/" . Files::splitAppendix($item->b_image);
            $imagefile = ARFACMS_STORAGE_PATH . "acrm" . DS . "i" . DS . Files::getAppendix($item->b_image, true) . DS . $item->b_image;
            if (Files::isFlash($imagefile)) {
                $_html .= HTMLControls::renderFlash($imageurl, $width, $height);
            } elseif (Files::isImage($imagefile)) {
                $_html .= HTMLControls::renderImage($imageurl, false, $width, $height, $name, $name, "", false);
            }
        } elseif ($item->b_custom_code) {
            $_html .= html_entity_decode($item->b_custom_code);
        } else return '';
        if (intval($this->getParam('Show_titles')) && $name) {
            $_html .= "<div class=\"itms_title\">" . $name . "</div>";
        }
        if (intval($this->getParam('Show_descriptions')) && $item->b_descr) {
            $_html .= "<div class=\"itms_descr\">" . $item->b_descr . "</div>";
        }
        if ($target) $_html .= "</a>";
        $_html .= "</div>";
        return $_html;
    }
}

?>

