<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: e70c681f (2024-09-25 19:38:20)
// 

use ArfaCMS\Core\Event;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Settings;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Widget;
use ArfaCMS\DB\Database;

defined('_ARFACMS_VALID') or die("Access denied");

class adminmenuWidget extends Widget {
	protected $tree_level = 0;
	protected $disabled_modules = array();
	protected $max_levels = 5;

	public function render() {
        $this->disabled_modules = Portal::getInstance()->getDisabledModules();

		if (\siteConfig::$treeDepth) $this->max_levels = (int)\siteConfig::$treeDepth;
		$this->tree_level = 0;

        $menuHTML = "<a class=\"navbar-brand d-".$this->getParam('menu_class_visibility', false)."-none\">".Text::_("Navigation")."</a>";
        $menuHTML.= "<button class=\"navbar-toggler\" type=\"button\" data-bs-toggle=\"offcanvas\" data-bs-target=\"#".$this->getParam('menu_div_id', false)."\" aria-controls=\"offcanvasNavbar2\" aria-label=\"Toggle navigation\">";
        $menuHTML.= "    <span class=\"navbar-toggler-icon\"></span>";
        $menuHTML.= "</button>";
		$menuHTML.= "<div id=\"".$this->getParam('menu_div_id', false)."\" class=\"offcanvas offcanvas-end\">";
		$menuHTML.= "    <div class=\"offcanvas-header pb-0 admin-top-menu-header\">";
		$menuHTML.= "        <button type=\"button\" class=\"btn-close\" data-bs-dismiss=\"offcanvas\"></button>";
		$menuHTML.= "    </div>";
		$menuHTML.= "    <div class=\"offcanvas-body pt-0 admin-top-menu-body\">";
		$menuHTML.= "    <ul class=\"navbar-nav justify-content-start flex-grow-1 pe-3\">";
		$menu_arr = $this->getBaseMenuArray();
		Event::raise("widget.adminmenu.base_data.prepared", array(), $menu_arr);
		$this->appendCustomMenuArray($menu_arr);
		Event::raise("widget.adminmenu.custom_data.prepared", array(), $menu_arr);
		foreach($menu_arr as $m_key=>$m_arr){
            if(count($m_arr["childs"])){
                $menuHTML.= "		<li class=\"nav-item dropdown admin-menu-".$m_key."\">";
                $menuHTML.= "			<a class=\"nav-link dropdown-toggle\" role=\"button2\" data-bs-toggle=\"dropdown\" href=\"#\">".Text::_($m_arr["title"])."</a>";
                $menuHTML.= $this->renderSubmenuUl($m_arr);
                $menuHTML.= "		</li>";
            } else {
                $menuHTML.= "		<li class=\"nav-item admin-menu-".$m_key."\">";
                $menuHTML.= "			<a class=\"nav-link\" href=\"".Router::_($m_arr["link"])."\">".Text::_($m_arr["title"])."</a>";
                $menuHTML.= "		</li>";
            }
		}
		$menuHTML.= "		</ul>";
		$menuHTML.= "	</div>";
		$menuHTML.= "</div>";
		
		$script="$(document).ready(function() {
					$('#".$this->getParam('menu_div_id', false)." ul.dropdown-menu [data-toggle=dropdown-submenu]').on('click', function(event) {
						event.preventDefault();
						event.stopPropagation();
						if($(this).hasClass('show')) {
							$(this).removeClass('show');
							$(this).siblings('ul').removeClass('show');
							$(this).siblings('ul').find('*').removeClass('show');
						} else {
							$(this).addClass('show');
							$(this).siblings('ul').addClass('show');
						}
					});
					$('#".$this->getParam('menu_div_id', false)." .dropdown').on('hide.bs.dropdown', function() {
                        $(this).find('*').removeClass('show');
                    })
				});";
		Portal::getInstance()->addScriptDeclaration($script);
		return $menuHTML;
	}
    protected function renderSubmenuUl($m_arr){
        $divider_html = "";
        $result = "";
        if(count($m_arr["childs"])) {
            $result = "<ul class=\"dropdown-menu\">";
            $divider = 0;
            foreach($m_arr["childs"] as $ms_key=>$ms_arr){
                if($ms_key != $divider){
                    $divider_html = "<li><hr class=\"dropdown-divider\"></li>";
                    $divider = $ms_key;
                }
                foreach($ms_arr as $_key=>$_arr) {
                    $result .= $divider_html;
                    $divider_html = "";
                    $link = Router::_($_arr["link"]);
                    $disabled_class = "";
                    if (in_array($_arr["module"], $this->disabled_modules)){
                        $link = "#";
                        $disabled_class = " disabled";
                    }
                    if(count($_arr['childs'])){
                        $result.= "<li><a class=\"dropdown-item dropdown-toggle".$disabled_class."\" data-toggle=\"dropdown-submenu\" href=\"#\">".Text::_($_arr["title"])."</a>";
                    } else {
                        $result.= "<li><a class=\"dropdown-item".$disabled_class."\" href=\"".$link."\">".Text::_($_arr["title"])."</a>";
                    }
                    $result.= $this->renderSubmenuUl($_arr);
                    $result.= "</li>";
                }
            }
            $result.= "</ul>";
        }
        return $result;
    }
	protected function getBaseMenuArray(){
		$menu_arr = array();
		/***********************************************************************************/
		$mm_name = "service";
		$menu_arr[$mm_name]=array("title"=>"Service menu", "link"=>"#", "module"=>"", "acl"=>"", "childs"=>array());
		$divider = 0;
		$menu_arr[$mm_name]["childs"][$divider] = array();
		/***********************************************************************************/
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Cache manager", "link"=>"index.php?module=service&view=cachemanager", "module"=>"service", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Clear users filters", "link"=>"index.php?module=service&view=userfilter", "module"=>"service", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Check ACL objects", "link"=>"index.php?module=service&view=aclrules", "module"=>"service", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Prepare metadata fields", "link"=>"index.php?module=conf&task=prepareFields", "module"=>"conf", "acl"=>"", "childs"=>array());
		if (defined("_ARFACMS_TRANSLATE")) {
			$divider++;
			$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Check Translate module", "link"=>"index.php?module=conf&task=checkTranslateList", "module"=>"conf", "acl"=>"", "childs"=>array());
		}
		$divider++;
        // @FIXME Temporary disabled !!! Imho, not need !!!
        // $menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"DB Service", "link"=>"index.php?module=service&view=db", "module"=>"service", "acl"=>"", "childs"=>array());
		if(intval(Settings::getVar("restruct_version")) < Portal::getInstance()->getVersionRevision()){
			$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Database restructuring", "link"=>"index.php?module=service&&view=updater&&layout=restructure", "module"=>"service", "acl"=>"", "childs"=>array());
		} else {
			$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Check for updates", "link"=>"index.php?module=service", "module"=>"service", "acl"=>"", "childs"=>array());
		}
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Installer", "link"=>"index.php?module=installer", "module"=>"installer", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"PHP Info", "link"=>"index.php?module=conf&task=pi", "module"=>"conf", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Mailer log", "link"=>"index.php?module=service&view=mailerlog", "module"=>"service", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Manage site maps", "link"=>"index.php?module=conf&view=sitemap", "module"=>"conf", "acl"=>"", "childs"=>array());
        // @TODO Temporary disabled !!!
        // Code not ready !!!
		// $menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Site map manual editor", "link"=>"index.php?module=conf&view=mansitemap", "module"=>"conf", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Media manager", "link"=>"index.php?module=service&view=mediamanager", "module"=>"service", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Image processor", "link"=>"index.php?module=service&view=imageprocessor", "module"=>"service", "acl"=>"", "childs"=>array());
		/***********************************************************************************/
		$mm_name = "config";
		$menu_arr[$mm_name]=array("title"=>"Configuration", "link"=>"#", "module"=>"", "acl"=>"", "childs"=>array());
		$divider = 0;
		$menu_arr[$mm_name]["childs"][$divider] = array();
		/***********************************************************************************/
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Settings", "link"=>"index.php?module=conf&view=config", "module"=>"conf", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Menu editor", "link"=>"index.php?module=menus", "module"=>"menus", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Modules", "link"=>"index.php?module=conf&view=modules", "module"=>"conf", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Plugins", "link"=>"index.php?module=conf&view=plugins", "module"=>"conf", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Widgets", "link"=>"index.php?module=conf&view=widgets", "module"=>"conf", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Countries, regions, cities", "link"=>"index.php?module=conf&view=cladr", "module"=>"conf", "acl"=>"", "childs"=>array());
        $menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Measures", "link"=>"index.php?module=conf&view=measures", "module"=>"conf", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Currencies", "link"=>"index.php?module=conf&view=currency", "module"=>"conf", "acl"=>"", "childs"=>array());
        $menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"List of taxes", "link"=>"index.php?conf=catalog&view=taxes", "module"=>"conf", "acl"=>"", "childs"=>array());
        $menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Payment types", "link"=>"index.php?module=conf&view=paymenttypes", "module"=>"conf", "acl"=>"", "childs"=>array());
        $menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Delivery types", "link"=>"index.php?module=conf&view=deliverytypes", "module"=>"conf", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Prepare visio", "link"=>"index.php?module=conf&task=selectVisio", "module"=>"conf", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Additional fields", "link"=>"index.php?module=conf&view=dopfields_groups", "module"=>"conf", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Template zones", "link"=>"index.php?module=conf&view=tmplzones", "module"=>"conf", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Redirect links", "link"=>"index.php?module=conf&view=redirectlinks", "module"=>"conf", "acl"=>"", "childs"=>array());
		/***********************************************************************************/
		$mm_name = "acl";
		$menu_arr[$mm_name]=array("title"=>"ACL", "link"=>"#", "module"=>"", "acl"=>"", "childs"=>array());
		$divider = 0;
		$menu_arr[$mm_name]["childs"][$divider] = array();
		/***********************************************************************************/
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Users", "link"=>"index.php?module=user", "module"=>"user", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Roles", "link"=>"index.php?module=aclmgr&view=roles", "module"=>"aclmgr", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Auth providers", "link"=>"index.php?module=user&view=auth_providers", "module"=>"user", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Blacklist", "link"=>"index.php?module=user&view=blacklist", "module"=>"user", "acl"=>"", "childs"=>array());
		/***********************************************************************************/
		$mm_name = "content";
		$menu_arr[$mm_name]=array("title"=>"Common content", "link"=>"#", "module"=>"", "acl"=>"", "childs"=>array());
		$divider = 0;
		$menu_arr[$mm_name]["childs"][$divider] = array();
		/***********************************************************************************/
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Articles", "link"=>"index.php?module=article", "module"=>"article", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Blogs categories", "link"=>"index.php?module=blog", "module"=>"blog", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"All blogs", "link"=>"index.php?module=blog&view=list&layout=all", "module"=>"blog", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Rights map by roles", "link"=>"index.php?module=blog&view=rights&layout=rolesmap", "module"=>"blog", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Rights map by users", "link"=>"index.php?module=blog&view=rights&layout=usersmap", "module"=>"blog", "acl"=>"", "childs"=>array());
		/***********************************************************************************/
		$mm_name = "multimedia";
		$menu_arr[$mm_name]=array("title"=>"Multimedia", "link"=>"#", "module"=>"", "acl"=>"", "childs"=>array());
		$divider = 0;
		$menu_arr[$mm_name]["childs"][$divider] = array();
		/***********************************************************************************/
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Image gallery", "link"=>"index.php?module=gallery", "module"=>"gallery", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Video gallery", "link"=>"index.php?module=videoset", "module"=>"videoset", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Banners categories", "link"=>"index.php?module=acrm&view=cats", "module"=>"acrm", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Banners clients", "link"=>"index.php?module=acrm&view=clients", "module"=>"acrm", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"ACRM", "link"=>"index.php?module=acrm&view=items&layout=all", "module"=>"acrm", "acl"=>"", "childs"=>array());
		/***********************************************************************************/
		$mm_name = "forum";
		$menu_arr[$mm_name]=array("title"=>"Forum", "link"=>"#", "module"=>"forum", "acl"=>"", "childs"=>array());
		$divider = 0;
		$menu_arr[$mm_name]["childs"][$divider] = array();
		/***********************************************************************************/
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Forum sections", "link"=>"index.php?module=forum", "module"=>"forum", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Rights map by roles", "link"=>"index.php?module=forum&view=rights&layout=rolesmap", "module"=>"forum", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Rights map by users", "link"=>"index.php?module=forum&view=rights&layout=usersmap", "module"=>"forum", "acl"=>"", "childs"=>array());
		/***********************************************************************************/
		$mm_name = "comments";
		$menu_arr[$mm_name]=array("title"=>"Comments", "link"=>"#", "module"=>"comments", "acl"=>"", "childs"=>array());
		$divider = 0;
		$menu_arr[$mm_name]["childs"][$divider] = array();
		/***********************************************************************************/
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Comments groups", "link"=>"index.php?module=comments", "module"=>"comments", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Rights map by roles", "link"=>"index.php?module=comments&view=rights", "module"=>"comments", "acl"=>"", "childs"=>array());
		/***********************************************************************************/
		$mm_name = "feedback";
		$menu_arr[$mm_name]=array("title"=>"Feedback", "link"=>"#", "module"=>"feedback", "acl"=>"", "childs"=>array());
		$divider = 0;
		$menu_arr[$mm_name]["childs"][$divider] = array();
		/***********************************************************************************/
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Messages from site", "link"=>"index.php?module=feedback&view=messages", "module"=>"feedback", "acl"=>"", "childs"=>array());
        $menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Feedback forms", "link"=>"index.php?module=feedback&view=forms", "module"=>"feedback", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Polls", "link"=>"index.php?module=polls", "module"=>"polls", "acl"=>"", "childs"=>array());
		/***********************************************************************************/
		$mm_name = "catalog";
		$menu_arr[$mm_name]=array("title"=>"Catalog", "link"=>"#", "module"=>"catalog", "acl"=>"", "childs"=>array());
		$divider = 0;
		$menu_arr[$mm_name]["childs"][$divider] = array();
		/***********************************************************************************/
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Goods groups and goods", "link"=>"index.php?module=catalog&view=goodsgroup", "module"=>"catalog", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Goods without groups", "link"=>"index.php?module=catalog&view=goods", "module"=>"catalog", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Vendors categories", "link"=>"index.php?module=catalog&view=vendor_cats", "module"=>"catalog", "acl"=>"", "childs"=>array());
		if(\catalogConfig::$multy_vendor) {
			$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Vendors links", "link"=>"index.php?module=catalog&view=users", "module"=>"catalog", "acl"=>"", "childs"=>array());
		}
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Manufacturers categories", "link"=>"index.php?module=catalog&view=manufacturer_cats", "module"=>"catalog", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Goods options", "link"=>"index.php?module=catalog&view=options", "module"=>"catalog", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Discounts and surcharges", "link"=>"index.php?module=catalog&view=discounts", "module"=>"catalog", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Data exchange in 1C format", "link"=>"index.php?module=catalog&view=exchange1c", "module"=>"catalog", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Export catalog data as CSV", "link"=>"index.php?module=catalog&view=export", "module"=>"catalog", "acl"=>"", "childs"=>array());
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Import catalog data as CSV", "link"=>"index.php?module=catalog&view=import", "module"=>"catalog", "acl"=>"", "childs"=>array());
		$divider++;
		$menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"Transition statistics from other sites", "link"=>"index.php?module=catalog&view=goods_stat", "module"=>"catalog", "acl"=>"", "childs"=>array());
        /***********************************************************************************/
        $mm_name = "orders";
        $menu_arr[$mm_name]=array("title"=>"Orders", "link"=>"#", "module"=>"orders", "acl"=>"", "childs"=>array());
        $divider = 0;
        $menu_arr[$mm_name]["childs"][$divider] = array();
        $menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"View orders", "link"=>"index.php?module=orders", "module"=>"orders", "acl"=>"", "childs"=>array());
		/***********************************************************************************/
		// $menu_arr[$mm_name]["childs"][$divider][] = array("title"=>"", "link"=>"", "module"=>"", "acl"=>"", "childs"=>array());
		/***********************************************************************************/
		return $menu_arr;
	}
	protected function appendCustomMenuArray(&$menu_arr){
		$sql = "SELECT am.*, COUNT(amc.mnu_id) AS mnu_childs FROM #__admin_menus AS am
				LEFT JOIN #__admin_menus AS amc ON amc.mnu_parent_id=am.mnu_id
				WHERE am.mnu_id>0
				GROUP BY am.mnu_id
				ORDER BY am.mnu_order,am.mnu_id";
		Database::getInstance()->setQuery($sql);
		$res=Database::getInstance()->loadObjectList();
		if(count($res)){
			$childs = array();
			foreach($res as $k=>$v){
				if(!isset($childs[$v->mnu_parent_id])) $childs[$v->mnu_parent_id] = array();
				$childs[$v->mnu_parent_id][] = $v;
			}
			$this->tree_level++;
			foreach($res as $k=>$v){
				if($v->mnu_parent_id == 0){
					$mm_name = "admin-menu-".$v->mnu_id;
					$menu_arr[$mm_name]=array("title"=>$v->mnu_name, "link"=>"#", "module"=>$v->mnu_module, "acl"=>"", "childs"=>array());
					$menu_arr[$mm_name]["childs"] = $this->appendCustomMenuChilds($v->mnu_id, $childs);
                    if(!count($menu_arr[$mm_name]["childs"])) $menu_arr[$mm_name]["link"] = $v->mnu_link;
				}
			}
		}
	}
	protected function appendCustomMenuChilds($mnu_id, $childs){
		$menu_arr = array();
		if($this->tree_level > $this->max_levels) return $menu_arr;
		$this->tree_level++;
		if(array_key_exists($mnu_id, $childs)){
			$divider = 0;
			foreach($childs[$mnu_id] as $k=>$v){
				if($v->mnu_parent_id == $mnu_id){
					if($v->mnu_name === "-") {
						$divider++;
						continue;
					}
					$menu_arr[$divider][] = array("title"=>$v->mnu_name, "link"=>$v->mnu_link, "module"=>$v->mnu_module, "acl"=>"", "childs"=>$this->appendCustomMenuChilds($v->mnu_id, $childs));
				}
			}
		}
		$this->tree_level--;
		return $menu_arr;
	}
    protected function removeItemByTitle(&$arr, $title, $strict = true){
        $result = false;
        foreach ($arr as $k=>&$v){
            if(is_array($v)){
                if(isset($v["title"]) && ($strict ? $v["title"]==$title : stristr($v["title"], $title) != FALSE)){
                    unset($arr[$k]);
                    $result = true;
                } else {
                    $result = ($this->removeItemByTitle($v, $title, $strict) ? true : $result);
                }
            }
        }
        return $result;
    }
    protected function removeItemByLink(&$arr, $link, $strict = true){
        $result = false;
        foreach ($arr as $k=>&$v){
            if(is_array($v)){
                if(isset($v["link"]) && ($strict ? $v["link"]==$link : stristr($v["link"], $link) != FALSE)){
                    unset($arr[$k]);
                    $result = true;
                } else {
                    $result = ($this->removeItemByLink($v, $link, $strict) ? true : $result);
                }
            }
        }
        return $result;
    }
}
?>