<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 1c690378 (2024-10-13 03:03:48)
// 

use ArfaCMS\Core\Date;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Util;
use ArfaCMS\Core\Widget;
use ArfaCMS\DB\Database;

defined('_ARFACMS_VALID') or die("Access denied");

class articlelistWidget extends Widget {
	protected function setParamsMask(){
		parent::setParamsMask();
		$this->addParam("include_articles_parents", "table_multiselect", 0, false, "SELECT a_id AS fld_id, CONCAT('(',a_id,') ',a_title) AS fld_name FROM #__articles ORDER BY fld_name");
		$this->addParam("exclude_articles_parents", "table_multiselect", 0, false, "SELECT a_id AS fld_id, CONCAT('(',a_id,') ',a_title) AS fld_name FROM #__articles ORDER BY fld_name");
		$this->addParam("article_count", "integer", 5);
		$this->addParam("show_date", "boolean", 0);
		$this->addParam("data_absent_text", "string", "");
		$this->addParam("menu_id", "table_select", 0, false, "SELECT mi_id AS fld_id, CONCAT('(',mi_id,') ',mi_name) AS fld_name FROM #__menus ORDER BY fld_name");
	}
	public function render() {
		$article_count	= $this->getParam('article_count');
		$arrpsid		= explode(";",$this->getParam('include_articles_parents'));
		$arrexpsid	= explode(";",$this->getParam('exclude_articles_parents'));
		$show_date = $this->getParam('show_date');
		if ($this->getParam('menu_id')) $mid="&amp;mid=".$this->getParam('menu_id'); else $mid="";
		if(count($arrpsid)==1 && empty($arrpsid[0]) ) $arrpsid=array();
		if(count($arrexpsid)==1 && empty($arrexpsid[0]) ) $arrexpsid=array();
		$body = "";
		$list = $this->getArticles($arrpsid,$arrexpsid,$article_count);
		if (count($list)) {
			foreach($list as $val) {
				$link=Router::_("index.php?module=article&amp;view=read&amp;psid=".$val->a_id."&amp;alias=".$val->a_alias.$mid);
				$link_aut=Router::_("index.php?module=user&view=info&psid=".$val->u_id);
				$body.='<div class="single-article row mb-3"><div class="article-title col-sm-12"><a href="'.$link.'">'.$val->title;
				if($show_date) $body.='&nbsp;<small>'.Date::fromSQL($val->date,true,true).'</small>';
				$body.='</a></div></div>';
			}
		}	else {	// Render failure
			$data_absent_text = $this->getParam('data_absent_text');
			$body .= "<div class=\"data-absent\">".($data_absent_text ? $data_absent_text : Text::_('Article not found'))."</div>";
		}
		return $body;
	}
	protected function getArticles($arrArticleId,$arrArticleIdEx,$post_count=5) {
		$db = Database::getInstance();
		$res=array(); $res_ex=array();
        if(count($arrArticleId)) {
			foreach($arrArticleId as $ArticleId) {
				$res[]=intval($ArticleId);
			}
		} else {
            $res[] = 0; // First level only
        }
		if(count($arrArticleIdEx)) {
			foreach($arrArticleIdEx as $ArticleId) {
				$res_ex[]=intval($ArticleId);
			}
		}
		$query = "SELECT a.a_id, a.a_alias, a.a_parent_id,
				a.a_title as title,a.a_date as date,
				a.a_author_id as u_id,	u.u_nickname as author
				FROM `#__articles` as a,`#__users` as u
				WHERE a.a_deleted=0 AND a.a_published=1";
		$query .= " AND a.a_author_id=u.u_id";
		if (count($res)) {
			$query .= " AND (a.a_parent_id IN (".implode(",",$res)."))";
		}
		if (count($res_ex)) {
			$query .= " AND (a.a_parent_id NOT IN (".implode(",",$res_ex)."))";
		}
		$query .= " ORDER BY a.a_date DESC";
		$query .= " LIMIT ".(int)$post_count;
		$db->setQuery($query);
		$posts = $db->loadObjectList();
		if (is_array($posts) && count($posts) > 0) {
			return $posts;
		}	else 	{
			return array();
		}
	}
}
?>