<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 0998a35f (2024-10-14 23:52:15)
// 

use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\simpleTreeTable;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Widget;
use ArfaCMS\MVC\Module;

defined('_ARFACMS_VALID') or die("Access denied");

class catalogmenuWidget extends Widget
{
    protected $_requiredModules = array("catalog");
    protected $_mobileWith = 992;

    protected function setParamsMask()
    {
        parent::setParamsMask();
        $this->addParam("Collapse_widget", "select", 0, false, array(1 => Text::_("Accordeon"), 2 => Text::_("Full screen")), Text::_("Mobile devices"));
        $this->addParam("Collapsed_widget_title", "string", "", false, null, Text::_("Mobile devices"));
        $this->addParam("Hide_main_title_on_mobiles", "boolean", 1);
        $this->addParam("Catalogmenu_ul_id", "string", "");
        $this->addParam("Catalogmenu_ul_class", "string", "flex-column", true, null);
        $this->addParam("Order_by", "select", "ggr_name", false, array(array("id" => "ggr_name", "name" => Text::_("Name")), array("id" => "ggr_ordering", "name" => Text::_("Ordering"))));
        $this->addParam("Render_type", "select", 3, true, array("2" => Text::_("Dropdown"), "3" => Text::_("Static menu"), "4" => Text::_("Accordeon")));
        $this->addParam("Hover_dropdown", "boolean", 0, false, null, Text::_("Variants") . ":<br />" . Text::_("Dropdown") . " (" . Text::_("Except mobiles") . ")");
        $this->addParam("Menu_data_toggle_separate", "boolean", 0, false, null, Text::_("Variants") . ":<br />" . Text::_("Dropdown") . "<br />" . Text::_("Accordeon"));
        $this->addParam("Expand_active", "boolean", 0, false, null, Text::_("Variants") . ":<br />" . Text::_("Static menu") . "<br />" . Text::_("Accordeon"));
        // $this->addParam("Expand_behaviour", "select", "location", false, array(array("id"=>"cookie","name"=>Text::_("By last expanded")),array("id"=>"location","name"=>Text::_("Current location"))));
        $this->addParam("Exclude_groups_ids", "multiselect", "0", false, $this->getGroupsItems());
        // Next line is for example of using another source for "Exclude_groups_ids"
        //$this->addParam("Exclude_groups_ids", "table_multiselect", 0, false, "SELECT ggr_id AS fld_id, CONCAT(ggr_name, ' (',ggr_id,')') AS fld_name FROM c_goods_group ORDER BY fld_name");
        $this->addParam("Max_levels", "integer", 5);
        $this->addParam("Root_id", "select", "0", false, $this->getGroupsItems());
        // $this->addParam("Reset_filter_on_click", "boolean", 0);
        $this->addParam("Use_controller", "select", "default", false, defined('_ADMIN_MODE') ? Module::getControllers("catalog", true) : null);
        $this->addParam("Title", "title", Text::_("Buttons"));
        $this->addParam("Cart_button", "boolean", 0);
        $this->addParam("Filter_button", "boolean", 0);
        $this->addParam("Vendors_button", "boolean", 0);
        $this->addParam("Manufacturers_button", "boolean", 0);
        $this->addParam("Favourites_button", "boolean", 0);
        $this->addParam("Compare_button", "boolean", 0);
    }

    public function prepare()
    {
        $aw_id = $this->getParam('aw_id');
        $widget_id = $this->getParam('Widget_ID', false, "widget_" . $this->getParam("aw_id"));
        $ul_id = $this->getParam('Catalogmenu_ul_id', false, ($widget_id ? $widget_id . "_mnu" : ""));
        $render_type = intval($this->getParam('Render_type'));
        $toggle_separate = intval($this->getParam('Menu_data_toggle_separate'));
        Portal::getInstance()->addStyleSheet("modules/catalog.css", !\seoConfig::$tmplCSSBackCompatibility);

        if ($ul_id) {
            $script = "$(document).ready(function(){";
            switch ($render_type) {
                case "2":
                case "4":
                    $script .= "w_menus_init('" . $ul_id . "', " . $render_type . ", " . $toggle_separate . ", " . ($render_type == 2 ? "true" : "false") . ", " . $this->getParam('Hover_dropdown') . ", " . $this->_mobileWith . ");";
                    break;
                case "3":
                default:
                    break;
            }
            if (Request::getSafe("module") == "catalog" && $this->getParam('Expand_active')) {
                switch ($render_type) {
                    case "3":
                    case "4":
                            $script .= "if (getCMSCookie('arfacms_catalog_group') != null) {";
                            $script .= "    var curr_catalog_group_" . $aw_id . " = '" . $ul_id . "_item_' + getCMSCookie('arfacms_catalog_group');";
                            $script .= "    $('#" . $ul_id . " ul').removeClass('show');";
                            $script .= "    $('#" . $ul_id . " li').removeClass('show').removeClass('selected').removeClass('has-active-child');";
                            $script .= "    $('#" . $ul_id . " a').removeClass('active');";

                            $script .= "    $('#" . $ul_id . " li#' + curr_catalog_group_" . $aw_id . ").addClass('selected').addClass('show');";
                            $script .= "    $('#" . $ul_id . " li#' + curr_catalog_group_" . $aw_id . " + ' > a').addClass('active');";
                            $script .= "    $('#" . $ul_id . " li#' + curr_catalog_group_" . $aw_id . ").parents('li').addClass('has-active-child').children('a').addClass('active');";
                            $script .= "    $('#" . $ul_id . " li#' + curr_catalog_group_" . $aw_id . ").parents('ul, li').addClass('show');";
                            $script .= "    $('#" . $ul_id . " li#' + curr_catalog_group_" . $aw_id . " + ' > ul').addClass('show');";
                            $script .= "}";
                        break;
                    case "2":
                    default:
                        break;
                }
            }
            switch ($this->getParam('Collapse_widget')) {
                case "1":
                    break;
                case "2":
                    break;
                default:
                    break;
            }
            /*
            if($this->getParam('Reset_filter_on_click')){
                $script.="
                    $('#".$ul_id." li:not(.has_childs)>a').on('click', function(e){
                        var href = $(this).attr('href');
                        e.preventDefault();
                        e.stopPropagation();
                        ajaxShowActivity();
                        $.ajax({
                            url : siteConfig['siteUrl']+'index.php',
                            data:({
                                type:'module',
                                module:'catalog',
                                view: 'goods',
                                option:'ajax',
                                full_reset: 1,
                                task:'resetfilter'
                            }),
                            dataType:'json',
                            success: function (data, textStatus) {
                                if (data.result=='OK') document.location=href;
                                ajaxHideActivity();
                            },
                            error: function (qq) {
                                ajaxHideActivity();
                                return false;
                            }
                        });
                    });
                ";
            }
            */
            $script .= "});"; // $(document).ready(function(){
            Portal::getInstance()->AddScriptDeclaration($script);
            if ($this->getParam('Hide_main_title_on_mobiles')) {
                Portal::getInstance()->addStyle("@media (max-width: " . $this->_mobileWith . "px){#" . $widget_id . " .aw-title {display:none !important;}}");
            }
        }
    }

    public function render()
    {
        if (\catalogConfig::$catalogDisabled) {
            $menuHTML = "";
            return $menuHTML;
        }
        $widget_id = $this->getParam('Widget_ID', false, "widget_" . $this->getParam("aw_id"));
        $ul_id = $this->getParam('Catalogmenu_ul_id', false, ($widget_id ? $widget_id . "_mnu" : ""));
        $render_type = intval($this->getParam('Render_type'));
        $_ul_class = $this->getParam('Catalogmenu_ul_class');
        $_ul_class .= ($_ul_class ? " " : "") . "nav";
        if ($render_type == 2) {
            $_ul_class .= ($this->getParam('Hover_dropdown') ? " hover-dropdown" : "");
        } elseif ($render_type == 3) {
            //
        } elseif ($render_type == 4) {
            //
        }
        $exc_grp_ids = $this->getParam('Exclude_groups_ids');
        $exc_grp_ids = implode(",", explode(";", $exc_grp_ids));
        $max_levels = $this->getParam('Max_levels');
        $root_id = $this->getParam('Root_id');
        $filter_button = $this->getParam('Filter_button');
        $cart_button = $this->getParam('Cart_button');
        $vendors_button = $this->getParam('Vendors_button');
        $manufacturers_button = $this->getParam('Manufacturers_button');
        $favourites_button = $this->getParam('Favourites_button');
        $compare_button = $this->getParam('Compare_button');
        $ordering = $this->getParam('Order_by');
        $model = $this->getGroupsTree($ordering, $max_levels, $exc_grp_ids);
        $widgetHTML = "";
        $menuHTML = "<div class=\"catalogmenu-widget-tree-wrapper tree-wrapper w-100 menu-render-type-" . $render_type . " mb-3\">" . $model->getTreeHTML($root_id, $ul_id, 0, $ul_id . "_item", $_ul_class) . "</div>";
        if ($filter_button || $cart_button || $manufacturers_button) {
            if (Module::getInstance()->getName() == "catalog") $controller = Module::getInstance("catalog")->get("controller");
            else $controller = "";
            $menuHTML .= "<div class=\"catalogmenu-widget-buttons\">";
            if ($filter_button && $cart_button) {
                $menuHTML .= "<div class=\"btn-group w-100\">";
            }
            if ($filter_button) $menuHTML .= "<a class=\"linkButton btn btn-primary" . ($cart_button ? " w-50" : " w-100") . "\" onclick=\"javascript:showFilter('catalog','goods','','0', '" . Text::_("Filter") . "', 0, '" . $controller . "'); return false;\" title=\"" . Text::_("Filter") . "\">" . Text::_("Filter") . "</a>";
            if ($cart_button) $menuHTML .= "<a rel=\"nofollow\" class=\"rel-popup-text linkButton btn btn-primary" . ($filter_button ? " w-50" : " w-100") . "\" href=\"" . Router::_("index.php?module=catalog&amp;task=ShowBasket&amp;option=ajax") . "\" title=\"" . Text::_("Basket") . "\">" . Text::_("Basket") . "</a>";
            if ($filter_button && $cart_button) {
                $menuHTML .= "</div>";
            }
            if ($vendors_button) $menuHTML .= "<a class=\"linkButton btn btn-primary w-100 mt-1\" href=\"" . Router::_("index.php?module=catalog&amp;view=vendors") . "\">" . Text::_("Vendors") . "</a>";
            if ($manufacturers_button) $menuHTML .= "<a class=\"linkButton btn btn-primary w-100 mt-1\" href=\"" . Router::_("index.php?module=catalog&amp;view=manufacturers") . "\">" . Text::_("Manufacturers") . "</a>";
            if ($favourites_button && Module::getInstance("catalog")->getParam("enable_favourites_goods")) $menuHTML .= "<a rel=\"nofollow\" class=\"linkButton btn btn-primary w-100 mt-1\" href=\"" . Router::_("index.php?module=catalog&amp;task=favourites") . "\">" . Text::_("Favourites list") . "</a>";
            if ($compare_button && Module::getInstance("catalog")->getParam("enable_compare_goods")) $menuHTML .= "<a rel=\"nofollow\" class=\"linkButton btn btn-primary w-100 mt-1\" href=\"" . Router::_("index.php?module=catalog&amp;task=compare") . "\">" . Text::_("Compare list") . "</a>";
            $menuHTML .= "</div>";
        }
        switch ($this->getParam('Collapse_widget')) {
            case "1":
                $adaptive_menu_title = $this->getParam('Collapsed_widget_title');
                $widgetHTML .= "<div class=\"catalogmenu-widget navbar navbar-expand-lg p-0\">";
                $widgetHTML .= "    <div class=\"widget-navbar-panel d-flex w-100 d-lg-none justify-content-between\">";
                $widgetHTML .= "       <a class=\"navbar-brand topmenu_label\">" . $adaptive_menu_title . "</a>";
                $widgetHTML .= "       <button type=\"button\" class=\"navbar-toggler\" data-bs-toggle=\"collapse\" data-bs-target=\"#" . $widget_id . "_wrapper\"><span class=\"navbar-toggler-icon\"></span></button>";
                $widgetHTML .= "    </div>";
                $widgetHTML .= "    <div id=\"" . $widget_id . "_wrapper\" class=\"collapse navbar-collapse flex-column pt-3 pt-lg-0\">" . $menuHTML . "</div>";
                $widgetHTML .= "</div>";
                break;
            case "2":
                $adaptive_menu_title = $this->getParam('Collapsed_widget_title');
                $widgetHTML .= "<div class=\"catalogmenu-widget navbar navbar-expand-lg p-0\">";
                $widgetHTML .= "    <div class=\"widget-navbar-panel d-flex w-100 d-lg-none justify-content-between\">";
                $widgetHTML .= "        <a class=\"navbar-brand topmenu_label\">" . $adaptive_menu_title . "</a>";
                $widgetHTML .= "        <button type=\"button\" class=\"navbar-toggler\" data-bs-toggle=\"offcanvas\" data-bs-target=\"#" . $widget_id . "_wrapper\"><span class=\"navbar-toggler-icon\"></span></button>";
                $widgetHTML .= "    </div>";
                $widgetHTML .= "    <div id=\"" . $widget_id . "_wrapper\" class=\"offcanvas offcanvas-end\">";
                $widgetHTML .= "        <div class=\"offcanvas-header\">";
                $widgetHTML .= "            <a class=\"navbar-brand topmenu_label\">" . $adaptive_menu_title . "</a>";
                $widgetHTML .= "            <button type=\"button\" class=\"btn-close\" data-bs-dismiss=\"offcanvas\"></button>";
                $widgetHTML .= "        </div>";
                $widgetHTML .= "        <div class=\"offcanvas-body flex-column pt-0\">" . $menuHTML . "</div>";
                $widgetHTML .= "    </div>";
                $widgetHTML .= "</div>";
                break;
            default:
                $widgetHTML = $menuHTML;
                break;
        }
        return $widgetHTML;
    }

    protected function getGroupsItems()
    {
        $data = array();
        if (!defined('_ADMIN_MODE')) return $data;
        $model = $this->getGroupsTree("ggr_ordering", 3, "");
        foreach ($model->getTreeArr() as $key => $val) {
            $data[] = array("id" => $val->id, "name" => str_repeat("- ", $val->level - 1) . $val->title);
        }
        return $data;
    }

    protected function getGroupsTree($ordering, $max_levels, $exc_grp_ids)
    {
        $model = new simpleTreeTable();
        $controller = $this->getParam('Use_controller');
        $model->table = "goods_group";
        $model->fld_id = "ggr_id";
        $model->fld_parent_id = "ggr_id_parent";
        $model->fld_title = "ggr_name";
        $model->fld_alias = "ggr_alias";
        $model->fld_enabled = "ggr_enabled";
        $model->fld_deleted = "ggr_deleted";
        $model->fld_orderby = $ordering;
        $model->element_link = "index.php?module=catalog" . ($controller ? "&amp;controller=" . $controller : "") . "&amp;view=goods&amp;psid=";
        $model->buildTreeArrays($exc_grp_ids, 0, 1, 1, $max_levels);
        return $model;
    }
}

?>