<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 0998a35f (2024-10-14 23:52:15)
// 

use ArfaCMS\Core\Menus;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Widget;
use ArfaCMS\DB\Database;

defined('_ARFACMS_VALID') or die("Access denied");

class menuWidget extends Widget
{
    private $data_id_attr = "data-id";
    private $data_canonical_attr = "data-canonical-id";
    protected $_mobileWith = 992;

    protected function setParamsMask()
    {
        parent::setParamsMask();
        $this->addParam("Collapse_widget", "select", 0, false, array(1 => Text::_("Accordeon"), 2 => Text::_("Full screen")), Text::_("Mobile devices"));
        $this->addParam("Collapsed_widget_title", "string", "", false, null, Text::_("Mobile devices"));
        $this->addParam("Hide_main_title_on_mobiles", "boolean", 1);
        $this->addParam("Render_type", "select", 3, true, array("2" => Text::_("Dropdown"), "3" => Text::_("Static menu"), "4" => Text::_("Accordeon")));
        $this->addParam("Hover_dropdown", "boolean", 0, false, null, Text::_("Variants") . ":<br />" . Text::_("Dropdown") . " (" . Text::_("Except mobiles") . ")");
        $this->addParam("Expand_active", "boolean", 0, false, null, Text::_("Variants") . ":<br />" . Text::_("Static menu") . "<br />" . Text::_("Accordeon"));
        $this->addParam("Menu_ul_id", "string", "");
        $this->addParam("Menu_class", "string", "flex-column", true);
        $this->addParam("Menu_li_class", "string", "");
        $this->addParam("Menu_a_class", "string", "");
        $this->addParam("Menu_ul_class", "string", "");
        $this->addParam("Menu_data_toggle_separate", "boolean", 0, false, null, Text::_("Variants") . ":<br />" . Text::_("Dropdown") . "<br />" . Text::_("Accordeon"));
        $this->addParam("Menu_root_id", "select", "", false, $this->getTopItems());
        $this->addParam("Menu_max_levels", "integer", 0);
        $this->addParam("Hide_menu_titles", "boolean", 0);
        $this->addParam("Translate", "boolean", 0);
    }

    public function prepare()
    {
        $widget_id = $this->getParam('Widget_ID');
        $ul_id = $this->getParam('Menu_ul_id', false, ($widget_id ? $widget_id . "_mnu" : ""));
        $li = $ul_id . "_item_" . $_SESSION['active_menu_id'];
        $li_canonical = "li[" . $this->data_canonical_attr . "='" . $_SESSION['active_menu_id'] . "']";
        $render_type = intval($this->getParam('Render_type'));
        $toggle_separate = intval($this->getParam('Menu_data_toggle_separate'));

        if ($ul_id) {
            $script = '$(document).ready(function(){';
            switch ($this->getParam('Collapse_widget')) {
                case "1":
                    break;
                case "2":
                    break;
                default:
                    break;
            }
            // Here we should consider mi_canonical_id (data-canonical-id)
            $script .= '    $("#' . $ul_id . ' a").removeClass("active");';
            $script .= '    $("#' . $ul_id . ' li").removeClass("selected").removeClass("has-active-child");';
            // ------------------------------------------------------------
            $script .= '    $("#' . $ul_id . ' li#' . $li . '").addClass("selected");';
            $script .= '    $("#' . $ul_id . ' li#' . $li . ' > a").addClass("active");';
            $script .= '    $("#' . $ul_id . ' li#' . $li . '").parents("li").addClass("has-active-child").children("a").addClass("active");';

            $script .= '    $("#' . $ul_id . ' ' . $li_canonical . '").addClass("selected");';
            $script .= '    $("#' . $ul_id . ' ' . $li_canonical . ' > a").addClass("active");';
            $script .= '    $("#' . $ul_id . ' ' . $li_canonical . '").parents("li").addClass("has-active-child").children("a").addClass("active");';

            // Init menu
            switch ($render_type) {
                case "2":
                case "4":
                    $script .= "w_menus_init('" . $ul_id . "', " . $render_type . ", " . $toggle_separate . ", " . ($render_type == 2 ? "true" : "false") . ", " . $this->getParam('Hover_dropdown') . ", " . $this->_mobileWith . ");";
                    break;
                case "3":
                default:
                    break;
            }

            // Expand menu
            if ($this->getParam('Expand_active')) {
                switch ($render_type) {
                    case "3":
                    case "4":
                        $script .= "$('#" . $ul_id . " .active').parents('ul,li').addClass('show');";
                        $script .= "$('#" . $ul_id . " .active').siblings('ul').addClass('show');";
                        break;
                    case "2":
                    default:
                        break;
                }
            }

            $script .= '});';

            Portal::getInstance()->AddScriptDeclaration($script);
            if ($this->getParam('Hide_main_title_on_mobiles')) {
                Portal::getInstance()->addStyle("@media (max-width: " . $this->_mobileWith . "px){#" . $widget_id . " .aw-title {display:none !important;}}");
            }
        }
    }

    public function render()
    {
        $widgetHTML = "";
        $widget_id = $this->getParam('Widget_ID', false, "widget_" . $this->getParam("aw_id"));
        $ul_id = $this->getParam('Menu_ul_id', false, ($widget_id ? $widget_id . "_mnu" : ""));
        $ul_hide_menu_titles = $this->getParam('Hide_menu_titles');
        $ul_class = $this->getParam('Menu_class') . ($ul_hide_menu_titles ? " hide_titles" : "");
        $ul_li_class = $this->getParam('Menu_li_class');
        $ul_ul_class = $this->getParam('Menu_ul_class');
        $ul_a_class = $this->getParam('Menu_a_class');
        $data_toggle = "";
        $data_toggle_separate = $this->getParam('Menu_data_toggle_separate');
        $root_id = $this->getParam('Menu_root_id');
        $translate = $this->getParam('Translate');
        $max_levels = $this->getParam('Menu_max_levels');
        /*
		switch($this->getParam('Render_type')){
			case "2":
				$ul_class = "nav".($ul_class ? " " : "").$ul_class.($this->getParam('Hover_dropdown') ? " hover-dropdown" : "");
				$ul_li_class = "nav-item".($ul_li_class ? " " : "").$ul_li_class;
				$ul_a_class = "nav-link".($ul_a_class ? " " : "").$ul_a_class;
				$ul_ul_class = "tree-submenu collapse".($ul_ul_class ? " " : "").$ul_ul_class;
				// $data_toggle = "q5555 dropdown".($data_toggle ? " " : "").$data_toggle;
				break;
			case "3":
				break;
			case "4":
				$ul_class = "nav".($ul_class ? " " : "").$ul_class;
				$ul_li_class = "dropdown".($ul_li_class ? " " : "").$ul_li_class;
				$ul_a_class = "dropdown-toggle".($ul_a_class ? " " : "").$ul_a_class;
				$ul_ul_class = "dropdown-menu".($ul_ul_class ? " " : "").$ul_ul_class;
				$data_toggle = "dropdown".($data_toggle ? " " : "").$data_toggle;
				break;
			default:

				break;

		}
        */
        $ul_class = "nav" . ($ul_class ? " " : "") . $ul_class . ($this->getParam('Hover_dropdown') && $this->getParam('Render_type') == 2 ? " hover-dropdown" : "");
        $ul_li_class = "nav-item" . ($ul_li_class ? " " : "") . $ul_li_class;
        $ul_a_class = "nav-link" . ($ul_a_class ? " " : "") . $ul_a_class;
        $ul_ul_class = "tree-submenu collapse" . ($ul_ul_class ? " " : "") . $ul_ul_class;

        if ($root_id) {
            $model = new Menus(false, true, $ul_class, $ul_li_class, $ul_a_class, $ul_ul_class, $data_toggle, $data_toggle_separate);
            $menuHTML = $model->render($root_id, $ul_id, $translate, $max_levels);
            $auto_close = "true";
            if ($this->getParam('Render_type') == 4) $auto_close = "false";
            $menuHTML = "<div class=\"menu-tree-wrapper tree-wrapper w-100 menu-render-type-" . $this->getParam('Render_type') . "\" data-bs-auto-close=\"" . $auto_close . "\">" . $menuHTML . "</div>";
            switch ($this->getParam('Collapse_widget')) {
                case "1":
                    $adaptive_menu_title = $this->getParam('Collapsed_widget_title');
                    $widgetHTML .= "<div class=\"menu-widget navbar navbar-expand-lg p-0\">";
                    $widgetHTML .= "    <div class=\"widget-navbar-panel d-flex w-100 d-lg-none justify-content-between\">";
                    $widgetHTML .= "       <a class=\"navbar-brand topmenu_label\">" . $adaptive_menu_title . "</a>";
                    $widgetHTML .= "       <button type=\"button\" class=\"navbar-toggler\" data-bs-toggle=\"collapse\" data-bs-target=\"#" . $widget_id . "_wrapper\"><span class=\"navbar-toggler-icon\"></span></button>";
                    $widgetHTML .= "    </div>";
                    $widgetHTML .= "    <div id=\"" . $widget_id . "_wrapper\" class=\"collapse navbar-collapse flex-column pt-3 pt-lg-0\">" . $menuHTML . "</div>";
                    $widgetHTML .= "</div>";
                    break;
                case "2":
                    $adaptive_menu_title = $this->getParam('Collapsed_widget_title');
                    $widgetHTML .= "<div class=\"menu-widget navbar navbar-expand-lg p-0\">";
                    $widgetHTML .= "    <div class=\"widget-navbar-panel d-flex w-100 d-lg-none justify-content-between\">";
                    $widgetHTML .= "        <a class=\"navbar-brand topmenu_label\">" . $adaptive_menu_title . "</a>";
                    $widgetHTML .= "        <button type=\"button\" class=\"navbar-toggler\" data-bs-toggle=\"offcanvas\" data-bs-target=\"#" . $widget_id . "_wrapper\"><span class=\"navbar-toggler-icon\"></span></button>";
                    $widgetHTML .= "    </div>";
                    $widgetHTML .= "    <div id=\"" . $widget_id . "_wrapper\" class=\"offcanvas offcanvas-end\">";
                    $widgetHTML .= "        <div class=\"offcanvas-header\">";
                    $widgetHTML .= "            <a class=\"navbar-brand topmenu_label\">" . $adaptive_menu_title . "</a>";
                    $widgetHTML .= "            <button type=\"button\" class=\"btn-close\" data-bs-dismiss=\"offcanvas\"></button>";
                    $widgetHTML .= "        </div>";
                    $widgetHTML .= "        <div class=\"offcanvas-body flex-column pt-0\">" . $menuHTML . "</div>";
                    $widgetHTML .= "    </div>";
                    $widgetHTML .= "</div>";
                    break;
                default:
                    $widgetHTML = $menuHTML;
                    break;
            }

        }
        return $widgetHTML;
    }

    public function getTopItems()
    {
        if (!defined('_ADMIN_MODE')) return null;
        $sql = "SELECT mi_id AS id, mi_name AS name FROM #__menus WHERE mi_parent_id=0 ORDER by mi_ordering";
        Database::getInstance()->setQuery($sql);
        return Database::getInstance()->loadObjectList();
    }
}

?>