<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 852dd4da (2024-10-07 21:00:27)
// 

use ArfaCMS\Core\Event;
use ArfaCMS\Core\Files;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Widget;
use ArfaCMS\DB\Database;

defined('_ARFACMS_VALID') or die("Access denied");

class randarticleWidget extends Widget
{
    protected function setParamsMask()
    {
        parent::setParamsMask();
        $this->addParam("ParentarticleId", "table_select", 0, false, "SELECT a_id AS fld_id, a_title AS fld_name FROM #__articles ORDER BY fld_name");
        $this->addParam("ShowDopTitle", "boolean", 1);
        $this->addParam("DopTitle", "string", "");
        $this->addParam("LinkDopTitle", "string", "");
        $this->addParam("shortTextLength", "integer", 200);
    }

    public function render()
    {
        $psid = $this->getParam('ParentarticleId');
        $show_Doptitle = $this->getParam('ShowDopTitle');
        $Doptitle = $this->getParam('DopTitle');
        $LinkDoptitle = $this->getParam('LinkDopTitle');
        $shortTextLength = $this->getParam('shortTextLength');
        // получаем случайную статью, если указан псид, то ищем в дочерних, иначе по всей базе, исключая удаленные и выключенные
        $widgetHTML = "";
        $art = $this->getRandArticle($psid);
        if ($art) {
            if ($show_Doptitle && $Doptitle) {
                $dop_href = Router::_($LinkDoptitle);
                $widgetHTML .= "<div class=\"a-toplabel\"><a href=\"" . $dop_href . "\">" . $Doptitle . "</a></div>";
            }
            $img = $this->getImage($art->a_thumb);
            if ($img) {
                $widgetHTML .= "<div class=\"a-thumb\"><img src=\"" . $img . "\" alt=\"\" /></div>";
            }
            $href = Router::_("index.php?module=article&view=read&psid=" . $art->a_id . "&alias=" . $art->a_alias);
            $widgetHTML .= "<div class=\"a-title\">";
            $widgetHTML .= "<a href=\"" . $href . "\">" . $art->a_title . "</a>";
            $widgetHTML .= "</div>";
            $widgetHTML .= "<div class=\"a-text\">";
            Event::raise('content.prepare', array("clean_all" => "1"), $art->a_text);
            $first_hr = mb_strpos($art->a_text, '<hr id="system-readmore"', 0);
            if ($first_hr) $widgetHTML .= mb_substr($art->a_text, 0, $first_hr);
            // else $widgetHTML.=mb_substr(strip_tags($art->a_text), 0, $shortTextLength)."...";
            else $widgetHTML .= Text::cutHtml($art->a_text, $shortTextLength);

            $widgetHTML .= "</div>";
            $widgetHTML .= "<div class=\"a-readmore\"><a href=\"" . $href . "\">" . Text::_('Read more') . "</a></div>";
        } else $widgetHTML .= "";
        return $widgetHTML;
    }

    public function getRandArticle($aid = 0)
    {
        $query = "SELECT a.a_id FROM #__articles AS a WHERE a.a_published=1 AND a.a_deleted=0";
        if ($aid) $query .= " AND a.a_parent_id=" . intval($aid);
        Database::getInstance()->setQuery($query);
        if ($ids_res = Database::getInstance()->loadObjectList()) {
            $rand = rand(0, count($ids_res) - 1);
            $a_id = $ids_res[$rand]->a_id;
            $query = "SELECT a.*, u.u_nickname AS author FROM #__articles AS a";
            $query.= " LEFT JOIN #__users AS u ON a.a_author_id=u.u_id";
            $query.= " WHERE a.a_id=" . $a_id . " AND a.a_deleted=0";
            Database::getInstance()->setQuery($query);
            Database::getInstance()->loadObject($art);
            if ($art->a_parent_id != 0) {
                $query = "SELECT a_title FROM #__articles WHERE a_id=" . $art->a_parent_id;
                Database::getInstance()->setQuery($query);
                $art->parentTitle = strval(Database::getInstance()->loadResult());
            }
            return $art;
        } else return false;
    }

    public function getImage($img)
    {
        $imgpath = ARFACMS_STORAGE_PATH . "article" . DS . 'thumbs' . DS . Files::splitAppendix($img, true);
        if (file_exists($imgpath) && is_file($imgpath)) {
            $imgurl = ARFACMS_STORAGE_LINK . "/article/thumbs/" . Files::splitAppendix($img);
        } else $imgurl = "";
        return $imgurl;
    }
}

?>