<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 6f542a89 (2024-10-11 15:14:29)
// 

use ArfaCMS\Core\HTMLControls;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Router;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\User;
use ArfaCMS\Core\Util;
use ArfaCMS\Core\Widget;
use ArfaCMS\MVC\Module;

defined('_ARFACMS_VALID') or die("Access denied");

class usermenuWidget extends Widget
{
    protected function setParamsMask()
    {
        parent::setParamsMask();
        $this->addParam("show_greeting", "boolean", 1);
        $this->addParam("login_popup", "boolean", 1);
        $this->addParam("show_register", "boolean", 1);
        $this->addParam("show_reminder", "boolean", 1);
        $this->addParam("show_post", "boolean", 1);
        $this->addParam("show_basket", "boolean", 1);
        $this->addParam("basket_popup", "boolean", 1);
        $this->addParam("show_admin", "boolean", 1);
    }

    public function prepare()
    {
        Portal::getInstance()->addStyleSheet("modules/catalog.css", !\seoConfig::$tmplCSSBackCompatibility);
    }

    public function render()
    {
        $loginHTML = "<div class=\"cabinet-menu\">";
        if (User::getInstance()->isLoggedIn()) {
            // выясняем есть ли у пользователя непрочитанная почта
            $countMail = Module::getInstance('mail')->getModel('letter')->getCountUnreadLetter(User::getInstance()->u_id);
            // $countMail = 345; // Just for test
            if ($this->getParam('show_greeting')) $loginHTML .= "<p class=\"greeting\">" . Text::_("Greeting") . ", " . User::getInstance()->u_nickname . "</p>";
            $loginHTML .= "<ul class=\"cabinet-links\">";
            $loginHTML .= "<li class=\"cabinet\"><a rel=\"nofollow\" title=\"" . Text::_("Cabinet") . "\" href=\"" . Router::_("index.php?module=user&amp;view=panel&at=tab_1") . "\"><span class=\"l-text\">" . Text::_("Cabinet") . "</span></a></li>";
            if ($this->getParam('show_post')) $loginHTML .= "<li class=\"my-mail".($countMail ? " has-mail" : "")."\"><a rel=\"nofollow\" title=\"" . Text::_("My mail") . "\" href=\"" . Router::_("index.php?module=mail") . "\"><span class=\"l-text\">" . Text::_("My mail") . "</span>" . ($countMail ? "<span class=\"badge\">" . $countMail . "</span>" : "") . "</a></li>";
            if (!\catalogConfig::$ordersDisabled && $this->getParam('show_basket')) {
                if ($this->getParam('basket_popup')) {
                    $loginHTML .= "<li class=\"my-basket\"><a class=\"rel-popup-text\" title=\"" . Text::_("Basket") . "\" rel=\"nofollow\" href=\"" . Router::_("index.php?module=catalog&amp;task=ShowBasket&amp;option=ajax") . "\"><span class=\"l-text\">" . Text::_("Basket") . "</span></a></li>";
                } else {
                    $loginHTML .= "<li class=\"my-basket\"><a rel=\"nofollow\" title=\"" . Text::_("Basket") . "\" href=\"" . Router::_("index.php?module=catalog&view=orders") . "\"><span class=\"l-text\">" . Text::_("Basket") . "</span></a></li>";
                }
            }
            if (User::getInstance()->isAdmin() && $this->getParam('show_admin')) {
                $loginHTML .= "<li class=\"admin-link\"><a rel=\"nofollow\" title=\"" . Text::_("Administration") . "\" href=\"" . Router::_(ADMIN_FOLDER . "/index.php") . "\"><span class=\"l-text\">" . Text::_("Administration") . "</span></a></li>";
            }
            $loginHTML .= "<li class=\"logout\"><a rel=\"nofollow\" title=\"" . Text::_("Logout") . "\" href=\"" . Router::_("index.php?option=logout") . "\"><span class=\"l-text\">" . Text::_("Logout") . "</span></a></li>";
            $loginHTML .= "</ul>";
        } else {
            $loginHTML .= "<ul class=\"cabinet-links\">";
            if ($this->getParam('login_popup')) {
                $loginHTML .= "<li class=\"login\"><a data-modal-size=\"medium\" title=\"" . Text::_("Log in") . "\" data-modal-backdrop=\"0\" rel=\"nofollow\" class=\"rel-popup-text\" href=\"" . Router::_("index.php?module=user&amp;view=login") . "\"><span class=\"l-text\">" . Text::_("Log in") . "</span></a></li>";
            } else {
                $suffix = '';
                $loginHTML .= "<div class=\"auth-block" . $suffix . "\">";
                $loginHTML .= "<form action=\"" . Router::_("index.php") . "\" method=\"post\">";
                $loginHTML .= "<input type=\"hidden\" name=\"option\" value=\"login\" />";
                $loginHTML .= "<input type=\"hidden\" name=\"return_url\" value=\"" . Util::getReturnUrl() . "\" />";
                $loginHTML .= "<div class=\"login-label mb-2\">" . (\backofficeConfig::$allowEmailLogin ? HTMLControls::renderLabelField(false, 'Email', true) : HTMLControls::renderLabelField(false, 'Login name', true)) . ":</div>";
                $loginHTML .= "<div class=\"login-input mb-3\"><input type=\"text\"  class=\"commonEdit loginEdit form-control\" name=\"username\" value=\"\" /></div>";
                $loginHTML .= "<div class=\"pwd-label mb-2\">" . HTMLControls::renderLabelField(false, 'Password', true) . ":</div>";
                $loginHTML .= "<div class=\"pwd-input mb-3\"><input type=\"password\"  class=\"commonEdit loginEdit form-control\" name=\"userpass\" value=\"\" /></div>";
                $loginHTML .= "<div class=\"remember-label form-check\"><input type=\"checkbox\" id=\"remember_mew\" name=\"remember\" class=\"form-check-input\" />" . HTMLControls::renderLabelField('remember_mew', 'Remember me', true) . "</div>";
                $loginHTML .= "<div id=\"loginButtons_w\" class=\"buttons mt-3\">" . HTMLControls::renderButton("login_w", Text::_('Log in'), "submit") . "</div>";
                $loginHTML .= "</form>";
                $loginHTML .= "</div>";
            }
            if (!\backofficeConfig::$noRegistration && $this->getParam('show_register')) $loginHTML .= "	<li class=\"register\"><a rel=\"nofollow\" title=\"" . Text::_("Register") . "\" href=\"" . Router::_("index.php?module=user&amp;view=register") . "\"><span class=\"l-text\">" . Text::_("Register") . "</span></a></li>";
            if ($this->getParam('show_reminder')) $loginHTML .= "<li class=\"remind-password\"><a rel=\"nofollow\" class=\"rel-popup-text\" title=\"" . Text::_("Remind password") . "\" data-modal-size=\"small\" data-modal-backdrop=\"0\" href=\"" . Router::_("index.php?module=user&amp;view=confirm&amp;layout=password") . "\"><span class=\"l-text\">" . Text::_("Remind password") . "</span></a></li>";
            if (!\catalogConfig::$ordersDisabled && $this->getParam('show_basket')) $loginHTML .= "<li class=\"my-basket\"><a class=\"rel-popup-text\" title=\"" . Text::_("Basket") . "\" rel=\"nofollow\" href=\"" . Router::_("index.php?module=catalog&amp;task=ShowBasket&amp;option=ajax") . "\"><span class=\"l-text\">" . Text::_("Basket") . "</span></a></li>";
            $loginHTML .= "</ul>";
        }

        $loginHTML .= "</div>";
        return $loginHTML;
    }
}

?>